
/**
  * <code>PilotDocHeader</code> class modified for use with SuperWaba
  * November 2002 by Alexander Gro <a.gross@graumueller.com>.
  * Class not accessible, use <code>PilotDoc</cods> class instead.
  */
package superwaba.ext.palm.io.doc;
import waba.io.*;

class PilotDocHeader
{
   public PilotDocHeader()
   {

   }

   /**
     * constant to indicate PilotDoc is uncompressed
     * @since 1.0
     */
   public final static int UNCOMPRESSED = 1;

   /**
     * constant to indicate PilotDoc is compressed
     * @since 1.0
     */
   public final static int COMPRESSED = 2;

   /**
     * size of text record
     * @since 1.0
     */
   public final static int textRecordSize = 4096;

   /**
     * version of PilotDoc
     * @since 1.0
     */
   public int version = 0; // 2 Word	1=plain text, 2=compressed

   /**
     * spare (unknown)
     * @since 1.0
     */
   public int spare = 0; // 2 Word

   /**
     * length of uncompressed PilotDoc
     * @since 1.0
     */
   public int storyLen = 0; // 4 DWord	in bytes, when decompressed

   /**
     * number of PilotDoc records
     * @since 1.0
     */
   public int numRecords = 0; // 2 Word	text records

   /**
     * uncompressed text record size
     * @since 1.0
     */
   public int recordSize = textRecordSize; // 2 Word	usually 0x1000

   /**
     * currently viewed position in the document
     * not used by all readers
     * @since 1.0
     */
   public int position = 0; // 4 DWord

   // 16 bytes for Document Header

   /**
     * The size of the DocumentHeader in bytes
     * @return the number of bytes in the DocumentHeader
     * @since 1.0
     */
   static int getSize()
   {
      return (16);
   }

   /**
     * Reads the DocumentHeader from the DataInput
     * @param the DataInput to read from
     * @since 1.0
     */
   void read(DataStream ds)
   {
      version = ds.readUnsignedShort();
      spare = ds.readUnsignedShort();
      storyLen = ds.readInt();
      numRecords = ds.readUnsignedShort();
      recordSize = ds.readUnsignedShort();
      position = ds.readInt();
   }

   /**
     * Writes the DocumentHeader to the DataInput
     * @param the DataInput to write to
     * @since 1.0
     */
   int write(DataStream out)
   {
      int totout = out.writeShort((short)version);
      totout += out.writeShort((short)spare);
      totout += out.writeInt(storyLen);
      totout += out.writeShort((short)numRecords);
      totout += out.writeShort((short)recordSize);
      totout += out.writeInt(position);
      return totout;
   }

   /**
     * indicates whether the document is compressed or not
     * @since 1.1
     */
   boolean isCompressed()
   {
      return version == COMPRESSED;
   }
}
