package superwaba.ext.xplat.html;

import waba.fx.*;
import waba.ui.*;

/**
 * The <code>Element</code> class is the base class for objects
 * in the HTML document.  These include regions of text, forms,
 * input controls of forms and in the future, images.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Element
{
  /**
   * The attributes associated with this element.
   */
  Attributes attr;

  /**
   * The set of rectangles associated with this element.
   */
  Region region = new Region();

  /**
   * Construct a new element given the attributes to be associated
   * with it.
   * @param attr attributes for this element.
   */
  public Element(Attributes attr)
  {
    this.attr = attr;
  }

  /**
   * The layout method is called after all elements have been
   * parsed.  Each element needs to update the layout context
   * provided by allocating the required space to view the element
   * in the page.
   * @param r HtmlRenderer instance that will display this element.
   * @param c LayoutContext instance to be updated.
   */
  public void layout(HtmlRenderer r, LayoutContext c)
  {
  }

  /**
   * This method is called to paint the element.
   * @param r HtmlRenderer associated with this element.
   * @param g Graphics context to use to draw.
   */
  public void onPaint(HtmlRenderer r, Graphics g)
  {
  }

  /**
   * This method is called when the mouse enters the region
   * of this element.
   * @param r HtmlRenderer associated with this element.
   * @param g Graphics context to use to draw.
   */
  public void mouseEnter(HtmlRenderer r, Event e)
  {
  }

  /**
   * This method is called when the mouse exits the region
   * of this element.
   * @param r HtmlRenderer associated with this element.
   * @param g Graphics context to use to draw.
   */
  public void mouseExit(HtmlRenderer r, Event e)
  {
  }

  /**
   * This method is called when the mouse is pressed over the
   * region defined by this element.
   * @param r HtmlRenderer associated with this element.
   * @param g Graphics context to use to draw.
   */
  public void mousePress(HtmlRenderer r, PenEvent e)
  {
  }

  /**
   * This method is called when the mouse is released over the
   * region defined by this element.
   * @param r HtmlRenderer associated with this element.
   * @param g Graphics context to use to draw.
   */
  public void mouseRelease(HtmlRenderer r, PenEvent e)
  {
  }

  /**
   * This utility function breaks the current line in the layout context
   * to start a new one.  Paragraph tags and various other items
   * break the current line and start a new one.
   * @param lc layout context to update.
   * @param forcebreak true if a break should be "forced" even with no text on the current line.
   */
  protected void breakLine(LayoutContext lc, boolean forcebreak)
  {
    if (lc.textOnLine || forcebreak)
    {
      lc.textOnLine = false;
      // first finish off last line.
      lc.x = 0;//attr.indent;
      lc.y += lc.lineHeight;
      lc.preferredHeight += lc.lineHeight;
      lc.lineHeight = 0;
    }
  }

  /**
   * This utility function breaks the current line in the layout context
   * to start a new one.  Paragraph tags and various other items
   * break the current line and start a new one.
   * @param lc layout context to update.
   */
  protected void breakLine(LayoutContext lc)
  {
    breakLine(lc, false);
  }

}
