package superwaba.ext.xplat.html;

import waba.util.*;

/**
 * The <code>Form</code> class is a document element created
 * for the &lt;FORM&gt; tag. It saves the method and action of the
 * form and causes the renderer to perform the method and action
 * when the form is submitted.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Form extends Element
{
  String method;
  String url;
  HtmlRenderer renderer;
  Vector inputs;

  /**
   * Construct a new for given the tag, arguments and
   * current set of attributes.
   * @param tag tag for this form (i.e. <FORM>)
   * @param args arguments for the tag.
   * @param attr HTML attributes to this point.
   */
  public Form(int tag, Hashtable args, Attributes attr)
  {
    super(attr);
    String val;

    inputs = new Vector(4);

    if ((val = (String)args.get("method")) != null)
      method = val;
    if ((val = (String)args.get("action")) != null)
      url = val;
  }

  /**
   * Override the layout method to break the form text onto
   * a new line.
   * @param r renderer to submit the form to.
   * @param lc layout context
   */
  public void layout(HtmlRenderer r, LayoutContext lc)
  {
    renderer = r;   // save the html renderer for the submit.
    breakLine(lc);  // forms start a new line.
  }

  /**
   * This method is called by the submit button of the form
   * to submit the contents of the form to the server.
   */
  public void submit()
  {
    String link = url;

    Vector v = getNamedValues();
    int n = v.size();
    String s;
    boolean firstarg = true;
    for (int i =0; i < n; i++)
    {
      s = (String)v.items[i];
      if (firstarg)
      {
         link = link + "?" + s;
         firstarg = false;
      }
      else
      {
         link = link + "&" + s;
      }
    }
    //System.out.println("goto " + link);
    renderer.linkSelection(link);
  }

  /**
   * This method extracts the form contents for use when the
   * for is submitted.  All named for elements are passed in the URL.
   * @return vector of name value pairs separated by an equal sign.
   */
  public Vector getNamedValues()
  {
    Vector rv = new Vector(4);

    Input item;
    String val;
    int n = inputs.size();
    for (int i =0; i < n; i++)
    {
      item = (Input)inputs.items[i];
      if (item.name != null)
      {
         val = item.getValue();
         if (val != null)
            rv.add(item.name + "=" + val);
      }
    }
    return rv;
  }
}
