package superwaba.ext.xplat.html;

import waba.util.*;
import waba.fx.*;

/**
 * The <code>Hr</code> class draws a horizontal line across the
 * HTML renderer that is associated with the HR tag.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Hr extends Element
{
  static final int HR_SPACE = 2;
  int y;
  int w;

  /**
   * Construct a new horizontal line object.
   * @param tag tag (i.e. HR)
   * @param args arguments (of which there are none)
   * @param attr attributes to this point.
   */
  public Hr(int tag, Hashtable args, Attributes attr)
  {
    super(attr);
  }

  /**
   * The layout method breaks the current line then adds
   * space for itself in the layout context.
   * @param r renderer in which this object is drawn.
   * @param lc layout context to be updated.
   */
  public void layout(HtmlRenderer r, LayoutContext lc)
  {
    breakLine(lc);
    y = lc.y + HR_SPACE;
    int sz = 2*HR_SPACE + 1;
    lc.y += sz;
    lc.preferredHeight += sz;
    w = lc.rendererWidth;
  }

  /**
   * Override onPaint to draw the horizontal line.
   * @param r renderer in which this object is drawn.
   * @param lc layout context to be updated.
   */
  public void onPaint(HtmlRenderer r, Graphics g)
  {
    g.drawLine(3, y, w-3, y);
  }
}
