package superwaba.ext.xplat.html;

import waba.ui.*;
import waba.fx.*;

/**
 * The <code>HtmlEdit</code> is a special subclass of the Waba edit
 * control to be used in an HTML form.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class HtmlEdit extends Edit implements HtmlControl
{
  HtmlRenderer r;
  int len;

  /**
   * Construct an HtmlEdit.  The default text length is 10 characters.
   */
  public HtmlEdit()
  {
    super();
    len = 10;
  }

  /**
   * Set the renderer associated with this HTML control.
   * @param r renderer for this control
   */
  public void setRenderer(HtmlRenderer r)
  {
    this.r = r;
    parent = r;
  }

  /**
   * Override the repaint method because since it's inside the form
   * it doesn't work properly.
   */
  public void repaint()
  {
    if (r != null)
       r.requestRepaint(this);
  }

  /**
   * Set the length of this field (based on the attributes of
   * the HTML tag).  This is used to determine the size of the
   * field during layout.
   * @param len length (in characters) of this field
   */
  public void setLength(int len)
  {
    this.len = len;
  }

  /**
   * Override the createGraphics method in order to get the
   * text edit control to work properly in a the scrolled
   * HTML renderer.  This simply calls super.createGraphics()
   * then calls Graphics.translate to make up for the scrolling.
   * @return graphics object for this Edit control.
   */
  public Graphics createGraphics()
  {
    Graphics g = super.createGraphics();
    //System.out.println("create graphics " + x + " " + y);
    g.translate(-r.hOffset, -r.vOffset);
    return g;
  }
}
