package superwaba.ext.xplat.html;

import waba.ui.*;
import waba.fx.*;
import waba.util.*;

/**
 * The <code>HtmlRenderer</code> accepts a Document returned
 * by the HtmlParser and renderers it.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class HtmlRenderer extends Container
{
  Vector listeners;

  int verticalScrollInc;
  int horizontalScrollInc;
  int preferredHeight;
  int preferredWidth;

  int viewheight;
  int viewwidth;
  int vOffset = 0;
  int hOffset = 0;

  LayoutContext lc;
  Document doc;
  Vector vdoc;
  Element mouseOver;

  /**
   * Construct a new HtmlRenderer.
   */
  public HtmlRenderer()
  {
    listeners = new Vector(1);

    verticalScrollInc = fmH;
    horizontalScrollInc = (fm.getTextWidth("Wm") / 2);
  }

  /**
   * Implement the Scrollable.setViewSize method and save the
   * viewheight and viewwidth in order to be able to layout the
   * HTML properly.
   * @param viewwidth width of viewable area.
   * @param viewheight height of the viewable area.
   */
  public void setViewSize(int viewwidth, int viewheight)
  {
    //System.out.println("set view size " + viewwidth + " " + viewheight);
    this.viewheight = viewheight;
    this.viewwidth = viewwidth;
  }

  /**
   * Return the preferred size of the renderer for use by the
   * scroller in determining the size of the scrollbars.
   * @return perferred dimensions of the renderer.
   */
  public int getPreferredWidth()
  {
    return preferredWidth;
  }

  /**
   * Return the preferred size of the renderer for use by the
   * scroller in determining the size of the scrollbars.
   * @return perferred dimensions of the renderer.
   */
  public int getPreferredHeight()
  {
    return preferredHeight;
  }

  /**
   * This method is called when the user adjusts the vertical
   * scrollbar.  The value provided indicates the number of pixels
   * that the text should be scrolled by.
   * @param val current scrollbar value.
   */
  public void verticalScroll(int val)
  {
    vOffset = val;
    getParentWindow().eraseBackgroundNow = true; // guich@400_34
    repaint();
  }

  /**
   * This method is called when the user adjusts the horizontal
   * scrollbar.  The value provided indicates the number of pixels
   * that the text should be scrolled by.
   * @param val current scrollbar value.
   */
  public void horizontalScroll(int val)
  {
    hOffset = val;
    getParentWindow().eraseBackgroundNow = true; // guich@400_34
    repaint();
  }

  /**
   * Invalidate the contents of the renderer.  This method will
   * call the layout method of each element in the document
   * in order to determine the complete layout of the HTML document.
   */
  public void invalidate()
  {
    // relayout the content.
    lc = new LayoutContext();

    //System.out.println("invalidate");
    lc.rendererWidth = viewwidth;
    lc.rendererHeight = viewheight;

    //Iterator iter = new Iterator(doc.children);
    Element item;
    int n = vdoc.size();
    for (int i =0; i < n; i++)
    {
      item = (Element)vdoc.items[i];
      item.layout(this, lc);
      //System.out.println(item);
    }
    //lc.preferredHeight += 5; // space at bottom.
    // end current line
    if (lc.textOnLine)
      lc.preferredHeight += lc.lineHeight;

    preferredWidth = lc.preferredWidth;
    preferredHeight = lc.preferredHeight;
    //System.out.println("preferred size: " + preferredWidth + " " + preferredHeight);
  }

  /**
   * This method is called to set the document to be displayed in
   * the renderer.  This method calls invalidate to layout the contents
   * then invalidates the scroller to update the scrollbars.
   * @param doc document to be rendered.
   */
  public void setDocument(Document doc)
  {
    this.doc = doc;
    vdoc = doc.children;

    // reset previous scrolling data
    vOffset = 0;
    hOffset = 0;

    invalidate();

    Control parent = getParent();
/* guich: need to find a substitute for Scroller
    if ((parent != null) && (parent instanceof Scroller))
    {
      //System.out.println("scroller invalidate");
      ((Scroller)parent).invalidate();
    }*/

    repaint();
  }

  /**
   * Return the amount that the scrollbar should scroll the display
   * when the vertical arrows are pressed.
   * @return the average character height for scrolling.
   */
  public int getHorizontalScrollIncrement()
  {
    return horizontalScrollInc;
  }

  /**
   * Return the amount that the scrollbar should scroll the display
   * when the horizontal arrows are pressed.
   * @return the character width for scrolling.
   */
  public int getVerticalScrollIncrement()
  {
    return verticalScrollInc;
  }

  /**
   * Implement the onPaint method to draw the document elements.
   * @param g graphics context to use for painting.
   */
  public void onPaint(Graphics g)
  {
    g.translate(-1*hOffset,-1*vOffset);

    if (doc != null)
    {
       Element item;
       int n = vdoc.size();
       for (int i =0; i < n; i++)
       {
         item = (Element)vdoc.items[i];
         item.onPaint(this, g);
      }
    }

    g.translate(hOffset,vOffset);
  }

  /**
   * Implement the onEvent method to distribute the event to the
   * appropriate document element.
   * @param event event to be processed.
   */
  public void onEvent(Event event)
  {
    if (doc == null)
      return;

    switch (event.type)
    {
       case PenEvent.PEN_MOVE:
       {
         // update pen event coordinates due to scrolling.
         PenEvent pevent = (PenEvent)event;
         pevent.x += hOffset;
         pevent.y += vOffset;

         boolean foundOne = false;
         Element item;
         int n = vdoc.size();
         for (int i =0; i < n; i++)
         {
            item = (Element)vdoc.items[i];
            if (item.region.contains(pevent.x, pevent.y))
            {
               foundOne = true;
               if (item != mouseOver)
               {
                  if (mouseOver != null)
                  mouseOver.mouseExit(this, event);
                  mouseOver = item;
                  item.mouseEnter(this, event);
               }
            }
         }
         if ((foundOne == false) && (mouseOver != null))
         {
            mouseOver.mouseExit(this, event);
            mouseOver = null;
         }
       }
       break;
       case PenEvent.PEN_UP:
       case PenEvent.PEN_DOWN:
       case PenEvent.PEN_DRAG:
       {
         // update pen event coordinates.
         PenEvent pevent = (PenEvent)event;
         pevent.x += hOffset;
         pevent.y += vOffset;

         Element item;
         int n = vdoc.size();
         for (int i =0; i < n; i++)
         {
            item = (Element)vdoc.items[i];
            if (item.region.contains(((PenEvent)event).x, ((PenEvent)event).y))
            {
               if (event.type == PenEvent.PEN_UP)
                  item.mouseRelease(this, pevent);
               else
                  item.mousePress(this, pevent);
            }
         }
       }
     }
     super.onEvent(event);
  }

  /**
   * Set the message to the text provided that is displayed at the
   * bottom of the browser window or whatever object contains
   * this renderer.
   * @param msg message to be displayed.
   */
  public void setMessage(String msg)
  {
    RendererListener item;
    int n = listeners.size();
    for (int i =0; i < n; i++)
    {
      item = (RendererListener)listeners.items[i];
      item.setMessage(msg);
    }
  }

  /**
   * This method is called when the user selects a anchor
   * object to bring up a new page.
   * @param url url to go to.
   */
  public void linkSelection(String url)
  {
    RendererListener item;
    int n = listeners.size();
    for (int i =0; i < n; i++)
    {
      item = (RendererListener)listeners.items[i];
      item.linkSelection(url);
    }
  }

  /**
   * Add a listener for rendering events.
   * @param l listener to be notified when events happen.
   */
  public void addRendererListener(RendererListener l)
  {
    listeners.add(l);
  }

  /**
   * Remove the listener provided from the list of listeners.
   * @param l listener to be notified when events happen.
   */
  public void removeRendererListener(RendererListener l)
  {
    listeners.del(l);
  }

  /**
   * This function is called by the form controls to request a
   * repaint for a particular region of the display.  This workaround
   * is required due to clipping and other issues.
   * @param c control to be repainted.
   */
  public void requestRepaint(Control c)
  {
    Graphics g = createGraphics();
    restoreClip(g);
    g.translate(-1*hOffset,-1*vOffset);
    Rect r = c.getRect();
    g.translate(r.x, r.y);
    g.setBackColor(Color.WHITE);
    g.fillRect(0, 0, r.width, r.height);
    c.onPaint(g);
    g.free();
  }

  /**
   * Restore the clip to the entire contents of the renderer.
   * @param g graphics to be updated.
   */
  public void restoreClip(Graphics g)
  {
    Rect rr = getRect();
    // the edit control messes with the clip, so fix it.
    g.setClip(hOffset + rr.x, vOffset + rr.y, rr.width, rr.height);
    //System.out.println("clip set to: " + String.valueOf(hOffset + rr.x) + " " + String.valueOf(vOffset + rr.y) + " " + rr.width + " " + rr.height);

  }
}
