package superwaba.ext.xplat.html;

import waba.util.*;
import waba.sys.*;
import waba.fx.*;
import waba.ui.*;

/**
 * The <code>HtmlTable</code> class is responsible for renderering HTML HtmlTable
 * elements.
 * <P>
 * Please note that the implementation of this class is not complete.
 * Events (mouse press and release) are not passed to HtmlTable cells.
 * The only attribute supported for the HtmlTable is the border attribute
 * which draws a rectangle around each cell.
 *
 * @author Jim Guistwite
 */

class HtmlTable extends Element implements ElementCollection
{
  Vector rows;
  Vector currentRow;
  TableCell currentCell;
  IntVector colWidths;
  IntVector rowHeights;
  int borderWidth;
  int width;
  int height;

  /**
   * Construct a HtmlTable given its tag, Hashtable and current attributes.
   * @param tag associated with the HtmlTable (i.e. HtmlTable)
   * @param p   arguments associated with this tag.
   * @param attr current attributes for text.
   */
  public HtmlTable(int tag, Hashtable p, Attributes attr)
  {
    super(attr);

    rows = new Vector(4);

    String val;
    if ((val = (String)p.get("border")) != null)
      borderWidth = Convert.toInt(val);
  }

  /**
   * Implement the onPaint method to paint each cell.
   * @param renderer HTML renderer into which paint ops should take place.
   * @param g graphics context to draw with.
   */
  public void onPaint(HtmlRenderer renderer, Graphics g)
  {
    // paint each cell.
    TableCell cell;
    int n = rows.size();
    Vector row;
    for (int i =0; i < n; i++)
    {
      row = (Vector)rows.items[i];
      int nn = row.size();
      for (int ii = 0; ii < nn; ii++)
      {
         cell = (TableCell)row.items[ii];
         cell.onPaint(renderer, g);
      }
    }
  }

  /**
   * Implement the mouseRelease method to pass the event to the
   * appropriate cell.
   *
   * @param renderer HTML renderer into which paint ops should take place.
   * @param event pen event to be processed.
   */
  public void mouseRelease(HtmlRenderer renderer, PenEvent event)
  {
    // forward event to appropriate cell.
  }

  /**
   * Implement the mousePress method to pass the event to the
   * appropriate cell.
   *
   * @param renderer HTML renderer into which paint ops should take place.
   * @param event pen event to be processed.
   */
  public void mousePress(HtmlRenderer renderer, PenEvent event)
  {
    // forward event to appropriate cell.
  }


  /**
   * Implement the layout method to layout the contents of the HtmlTable.
   * <P>
   * This first step is to determine the size of each row and column
   * by performing a layout operation on the contents of each cell.
   * Once this size is known, the algorithm attempts to resize each
   * column by its percentage of the entire with in order to make the
   * HtmlTable fit the width of the renderer.  Once this is accomplished,
   * each column width is determined and the layout method for each
   * cell can be called.
   * be
   *
   * @param renderer HTML renderer into which paint ops should take place.
   * @param lc layout context to update.
   */
  public void layout(HtmlRenderer renderer, LayoutContext lc)
  {
    int n;
    // determine each cell's size and update the layout context appropriately.

    // tables start a new line.
    breakLine(lc, false);

    colWidths = new IntVector(4);
    rowHeights = new IntVector(4);

    TableCell cell;
    n = rows.size();
    Vector row;
    for (int i =0; i < n; i++)
    {
      row = (Vector)rows.items[i];
      int nn = row.size();
      for (int columnIdx = 0; columnIdx < nn; columnIdx++)
      {
         cell = (TableCell)row.items[columnIdx];
         //columnCount += cell.colSpan;

         Coord psize = cell.determinePreferredSize(renderer, 1024, 1024);
         if (columnIdx >= colWidths.getCount())
            colWidths.add(psize.x);
         else
         {
           int integ = colWidths.items[columnIdx];
           if (psize.x > integ)
              colWidths.setElementAt(psize.x, columnIdx);
         }
       }
    }

    n = colWidths.getCount();
    int totalWidth = borderWidth*n;
    for (int ii = 0; ii < n; ii++)
    {
      int i = colWidths.items[ii];
      totalWidth += i;
    }

    if (totalWidth > lc.rendererWidth)
    {
      int diff = totalWidth - lc.rendererWidth;
      float percentage;
      for (int ii = 0; ii < n; ii++)
      {
         int i = colWidths.items[ii];
         percentage = (float)i / (float)totalWidth;
         colWidths.setElementAt((int)(i - percentage*diff),i);
      }
      // run through each with these to recalculate column widths.
      n = rows.size();
      for (int i =0; i < n; i++)
      {
         row = (Vector)rows.items[i];
         int nn = row.size();
         for (int columnIdx = 0; columnIdx < nn; columnIdx++)
         {
           cell = (TableCell)row.items[columnIdx];
           int integ = colWidths.items[columnIdx];
           cell.determinePreferredSize(renderer, integ, 1024);
           if (cell.psize.x > integ)
              colWidths.setElementAt(cell.psize.x, columnIdx);
         }
      }
    }

    width = borderWidth*n;
    for (int ii = 0; ii < n; ii++)
    {
      int i = colWidths.items[ii];
      width += i;
    }

    height = 0;

    // now we can layout each cell.
    n = rows.size();
    int currentx = (int)(attr.indent);
    int currenty = lc.y;
    int maxheight = 0;
    for (int i =0; i < n; i++)
    {
      row = (Vector)rows.items[i];
      int nn = row.size();

      currentx = (int)(attr.indent);
      currenty = currenty + maxheight + borderWidth;

      maxheight = 0;

      for (int columnIdx = 0; columnIdx < nn; columnIdx++)
      {
         cell = (TableCell)row.items[columnIdx];

         int integ = colWidths.items[columnIdx];
         //cell.setSize(integ.i, 400);
         LayoutContext lc2 = new LayoutContext();
         lc2.rendererWidth = cell.psize.x;
         lc2.rendererHeight = cell.psize.y;
         cell.layout(renderer, lc2);
         if (lc2.preferredHeight > maxheight)
            maxheight = lc2.preferredHeight;

         cell.translate(currentx, currenty);
         currentx += (integ + borderWidth);
      }

      height += maxheight+borderWidth;
      rowHeights.add(maxheight);
    }

    if (width > lc.preferredWidth)
       lc.preferredWidth = width;
    lc.preferredHeight += height;
  }

  public void startRow(Hashtable hashtable, Attributes attr)
  {
    currentRow = new Vector(4);
    rows.add(currentRow);
  }

  public void endRow()
  {
    currentRow = null;
  }

  public void startCell(Hashtable hashtable)
  {
    if (currentRow != null)
    {
      currentCell = new TableCell();
      currentRow.add(currentCell);
    }
  }

  public void endCell()
  {
    currentCell = null;
  }

  class TableCell
  {
    Vector elements;
    int colSpan;
    Coord psize;

    public TableCell()
    {
      elements = new Vector(2);
      colSpan = 1;
      psize = new Coord(0,0);
    }

    public void layout(HtmlRenderer renderer, LayoutContext lc2)
    {
      int n = elements.size();
      Element item;
      for (int i = 0; i < n; i++)
      {
         item = (Element)elements.items[i];
         item.layout(renderer, lc2);
      }
      // end current line
      if (lc2.textOnLine)
         lc2.preferredHeight += lc2.lineHeight;
    }

    public void translate(int x, int y)
    {
      Element item;
      int n = elements.getCount();
      for(int i = 0; i < n; i++)
      {
         item = (Element)elements.items[i];
         item.region.translate(x, y);
      }
    }

    public Coord determinePreferredSize(HtmlRenderer renderer)
    {
      return determinePreferredSize(renderer, psize.x, psize.y);
    }

    public Coord determinePreferredSize(HtmlRenderer renderer, int width, int height)
    {
      LayoutContext lc2 = new LayoutContext();
      lc2.rendererWidth = width;
      lc2.rendererHeight = height;
      int n = elements.getCount();
      for(int i = 0; i < n; i++)
      {
         Element item = (Element)elements.items[i];
         item.layout(renderer, lc2);
      }
      // end current line
      if (lc2.textOnLine)
         lc2.preferredHeight += lc2.lineHeight;

      psize.x = lc2.preferredWidth;
      psize.y = lc2.preferredHeight;
      return psize;
    }

    public void onPaint(HtmlRenderer renderer, Graphics g)
    {
      if (borderWidth > 0)
      {
         g.drawRect(0, 0, width, height);
         int n = rowHeights.size();
         int i;
         int x = 0;
         int y = 0;
         for (int ii = 0; ii < n; ii++)
         {
           i = rowHeights.items[ii];
           y += i+1;
           if (ii < n-1)
             g.drawLine(0, y, width, y);
         }
         n = colWidths.size();
         for (int ii = 0; ii < n; ii++)
         {
           i = colWidths.items[ii];
           x += i+1;
           if (ii < n-1)
             g.drawLine(x, 0, x, height-1);
         }
      }

      Element item;
      Rect r;
      int n = elements.size();
      for(int i = 0; i < n; i++)
      {
         item = (Element)elements.items[i];
         //r = item.region.getBounds();
         //g.setClip(r.x, r.y, r.width, r.height);
         item.onPaint(renderer, g);
         //renderer.restoreClip(g);
         //g.drawRect(r.x, r.y, r.width, r.height);
      }
    }
  }

  public void add(Element e)
  {
    if (currentCell != null)
       currentCell.elements.add(e);
  }
}
