package superwaba.ext.xplat.html;

/**
 * The <code>HtmlTag</code> defines the set of tags with integer
 * values that are supported by the renderer.  Rather than doing
 * string comparisons thoughout the renderer, each tag is converted
 * to its integer number here to speed up processing of the document.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public final class HtmlTag
{
  public static final int A = 1;
  public static final int ACRONYM = 2;
  public static final int ADDRESS = 3;
  public static final int APPLET = 4;
  public static final int AREA = 5;
  public static final int B= 6;
  public static final int BASE = 7;
  public static final int BASEFONT = 8;
  public static final int BGSOUND = 9;
  public static final int BIG = 10;
  public static final int BLOCKQUOTE = 11;
  public static final int BODY = 12;
  public static final int BR = 13;
  public static final int BUTTON = 14;
  public static final int CAPTION = 15;
  public static final int CENTER = 16;
  public static final int CITE = 17;
  public static final int CODE = 18;
  public static final int COL = 19;
  public static final int COLGROUP = 20;
  public static final int COMMENT = 21;
  public static final int DD = 22;
  public static final int DEL = 23;
  public static final int DFN = 24;
  public static final int DIR = 25;
  public static final int DIV = 26;
  public static final int DL = 27;
  public static final int DT = 28;
  public static final int EM = 29;
  public static final int EMBED = 30;
  public static final int FIELDSET = 31;
  public static final int FONT = 32;
  public static final int FORM = 33;
  public static final int FRAME = 34;
  public static final int FRAMESET = 35;
  public static final int HEAD = 36;
  public static final int H1 = 37;
  public static final int H2 = 38;
  public static final int H3 = 39;
  public static final int H4 = 40;
  public static final int H5 = 41;
  public static final int H6 = 42;
  public static final int HR = 43;
  public static final int HTML = 44;
  public static final int I = 45;
  public static final int IFRAME = 46;
  public static final int IMG = 47;
  public static final int INPUT = 48;
  public static final int INS = 49;
  public static final int ISINDEX = 50;
  public static final int KBD = 51;
  public static final int LABEL = 52;
  public static final int LEGEND = 53;
  public static final int LI = 54;
  public static final int LINK = 55;
  public static final int LISTING = 56;
  public static final int MAP = 57;
  public static final int MARQUEE = 58;
  public static final int MENU = 59;
  public static final int META = 60;
  public static final int NOBR = 61;
  public static final int NOFRAMES = 62;
  public static final int NOSCRIPT = 63;
  public static final int OBJECT = 64;
  public static final int OL = 65;
  public static final int OPTION = 66;
  public static final int P = 67;
  public static final int PARAM = 68;
  public static final int PLAINTEXT = 69;
  public static final int PRE = 70;
  public static final int Q = 71;
  public static final int S = 72;
  public static final int SAMP = 73;
  public static final int SCRIPT = 74;
  public static final int SELECT = 75;
  public static final int SMALL = 76;
  public static final int SPAN = 77;
  public static final int STRIKE = 78;
  public static final int STRONG = 79;
  public static final int STYLE = 80;
  public static final int SUB = 81;
  public static final int SUP = 82;
  public static final int HtmlTable = 83;
  public static final int TBODY = 84;
  public static final int TD = 85;
  public static final int TEXTAREA = 86;
  public static final int TFOOT = 87;
  public static final int TH = 88;
  public static final int THEAD = 89;
  public static final int TITLE = 90;
  public static final int TR = 91;
  public static final int TT = 92;
  public static final int U = 93;
  public static final int UL = 94;
  public static final int VAR = 95;
  public static final int WBR = 96;
  public static final int XMP = 97;

  public static final int TOTALTAGS = 98;

  private static final char[] idx = {1, // a
				     6, // b
				     15,// c
				     22,// d
				     29,// e
				     31,// f
				     36,//(char)-1, // g
				     36,// h
				     45,// i
				     51,//(char)-1, // j

				     51,// k
				     52,// l
				     57,// m
				     61,// n
				     64,// o
				     67,// p
				     71,// q
				     72,//(char)-1,//r
				     72,// s
				     83,// t
				     93,// u
				     95,// v
				     96,// w,
				     97,// x
				     98,//(char)-1,
				     (char)-1};

  public static final String[] tags = {"unknown",
				       /*1*/"a","acronym","address","applet","area",
				       /*6*/"b","base","basefont","bgsound","big","blockquote","body","br","button",
				       /*15*/"caption","center","cite","code","col","colgroup","comment",
				       /*22*/"dd","del","dfn","dir","div","dl","dt",
				       /*29*/"em","embed",
				       /*31*/"fieldset","font","form","frame","frameset",
				       /*36*/"head","h1","h2","h3","h4","h5","h6","hr","html",
				       /*45*/"i","iframe","img","input","ins","isindex",
				       /*51*/"kbd",
				       /*52*/"label","legend","li","link","listing",
				       /*57*/"map","marquee","menu", "meta",
				       /*61*/"nobr","noframes","noscript",
				       /*64*/"object", "ol", "option",
				       /*67*/"p", "param", "plaintext", "pre",
				       /*71*/"q",
				       /*72*/"s","samp","script","select","small","span","strike","strong","style","sub","sup",
				       /*83*/"HtmlTable","tbody","td","textarea","tfoot","th","thead","title","tr","tt",
				       /*93*/"u","ul",
				       /*95*/"var","wbr",
				       /*97*/"xmp"};

  /**
   * Return the integer value corresponding to the tag provided.
   * @param tag tag to be translated (must be in lowercase).
   * @return integer value for the tag provided or -1.
   */
  public static int toInt(String tag)
  {
    int letter = tag.charAt(0)-'a';
    int end = idx[letter+1];   // guich@330_31: why go to end? go to letter+1
    for(int i=idx[letter];i<end;i++)
      if (tag.equals(tags[i]))
         return i;
    return -1;
  }
}
