package superwaba.ext.xplat.html;

import waba.util.*;
import waba.sys.*;

/**
 * The <code>HtmlTokenizer</code> converts an input string into
 * a sequence of HtmlTags.  This uses a listeners mechanism
 * similar to the SAX parser for XML.  The caller registers
 * an instance of HtmlListener with the tokenizer and the listener
 * is called when a tag starts, ends and when characters or comments
 * are read.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class HtmlTokenizer
{
   HtmlListener listener;
   int newlineSignificant;
   boolean inTag;
   boolean inKey;
   boolean inVal;
   boolean inQuote;
   boolean inComment;
   char quoteChar;
   Hashtable props;
   String key;
   String tag;

   /**
    * Construct a new HtmlTokenizer.
    */
   public HtmlTokenizer()
   {
     props = new Hashtable(50);
   }

   /**
    * Parse the input provided calling the listeners
    * for each tag and character sequence read.
    * @param input HTML content to be parsed.
    */
   public void parse(String input)
   {
      char c;
      char nextc;

      CharBuf buf = new CharBuf();
      char chars[] = input.toCharArray();

      int len = chars.length;

      for (int i = 0; i < len; i++)
      {
         c = chars[i];
         if (i != (len-1))
            nextc = chars[i+1];
         else
            nextc = 0;

         if ((c == '\'' || c == '"') && inVal)
         {
            if (inQuote)
            {
               if (quoteChar == c)
                  inQuote = false;
               else
                  buf.append(c);
            }
            else
            {
               inQuote = true;
               quoteChar = c;
            }
         }
         else
         if (inQuote)
         {
            buf.append(c);
         }
         else
         if (c == '<')
         {
            buf.trim();
            if (buf.len != 0)
            {
               if (listener != null)
               {
                  listener.characters(buf.toString());
                  buf.clear();
               }
            }
            if (nextc == '!')
            {
               //System.out.println("begin comment");
               inComment = true;
               inKey = false;
               //System.out.println("inKey is now off!");
               inVal = false;
               inTag = false;
            }
            else
            {
               inTag = true;
            }
         }
         else
         if (c == '>')
         {
            if (buf.len != 0)
            {
               if (inComment)
               {
                  inComment = false;
                  //System.out.println("end comment");
                  if (listener != null)
                  {
                     //FINISH: fix string
                     listener.comment(buf.toString());
                  }
                  buf.clear();
               }
               else
               {
                  if (inVal && key != null)
                  {
                     props.put(key, buf.toString());
                     buf.clear();
                     inVal = false;
                  }
                  else
                  {
                     tag = buf.toString();
                     buf.clear();
                  }
                  if (tag != null && tag.length() != 0 && listener != null)
                  {
                     //System.out.println("tag: " + tag);
                     if (tag.charAt(0) == '/')
                     {
                        //System.out.println("end element " + tag);
                        listener.endElement(HtmlTag.toInt(tag.substring(1,tag.length())));
                     }
                     else
                     {
                        //System.out.println("start element " + tag);
                        listener.startElement(HtmlTag.toInt(tag), props);
                     }
                     props = new Hashtable(50);
                  }
                  inVal = false;
                  inTag = false;
               }
            }
         }
         else
         if (c == '\n')
         {
            if (newlineSignificant > 0)
               buf.append(c);
         }
         else
         if (c == ' ')
         {
            if (inTag)
            {
               tag = buf.toString();
               buf.clear();
               inTag = false;
               inKey = true;
            }
            else
            if (inVal)
            {
               props.put(key, buf.toString());
               buf.clear();
               key = null;
               inVal = false;
               inKey = true;
            }
            else
            if (inKey)
            {
               // skip spaces in key.
            }
            else
              buf.append(c);
         }
         else
         if (c == '&')
         {
            int jmax = len + 5;
            for (int j = i; j < jmax; j++)
            {
               if (chars[j] == ';')
               {
                  String thing = input.substring(i+1, j);
                  i = j;
                  //System.out.println("thing: " + thing);
                  if (thing.equals("lt"))
                  {
                     buf.append('<');
                  }
                  else
                  if (thing.equals("gt"))
                  {
                     buf.append('>');
                  }
                  else
                  if (thing.equals("amp"))
                  {
                     buf.append('&');
                  }
                  else
                  if (thing.equals("quot"))
                  {
                     buf.append('"');
                  }
                  else
                  if (thing.charAt(0) == '#')
                  {
                     int num = Convert.toInt(thing.substring(1, thing.length()));
                     if (num >= 0)
                        buf.append(Convert.toString((char)num).charAt(0));
                  }
                  break;
               }
            }
         }
         else
         if (inKey && c == '=')
         {
            key = buf.toString();
            buf.clear();
            inKey = false;
            //System.out.println("inKey is now off!");
            inVal = true;
         }
         else
         if (inKey || inTag)
         {
            buf.append(Convert.toLowerCase(c));
         }
         else
         {
            //System.out.println("appending " + c);
            buf.append(c);
         }
      }
   }

   /**
    * Set the flag to indicate that newline characters are significant.
    * This is the case if we're in a &lt;PRE&gt; block or something like that.
    * @param val true if newline characters are significant.
    */
   public void setNewlineSignificant(boolean val)
   {
      newlineSignificant += (val ? 1 : -1);
   }
}