package superwaba.ext.xplat.html;

import waba.util.*;
import waba.sys.*;
import waba.fx.*;
import waba.ui.*;

/**
 * The <code>Input</code> class is created when an HTML form control
 * object is found in the HTML.  This could be a button, submit button,
 * checkbox, or edit box.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Input extends Element
{
  Control control;
  HtmlControl htmlcontrol;
  String type;
  String name;

  /**
   * Construct a new Input control.
   * @param tag HTML tag for this control.
   */
  public Input(int tag, Hashtable p, Attributes attr)
  {
    super(attr);

    attr.form.inputs.add(this); // add this input control to the form.

    String val;
    String value = (String)p.get("value");
    name = (String)p.get("name");
    if ((val = (String)p.get("type")) != null)
    {
      this.type = val;
      if (val.equalsIgnoreCase("button"))
      {
         HtmlButton h = new HtmlButton(value == null ? "" : value);
         control = h;
         htmlcontrol = h;
      }
      else
      if (val.equalsIgnoreCase("submit"))
      {
         if (value == null)
            value = "Submit";
         HtmlButton b = new HtmlButton(value);
         control = b;
         htmlcontrol = b;
         /* guich: removed actionlistener. need to find a substitute
         b.addActionListener(new ActionListener()
         {
            public void actionPerformed(ActionEvent e)
            {
               //System.out.println("submit pressed");
               // find the form and submit it.
               Input.this.attr.form.submit();
            }
         });*/
      }
      else
      if (val.equalsIgnoreCase("checkbox"))
      {
         HtmlCheck h = new HtmlCheck(value == null ? "" : value);
         htmlcontrol = h;
         control = h;
      }
      else
      {
         //System.out.println("type defaulted to edit for " + val);
         if (value == null)
            value = "";
         HtmlEdit c = new HtmlEdit();
         if ((val = (String)p.get("size")) != null)
         {
           int len = Convert.toInt(val);
           if (len > 0)
             c.setLength(len);
         }
         c.setText(value);
         control = c;
         htmlcontrol = c;
      }
    }
  }

  public void onPaint(HtmlRenderer renderer, Graphics g)
  {
    //System.out.println("on paint!");
    if (control != null)
    {
      Coord c = control.getPos();
      g.translate(c.x, c.y);
      control.onPaint(g);
      g.translate(-c.x, -c.y);

      if (control instanceof Edit)
         renderer.restoreClip(g);
    }
  }

  public void mouseRelease(HtmlRenderer renderer, PenEvent event)
  {
    //System.out.println("Input::mouseRelease");
    forwardEvent(renderer, event);
  }

  public void mousePress(HtmlRenderer renderer, PenEvent event)
  {
    //System.out.println("Input::mousePress");
    if (control instanceof Edit)
      MainWindow.getMainWindow().setFocus(control);
    forwardEvent(renderer, event);
  }

  private void forwardEvent(HtmlRenderer renderer, PenEvent event)
  {
    //translate the position of the event.
    Coord c = control.getPos();
    event.x -= c.x;
    event.y -= c.y;

    //System.out.println("mouse event at " + event.x + " " + event.y);
    //System.out.println("control rect: " + r.width + " " + r.height);
    control.onEvent(event);
  }

  public void layout(HtmlRenderer renderer, LayoutContext lc)
  {
    //System.out.println("Input::layout " + lc);
    if (control != null)
    {
       htmlcontrol.setRenderer(renderer);

      Font font = new Font("Helvetica", Font.PLAIN, 12);
      FontMetrics fm = renderer.getFontMetrics(font);
      int fmH = fm.getHeight();

      int width = 0;
      int height = 0;
      if (control instanceof Button)
      {
         width = fm.getTextWidth(((Button)control).getText()) + 14;
         height = fmH + 3;
      }
      else
      if (control instanceof Check)
      {
         width = fm.getTextWidth(((Check)control).getText()) + 17;
         height = fmH + 3;
      }
      else
      if (control instanceof HtmlEdit)
      {
         int acw = (fm.getTextWidth("Wi"))/2;
         width = acw * ((HtmlEdit)control).len;
         height = fmH + (Settings.isColor?5:3);
      }

      //System.out.println("control set to " + lc.x + " " + lc.y + " " + width + " " + height);
      //FINISH: see if it will fit.
      if ((lc.x + width) > lc.rendererWidth)
         breakLine(lc); // break to a new line.

      if (width > lc.preferredWidth)
         lc.preferredWidth = width;

      control.setRect(lc.x, lc.y, width, height);
      region.add(new Rect(lc.x, lc.y, width, height));
      lc.x += width;
      lc.textOnLine = true;

      if (height > lc.lineHeight)
         lc.lineHeight = height;
    }
  }

  public String getValue()
  {
    if (control instanceof Check)
    {
      if (((Check)control).getChecked())
         return "on";
    }
    else
    if (control instanceof HtmlEdit)
    {
       return ((Edit)control).getText();
    }

    return null;
  }
}
