package superwaba.ext.xplat.html;

import waba.sys.*;
import waba.util.*;

/**
 * The <code>ListAttr</code> class is stored in the Attributes
 * object and encodes information about the current list that may
 * be open.  This includes the type of the list and the current value.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class ListAttr
{
  public static final int LIST_BULLET = 0;
  public static final int LIST_NUMBER = 1;
  public static final int LIST_UPPERALPHA = 2;
  public static final int LIST_LOWERALPHA = 3;
  public static final int LIST_UPPERROMAN = 4;
  public static final int LIST_LOWERROMAN = 5;

  /**
   * Type of the list (e.g. LIST_BULLET).
   */
  public int listType;


  /**
   * Current index in the sequence of this type.
   */
  public int idx;

  /**
   * Construct a new ListAttr instance using the tag provided
   * and any attributes from the HTML.
   * @param tag HTML tag.
   * @param attr attributes of the tag used to determine the list type.
   */
  public ListAttr(int tag, Hashtable attr)
  {
    String type = (String)attr.get("type");
    String start = (String)attr.get("start");

    if (type == null)
    {
      if (tag == HtmlTag.OL)
      {
         listType = LIST_NUMBER;  // default for ordered list.
         idx = start == null ? 1 : Convert.toInt(start);
      }
      else
         listType = LIST_BULLET;
    }
    else
    {
      switch (type.charAt(0))
      {
         // limitation: map romans to ints
         case '1':
         case 'i':
         case 'I':
            listType = LIST_NUMBER;
            idx = start == null ? 1 : Convert.toInt(start);
            break;
         case 'a':
            listType = LIST_LOWERALPHA;
            idx = start == null ? (int)'a' : (int)start.charAt(0);
            break;
         case 'A':
            listType = LIST_UPPERALPHA;
            idx = start == null ? (int)'a' : (int)start.charAt(0);
            break;
         default:
            listType = LIST_BULLET;
      }
    }
  }

  /**
   * Get the next value of the sequence for the given list type
   * and increment the value.
   * @return next value in the sequence.
   */
  public String getAndIncrement()
  {
    switch (listType)
    {
      case LIST_NUMBER:
         idx++;
         return idx + ".";
      case LIST_LOWERALPHA:
      case LIST_UPPERALPHA:
         idx++;
         return (char)idx + ".";
    }
    return "";
  }
}

