package superwaba.ext.xplat.html;

import waba.util.*;

/**
 * The <code>Paragraph</code> class is an element added to the document.
 * During layout, this object simply breaks the current line and leaves
 * some space for the next line to start.  The <br> tag also uses the
 * class but doesn't leave any space.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Paragraph extends Element
{
  boolean para;
  static final int PARAGRAPH_SPACE = 5;

  /**
   * Construct a new Paragraph object using the tag, args and attr provided.
   * @param tag HTML tag.
   * @param args HTML tag arguments.
   * @param attr current attributes.
   */
  public Paragraph(int tag, Hashtable args, Attributes attr)
  {
    super(attr);

    para = (tag == HtmlTag.P);
    this.attr = attr;
  }

  /**
   * Layout this element by breaking the current line and
   * adding some space if this is a paragraph.
   * @param r HtmlRenderer to draw in.
   * @param lc LayoutContext to update.
   */
  public void layout(HtmlRenderer r, LayoutContext lc)
  {
    breakLine(lc);
    if (para)
    {
      lc.y += PARAGRAPH_SPACE;
      lc.preferredHeight += PARAGRAPH_SPACE;
    }
  }
}
