package superwaba.ext.xplat.html;

import waba.util.*;
import waba.fx.*;

/**
 * A <code>region</code> defines a set of one or more rectangles
 * that define a particular subset of this screen.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class Region
{
  public Vector rectangles;

  /**
   * Create an new region that is initially empty.
   */
  public Region()
  {
    rectangles = new Vector(1);
  }

  /**
   * Add a rectangle to the region.
   * @param r rectangle to be added to the region.
   */
  public void add(Rect r)
  {
    rectangles.add(r);
  }

  /**
   * Add the region provided to this region.  For now,
   * this simply involves copying the rectangles from the
   * region provided into this region.
   * @param r region to be copied into this region.
   */
  public void add(Region r)
  {
    int n = r.rectangles.getCount(); // ARGH! NEVER IN YOUR LIKE PUT A METHOD AS THE END CONDITION OF A LOOP!
    for(int i=0;i<n;i++)
      rectangles.add(r.rectangles.items[i]);
  }

  /**
   * Returns true if the region contains the point defined
   * by the x and y coordinates provided.
   * @param x x coordinate.
   * @param y y coordinate.
   */
  public boolean contains(int x, int y)
  {
    int n = rectangles.getCount(); // ARGH! NEVER IN YOUR LIKE PUT A METHOD AS THE END CONDITION OF A LOOP!
    for(int i=0;i<n;i++)
    {
      Rect r = (Rect)rectangles.items[i];
      //System.out.println("rect " + r.x + " " + r.y + " " + r.width + " " + r.height);
      if (x >= r.x && x <= (r.x + r.width) && y >= r.y && y <= (r.y + r.height))
         return true;
    }
    return false;
  }

  /**
   * Return true if the region covers no area.
   * @return true if the region is empty.
   */
  public boolean isEmpty()
  {
    return (rectangles.getCount() == 0);
  }

  /**
   * Translate the region by the given x and y offsets.
   * @param x horizontal offset to be applied to region.
   * @param y vertical offset to be applied to region.
   */
  public void translate(int x, int y)
  {
    int n = rectangles.getCount(); // ARGH! NEVER IN YOUR LIKE PUT A METHOD AS THE END CONDITION OF A LOOP!
    for(int i=0;i<n;i++)
    {
      Rect r = (Rect)rectangles.items[i];
      r.x += x;
      r.y += y;
    }
  }

  /**
   * Return the bounding box of the entire region.
   * This is a single rectangle that includes all of
   * the regions subrectangles.
   * @return bounding box of the region.
   */
  public Rect getBounds()
  {
    if (rectangles.getCount() == 0)
      return new Rect(0,0,0,0);

    int minx = 9999;
    int maxx = -9999;
    int miny = 9999;
    int maxy = -9999;
    int n = rectangles.getCount(); // ARGH! NEVER IN YOUR LIKE PUT A METHOD AS THE END CONDITION OF A LOOP!
    for(int i=0;i<n;i++)
    {
      Rect r = (Rect)rectangles.items[i];
      if (r.x < minx)
         minx = r.x;
      if ((r.x + r.width) > maxx)
         maxx = r.x + r.width;
      if (r.y < miny)
         miny = r.y;
      if ((r.y + r.height) > maxy)
         maxy = r.y + r.height;
    }
    return new Rect(minx, miny, maxx - minx, maxy - miny);
  }
}
