package superwaba.ext.xplat.html;

/**
 * The <code>StringTokenizer</code> class attempts to implement
 * the functionality of the java.util.StringTokenizer class.
 * It allows the developer to parse a string into a sequence of
 * tokens delimited by a set of characters.
 *
 * @version $Header: $
 * @author Jim Guistwite
 */

public class StringTokenizer
{
  private String str;
  private String delim;
  private int position;
  private boolean hasMore;
  private char lastDelimeter = 0;

  /**
   * Construct a new StringTokenizer with the string provided.
   * The delimeter string defaults to ",".
   * @param str string to be tokenized.
   */
  public StringTokenizer(String str) {
    this(str, ",");
  }

  /**
   * Construct a new StringTokenizer with the string provided
   * and a string of delimeters.
   * @param str string to be tokenizer.
   * @param tok string of delimeter characters.
   */
  public StringTokenizer(String str, String delim) {
    this.str = str;
    this.delim = delim;
    position = 0;
    hasMore = (str.length() != 0);
  }

  /**
   * Returns true if there are more tokens to be returned by the
   * nextToken() method.
   * @return true if there are more tokens
   */
  public boolean hasMoreTokens() {
    return hasMore;
  }

  /**
   * Returns the delimeter that was used to create the last
   * token via the nextToken method.
   * @return last delimeter character.
   */
  public char getLastDelimeter() {
    return lastDelimeter;
  }

  /**
   * Return the next token in the string.
   * @return next token in the string.
   */
  public String nextToken() {
    if (hasMore == false)
      return null;

    int len = str.length();
    int idx = -1;
    for(int i=position;i<len;i++) {
      if (indexOf(delim, str.charAt(i)) >= 0) {
	lastDelimeter = str.charAt(i);
	idx = i;
	break;
      }
    }

    String rv;
    if (idx < 0) {
      rv = str.substring(position, len);
      hasMore = false;
    }
    else {
      rv = str.substring(position,idx);
      position = idx + 1;
    }

    return rv;
  }

  /**
   * Private method to return the index of the character c in
   * the string str1 provided.
   */
  private int indexOf(String str1, char c) {
    for(int i=0;i<str1.length();i++) {
      if (str1.charAt(i) == c) {
	return i;
      }
    }
    return -1;
  }
}
