package superwaba.ext.xplat.io;

// Class created by Rick Grehan
// http://www.fawcette.com/javapro/2002_01/magazine/columns/javatogo/default_pf.asp
import waba.io.*;

/**
 * class SerialSocket
 * This is the client-side class. It implements a socket-through-a-serial-port.
 * Created by Rick Grehan and ported to SuperWaba by Guich.
*/
public class SerialSocket extends waba.io.Socket
{
   public static final byte CREATE = 1;
   public static final byte READ = 2;
   public static final byte WRITE = 3;
   public static final byte CLOSE = 4;

   /** 10 sec timeout in millisecs for default */
   private int timeout = 10000;

   /** Serial port object */
   private SerialPort sport;

   /** DataStream object */
   private DataStream ds;

   /** temp buffer */
   private byte []tbuf = new byte[2];

   /** Create and open the Socket */
   public SerialSocket(String host, int port, int baud)
   {
      // Open serial port 0
      sport = new SerialPort(0, baud);
      if(!sport.isOpen())
      {
         lastError = sport.lastError;
         sport = null;
         return;
      }
      ds = new DataStream(sport);

      // Prepare packet
      ds.writeByte(CREATE);
      ds.writeString(host);
      ds.writeInt(port);

      // Flush the port
      flush();

      // Check for reply.
      boolean ack = ds.readBoolean();
      waba.sys.Vm.debug("SerialSocket creation ack: "+ack);

      if (!ack)
      {
         sport.close();
         lastError = sport.lastError;
         sport = null;
      }
   }

   /** Return true if socket is open - false otherwise */
   public boolean isOpen()
   {
      return sport != null;
   }

   /** Read bytes from the Socket into a byte array */
   public int readBytes(byte[] buf, int start, int count)
   {
      // Flush the port before write
      flush();

      // Transmit request
      ds.writeByte(READ);
      ds.writeInt(count);

      // Receive response
      boolean ack = ds.readBoolean();
      if (ack)
         ds.readBytes(buf, start,count);

      return ack ? count : -1;
   }

   /** Write bytes from the Socket into a byte array */
   public int writeBytes(byte[] buf, int start, int count)
   {
      // Flush the port before write
      flush();

      // Transmit request
      ds.writeByte(WRITE);
      ds.writeInt(count);
      ds.writeBytes(buf, start, count);

      // Get response
      boolean ack = ds.readBoolean();
      return ack ? count : -1;
   }

   /** Close the Socket */
   public boolean close()
   {
      // Flush the port before write
      flush();

      // Build packet
      ds.writeByte(CLOSE);

      // Get response
      boolean ack = ds.readBoolean();

      // close port even if error
      lastError = sport.lastError;
      sport.close();
      sport = null;
      return ack;
   }

   /** Shutdown the server. */
   public void shutdownServer()
   {
      flush();
      ds.writeByte(-1); // just send an invalid command.
   }

   /** Sets the read timeout. Note that this is the timeout for a Serial connection.
     * The default is 10 seconds. */
   public boolean setReadTimeout(int millis)
   {
      this.timeout = millis;
      return sport.setReadTimeout(millis);
   }
   // Flush any input
   private void flush()
   {
      // Set timeout for instant return
      sport.setReadTimeout(0);

      while(sport.readCheck()!=0)
         sport.readBytes(tbuf,0,1);

      // Return timeout to normal
      sport.setReadTimeout(timeout);
   }
}
