package superwaba.ext.xplat.io;

import waba.io.*;
import waba.sys.*;

/** Proxy for Serial Socket
  * Created by Rick Grehan and ported to SuperWaba by Guich.
  */

public class SerialSocketServer
{
   public static final byte CREATE = 1;
   public static final byte READ = 2;
   public static final byte WRITE = 3;
   public static final byte CLOSE = 4;

   Socket socket;
   SerialPort sport;
   DataStream ds;

   /** Construct a serial server to listen at <code>port</code> using <code>baud</code>
     * rate */
   public SerialSocketServer(int port, int baud)
   {
      // Open the serial port
      sport = new SerialPort(port,baud);
      sport.setReadTimeout(100000000);
      // attach a DataStream to it to make things easier
      ds = new DataStream(sport);
   }

   // Process create command
   public void create()
   {
      String url = ds.readString();
      int port = ds.readInt();
      System.out.println("create connection to "+url+':'+port);

      // Create a socket
      try
      {
         socket = new Socket(url, port);
      } catch (Exception e) {System.out.println("Exception when creating port: "+e.getMessage());}

      // Return results
      ds.writeBoolean(socket.isOpen());
   }

   // Process readBytes command
   public void readBytes()
   {
      // Get # of bytes to read and
      int len = ds.readInt();
      println("reading "+len+" bytes");
      // allocate a buffer
      byte[] rbuf = new byte[len];

      // Read the bytes
      int read = socket.readBytes(rbuf,0,len);
      println("socket.readBytes returned "+read+" bytes");

      // Return the response
      ds.writeBoolean(read == len);
      if (read == len && len > 0)
         ds.writeBytes(rbuf,0,len);
   }

   // Process writeBytes command
   public void writeBytes()
   {
      // Get the # of bytes to write
      int len = ds.readInt();
      println("writting "+len+" bytes");
      // allocate a buffer
      byte[] sbuf = new byte[len];
      ds.readBytes(sbuf,0,len);

      // Write the packet
      int written = socket.writeBytes(sbuf,0,len);
      println("socket.writeBytes returned "+written+" bytes");

      // Return the response
      ds.writeBoolean(written == len);
   }

   // Process close command
   public void close()
   {
      // Close the socket
      boolean ok = socket.close();
      println("close "+(ok?"successful":"not successful"));
      socket=null;
      ds.writeBoolean(ok);
   }

   // Flush any input
   private void flush()
   {
      byte []tbuf = new byte[2];
      // Set timeout for instant return
      sport.setReadTimeout(0);

      while(sport.readCheck()!=0)
         sport.readBytes(tbuf,0,1);

      // Return timeout to normal
      sport.setReadTimeout(100000000);
   }

   public void listen()
   {
      println("Idle");
      flush();
      // See if there's anything to be done
      while (true)
      {
         int command = ds.readByte();

         // Get the command and act accordingly
         switch(command)
         {
            case CREATE: create();     break;
            case READ:   readBytes();  break;
            case WRITE:  writeBytes(); break;
            case CLOSE:  close();      break;
            default:
               Vm.debug("Unknown command received: "+command);
               //return;
         }
      }
   }

   private static boolean showDebug=true;
   private static void println(String s)
   {
      if (showDebug) System.out.println(s);
   }

   public static void main(String args[])
   {
      if (args.length == 0)
         Vm.debug("Syntax: SerialSocketServer port baud </q>\n/q is optional and will hide all messages");
      else
      {
         int port = Convert.toInt(args[0]);
         if (port == 0) port = 1;
         int baud = Convert.toInt(args[1]);
         if (baud == 0) baud = 19200;
         if (args.length == 3)
            showDebug = false;
         new SerialSocketServer(port,baud).listen();
      }
   }
}
