package superwaba.ext.xplat.io.gps;


import waba.ui.*;
import waba.fx.*;
import waba.io.*;
import waba.sys.*;

/** Control that display GPS coordinates readen from the serial port. */

public class GPS extends Control
{
   SerialPort sp;
   byte[] buf=new byte[200];
   int bufCount=0;
   int bufPos=0;
   int cnt;
   int maxAvailable=0;
   String[] text = new String[]{"","","",""};
   double[] location=new double[2];
   Time lastFix=new Time();
   boolean dataValid;

   public GPS()
   {
      sp=new SerialPort(0,9600);
      sp.setReadTimeout(2000);
      sp.setFlowControl(false);
      text[0]="GPS Initialising";
      addTimer(100);
      nextMessage();
   }

   public GPS(SerialPort sp,int readInterval)
   {
      this.sp=sp;
      sp.setReadTimeout(2000);
      sp.setFlowControl(false);
      text[0]="GPS Initialising";
      addTimer(readInterval);
      if (!sp.isOpen())
         text[0] = "Can't open port";
      else
         nextMessage();
   }

   public void onEvent(Event e)
   {
      if (e.type==ControlEvent.TIMER)
         processInput();
   }

   public void onPaint(Graphics g)
   {
      g.setForeColor(foreColor);
      g.setBackColor(backColor);
      g.fillRect(0,0,width,height);

      if (text!=null)
      {
         int y = 0;
         for (int i =0; i < text.length; i++,y+=fmH)
            g.drawText(text[i], 0, y);
      }
   }


   private void processInput()
   {
      String message;
      if ((message=nextMessage())!=null)
      {
         String[] sp=split(message);
         if (sp[0].equals("$GPGLL"))
         {
            location[0]=toCoordinate(sp[1],sp[2]);
            location[1]=toCoordinate(sp[3],sp[4]);
            if (sp[5]!=null)
            {
               lastFix.hour=Convert.toInt(sp[5].substring(0,2));
               lastFix.minute=Convert.toInt(sp[5].substring(2,4));
               lastFix.second=Convert.toInt(sp[5].substring(4,6));
               lastFix.millis=0;
            }

            dataValid=(sp[6]!=null&&sp[6].equals("A"));

            if (location.length>0)
            {
               text[0] = "lat: " + location[0];
               text[1] = "lon: " + location[1];
               text[2] = "fix: " + lastFix.toString();
               text[3] = "valid: " + dataValid;
            }
            repaint();
         }
      }
      else
      {
         text[0] ="No message " + (cnt++);
         text[1] = "";
         text[2] = "";
         text[3] = "";
         repaint();
      }
   }


   /** returns the preffered width of this control.*/
   public int getPreferredWidth()
   {
      if (text!=null)
         return FILL;
      int w = 0;
      for (int i =0; i < text.length; i++)
         w = Math.max(w, fm.getTextWidth(text[i]));
      return w;
   }

   /** returns the preffered height of this control.*/
   public int getPreferredHeight()
   {
      return fmH*text.length;
   }

   /** returns the location array.*/
   public double[] getLocation()
   {
      return location;
   }
   /** returns the latitude.*/
   public double getLatitude()
   {
      return location[0];
   }

   /** returns the longitude.*/
   public double getLongitude()
   {
      return location[1];
   }

   /** returns the time of the last fix.*/
   public Time getLastFix()
   {
      return lastFix;
   }

   private String nextMessage()
   {
      StringBuffer sb=null;
      while (true)
      {
         while (bufCount-->0)
         {
            char c=(char)buf[bufPos++];
            if ((c==13||c==10)&&sb!=null)
               return sb.toString();
            if (c=='$'&&sb==null)
               sb=new StringBuffer();
            if (sb!=null)
               sb.append(c);
         }
         int available=sp.readCheck();
         if (available==0)
            return "0 available";
         if (available!=-1)
            maxAvailable=available;
         else
            sp.readBytes(buf,0,1);

         int toRead=available>buf.length||available==-1?buf.length:available;
         bufCount=sp.readBytes(buf,0,toRead);
         bufPos=0;
         if (bufCount==-1)
            return ">negative one";
         if (bufCount==0)
            return "0 read ("+toRead+" to read ("+available+" available ("+maxAvailable+")))";
      }
   }

   private String[] split(String s)
   {
      if (s==null)
         return null;
      char[] c=s.toCharArray();

      int size=c.length;
      if (c[size-3]=='*')
         size-=3;
      int cnt=1;
      for(int i=0;i<size;i++)
         if (c[i]==',')
           cnt++;
      String[] ret=new String[cnt];
      int last=0;
      int pos=0;
      for(int i=0;i<=size;i++)
         if (i==size||c[i]==',')
         {
            if (i>last)
               ret[pos]=new String(c,last,i-last);
            pos++;
            last=i+1;
         }
      return ret;
   }

   public double toCoordinate(String s,String dir)
   {
      if (s==null||dir==null)
         return 0;
      double deg=0;
      char[] cha=s.toCharArray();
      int i;
      int size=cha.length;
      for(i=size-1;i>=0&&cha[i]!='.';i--);
      if (i>=0)
      {
         int divider=1;
         for (int d=0;d<size-i-1;d++)
            divider*=10;
         deg=(double)Convert.toInt(s.substring(0,i-2))+
            ((double)Convert.toInt(s.substring(i-2,i))+
             (double)Convert.toInt(s.substring(i+1,size))/divider)/60;
         if (dir.equals("S")||dir.equals("W"))
            deg=-deg;
      }
      return deg;
   }
}
