/*********************************************************************************
 *  Copyright (C) 2002 Guilherme Campos Hazan <guich@superwaba.org.>             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library is NOT free software.                                           *
 *  Once you paid for it, you can use it in how many devices you want.           *
 *                                                                               *
 *********************************************************************************/

package superwaba.ext.xplat.io.print;

import superwaba.ext.xplat.io.print.PrinterException;

/** This class represents a printer font. It has the same attributes of the original
  * waba.fx.Font, but it works in a different way. It also has two more attributes to
  * be used only in supported printers: ITALIC and UNDERLINE.
  */

public class Font
{
   private String name;
   private int style;
   private int size;

   /** A plain font style. May be ORed with ITALIC and/or UNDERLINE. */
   public static final int PLAIN  = 0;

   /** A bold font style.  May be ORed with ITALIC and/or UNDERLINE. */
   public static final int BOLD   = 1;

   /** An italic font style. */
   public static final int ITALIC = 2;

   /** An underline font style. */
   public static final int UNDERLINE = 4;

   /**
   * Creates a font of the given name, style and size. Font styles are defined
   * in this class.
   * @see #PLAIN
   * @see #BOLD
   * @see #ITALIC
   * @see #UNDERLINE
   * @see Graphics
   */
   public Font(String name, int style, int size)
   {
	   this.name = name;
	   this.style = style;
	   this.size = size;
   }

   /** Returns the name of the font. */
   public String getName()
   {
	   return name;
   }

   /** Returns the size of the font. */
   public int getSize()
   {
	   return size;
   }

   /**
   * Returns the style of the font. Font styles are defined in this class.
   * @see #PLAIN
   * @see #BOLD
   */
   public int getStyle()
   {
	   return style;
   }
}