/*********************************************************************************
 *  Copyright (C) 2002 Guilherme Campos Hazan <guich@superwaba.org.>             *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library is NOT free software.                                           *
 *  Once you paid for it, you can use it in how many devices you want.           *
 *                                                                               *
 *********************************************************************************/

package superwaba.ext.xplat.io.print;

import superwaba.ext.xplat.io.print.PrinterException;
import superwaba.ext.xplat.io.print.NotSupportedException;

/** This class is used to send graphic command to the printer. You can get an
  * instance of it with the <code>PrintManager.getGraphics</code> method.
  * <p><b>Very important!</b> Some printers will not support drawing in an alleatory way.
  * Try always to draw things from top to bottom.
  */
public abstract class Graphics
{
	/** Package accessed constructor. Use PrintManager.getGraphics() to get an instance of this object. */
	protected Graphics() throws NotSupportedException
	{
	}

   /** Sets the font to the given font. */
	public abstract void setFont(superwaba.ext.xplat.io.print.Font font) throws PrinterException, NotSupportedException;

	/** Draws an image as in the given position. Note that in Palm OS < 3.5, the
	  * image is converted to black and white before printing. Also, some printers
	  * may not support grayscale or color images even in newer Palm OS versions.
	  */
	public abstract void drawImage(waba.fx.Image img, int x, int y) throws PrinterException, NotSupportedException;

	/** Draws an image scaled with the given scale. You may use values like 2 or 0.5.
	  * Not all printers support this feature
	  */
	public abstract void drawImage(waba.fx.Image img, int x, int y, float scale) throws PrinterException, NotSupportedException;

	/** Draws the given text in the given position, using the currently selected font and using
	  * the current foreground color.
	  */
	public abstract void drawText(String text, int x, int y) throws PrinterException, NotSupportedException;

	/** In most printers, setting from anything than Color.BLACK will turn gray what you draw. */
	public abstract void setForeColor(waba.fx.Color c);

	/** In most printers, setting from anything than Color.BLACK will turn gray what you draw. */
	public abstract void setBackColor(waba.fx.Color c);

   /** Draws a line from ax,ay to bx,by using the current foreground color. Note that some printers do not support diagonal lines. */
	public abstract void drawLine(int ax, int ay, int bx, int by) throws PrinterException, NotSupportedException; // gray & black

   /** Draws a round rectangle with the given parameters using the current foreground color. */
	public abstract void drawRoundRect(int x, int y, int width, int height, int r) throws PrinterException, NotSupportedException; // gray & black

   /** Draws a rectangle with the given parameters using the current foreground color. */
	public abstract void drawRect(int x, int y, int width, int height) throws PrinterException, NotSupportedException; // gray & black

   /** Fills a round rectangle with the given parameters using the current background color. */
	public abstract void fillRoundRect(int x, int y, int width, int height, int r) throws PrinterException, NotSupportedException; // black

   /** Fills a rectangle with the given parameters using the current background color. */
	public abstract void fillRect(int x, int y, int width, int height) throws PrinterException, NotSupportedException; // black
}