/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2001-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 2003 Dave Slaughter <dslaughter@safeway.co.uk>                 *
 *  Copyright (C) 2000 Tom Cuthill                                               *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package superwaba.ext.xplat.io.scanner;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Frame;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Imlementation of scanner which can run under the JDK.  The scanner is
 * emulated by opening a windows where a barcode can be typed in.
 * <p>This class is for internal use only. There are no public methods.
 */

public class NativeMethods4JDK
{
   static private Frame scanFrame = null;
   static private boolean isActive = false;
   static private String barcode = null;

   /**
	* Frame to display emulate allow entry of a barcode to emulate the
	* scanner.
	*/
	static private class ScanFrame extends Frame implements ActionListener
	{
		TextField scanText;

		ScanFrame()
		{
			super("Scanner");
			scanText = new TextField(40);
			add(scanText, BorderLayout.CENTER);
			Button okB = new Button("OK");
			okB.addActionListener(this);
			scanText.addActionListener(this);
			add(okB, BorderLayout.EAST);
			// move out of the way of the window
			setLocation(250, 0);
			pack();
		}

		public void actionPerformed(ActionEvent e)
		{
			setVisible(false);
			isActive = false;
			barcode = scanText.getText();
			Scanner._onEvent(1);
			if (isActive) {
				setVisible(true);
			}
		}
	}

  /**
	* Get the frame for entering a barcode.
	* @return Frame
	*/
   static private Frame getScanFrame()
   {
	   if (scanFrame == null)
		   scanFrame = new ScanFrame();
	   return scanFrame;
   }

   static boolean scannerActivate()
   {
		getScanFrame().setVisible(true);
		isActive = true;
		return true;
   }
   static boolean scannerSetBarcodeParam(int barcodeType, boolean enable)
   {
      return true;
   }
   static boolean scannerSetBarcodeLength(int barcodeType, int lengthType, int length1, int length2)
   {
      return true;
   }
   static boolean scannerSetParam(int type, int barcodeType, int value)
   {
      return true;
   }
   static boolean scannerCommitBarcodeParams()
   {
      return true;
   }
   static String scannerGetData()
   {
      return barcode;
   }
   static String scannerGetScanManagerVersion()
   {
      return "Java emulation 1.0";
   }
   static String scannerGetScanPortDriverVersion()
   {
      return "Java emulation 1.0";
   }
   static boolean scannerDeactivate()
   {
   	if (scanFrame != null)
   		scanFrame.setVisible(false);
   	isActive=false;
   	return true;
   }
}