/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2001-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 2000 Tom Cuthill                                               *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package superwaba.ext.xplat.io.scanner;

/**
 * ScanEvent is an event thrown by the Symbol barcode scanner.
 * @author Tom Cuthill (Palm Solutions) 16/3/2000
 */
public class ScanEvent extends waba.ui.Event
{
   /** The event type for a scanner scanning event. Note that since they are set at runtime, you cannot use them in a switch statement. */
   public static final int SCANNED = waba.ui.Event.getNextAvailableEventId(); // dinamically assigned
   /** The event type for a scanner low battery event. Note that since they are set at runtime, you cannot use them in a switch statement. */
   public static final int BATTERY_ERROR = waba.ui.Event.getNextAvailableEventId(); // dinamically assigned
   /** The event type for a scanner triggered: A scan attempt was initiated - hard or soft trigger. Note that since they are set at runtime, you cannot use them in a switch statement. */
   public static final int TRIGGERED = waba.ui.Event.getNextAvailableEventId(); // dinamically assigned

   /**
   * The data resulting from the scan when type is <code>SCANNED</code>.
   * String matches the data returned
   * by ScanGetDecodedData function in Symbol's ScanMgr library.
   */
   public String data;

   /**
   * Constructs a scan event of the given type.
   * target is always null.
   * guich@200b2: added a parameter to the constructor.
   */
   public ScanEvent(int type)
   {
      this.type = type;
      target = null;
      if (type == SCANNED)
         data = Scanner.getData();
   }
}

