
/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
 *                 and Andy DePue <sir_anodos@yahoo.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/
package superwaba.ext.xplat.sql;

/**
  * This class is a wrapper around java.util.Date to allow the JDBC
  * driver to identify the value as a SQL Date.
  */

public class Date
{
   protected String _toString;
   protected waba.sys.Time _date;

   /**
     * This method initializes a new instance of this class with the
     * specified year, month, and day.  While this method is deprecated in the
     * latest JDBC, it is the only way to do it for now in SW, as SW does not
     * have a 'long' representation of time, and the conversion overhead is
     * considered too great for small devices (this may change in the future).
     *
     * @param year The year of this date minue 1900.
     * @param month The month of this date (0-11).
     * @param day The day of this date (1-31).
     *
     *
     */
   public Date(int year, int month, int day)
   {
      _date = new waba.sys.Time();
      _date.year = year;
      _date.month = month;
      _date.year = year;
   }
   protected Date(String str, byte format)
   {
      waba.util.Date d = new waba.util.Date(str, format);
      _date = new waba.sys.Time();
      _date.year = d.getYear() - 1900;
      _date.month = d.getMonth();
      _date.day = d.getDay();
   }
   protected Date(waba.sys.Time time)
   {
      _date = time;
   }
   protected Date()
   {
      _date = new waba.sys.Time();
   }

   /**
     * This method returns a new instance of this class by parsing a
     * date in JDBC format into a Java date.
     * JDBC format is DATE_YMD.
     * @param str The string to parse.
     * @return The resulting <code>java.sql.Date</code> value.
     */
   public static Date valueOf(String str)
   {
      return new Date(str, waba.sys.Settings.DATE_YMD);
   }
   public int getDate()
   {
      return _date.day;
   }
   public void setDate(int date)
   {
      _date.day = date;
      _toString = null;
   }
   public int getMonth()
   {
      return _date.month;
   }
   public void setMonth(int month)
   {
      _date.month = month;
      _toString = null;
   }
   public int getYear()
   {
      return _date.year;
   }
   public void setYear(int year)
   {
      _date.year = year;
      _toString = null;
   }

   /**
     * This method returns this date in JDBC format.  JDBC format is DATE_YMD.
     *
     * @return This date as a string.
     */
   public String toString()
   {
      if(_toString == null)
         _toString = convertDateToString();
      return _toString;
   }
   protected String convertDateToString()
   {
      return waba.util.Date.formatDate(_date.day, _date.month, _date.year + 1900, waba.sys.Settings.DATE_YMD);
   }
}
