
/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
 *                 and Andy DePue <sir_anodos@yahoo.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/
package superwaba.ext.xplat.sql;
import waba.sys.Convert;

/**
  * This class is a wrapper around java.util.Date to allow the JDBC
  * driver to identify the value as a SQL Time.
  *
  */

public class Time extends Date
{

   /**
     * This method returns a new instance of this class by parsing a
     * time in JDBC format into a Java date.  JDBC format is HH:mm:ss
     *
     * @param str The string to parse.
     * @return The resulting <code>java.sql.Time</code> value.
     */
   public static Date valueOf(String str)
   {
      return new Time(parseTime(str, 0));
   }
   protected static waba.sys.Time parseTime(String str, int begin)
   {
      char[] chars = str.toCharArray();
      char ch;
      waba.sys.Time time = new waba.sys.Time();
      StringBuffer buf = new StringBuffer();
      int i = begin, size = chars.length;
      for(;i < size && (ch = chars[i]) != ':';i++)
         buf.append(ch);
      time.hour = Convert.toInt(buf.toString());
      buf.setLength(0);
      for(i++;i < size && (ch = chars[i]) != ':';i++)
         buf.append(ch);
      time.minute = Convert.toInt(buf.toString());
      buf.setLength(0);
      for(i++;i < size && (ch = chars[i]) != ':';i++)
         buf.append(ch);
      time.second = Convert.toInt(buf.toString());
      return time;
   }

   /**
     * This method initializes a new instance of this class with the
     * specified year, month, and day.
     * While this method is deprecated in the
     * latest JDBC, it is the only way to do it for now in SW, as SW does not
     * have a 'long' representation of time, and the conversion overhead is
     * considered too great for small devices (this may change in the future).
     *
     * @param hour The hour for this Time (0-23)
     * @param minute The minute for this time (0-59)
     * @param second The second for this time (0-59)
     */
   public Time(int hour, int minute, int second)
   {
      super();
      _date.hour = hour;
      _date.minute = minute;
      _date.second = second;
   }
   protected Time(waba.sys.Time time)
   {
      super(time);
   }
   public int getHour()
   {
      return _date.hour;
   }
   public void setHour(int hour)
   {
      _date.hour = hour;
      _toString = null;
   }
   public int getMinute()
   {
      return _date.minute;
   }
   public void setMinute(int minute)
   {
      _date.minute = minute;
      _toString = null;
   }
   public int getSecond()
   {
      return _date.second;
   }
   public void setSecond(int second)
   {
      _date.second = second;
      _toString = null;
   }

   /**
     * This method returns this date in JDBC format.
     *
     * @return This date as a string.
     */
   public String toString()
   {
      if(_toString == null)
         _toString = convertTimeToString();
      return _toString;
   }
   protected String convertTimeToString()
   {
      StringBuffer buf = new StringBuffer();
      buf.append(Convert.zeroPad(String.valueOf(_date.hour), 2)).append(':');
      buf.append(Convert.zeroPad(String.valueOf(_date.minute), 2)).append(':');
      buf.append(Convert.zeroPad(String.valueOf(_date.second), 2));
      return buf.toString();
   }
}
