
/*********************************************************************************
  *  SuperWaba Virtual Machine, version 4                                         *
  *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
  *                 and Andy DePue <sir_anodos@yahoo.com>                         *
  *  All Rights Reserved                                                          *
  *                                                                               *
  *  This library and virtual machine is free software; you can redistribute      *
  *  it and/or modify it under the terms of the Amended GNU Lesser General        *
  *  Public License distributed with this software.                               *
  *                                                                               *
  *  This library and virtual machine is distributed in the hope that it will     *
  *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
  *                                                                               *
  *  All software that links or interacts with this library and virtual machine,  *
  *  or any derivative works, require the software to prominently display the     *
  *  following notice:                                                            *
  *                                                                               *
  *                   Created with SuperWaba                                      *
  *                  http://www.superwaba.org                                     *
  *                                                                               *
  *  Please see the software license located at SuperWabaSDK/license.txt          *
  *  for more details.                                                            *
  *                                                                               *
  *  You should have received a copy of the License along with this software;     *
  *  if not, write to                                                             *
  *                                                                               *
  *     Guilherme Campos Hazan                                                    *
  *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
  *     Rio de Janeiro / RJ - Brazil                                              *
  *     Cep: 22050-000                                                            *
  *     E-mail: support@superwaba.com.br                                          *
  *                                                                               *
  *********************************************************************************/
package superwaba.ext.xplat.sql.db2e;
import superwaba.ext.xplat.sql.*;
import waba.util.Hashtable;
import superwaba.ext.xplat.sql.db2e.db2ex.*;

/** Creates a connection with the IBM DB2everyplace client */

public class Db2eConnection extends Connection
{
   private Db2e _db;
   private SQLWarning _warnings;
   private int _connection;
   private Driver _driver;
   private boolean _autoCommit = true;
   private boolean _open = false;
   public Db2eConnection(Driver driver, Db2e db, int envHandle, String db2eURL, Hashtable info) throws SQLException
   {
      _driver = driver;
      _db = db;
      try
      {

         //      Vm.debug("Allocating DBC handle.");
         _connection = _db.allocHandle(Db2e.SQL_HANDLE_DBC, envHandle);

         //      Vm.debug("Connecting to DB.");
         _db.connect(_connection, db2eURL, (String)info.get("user"), (String)info.get("password"));

         //      Vm.debug("Registering this connection.");
         Driver.registerConnection(this);
         _open = true;

      //      Vm.debug("New Db2eConnection finished.");
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }
   void handleError(DB2Exception error) throws SQLException
   {
      Driver.handleError(error);
   }

   /** This method clears all warnings that have occurred on this connection.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void clearWarnings() throws SQLException
   {
      _warnings = null;
   }

   /** This method immediately closes this database connection.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void close() throws SQLException
   {
      try
      {
         _db.disconnect(_connection);
         _open = false;
         _db.freeHandle(Db2e.SQL_HANDLE_DBC, _connection);
         Driver.deregisterConnection(this);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method commits any SQL statements executed on this connection since
     * the last commit or rollback.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void commit() throws SQLException
   {
      try
      {
         _db.endTran(Db2e.SQL_HANDLE_DBC, _connection, Db2e.SQL_COMMIT);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method creates a new SQL statement.  The default result set type
     * and concurrency will be used.
     *
     * @return A new <code>Statement</code> object.
     * @exception SQLException If an error occurs.
     * @see Statement
     *
     */
   public Statement createStatement() throws SQLException
   {

      //    Vm.debug("Creating new Db2eStatement.");
      return new Db2eStatement(this, _connection, _db);
   }

   /** This method creates a new SQL statement with the specified type and
     * concurrency.  Valid values for these parameters are specified in the
     * <code>ResultSet</code> class.
     *
     * @param resultSetType The type of result set to use for this statement.
     * @param resultSetConcurrency.  The type of concurrency to be used in
     *        the result set for this statement.
     * @return A new <code>Statement</code> object.
     * @exception SQLException If an error occurs.
     * @see Statement
     * @see ResultSet
     *
     */
   public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException
   {
      if(resultSetType == ResultSet.TYPE_SCROLL_INSENSITIVE)
         throw new SQLException("Unsupported result set type.");
      if(resultSetConcurrency != ResultSet.CONCUR_READ_ONLY)
         throw new SQLException("Unsupported result set concurrency.");

      // TO DO: actually care about resultSetType and resultSetConcurrency.
      return new Db2eStatement(this, _connection, _db, resultSetType);
   }

   /** This method tests whether or not auto commit mode is currently enabled.
     * In auto commit mode,  every SQL statement is committed its own transaction.
     * Otherwise a transaction must be explicitly committed or rolled back.
     *
     * @return <code>true</code> if auto commit mode is enabled,
     * <code>false</code> otherwise.
     *
     * @exception SQLException If an error occurs.
     *
     * @see #commit
     * @see #rollback
     *
     */
   public boolean getAutoCommit() throws SQLException
   {
      return _autoCommit;
   }

   /** This method returns the name of the catalog in use by this connection,
     * if any.
     *
     * @return The name of the catalog, or <code>null</code> if one does not
     *         exist or catalogs are not supported by this database.
     * @exception SQLException If an error occurs.
     *
     */
   public String getCatalog() throws SQLException
   {
      return null;
   }

   /** This method returns the meta data for this database connection.
     *
     * @return The meta data for this database.
     * @exception SQLException If an error occurs.
     * @see DatabaseMetaData
     *
     */
   public DatabaseMetaData getMetaData() throws SQLException
   {
      return new Db2eMetaData(_driver, this);
   }

   /** This method returns the current transaction isolation mode.  This will
     * be one of the constants defined in this interface.
     *
     * @return The transaction isolation level.
     * @exception SQLException If an error occurs.
     *
     */
   public int getTransactionIsolation() throws SQLException
   {
      return TRANSACTION_READ_COMMITTED;
   }

   /** This method returns the first warning that occurred on this connection,
     * if any.  If there were any subsequence warnings, they will be chained
     * to the first one.
     *
     * @return The first <code>SQLWarning</code> that occurred, or
     *         <code>null</code> if there have been no warnings.
     * @exception SQLException If an error occurs.
     *
     */
   public SQLWarning getWarnings() throws SQLException
   {
      return _warnings;
   }

   /** This method tests whether or not this connection has been closed.
     *
     * @return <code>true</code> if the connection is closed, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean isClosed() throws SQLException
   {
      return !_open;
   }

   /** This method creates a new <code>PreparedStatement</code> for the specified
     * SQL string.  This method is designed for use with parameterized
     * statements.  The default result set type and concurrency will be used.
     *
     * @param The SQL statement to use in creating this
     *        <code>PreparedStatement</code>.
     * @return A new <code>PreparedStatement</code>.
     * @exception SQLException If an error occurs.
     * @see PreparedStatement
     *
     */
   public PreparedStatement prepareStatement(String sql) throws SQLException
   {
      return new Db2eStatement(this, _connection, _db, sql);
   }

   /** This method creates a new <code>PreparedStatement</code> for the specified
     * SQL string.  This method is designed for use with parameterized
     * statements.  The specified result set type and concurrency will be used.
     * Valid values for these parameters are specified in the
     * <code>ResultSet</code> class.
     *
     * @param The SQL statement to use in creating this
     *        <code>PreparedStatement</code>.
     * @param resultSetType The type of result set to use for this statement.
     * @param resultSetConcurrency.  The type of concurrency to be used in
     *        the result set for this statement.
     * @return A new <code>PreparedStatement</code>.
     * @exception SQLException If an error occurs.
     * @see PreparedStatement
     * @see ResultSet
     *
     */
   public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
   {
      if(resultSetType == ResultSet.TYPE_SCROLL_INSENSITIVE)
         throw new SQLException("Unsupported result set type.");
      if(resultSetConcurrency != ResultSet.CONCUR_READ_ONLY)
         throw new SQLException("Unsupported result set concurrency.");

      // TO DO: actually care about resultSetType and resultSetConcurrency.
      return new Db2eStatement(this, _connection, _db, resultSetType, sql);
   }

   /** This method rolls back any SQL statements executed on this connection
     * since the last commit or rollback.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void rollback() throws SQLException
   {
      try
      {
         _db.endTran(Db2e.SQL_HANDLE_DBC, _connection, Db2e.SQL_ROLLBACK);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method turns auto commit mode on or off.  In auto commit mode,
     * every SQL statement is committed its own transaction.  Otherwise a
     * transaction must be explicitly committed or rolled back.
     *
     * @param autoCommit <code>true</code> to enable auto commit mode,
     *        <code>false</code> to disable it.
     * @exception SQLException If an error occurs.
     * @see #commit
     * @see #rollback
     *
     */
   public void setAutoCommit(boolean autoCommit) throws SQLException
   {
      try
      {
         _db.setAutoCommit(_connection, autoCommit);
         _autoCommit = autoCommit;
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method sets the name of the catalog in use by this connection.
     * Note that this method does nothing if catalogs are not supported by
     * this database.
     *
     * @param catalog The name of the catalog to use for this connection.
     * @exception SQLException If an error occurs.
     *
     */
   public void setCatalog(String catalog) throws SQLException
   {

   }

   /** This method sets the current transaction isolation mode.  This must
     * be one of the constants defined in this interface.
     *
     * @param level The transaction isolation level.
     * @exception SQLException If an error occurs.
     *
     */
   public void setTransactionIsolation(int level) throws SQLException
   {
      if(level != TRANSACTION_READ_COMMITTED)
         throw new SQLException("Unsupported transaction isolation.");
   }
}
