
/*********************************************************************************
  *  SuperWaba Virtual Machine, version 4                                         *
  *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
  *                 and Andy DePue <sir_anodos@yahoo.com>                         *
  *  All Rights Reserved                                                          *
  *                                                                               *
  *  This library and virtual machine is free software; you can redistribute      *
  *  it and/or modify it under the terms of the Amended GNU Lesser General        *
  *  Public License distributed with this software.                               *
  *                                                                               *
  *  This library and virtual machine is distributed in the hope that it will     *
  *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
  *                                                                               *
  *  All software that links or interacts with this library and virtual machine,  *
  *  or any derivative works, require the software to prominently display the     *
  *  following notice:                                                            *
  *                                                                               *
  *                   Created with SuperWaba                                      *
  *                  http://www.superwaba.org                                     *
  *                                                                               *
  *  Please see the software license located at SuperWabaSDK/license.txt          *
  *  for more details.                                                            *
  *                                                                               *
  *  You should have received a copy of the License along with this software;     *
  *  if not, write to                                                             *
  *                                                                               *
  *     Guilherme Campos Hazan                                                    *
  *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
  *     Rio de Janeiro / RJ - Brazil                                              *
  *     Cep: 22050-000                                                            *
  *     E-mail: support@superwaba.com.br                                          *
  *                                                                               *
  *********************************************************************************/
package superwaba.ext.xplat.sql.db2e;
import superwaba.ext.xplat.sql.*;

/** With this class you can get some information regarding a database */

public class Db2eMetaData extends DatabaseMetaData
{
   private Db2eConnection _con;
   private Driver _driver;
   Db2eMetaData(Driver driver, Db2eConnection con)
   {
      _con = con;
      _driver = driver;
   }

   /** This method returns the list of database catalogs as a
     * <code>ResultSet</code> with one column - TABLE_CAT - that is the
     * name of the catalog.
     *
     * @return A <code>ResultSet</code> with all the requested catalogs.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getCatalogs() throws SQLException
   {
      throw new SQLException("Not supported");
   }

   /** This method returns a list of the tables columns for
     * the requested tables.  This is returned in the form of a
     * <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>TABLE_CAT - The catalog the table is in, which may be
     * <code>null</code>.
     * <li>TABLE_SCHEM - The schema the tables is in, which may be
     * <code>null</code>.
     * <li>TABLE_NAME - The name of the table.
     * <li>COLUMN_NAME - The name of the column
     * <li>DATA_TYPE - The SQL type of the column. This is one of the constants
     * defined in <code>Types</code>.
     * <li>TYPE_NAME - The string name of the data type for this column.
     * <li>COLUMN_SIZE - The size of the column.
     * <li>Unused
     * <li>NUM_PREC_RADIX - The radix of the column.
     * <li>NULLABLE - Whether or not the column is NULLABLE.  This is one of
     * the constants defined in this class (<code>tableNoNulls</code>,
     * <code>tableNullable</code>, or <code>tableNullableUnknown</code>)
     * <li>REMARKS - A description of the column.
     * <li>COLUMN_DEF - The default value for the column, may be <code>null</code>.
     * <li>SQL_DATA_TYPE - Unused
     * <li>SQL_DATETIME_SUB - Unused
     * <li>CHAR_OCTET_LENGTH - For character columns, the maximum number of bytes
     * in the column.
     * <li>ORDINAL_POSITION - The index of the column in the table.
     * <li>IS_NULLABLE - "NO" means no, "YES" means maybe, and an empty string
     * means unknown.
     * </ol>
     *
     * @param catalog The name of the catalog to return table from,
     * or "" to return tables from all catalogs.
     * @param schemaPattern A schema pattern for the schemas to return
     * tables from, or "" to return tables from all schemas.
     * @param namePattern The pattern of tables names to return.
     * @param columnPattern The pattern of column names to return.
     * @returns A <code>ResultSet</code> with all the requested tables.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getColumns(String catalog, String schemaPattern, String tableNamePattern, String columnNamePattern) throws SQLException
   {
      throw new SQLException("Not supported");
   }

   /** This method returns the <code>Connection</code> object that was used
     * to generate the metadata in this object.
     *
     * @return The connection for this object.
     * @exception SQLException If an error occurs.
     *
     */
   public Connection getConnection() throws SQLException
   {
      return _con;
   }

   /** This method returns a description of how one table imports another
     * table's primary key as a foreign key.  The information is
     * returned as a <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>PKTABLE_CAT - The catalog of the table the key was imported from.
     * <li>PKTABLE_SCHEM - The schema of the table the key was imported from.
     * <li>PKTABLE_NAME - The name of the table the key was imported from.
     * <li>PKCOLUMN_NAME - The name of the column that was imported.
     * <li>FKTABLE_CAT - The foreign key catalog name.
     * <li>FKTABLE_SCHEM - The foreign key schema name.
     * <li>FKTABLE_NAME - The foreign key table name.
     * <li>FKCOLUMN_NAME - The foreign key column name.
     * <li>KEY_SEQ - The sequence number of the column within the foreign key.
     * <li>UPDATE_RULE - How the foreign key behaves when the primary key is
     * updated.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, <code>importedKeySetDefault</code>, or
     * <code>importedKeyRestrict</code>).
     * <li>DELETE_RULE - How the foreign key behaves when the primary key is
     * deleted.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, or <code>importedKeySetDefault</code>)
     * <li>FK_NAME - The name of the foreign key.
     * <li>PK_NAME - The name of the primary key.
     * <li>DEFERRABILITY - The deferrability value.  This is one of the
     * constants defined in this table (<code>importedKeyInitiallyDeferred</code>,
     * <code>importedKeyInitiallyImmediate</code>, or
     * <code>importedKeyNotDeferrable</code>).
     * </ol>
     *
     * @param primCatalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or <code>null</code>
     *        to return information from all catalogs, on the exporting side.
     * @param primSchema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema, on the exporting side.
     * @param primTable The table name to return information for, on the exporting
     *        side.
     * @param forCatalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or <code>null</code>
     *        to return information from all catalogs, on the importing side.
     * @param forSchema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema on the importing side.
     * @param forTable The table name to return information for on the importing
     *        side.
     * @return A <code>ResultSet</code> with the requested information
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getCrossReference(String primaryCatalog, String primarySchema, String primaryTable, String foreignCatalog, String foreignSchema, String foreignTable) throws SQLException
   {
      throw new SQLException("Not supported");
   }

   /** @since 1.4
     *
     */
   public int getDatabaseMajorVersion() throws SQLException
   {

      // TO DO: implement function in Db2e class (and natives) to retrieve DB version

      // from DB2e CLI.
      return 8;
   }

   /** @since 1.4
     *
     */
   public int getDatabaseMinorVersion() throws SQLException
   {

      // TO DO: implement function in Db2e class (and natives) to retrieve DB version

      // from DB2e CLI.
      return 1;
   }

   /** This method returns the name of the database product.
     *
     * @return The database product.
     * @exception SQLException If an error occurs.
     *
     */
   public String getDatabaseProductName() throws SQLException
   {

      // TO DO: implement function in Db2e class (and natives) to retrieve name

      // from DB2e CLI.
      return "IBM DB2 Everyplace";
   }

   /** This method returns the version of the database product.
     *
     * @return The version of the database product.
     * @exception SQLException If an error occurs.
     *
     */
   public String getDatabaseProductVersion() throws SQLException
   {
      return "v 8.1";
   }

   /** This method returns the default transaction isolation level of the
     * database.
     *
     * @return The default transaction isolation level of the database.
     * @exception SQLException If an error occurs.
     * @see Connection
     *
     */
   public int getDefaultTransactionIsolation() throws SQLException
   {
      return Connection.TRANSACTION_READ_COMMITTED;
   }

   /** This method returns the major version number of the JDBC driver.
     *
     * @return The major version number of the JDBC driver.
     *
     */
   public int getDriverMajorVersion()
   {
      return _driver.getMajorVersion();
   }

   /** This method returns the minor version number of the JDBC driver.
     *
     * @return The minor version number of the JDBC driver.
     *
     */
   public int getDriverMinorVersion()
   {
      return _driver.getMinorVersion();
   }

   /** This method returns the name of the JDBC driver.
     *
     * @return The name of the JDBC driver.
     * @exception SQLException If an error occurs.
     *
     */
   public String getDriverName() throws SQLException
   {

      // Not sure exactly what this is supposed to return?
      return "DB2 Everyplace WDBC";
   }

   /** This method returns the version of the JDBC driver.
     *
     * @return The version of the JDBC driver.
     * @exception SQLException If an error occurs.
     *
     */
   public String getDriverVersion() throws SQLException
   {
      return "v 0.5a";
   }

   /** This method returns a list of the table's which use this table's
     * primary key as a foreign key.  The information is
     * returned as a <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>PKTABLE_CAT - The catalog of the table the key was imported from.
     * <li>PKTABLE_SCHEM - The schema of the table the key was imported from.
     * <li>PKTABLE_NAME - The name of the table the key was imported from.
     * <li>PKCOLUMN_NAME - The name of the column that was imported.
     * <li>FKTABLE_CAT - The foreign key catalog name.
     * <li>FKTABLE_SCHEM - The foreign key schema name.
     * <li>FKTABLE_NAME - The foreign key table name.
     * <li>FKCOLUMN_NAME - The foreign key column name.
     * <li>KEY_SEQ - The sequence number of the column within the foreign key.
     * <li>UPDATE_RULE - How the foreign key behaves when the primary key is
     * updated.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, <code>importedKeySetDefault</code>, or
     * <code>importedKeyRestrict</code>).
     * <li>DELETE_RULE - How the foreign key behaves when the primary key is
     * deleted.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, or <code>importedKeySetDefault</code>)
     * <li>FK_NAME - The name of the foreign key.
     * <li>PK_NAME - The name of the primary key.
     * <li>DEFERRABILITY - The deferrability value.  This is one of the
     * constants defined in this table (<code>importedKeyInitiallyDeferred</code>,
     * <code>importedKeyInitiallyImmediate</code>, or
     * <code>importedKeyNotDeferrable</code>).
     * </ol>
     *
     * @param catalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or <code>null</code>
     *        to return information from all catalogs.
     * @param schema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema.
     * @param table The table name to return information for.
     * @return A <code>ResultSet</code> with the requested information
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getExportedKeys(String catalog, String schema, String table) throws SQLException
   {
      throw new SQLException("Not supported");
   }

   /** This metohd returns the quote string for SQL identifiers.
     * Returns a space " " if identifier quoting isn't supported.
     * @return The quote string for SQL identifers, or a space if quoting
     *         is not supported.
     * @exception SQLException If an error occurs.
     *
     */
   public String getIdentifierQuoteString() throws SQLException
   {

      // Not sure what this should be for DB2e?
      return " ";
   }

   /** This method returns a list of the table's foreign keys.  These are
     * returned as a <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>PKTABLE_CAT - The catalog of the table the key was imported from.
     * <li>PKTABLE_SCHEM - The schema of the table the key was imported from.
     * <li>PKTABLE_NAME - The name of the table the key was imported from.
     * <li>PKCOLUMN_NAME - The name of the column that was imported.
     * <li>FKTABLE_CAT - The foreign key catalog name.
     * <li>FKTABLE_SCHEM - The foreign key schema name.
     * <li>FKTABLE_NAME - The foreign key table name.
     * <li>FKCOLUMN_NAME - The foreign key column name.
     * <li>KEY_SEQ - The sequence number of the column within the foreign key.
     * <li>UPDATE_RULE - How the foreign key behaves when the primary key is
     * updated.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, <code>importedKeySetDefault</code>, or
     * <code>importedKeyRestrict</code>).
     * <li>DELETE_RULE - How the foreign key behaves when the primary key is
     * deleted.  This is one of the constants defined in this class
     * (<code>importedNoAction</code>, <code>importedKeyCascade</code>,
     * <code>importedKeySetNull</code>, or <code>importedKeySetDefault</code>)
     * <li>FK_NAME - The name of the foreign key.
     * <li>PK_NAME - The name of the primary key.
     * <li>DEFERRABILITY - The deferrability value.  This is one of the
     * constants defined in this table (<code>importedKeyInitiallyDeferred</code>,
     * <code>importedKeyInitiallyImmediate</code>, or
     * <code>importedKeyNotDeferrable</code>).
     * </ol>
     *
     * @param catalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or <code>null</code>
     *        to return information from all catalogs.
     * @param schema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema.
     * @param table The table name to return information for.
     *
     * @return A <code>ResultSet</code> with the foreign key columns.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getImportedKeys(String catalog, String schema, String table) throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** This method returns information about a tables indices and statistics.
     * It is returned as a <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>TABLE_CAT - The catalog of the table, which may be <code>null</code>.
     * <li>TABLE_SCHEM - The schema of the table, which may be <code>null</code>.
     * <li>TABLE_NAME - The name of the table.
     * <li>NON_UNIQUE - Are index values non-unique?
     * <li>INDEX_QUALIFIER The index catalog, which may be <code>null</code>
     * <li>INDEX_NAME - The name of the index.
     * <li>TYPE - The type of index, which will be one of the constants defined
     * in this interface (<code>tableIndexStatistic</code>,
     * <code>tableIndexClustered</code>, <code>tableIndexHashed</code>, or
     * <code>tableIndexOther</code>).
     * <li>ORDINAL_POSITION - The sequence number of this column in the index.
     * This will be 0 when the index type is <code>tableIndexStatistic</code>.
     * <li>COLUMN_NAME - The name of this column in the index.
     * <li>ASC_OR_DESC - "A" for an ascending sort sequence, "D" for a
     * descending sort sequence or <code>null</code> if a sort sequence is not
     * supported.
     * <li>CARDINALITY - The number of unique rows in the index, or the number
     * of rows in the table if the index type is <code>tableIndexStatistic</code>.
     * <li>PAGES - The number of pages used for the index, or the number of pages
     * in the table if the index type is <code>tableIndexStatistic</code>.
     * <li>FILTER_CONDITION - The filter condition for this index, which may be
     * <code>null</code>.
     * </ol>
     *
     * @param catalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or
     *        <code>null</code> to return information from all catalogs.
     * @param schema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema.
     * @param table The table name to return information for.
     * @param unique <code>true</code> to return only unique indexes,
     *        <code>false</code> otherwise.
     * @param approx <code>true</code> if data values can be approximations,
     *        <code>false</code> otherwise.
     * @return A <code>ResultSet</code> with the requested index information
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getIndexInfo(String catalog, String schema, String table, boolean unique, boolean approximate) throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** @since 1.4
     *
     */
   public int getJDBCMajorVersion() throws SQLException
   {
      return 1;
   }

   /** @since 1.4
     *
     */
   public int getJDBCMinorVersion() throws SQLException
   {
      return 0;
   }

   /** This method returns the number of hex characters allowed in an inline
     * binary literal.
     *
     * @return The number of hex characters allowed in a binary literal, 0 meaning
     *         either an unknown or unlimited number.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxBinaryLiteralLength() throws SQLException
   {
      return (32 * 1024);
   }

   /** This method returns the maximum length of a catalog name.
     *
     * @return The maximum length of a catalog name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxCatalogNameLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum length of a character literal.
     *
     * @return The maximum length of a character literal.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxCharLiteralLength() throws SQLException
   {
      return (32 * 1024);
   }

   /** This method returns the maximum length of a column name.
     *
     * @return The maximum length of a column name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnNameLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum number of columns in a GROUP BY statement.
     *
     * @return The maximum number of columns in a GROUP BY statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnsInGroupBy() throws SQLException
   {
      return 15;
   }

   /** This method returns the maximum number of columns in an index.
     *
     * @return The maximum number of columns in an index.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnsInIndex() throws SQLException
   {
      return 15;
   }

   /** This method returns the maximum number of columns in an ORDER BY statement.
     *
     * @return The maximum number of columns in an ORDER BY statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnsInOrderBy() throws SQLException
   {
      return 15;
   }

   /** This method returns the maximum number of columns in a SELECT statement.
     *
     * @return The maximum number of columns in a SELECT statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnsInSelect() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum number of columns in a table.
     *
     * @return The maximum number of columns in a table.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxColumnsInTable() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum number of connections this client
     * can have to the database.
     *
     * @return The maximum number of database connections.
     * @SQLException If an error occurs.
     *
     */
   public int getMaxConnections() throws SQLException
   {
      return 1;
   }

   /** This method returns the maximum length of a cursor name.
     *
     * @return The maximum length of a cursor name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxCursorNameLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum length of an index.
     *
     * @return The maximum length of an index.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxIndexLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum length of a procedure name.
     *
     * @return The maximum length of a procedure name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxProcedureNameLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum size of a row in bytes.
     *
     * @return The maximum size of a row.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxRowSize() throws SQLException
   {
      return (64 * 1024);
   }

   /** This method returns the maximum length of a schema name.
     *
     * @return The maximum length of a schema name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxSchemaNameLength() throws SQLException
   {
      return 0;
   }

   /** This method includes the maximum length of a SQL statement.
     *
     * @return The maximum length of a SQL statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxStatementLength() throws SQLException
   {
      return (64 * 1024);
   }

   /** This method returns the maximum number of statements that can be
     * active at any time.
     *
     * @return The maximum number of statements that can be active at any time.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxStatements() throws SQLException
   {

      // For version 8.1.

      // It is only 10 for earlier versions.
      return 20;
   }

   /** This method returns the maximum length of a table name.
     *
     * @return The maximum length of a table name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxTableNameLength() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum number of tables that may be referenced
     * in a SELECT statement.
     *
     * @return The maximum number of tables allowed in a SELECT statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxTablesInSelect() throws SQLException
   {
      return 0;
   }

   /** This method returns the maximum length of a user name.
     *
     * @return The maximum length of a user name.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxUserNameLength() throws SQLException
   {
      return 0;
   }

   /** This method tests whether or not the maximum row size includes BLOB's
     *
     * @return <code>true</code> if the maximum row size includes BLOB's,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean doesMaxRowSizeIncludeBlobs() throws SQLException
   {
      return false;
   }

   /** This method returns a list of a table's primary key columns.  These
     * are returned as a <code>ResultSet</code> with the following columns.
     * <p>
     * <ol>
     * <li>TABLE_CAT - The catalog of the table, which may be <code>null</code>.
     * <li>TABLE_SCHEM - The schema of the table, which may be <code>null</code>.
     * <li>TABLE_NAME - The name of the table.
     * <li>COLUMN_NAME - The name of the column.
     * <li>KEY_SEQ - The sequence number of the column within the primary key.
     * <li>PK_NAME - The name of the primary key, which may be <code>null</code>.
     * </ol>
     *
     * @param catalog The catalog to retrieve information from, or the empty string
     *        to return entities not associated with a catalog, or <code>null</code>
     *        to return information from all catalogs.
     * @param schema The schema to retrieve information from, or the empty string
     *        to return entities not associated with a schema.
     * @param table The table name to return information for.
     * @param columnPattern A pattern of column names to return information for.
     * @return A <code>ResultSet</code> with the primary key columns.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getPrimaryKeys(String catalog, String schema, String table) throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** @since 1.4
     *
     */
   public int getSQLStateType() throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** This method returns the list of database schemas as a
     * <code>ResultSet</code>, with one column - TABLE_SCHEM - that is the
     * name of the schema.
     *
     * @return A <code>ResultSet</code> with all the requested schemas.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getSchemas() throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** This method returns the string used to escape wildcards in search strings.
     *
     * @return The string used to escape wildcards in search strings.
     * @exception SQLException If an error occurs.
     *
     */
   public String getSearchStringEscape() throws SQLException
   {

      // I don't know??  What is it for DB2e?
      return "";
   }

   /** This method returns a list of the requested table as a
     * <code>ResultSet</code> with the following columns:
     * <p>
     * <ol>
     * <li>TABLE_CAT - The catalog the table is in, which may be <code>null</code>.
     * <li>TABLE_SCHEM - The schema the table is in, which may be <code>null</code>.
     * <li>TABLE_NAME - The name of the table.
     * <li>TABLE_TYPE - A string describing the table type.  This will be one
     * of the values returned by the <code>getTableTypes()</code> method.
     * <li>REMARKS - Comments about the table.
     * </ol>
     *
     * @param catalog The name of the catalog to return tables from,
     *        or "" to return tables from all catalogs.
     * @param schemaPattern A schema pattern for the schemas to return tables
     *        from, or "" to return tables from all schemas.
     * @param namePattern The pattern of table names to return.
     * @param types The list of table types to include; null returns all types.
     * @returns A <code>ResultSet</code> with all the requested tables.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet getTables(String catalog, String schemaPattern, String tableNamePattern, String[] types) throws SQLException
   {
      throw new SQLException("Not Supported");
   }

   /** This method returns the URL for this database.
     *
     * @return The URL string for this database, or <code>null</code> if it
     *         is not known.
     * @exception SQLException If an error occurs.
     *
     */
   public String getURL() throws SQLException
   {

      // DB2e currently ignores the URL, so I return a dummy one.
      return "wdbc:db2e:server";
   }

   /** This method returns the database username for this connection.
     *
     * @return The database username.
     * @exception SQLException If an error occurs.
     *
     */
   public String getUserName() throws SQLException
   {

      // DB2e currently ignores user name.
      return "user";
   }

   /** This method tests whether or not the database supports batch updates.
     *
     * @return <code>true</code> if batch updates are supported,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsBatchUpdates() throws SQLException
   {
      return false;
   }

   /** @since 1.4
     *
     */
   public boolean supportsMultipleOpenResults() throws SQLException
   {
      return false;
   }

   /** This method tests whether multiple result sets for a single statement are
     * supported.
     *
     * @return <code>true</code> if multiple result sets are supported for a
     *         single statement, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsMultipleResultSets() throws SQLException
   {
      return false;
   }

   /** This method test whether or not multiple transactions may be open
     * at once, as long as they are on different connections.
     *
     * @return <code>true</code> if multiple transactions on different
     *         connections are supported, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsMultipleTransactions() throws SQLException
   {
      return false;
   }

   /** @since 1.4
     * NOTE: removed from SW
     *
     */
   public boolean supportsNamedParameters() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not columns can be defined as NOT NULL.  A
     * fully JDBC compliant driver always returns <code>true</code>.
     *
     * @return <code>true</code> if NOT NULL columns are supported,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsNonNullableColumns() throws SQLException
   {
      return true;
   }

   /** This method tests whether or not the database supports cursors
     * remaining open across commits.
     *
     * @return <code>true</code> if cursors can remain open across commits,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsOpenCursorsAcrossCommit() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not the database supports cursors
     * remaining open across rollbacks.
     *
     * @return <code>true</code> if cursors can remain open across rollbacks,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsOpenCursorsAcrossRollback() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not the database supports statements
     * remaining open across commits.
     *
     * @return <code>true</code> if statements can remain open across commits,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsOpenStatementsAcrossCommit() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not the database supports statements
     * remaining open across rollbacks.
     *
     * @return <code>true</code> if statements can remain open across rollbacks,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsOpenStatementsAcrossRollback() throws SQLException
   {
      return false;
   }

   /** This method tests whether the specified result set type and result set
     * concurrency type are supported by the database.
     *
     * @param type The desired result type, which is one of the constants
     *        defined in <code>ResultSet</code>.
     * @param concur The desired concurrency type, which is one of the constants
     *        defined in <code>ResultSet</code>.
     * @return <code>true</code> if the result set type is supported,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     * @see ResultSet
     *
     */
   public boolean supportsResultSetConcurrency(int type, int concurrency) throws SQLException
   {
      if(type == ResultSet.TYPE_SCROLL_INSENSITIVE)
         return false;
      switch(concurrency)
      {
         case ResultSet.CONCUR_READ_ONLY:
            return true;

         case ResultSet.CONCUR_UPDATABLE:
            return false;
      }
      return false;
   }

   /** This method tests whether or not the datbase supports the specified
     * result type.
     *
     * @param type The desired result type, which is one of the constants
     * defined in <code>ResultSet</code>.
     *
     * @return <code>true</code> if the result set type is supported,
     * <code>false</code> otherwise.
     *
     * @exception SQLException If an error occurs.
     *
     * @see ResultSet
     *
     */
   public boolean supportsResultSetType(int type) throws SQLException
   {
      switch(type)
      {
         case ResultSet.TYPE_FORWARD_ONLY:
            return true;

         case ResultSet.TYPE_SCROLL_INSENSITIVE:
            return false;

         case ResultSet.TYPE_SCROLL_SENSITIVE:
            return true;
      }
      return false;
   }

   /** This method tests whether or not the database supports the specified
     * transaction isolation level.
     *
     * @param level The transaction isolation level.
     *
     * @return <code>true</code> if the specified transaction isolation level
     *         is supported, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsTransactionIsolationLevel(int level) throws SQLException
   {
      switch(level)
      {
         case Connection.TRANSACTION_READ_COMMITTED:
            return true;

         case Connection.TRANSACTION_READ_UNCOMMITTED:
         case Connection.TRANSACTION_REPEATABLE_READ:
         case Connection.TRANSACTION_SERIALIZABLE:
         default:
            return false;
      }
   }

   /** This method tests whether or not the database supports transactions.
     *
     * @return <code>true</code> if the database supports transactions,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean supportsTransactions() throws SQLException
   {
      return true;
   }
}
