
/*********************************************************************************
  *  SuperWaba Virtual Machine, version 4                                         *
  *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
  *                 and Andy DePue <sir_anodos@yahoo.com>                         *
  *  All Rights Reserved                                                          *
  *                                                                               *
  *  This library and virtual machine is free software; you can redistribute      *
  *  it and/or modify it under the terms of the Amended GNU Lesser General        *
  *  Public License distributed with this software.                               *
  *                                                                               *
  *  This library and virtual machine is distributed in the hope that it will     *
  *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
  *                                                                               *
  *  All software that links or interacts with this library and virtual machine,  *
  *  or any derivative works, require the software to prominently display the     *
  *  following notice:                                                            *
  *                                                                               *
  *                   Created with SuperWaba                                      *
  *                  http://www.superwaba.org                                     *
  *                                                                               *
  *  Please see the software license located at SuperWabaSDK/license.txt          *
  *  for more details.                                                            *
  *                                                                               *
  *  You should have received a copy of the License along with this software;     *
  *  if not, write to                                                             *
  *                                                                               *
  *     Guilherme Campos Hazan                                                    *
  *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
  *     Rio de Janeiro / RJ - Brazil                                              *
  *     Cep: 22050-000                                                            *
  *     E-mail: support@superwaba.com.br                                          *
  *                                                                               *
  *********************************************************************************/
package superwaba.ext.xplat.sql.db2e;
import waba.util.IntHashtable;
import superwaba.ext.xplat.sql.*;
import waba.sys.Vm;
import superwaba.ext.xplat.sql.db2e.db2ex.*;

// ResetSet interface below.

class Db2eResultSet extends ResultSet
{
   private int _stmtHandle;
   private Db2e _db;
   private int _curRow = 0;
   private WdbcColumnInfo[] _columns;
   private IntHashtable _columnByName;
   private int _colCount = 0;
   private int _maxFieldSize = 0;
   private Object[] _boundColumns;
   private boolean _wasNull = false;
   private Db2eStatement _parent;
   private Db2eRsmd _meta;
   public Db2eResultSet(Db2e db, Db2eStatement parent, int stmtHandle, int maxFieldSize)
   {
      _db = db;
      _parent = parent;
      _stmtHandle = stmtHandle;
      _maxFieldSize = maxFieldSize;
      _meta = new Db2eRsmd();
   }
   private void handleError(DB2Exception error) throws SQLException
   {

      /*    Vm.debug("DB2 state: " + (new String(error.getSQLState())));
      Vm.debug("DB2 native: " + error.getNativeErr());
      Vm.debug("DB2 SQL Error: " + error.getSQLErr()); */
      throw new SQLException(error.getMessage(), new String(error.getSQLState()), error.getNativeErr());
   }
   private void notSupported() throws SQLException
   {
      throw new SQLException("Not Supported");
   }
   public void setMaxFieldSize(int maxFieldSize) throws SQLException
   {
      _maxFieldSize = 0;
   }
   public void reset(boolean resetColumns) throws SQLException
   {
      if(resetColumns)
      {
         _columns = null;
         _colCount = 0;
         _boundColumns = null;
      }
      _curRow = 0;
   }

   /** This method positions the result set to the specified absolute row.
     * Positive numbers are row offsets from the beginning of the result
     * set (numbering starts from row 1) and negative numbers are row offsets
     * from the end of the result set (numbering starts from -1).
     *
     * @param row The row to position the result set to.
     *
     * @return <code>true</code> if the current position was changed,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean absolute(int row) throws SQLException
   {
      try
      {

         //      if(row != _curRow)

         //      {
         _curRow = row;
         return _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_ABSOLUTE, row);

      //      }

      //      return true;
      }
      catch(DB2Exception error)
      {
         handleError(error);
         return false;
      }
   }

   /** This method repositions the cursor to after the last row in the result
     * set.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void afterLast() throws SQLException
   {
      try
      {
         int lastRow = _parent.getUpdateCount();

         // Go past last.
         lastRow++;
         if(lastRow != _curRow)
         {
            _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_ABSOLUTE, lastRow);
            _curRow = lastRow;
         }
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method repositions the cursor to before the first row in the
     * result set.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void beforeFirst() throws SQLException
   {
      try
      {
         if(_curRow != 0)
         {
            _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_ABSOLUTE, 0);
            _curRow = 0;
         }
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method cancels any changes that have been made to a row.  If
     * the <code>rowUpdate</code> method has been called, then the changes
     * cannot be undone.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void cancelRowUpdates() throws SQLException
   {
      notSupported();
   }

   /** This method deletes the current row in the database.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void deleteRow() throws SQLException
   {
      notSupported();
   }
   protected void loadColumns() throws SQLException
   {
      int colCount = _meta.getColumnCount();
      _columns = new WdbcColumnInfo[colCount + 1];
      _columnByName = new IntHashtable(colCount + 1, 1.0f);

      //      Vm.debug("loadColumns - colCount=" + colCount);
      for(int i = 1;i <= colCount;i++)
      {
         try
         {
            WdbcColumnInfo ci = new WdbcColumnInfo();
            _db.describeCol(_stmtHandle, i, ci);
            switch(ci.type)
            {
               case Db2e.SQL_DECIMAL:
                  ci.displaySize = ci.scale + ci.precision + 1;
                  break;

               case Db2e.SQL_INTEGER:
                  ci.displaySize = 12;
                  break;

               case Db2e.SQL_SMALLINT:
                  ci.displaySize = 6;
                  break;

               case Db2e.SQL_TYPE_DATE:
                  ci.displaySize = 10;
                  break;

               case Db2e.SQL_TYPE_TIME:
                  ci.displaySize = 9;
                  break;

               case Db2e.SQL_TYPE_TIMESTAMP:
                  ci.displaySize = 20;
                  break;

               default:
                  ci.displaySize = ci.precision;
                  break;
            }
            ci.type = Db2eStatement.CONV_FROM_DB2E.get(ci.type);
            _columns[i] = ci;
            _columnByName.put(ci.name.hashCode(), i);
         }
         catch(DB2Exception error)
         {
            handleError(error);
         }
      }
   }

   /** This method returns the column index of the specified named column.
     *
     * @param column The name of the column.
     * @return The index of the column.
     * @exception SQLException If an error occurs.
     *
     */
   public int findColumn(String columnName) throws SQLException
   {

      // This is supposed to be faster than using 'if(_columnByName == null)'

      // since _columnByName should != null most of the time.
      try
      {
         return _columnByName.get(columnName.hashCode());
      }
      catch(NullPointerException npe)
      {
         loadColumns();
         return _columnByName.get(columnName.hashCode());
      }
   }

   /** This method repositions the cursor on the first row in the
     * result set.
     *
     * @return <code>true</code> if the cursor is on a valid row;
     *         <code>false</code> if there are no rows in the result set.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean first() throws SQLException
   {
      try
      {
         if(_curRow != 1)
         {
            boolean ret = _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_FIRST, 0);
            _curRow = 1;
            return ret;
         }
         return true;
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      return false;
   }
   public Blob getBlob(String colName) throws SQLException
   {
      return getBlob(findColumn(colName));
   }
   public boolean getBoolean(String columnName) throws SQLException
   {
      return getBoolean(findColumn(columnName));
   }
   public byte getByte(String columnName) throws SQLException
   {
      return getByte(findColumn(columnName));
   }
   public byte[] getBytes(String columnName) throws SQLException
   {
      return getBytes(findColumn(columnName));
   }
   public Date getDate(String columnName) throws SQLException
   {
      return getDate(findColumn(columnName));
   }
   public double getDouble(String columnName) throws SQLException
   {
      return getDouble(findColumn(columnName));
   }
   public float getFloat(String columnName) throws SQLException
   {
      return getFloat(findColumn(columnName));
   }
   public int getInt(String columnName) throws SQLException
   {
      return getInt(findColumn(columnName));
   }
   public long getLong(String columnName) throws SQLException
   {
      return getLong(findColumn(columnName));
   }
   public Object getObject(String columnName) throws SQLException
   {
      return getObject(findColumn(columnName));
   }
   public Ref getRef(String colName) throws SQLException
   {
      return getRef(findColumn(colName));
   }
   public short getShort(String columnName) throws SQLException
   {
      return getShort(findColumn(columnName));
   }
   public String getString(String columnName) throws SQLException
   {
      return getString(findColumn(columnName));
   }
   public Time getTime(String columnName) throws SQLException
   {
      return getTime(findColumn(columnName));
   }
   public Timestamp getTimestamp(String columnName) throws SQLException
   {
      return getTimestamp(findColumn(columnName));
   }

   /** This method returns the specified column value as a BLOB.
     *
     * @param index The index of the column value to return.
     * @return The value of the column as a BLOB.
     * @exception SQLException If an error occurs.
     *
     */
   public Blob getBlob(int i) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBlob blob;
      try
      {
         blob = (Db2eBlob)_boundColumns[i];
         try
         {
            if(blob._bb.length == Db2e.SQL_NULL_DATA)
            {
               _wasNull = true;
               return null;
            }
            _wasNull = false;
            return blob;
         }
         catch(NullPointerException npe)
         {


         // Handles if 'blob' is null.

         // Note, this falls through so that both NullPointerException catches

         // can share code.
         }
      }
      catch(NullPointerException npe)
      {

         // Handles if '_boundColumns' is null.
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }
      int size = _maxFieldSize;
      if(size == 0)
         size = _meta.getColumnDisplaySize(i);

      // Make a new blob and bind it.
      blob = new Db2eBlob(size + 2);
      _boundColumns[i] = blob;

      // Bind column.

      // We use two try/catch blocks so that we can still get the data

      // if it is just a warning.
      try
      {
         _db.bindBlob(_stmtHandle, i, blob._bb);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }

      // Get the value for this row.
      try
      {
         blob._bb.length = _db.getBlob(_stmtHandle, i, blob.getByteBuffer());
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      if(blob._bb.length == Db2e.SQL_NULL_DATA)
      {
         _wasNull = true;
         return null;
      }
      _wasNull = false;
      return blob;
   }

   /** This method returns the value of the specified column as a Java
     * <code>boolean</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>boolean</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean getBoolean(int columnIndex) throws SQLException
   {
      return getInt(columnIndex) != 0;
   }

   /** This method returns the value of the specified column as a Java
     * <code>byte</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>byte</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public byte getByte(int columnIndex) throws SQLException
   {
      return (byte)(getInt(columnIndex) & 0xFF);
   }

   /** This method returns the value of the specified column as a Java
     * byte array.
     *
     * @param index The index of the column to return.
     * @return The column value as a byte array
     * @exception SQLException If an error occurs.
     *
     */
   public byte[] getBytes(int columnIndex) throws SQLException
   {
      Blob blob = getBlob(columnIndex);
      if(blob == null)
         return null;
      return blob.getBytes(0, (int)blob.length());
   }

   /** This method returns the concurrency type of this result set.  This will
     * be one of the CONCUR_* constants defined in this interface.
     *
     * @return The result set concurrency type.
     * @exception SQLException If an error occurs.
     *
     */
   public int getConcurrency() throws SQLException
   {
      return CONCUR_READ_ONLY;
   }

   /** This method returns the name of the database cursor used by this
     * result set.
     *
     * @return The name of the database cursor used by this result set.
     * @exception SQLException If an error occurs.
     *
     */
   public String getCursorName() throws SQLException
   {
      return null;
   }

   /** This method returns the value of the specified column as a Java
     * <code>java.sql.Date</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>java.sql.Date</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public Date getDate(int columnIndex) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBoundDate bd;
      try
      {
         bd = (Db2eBoundDate)_boundColumns[columnIndex];
         try
         {
            if((_wasNull = (bd.nullIndicator == Db2e.SQL_NULL_DATA)))
               return null;
            bd.translateFromDb2e();
            return new Date(bd.year, bd.month, bd.day);
         }

         // Handles if bts was null.
         catch(NullPointerException npe)
         {


         // Note, this will just fall through... that way, both null handlers

         // can share the same bts binding code.
         }
      }

      // Handles if _boundColumns was null.
      catch(NullPointerException npe)
      {
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }
      bd = new Db2eBoundDate();
      _boundColumns[columnIndex] = bd;
      try
      {
         _db.bindDate(_stmtHandle, columnIndex, bd);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      try
      {
         _db.getDate(_stmtHandle, columnIndex, bd);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      if((_wasNull = (bd.nullIndicator == Db2e.SQL_NULL_DATA)))
         return null;
      bd.translateFromDb2e();
      return new Date(bd.year, bd.month, bd.day);
   }

   /** This method returns the value of the specified column as a Java
     * <code>double</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>double</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public double getDouble(int columnIndex) throws SQLException
   {
      notSupported();
      return 0.0;
   }

   /** This method returns the value of the specified column as a Java
     * <code>float</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>float</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public float getFloat(int columnIndex) throws SQLException
   {
      notSupported();
      return 0.0f;
   }

   /** This method returns the value of the specified column as a Java
     * <code>int</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>int</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public int getInt(int columnIndex) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBoundInt bi;
      try
      {
         bi = (Db2eBoundInt)_boundColumns[columnIndex];
         try
         {
            _wasNull = bi.nullIndicator == Db2e.SQL_NULL_DATA;
            return bi.value;
         }

         // Handles if bi was null.
         catch(NullPointerException npe)
         {


         // Note, this will just fall through... that way, both null handlers

         // can share the same bi binding code.
         }
      }

      // Handles if _boundColumns was null.
      catch(NullPointerException npe)
      {
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }
      bi = new Db2eBoundInt();
      _boundColumns[columnIndex] = bi;
      try
      {
         _db.bindInteger(_stmtHandle, columnIndex, bi);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      try
      {
        _db.getInteger(_stmtHandle, columnIndex, bi);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      _wasNull = bi.nullIndicator == Db2e.SQL_NULL_DATA;
      return bi.value;
   }

   /** This method returns the value of the specified column as a Java
     * <code>long</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>long</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public long getLong(int columnIndex) throws SQLException
   {
      return (long)getInt(columnIndex);
   }

   /** This method returns the value of the specified column as a Java
     * <code>Object</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as an <code>Object</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public Object getObject(int columnIndex) throws SQLException
   {
      notSupported();
      return null;
   }

   /** This method returns a <code>Ref</code> for the specified column which
     * represents the structured type for the column.
     *
     * @param index  The index of the column to return.
     * @return A <code>Ref</code> object for the column
     * @exception SQLException If an error occurs.
     *
     */
   public Ref getRef(int i) throws SQLException
   {
      notSupported();
      return null;
   }

   /** This method returns the current row number in the cursor.  Numbering
     * begins at index 1.
     *
     * @return The current row number, or 0 if there is not current row.
     * @exception SQLException If an error occurs.
     *
     */
   public int getRow() throws SQLException
   {
      return _curRow;
   }

   /** This method returns the value of the specified column as a Java
     * <code>short</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>short</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public short getShort(int columnIndex) throws SQLException
   {
      return (short)getInt(columnIndex);
   }

   /** This method returns a the <code>Statement</code> that was used to
     * produce this result set.
     *
     * @return The <code>Statement</code> used to produce this result set.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public Statement getStatement() throws SQLException
   {
      return _parent;
   }

   /** This method returns the value of the specified column as a Java
     * <code>String</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>String</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public String getString(int columnIndex) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBoundString bs;
      try
      {
         bs = (Db2eBoundString)_boundColumns[columnIndex];
         try
         {
            if((_wasNull = (bs.getLength() == Db2e.SQL_NULL_DATA)))
               return null;
            return bs.toString();
         }

         // Handles if bs was null.
         catch(NullPointerException npe)
         {


         //          Vm.debug("getString - NPE 2.");

         // Note, this will just fall through... that way, both null handlers

         // can share the same bs binding code.
         }
      }

      // Handles if _boundColumns was null.
      catch(NullPointerException npe)
      {

         //        Vm.debug("getString - NPE 1.");
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }

      //      Vm.debug("getString - common.");
      int size = _maxFieldSize;
      if(size == 0)
         size = _meta.getColumnDisplaySize(columnIndex);

      //      Vm.debug("size = " + size);

      // Make a new blob and bind it.
      bs = new Db2eBoundString(size + 2);
      _boundColumns[columnIndex] = bs;

      //      Vm.debug("Calling bindString.");
      try
      {
         _db.bindString(_stmtHandle, columnIndex, bs);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }

      //      Vm.debug("Calling getString");
      try
      {
         bs.setLength(_db.getString(_stmtHandle, columnIndex, bs.getBuf()));
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }

      //      Vm.debug("Checking null.");
      if((_wasNull = bs.getLength() == Db2e.SQL_NULL_DATA))
         return null;

      //      Vm.debug("Not null, returning string.");
      return bs.toString();
   }

   /** This method returns the value of the specified column as a Java
     * <code>java.sql.Time</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>java.sql.Time</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public Time getTime(int columnIndex) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBoundTime bt;
      try
      {
         bt = (Db2eBoundTime)_boundColumns[columnIndex];
         try
         {
            if((_wasNull = (bt.nullIndicator == Db2e.SQL_NULL_DATA)))
               return null;
            bt.translateFromDb2e();
            return new Time(bt.hour, bt.minute, bt.second);
         }

         // Handles if bts was null.
         catch(NullPointerException npe)
         {


         // Note, this will just fall through... that way, both null handlers

         // can share the same bts binding code.
         }
      }

      // Handles if _boundColumns was null.
      catch(NullPointerException npe)
      {
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }
      bt = new Db2eBoundTime();
      _boundColumns[columnIndex] = bt;
      try
      {
         _db.bindTime(_stmtHandle, columnIndex, bt);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      try
      {
         _db.getTime(_stmtHandle, columnIndex, bt);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      if((_wasNull = (bt.nullIndicator == Db2e.SQL_NULL_DATA)))
         return null;
      bt.translateFromDb2e();
      return new Time(bt.hour, bt.minute, bt.second);
   }

   /** This method returns the value of the specified column as a Java
     * <code>java.sql.Timestamp</code>.
     *
     * @param index The index of the column to return.
     * @return The column value as a <code>java.sql.Timestamp</code>.
     * @exception SQLException If an error occurs.
     *
     */
   public Timestamp getTimestamp(int columnIndex) throws SQLException
   {

      // This method is optimized to remove if(... == null) on the most

      // common code path, because the VM already checks the 'null' status

      // on object accesses.  While this speeds up the most common code path,

      // the least common code path takes much longer to execute than it would

      // with a simple 'if/else' statement (because a new NullPointerException

      // object must be instantiated, the class exception handlers must be

      // iterated, and the NullPointerException must then be compared to the

      // exception handler's exception entry to see if they are compatible...

      // considerably more involved than a simple 'if/else').

      // This works here, because the NullPointerException should happen once

      // on a ResultSet, and _all_ further accesses will take the most common

      // code path.
      Db2eBoundTimestamp bts;
      try
      {
         bts = (Db2eBoundTimestamp)_boundColumns[columnIndex];
         try
         {
            if((_wasNull = (bts.nullIndicator == Db2e.SQL_NULL_DATA)))
               return null;
            bts.translateFromDb2e();
            return new Timestamp(bts.year, bts.month, bts.day, bts.hour, bts.minute, bts.second, bts.fraction);
         }

         // Handles if bts was null.
         catch(NullPointerException npe)
         {


         // Note, this will just fall through... that way, both null handlers

         // can share the same bts binding code.
         }
      }

      // Handles if _boundColumns was null.
      catch(NullPointerException npe)
      {
         _boundColumns = new Object[_meta.getColumnCount() + 1];
      }
      bts = new Db2eBoundTimestamp();
      _boundColumns[columnIndex] = bts;
      try
      {
         _db.bindTimestamp(_stmtHandle, columnIndex, bts);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      try
      {
         _db.getTimestamp(_stmtHandle, columnIndex, bts);
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
      if((_wasNull = (bts.nullIndicator == Db2e.SQL_NULL_DATA)))
         return null;
      bts.translateFromDb2e();
      return new Timestamp(bts.year, bts.month, bts.day, bts.hour, bts.minute, bts.second, bts.fraction);
   }

   /** This method returns the result set type of this result set.  This will
     * be one of the TYPE_* constants defined in this interface.
     *
     * @return The result set type.
     * @exception SQLException If an error occurs.
     *
     */
   public int getType() throws SQLException
   {
      return _parent.getResultSetType();
   }

   /** This method inserts the current row into the database.  The result set
     * must be positioned on the insert row in order to call this method
     * successfully.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void insertRow() throws SQLException
   {
      notSupported();
   }

   /** This method tests whether or not the cursor is after the last row
     * in the result set.
     *
     * @return <code>true</code> if the cursor is positioned after the last
     *         row, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean isAfterLast() throws SQLException
   {
      return _curRow > _parent.getUpdateCount();
   }

   /** This method tests whether or not the cursor is before the first row
     * in the result set.
     *
     * @return <code>true</code> if the cursor is positioned before the first
     *         row, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean isBeforeFirst() throws SQLException
   {
      return _curRow < 1;
   }

   /** This method tests whether or not the cursor is positioned on the first
     * row in the result set.
     *
     * @return <code>true</code> if the cursor is positioned on the first
     *         row, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean isFirst() throws SQLException
   {
      return _curRow == 1;
   }

   /** This method tests whether or not the cursor is on the last row
     * in the result set.
     *
     * @return <code>true</code> if the cursor is positioned on the last
     *         row, <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean isLast() throws SQLException
   {
      return _curRow == _parent.getUpdateCount();
   }

   /** This method repositions the cursor on the last row in the result
     * set.
     *
     * @return <code>true</code> if the cursor is on a valid row;
     *         <code>false</code> if there are no rows in the result set.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean last() throws SQLException
   {
      try
      {
         int lastRow = _parent.getUpdateCount();

         // Go past last.
         if(lastRow != _curRow)
         {
            boolean ret = _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_ABSOLUTE, lastRow);
            _curRow = lastRow;
            return ret;
         }
         return true;
      }
      catch(DB2Exception error)
      {
         handleError(error);
         return false;
      }
   }
   public void moveToCurrentRow() throws SQLException
   {
      notSupported();
   }
   public void moveToInsertRow() throws SQLException
   {
      notSupported();
   }

   /** This method advances to the next row in the result set.  Any streams
     * open on the current row are closed automatically.
     *
     * @return <code>true</code> if the next row exists, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean next() throws SQLException
   {
      try
      {
         _curRow++;
         return _db.fetch(_stmtHandle);
      }
      catch(DB2Exception error)
      {
         handleError(error);
         return false;
      }
   }

   /** This method moves the current position to the previous row in the
     * result set.
     *
     * @return <code>true</code> if the previous row exists, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean previous() throws SQLException
   {
      try
      {
         boolean r = _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_PRIOR, 1);
         if(r)
            _curRow--;
         return r;
      }
      catch(DB2Exception error)
      {
         handleError(error);
         return false;
      }
   }

   /** This method refreshes the contents of the current row from the database.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void refreshRow() throws SQLException
   {
      notSupported();
   }

   /** This method moves the result set position relative to the current row.
     * The offset can be positive or negative.
     *
     * @param row The relative row position to move to.
     * @return <code>true</code> if the current position was changed,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean relative(int rows) throws SQLException
   {
      try
      {
         boolean r = _db.fetchScroll(_stmtHandle, Db2e.SQL_FETCH_RELATIVE, rows);
         _curRow += rows;
         return r;
      }
      catch(DB2Exception error)
      {
         handleError(error);
         return false;
      }
   }

   /** This method tests whether or not the current row in the result set
     * has been deleted.  Deletes must be visible in order of this method to
     * detect the deletion.
     *
     * @return <code>true</code> if the row has been deleted, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean rowDeleted() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not the current row in the result set
     * has been inserted.  Inserts must be visible in order of this method to
     * detect the insert.
     *
     * @return <code>true</code> if the row has been inserted, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean rowInserted() throws SQLException
   {
      return false;
   }

   /** This method tests whether or not the current row in the result set
     * has been updated.  Updates must be visible in order of this method to
     * detect the update.
     *
     * @return <code>true</code> if the row has been updated, <code>false</code>
     *         otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean rowUpdated() throws SQLException
   {
      return false;
   }

   /** @since 1.4
     *
     */
   public void updateBlob(int columnIndex, Blob x) throws SQLException
   {
      notSupported();
   }
   public void updateBlob(String columnName, Blob x) throws SQLException
   {
      notSupported();
   }
   public void updateBoolean(int columnIndex, boolean x) throws SQLException
   {
      notSupported();
   }
   public void updateBoolean(String columnName, boolean x) throws SQLException
   {
      notSupported();
   }
   public void updateByte(int columnIndex, byte x) throws SQLException
   {
      notSupported();
   }
   public void updateByte(String columnName, byte x) throws SQLException
   {
      notSupported();
   }
   public void updateBytes(String columnName, byte[] x) throws SQLException
   {
      notSupported();
   }
   public void updateBytes(int columnIndex, byte[] x) throws SQLException
   {
      notSupported();
   }
   public void updateDate(int columnIndex, Date x) throws SQLException
   {
      notSupported();
   }
   public void updateDate(String columnName, Date x) throws SQLException
   {
      notSupported();
   }
   public void updateDouble(int columnIndex, double x) throws SQLException
   {
      notSupported();
   }
   public void updateDouble(String columnName, double x) throws SQLException
   {
      notSupported();
   }
   public void updateFloat(int columnIndex, float x) throws SQLException
   {
      notSupported();
   }
   public void updateFloat(String columnName, float x) throws SQLException
   {
      notSupported();
   }
   public void updateInt(String columnName, int x) throws SQLException
   {
      notSupported();
   }
   public void updateInt(int columnIndex, int x) throws SQLException
   {
      notSupported();
   }
   public void updateLong(String columnName, long x) throws SQLException
   {
      notSupported();
   }
   public void updateLong(int columnIndex, long x) throws SQLException
   {
      notSupported();
   }
   public void updateNull(int columnIndex) throws SQLException
   {
      notSupported();
   }
   public void updateNull(String columnName) throws SQLException
   {
      notSupported();
   }
   public void updateObject(String columnName, Object x) throws SQLException
   {
      notSupported();
   }
   public void updateObject(int columnIndex, Object x) throws SQLException
   {
      notSupported();
   }
   public void updateObject(String columnName, Object x, int scale) throws SQLException
   {
      notSupported();
   }
   public void updateObject(int columnIndex, Object x, int scale) throws SQLException
   {
      notSupported();
   }
   public void updateRef(int columnIndex, Ref x) throws SQLException
   {
      notSupported();
   }
   public void updateRef(String columnName, Ref x) throws SQLException
   {
      notSupported();
   }
   public void updateRow() throws SQLException
   {
      notSupported();
   }
   public void updateShort(int columnIndex, short x) throws SQLException
   {
      notSupported();
   }
   public void updateShort(String columnName, short x) throws SQLException
   {
      notSupported();
   }
   public void updateString(int columnIndex, String x) throws SQLException
   {
      notSupported();
   }
   public void updateString(String columnName, String x) throws SQLException
   {
      notSupported();
   }
   public void updateTime(int columnIndex, Time x) throws SQLException
   {
      notSupported();
   }
   public void updateTime(String columnName, Time x) throws SQLException
   {
      notSupported();
   }
   public void updateTimestamp(String columnName, Timestamp x) throws SQLException
   {
      notSupported();
   }
   public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException
   {
      notSupported();
   }

   /** This method tests whether the value of the last column that was fetched
     * was actually a SQL NULL value.
     *
     * @return <code>true</code> if the last column fetched was a NULL,
     *         <code>false</code> otherwise.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean wasNull() throws SQLException
   {
      return _wasNull;
   }

   /** This method clears all warnings associated with this result set.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void clearWarnings() throws SQLException
   {


   //    _warnings = null;
   }

   /** This method closes the result set and frees any associated resources.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void close() throws SQLException
   {
      _parent.closeResultSet(this);
   }

   /** This method returns the current fetch direction for this result set.
     *
     * @return The fetch direction for this result set.
     * @exception SQLException If an error occurs.
     *
     */
   public int getFetchDirection() throws SQLException
   {
      return FETCH_FORWARD;
   }

   /** This method returns the current number of rows that will be fetched
     * from the database at a time.
     *
     * @return The current fetch size for this result set.
     * @exception SQLException If an error occurs.
     *
     */
   public int getFetchSize() throws SQLException
   {
      return 1;
   }

   /** This method returns data about the columns returned as part of the
     * result set as a <code>ResultSetMetaData</code> instance.
     *
     * @return The <code>ResultSetMetaData</code> instance for this result set.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSetMetaData getMetaData() throws SQLException
   {
      return _meta;
   }

   /** This method returns the first SQL warning associated with this result
     * set.  Any additional warnings will be chained to this one.
     *
     * @return The first SQLWarning for this result set, or <code>null</code> if
     *         there are no warnings.
     * @exception SQLException If an error occurs.
     *
     */
   public SQLWarning getWarnings() throws SQLException
   {
      return null;
   }

   /** This method provides a hint to the driver about which direction the
     * result set will be processed in.
     *
     * @param direction The direction in which rows will be processed. (Values?)
     * @exception SQLException If an error occurs.
     *
     */
   public void setFetchDirection(int direction) throws SQLException
   {
      if(direction != FETCH_FORWARD)
         throw new SQLException("Unsupported fetch direction.");
   }

   /** This method provides a hint to the driver about how many rows at a
     * time it should fetch from the database.
     *
     * @param rows The number of rows the driver should fetch per call.
     * @exception SQLException If an error occurs.
     *
     */
   public void setFetchSize(int rows) throws SQLException
   {
      if(rows != 1)
         throw new SQLException("Unsupported fetch size.");
   }

   // ResultSetMetaData interface methods below.

   class Db2eRsmd extends ResultSetMetaData
   {

      /** This method returns the name of the catalog containing the specified
        * column.
        *
        * @param index The index of the column to check the catalog name for.
        * @return The name of the catalog containing the column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getCatalogName(int column) throws SQLException
      {
         return null;
      }

      /** This method returns the name of the Java class which will be used to
        * create objects representing the data in this column.
        *
        * @param index The index of the column to check.
        * @return The name of the Java class that will be used for values in
        *         this column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getColumnClassName(int column) throws SQLException
      {
         notSupported();
         return null;
      }

      /** This method returns the number of columns in the result set.
        *
        * @return The number of columns in the result set.
        * @exception SQLException If an error occurs.
        *
        */
      public int getColumnCount() throws SQLException
      {
         if(_colCount == 0)
         {
            try
            {
               return _colCount = _db.getNumResultCols(_stmtHandle);
            }
            catch(DB2Exception error)
            {
               handleError(error);
               return 0;
            }
         }
         return _colCount;
      }

      /** This method returns the maximum number of characters that can be used
        * to display a value in this column.
        *
        * @param index The index of the column to check.
        * @return The maximum number of characters that can be used to display a
        *         value for this column.
        * @exception SQLException If an error occurs.
        *
        */
      public int getColumnDisplaySize(int column) throws SQLException
      {
         try
         {

            //        Vm.debug("getColumnDisplaySize(" + column + ")");
            return _columns[column].displaySize;
         }
         catch(NullPointerException npe)
         {

            //        Vm.debug("getColumnDisplaySize - NPE");
            loadColumns();

            //        Vm.debug("loaded columns.");
            return _columns[column].displaySize;
         }
      }

      /** This method returns a string that should be used as a caption for this
        * column for user display purposes.
        *
        * @param index The index of the column to check.
        * @return A display string for the column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getColumnLabel(int column) throws SQLException
      {
         return getColumnName(column);
      }

      /** This method returns the name of the specified column.
        *
        * @param index The index of the column to return the name of.
        * @return The name of the column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getColumnName(int column) throws SQLException
      {
         try
         {
            return _columns[column].name;
         }
         catch(NullPointerException npe)
         {
            loadColumns();
            return _columns[column].name;
         }
      }

      /** This method returns the SQL type of the specified column.  This will
        * be one of the constants from <code>Types</code>.
        *
        * @param index The index of the column to check the SQL type of.
        * @return The SQL type for this column.
        * @exception SQLException If an error occurs.
        * @see Types
        *
        */
      public int getColumnType(int column) throws SQLException
      {
         try
         {
            return _columns[column].type;
         }
         catch(NullPointerException npe)
         {
            loadColumns();
            return _columns[column].type;
         }
      }

      /** This method returns the name of the SQL type for this column.
        *
        * @param index The index of the column to check the SQL type name for.
        * @return The name of the SQL type for this column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getColumnTypeName(int column) throws SQLException
      {
         notSupported();
         return null;
      }

      /** This method returns the precision of the specified column, which is the
        * number of decimal digits it contains.
        *
        * @param index The index of the column to check the precision on.
        * @return The precision of the specified column.
        * @exception SQLException If an error occurs.
        *
        */
      public int getPrecision(int column) throws SQLException
      {
         try
         {
            return _columns[column].precision;
         }
         catch(NullPointerException npe)
         {
            loadColumns();
            return _columns[column].precision;
         }
      }

      /** This method returns the scale of the specified column, which is the
        * number of digits to the right of the decimal point.
        *
        * @param index The index column to check the scale of.
        * @return The scale of the column.
        * @exception SQLException If an error occurs.
        *
        */
      public int getScale(int column) throws SQLException
      {
         try
         {
            if(_columns[column].allowNulls)
               return columnNullable;
            return columnNoNulls;
         }
         catch(NullPointerException npe)
         {
            loadColumns();
            if(_columns[column].allowNulls)
               return columnNullable;
            return columnNoNulls;
         }
      }

      /** This method returns the name of the schema that contains the specified
        * column.
        *
        * @param index The index of the column to check the schema name for.
        * @return The name of the schema that contains the column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getSchemaName(int column) throws SQLException
      {
         notSupported();
         return null;
      }

      /** This method returns the name of the table containing the specified
        * column.
        *
        * @param index The index of the column to check the table name for.
        * @return The name of the table containing the column.
        * @exception SQLException If an error occurs.
        *
        */
      public String getTableName(int column) throws SQLException
      {
         notSupported();
         return null;
      }

      /** This method test whether or not the column is an auto-increment column.
        * Auto-increment columns are read-only.
        *
        * @param index The index of the column to test.
        * @return <code>true</code> if the column is auto-increment, <code>false</code>
        *         otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isAutoIncrement(int column) throws SQLException
      {
         return false;
      }

      /** This method tests whether or not a column is case sensitive in its values.
        *
        * @param index The index of the column to test.
        * @return <code>true</code> if the column value is case sensitive,
        *         <code>false</code> otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isCaseSensitive(int column) throws SQLException
      {
         return true;
      }

      /** This method tests whether or not the column stores a monetary value.
        *
        * @param index The index of the column to test.
        * @return <code>true</code> if the column contains a monetary value,
        *         <code>false</code> otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isCurrency(int column) throws SQLException
      {
         return false;
      }

      /** This method tests whether or not the column is writable.  This
        * does guarantee that a write will be successful.
        *
        * @param index The index of the column to check for writability.
        * @return <code>true</code> if the column is writable,
        *         <code>false</code> otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isDefinitelyWritable(int column) throws SQLException
      {
         return true;
      }

      /** This method returns a value indicating whether or not the specified
        * column may contain a NULL value.
        *
        * @param index The index of the column to test.
        * @return A constant indicating whether or not the column can contain NULL,
        *         which will be one of <code>columnNoNulls</code>,
        *         <code>columnNullable</code>, or <code>columnNullableUnknown</code>.
        * @exception SQLException If an error occurs.
        *
        */
      public int isNullable(int column) throws SQLException
      {
         try
         {
            if(_columns[column].allowNulls)
               return columnNullable;
            return columnNoNulls;
         }
         catch(NullPointerException npe)
         {
            loadColumns();
            if(_columns[column].allowNulls)
               return columnNullable;
            return columnNoNulls;
         }
      }

      /** This method tests whether or not the specified column is read only.
        *
        * @param index The index of the column to check.
        * @return <code>true</code> if the column is read only, <code>false</code>
        *         otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isReadOnly(int column) throws SQLException
      {
         return false;
      }

      /** This method tests whether not the specified column can be used in
        * a WHERE clause.
        *
        * @param index The index of the column to test.
        * @return <code>true</code> if the column may be used in a WHERE clause,
        *         <code>false</code> otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isSearchable(int column) throws SQLException
      {
         return true;
      }

      /** This method tests whether or not the value of the specified column
        * is signed or unsigned.
        *
        * @param index The index of the column to test.
        * @return <code>true</code> if the column value is signed, <code>false</code>
        *         otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isSigned(int column) throws SQLException
      {
         return true;
      }

      /** This method tests whether or not the column may be writable.  This
        * does not guarantee that a write will be successful.
        *
        * @param index The index of the column to check for writability.
        * @return <code>true</code> if the column may be writable,
        *         <code>false</code> otherwise.
        * @exception SQLException If an error occurs.
        *
        */
      public boolean isWritable(int column) throws SQLException
      {
         return true;
      }
   }
}

class Db2eBlob extends Blob
{
   public Db2eBoundBlob _bb;
   public Db2eBlob(int maxSize)
   {
      Db2eBoundBlob bb = _bb;
      bb.blob = new byte[maxSize];
      bb.length = 0;
   }

   /** This method returns a byte array that may be larger than the actual
     * data retrieved.  The caller of this method must be sure to limit their
     * access to only the portion of the byte array that was actually retrieved.
     * That is, the element indices of 0 to Blob.length() - 1.
     * This method is unique to SuperWaba and was added to allow for column
     * binding optimization (to avoid creating a new byte array as getBytes must
     * usually do).
     * There is no promise given that this will be more effecient.  It is
     * dependent on the driver implementation, but it should never be less effecient
     * than calling getBytes(0, blob.getLength()).
     *
     * @return The internal byte buffer used for the BLOB.
     * @exception SQLException if an error occurs.
     *
     */
   public byte[] getByteBuffer() throws SQLException
   {
      return _bb.blob;
   }

   /** This method returns up to the requested bytes of this BLOB as a
     * <code>byte</code> array.
     *
     * @param pos The index into the BLOB to start returning bytes from.
     * @param length The requested number of bytes to return.
     * @return The requested bytes from the BLOB.
     * @exception SQLException If an error occurs.
     *
     */
   public byte[] getBytes(long pos, int length) throws SQLException
   {
      Db2eBoundBlob bb = _bb;
      if(pos == 0 && length == bb.length)
         return bb.blob;
      byte[] ret = new byte[length];
      Vm.copyArray(bb.blob, (int)pos, ret, 0, length);
      return ret;
   }

   /** This method returns the number of bytes in the BLOB.
     *
     * @return The number of bytes in the BLOB.
     * @exception SQLException If an error occurs.
     *
     */
   public long length() throws SQLException
   {
      return (long)_bb.length;
   }

   /** @exception SQLException If an error occurs.
     * @since 1.4
     *
     */
   public int setBytes(long pos, byte[] bytes) throws SQLException
   {
      Vm.copyArray(bytes, 0, _bb.blob, (int)pos, bytes.length);
      return bytes.length;
   }

   /** @exception SQLException If an error occurs.
     * @since 1.4
     *
     */
   public int setBytes(long pos, byte[] bytes, int offset, int len) throws SQLException
   {
      Vm.copyArray(bytes, offset, _bb.blob, (int)pos, len);
      return len;
   }
}

class WdbcColumnInfo extends Db2eColumnInfo
{
   public int displaySize;
}
