
/*********************************************************************************
  *  SuperWaba Virtual Machine, version 4                                         *
  *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
  *                 and Andy DePue <sir_anodos@yahoo.com>                         *
  *  All Rights Reserved                                                          *
  *                                                                               *
  *  This library and virtual machine is free software; you can redistribute      *
  *  it and/or modify it under the terms of the Amended GNU Lesser General        *
  *  Public License distributed with this software.                               *
  *                                                                               *
  *  This library and virtual machine is distributed in the hope that it will     *
  *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
  *                                                                               *
  *  All software that links or interacts with this library and virtual machine,  *
  *  or any derivative works, require the software to prominently display the     *
  *  following notice:                                                            *
  *                                                                               *
  *                   Created with SuperWaba                                      *
  *                  http://www.superwaba.org                                     *
  *                                                                               *
  *  Please see the software license located at SuperWabaSDK/license.txt          *
  *  for more details.                                                            *
  *                                                                               *
  *  You should have received a copy of the License along with this software;     *
  *  if not, write to                                                             *
  *                                                                               *
  *     Guilherme Campos Hazan                                                    *
  *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
  *     Rio de Janeiro / RJ - Brazil                                              *
  *     Cep: 22050-000                                                            *
  *     E-mail: support@superwaba.com.br                                          *
  *                                                                               *
  *********************************************************************************/
package superwaba.ext.xplat.sql.db2e;
import waba.util.IntHashtable;
import superwaba.ext.xplat.sql.*;
import waba.sys.Vm;
import superwaba.ext.xplat.sql.db2e.db2ex.*;

/**
  * This implements Statement and PreparedStatement to save
  * on overall package size.
  */

public class Db2eStatement extends PreparedStatement
{
   static final IntHashtable CONV_FROM_DB2E = new IntHashtable(10, (float)1.0);

   {

      //    Vm.debug("Entering Db2eStatement static.");
      CONV_FROM_DB2E.put(Db2e.SQL_BLOB, Types.BLOB);
      CONV_FROM_DB2E.put(Db2e.SQL_CHAR, Types.CHAR);
      CONV_FROM_DB2E.put(Db2e.SQL_DECIMAL, Types.DECIMAL);
      CONV_FROM_DB2E.put(Db2e.SQL_INTEGER, Types.INTEGER);
      CONV_FROM_DB2E.put(Db2e.SQL_SMALLINT, Types.SMALLINT);
      CONV_FROM_DB2E.put(Db2e.SQL_TYPE_DATE, Types.DATE);
      CONV_FROM_DB2E.put(Db2e.SQL_TYPE_TIME, Types.TIME);
      CONV_FROM_DB2E.put(Db2e.SQL_TYPE_TIMESTAMP, Types.TIMESTAMP);
      CONV_FROM_DB2E.put(Db2e.SQL_VARCHAR, Types.VARCHAR);
   }
   static final IntHashtable CONV_FROM_WDBC = new IntHashtable(10, (float)1.0);

   {
      CONV_FROM_WDBC.put(Types.BLOB, Db2e.SQL_BLOB);
      CONV_FROM_WDBC.put(Types.CHAR, Db2e.SQL_CHAR);
      CONV_FROM_WDBC.put(Types.DECIMAL, Db2e.SQL_DECIMAL);
      CONV_FROM_WDBC.put(Types.INTEGER, Db2e.SQL_INTEGER);
      CONV_FROM_WDBC.put(Types.SMALLINT, Db2e.SQL_SMALLINT);
      CONV_FROM_WDBC.put(Types.DATE, Db2e.SQL_TYPE_DATE);
      CONV_FROM_WDBC.put(Types.TIME, Db2e.SQL_TYPE_TIME);
      CONV_FROM_WDBC.put(Types.TIMESTAMP, Db2e.SQL_TYPE_TIMESTAMP);
      CONV_FROM_WDBC.put(Types.VARCHAR, Db2e.SQL_VARCHAR);

   //    Vm.debug("Leaving Db2eStatement static.");
   }
   protected SQLWarning _warnings = null;
   protected Db2eConnection _connection;
   protected Db2e _db;
   protected int _conHandle;
   protected int _statement;
   protected int _maxFieldSize = 0;
   protected boolean _resultOpen = false;
   protected Db2eResultSet _results = null;
   protected Db2eResultSet.Db2eRsmd _meta = null;
   protected int _count = -1;
   protected int _resultSetType = ResultSet.TYPE_FORWARD_ONLY;
   private Object[] _params;
   private boolean[] _paramBound;
   private int[] _paramType;
   Db2eStatement(Db2eConnection connection, int conHandle, Db2e db) throws SQLException
   {
      _connection = connection;
      _conHandle = conHandle;
      _db = db;
      try
      {

         //      Vm.debug("Allocating new statement handle.");
         _statement = db.allocHandle(Db2e.SQL_HANDLE_STMT, conHandle);

      //      Vm.debug("Handle allocated.");
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }
   Db2eStatement(Db2eConnection connection, int conHandle, Db2e db, int resultSetType) throws SQLException
   {
      _connection = connection;
      _conHandle = conHandle;
      _db = db;
      _resultSetType = resultSetType;
      try
      {

         //      Vm.debug("Allocating new statement handle.");
         _statement = db.allocHandle(Db2e.SQL_HANDLE_STMT, conHandle);

         //      Vm.debug("Handle allocated.");
         if(_resultSetType != ResultSet.TYPE_FORWARD_ONLY)
            _db.setCursorType(_statement, Db2e.SQL_CURSOR_STATIC);

      //      Vm.debug("Cursor type set.");
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }
   Db2eStatement(Db2eConnection connection, int conHandle, Db2e db, String prepareSQL) throws SQLException
   {
      this(connection, conHandle, db);
      try
      {

         //      Vm.debug("Calling prepare: " + prepareSQL);
         _db.prepare(_statement, prepareSQL);

      //      Vm.debug("Prepare called.");
      }
      catch(DB2Exception error)
      {

         //      Vm.debug("Caught: " + error);
         handleError(error);
      }
   }
   Db2eStatement(Db2eConnection connection, int conHandle, Db2e db, int resultSetType, String prepareSQL) throws SQLException
   {
      this(connection, conHandle, db, resultSetType);
      try
      {

         //      Vm.debug("Calling prepare: " + prepareSQL);
         _db.prepare(_statement, prepareSQL);

      //      Vm.debug("Prepare called.");
      }
      catch(DB2Exception error)
      {

         //      Vm.debug("Caught: " + error);
         handleError(error);
      }
   }
   protected void notSupported() throws SQLException
   {
      throw new SQLException("Not Supported");
   }
   protected void handleError(DB2Exception error) throws SQLException
   {
      Driver.handleError(error);
   }
   protected boolean prepResultSet(boolean resetColumns) throws SQLException
   {
      if(_results == null)
      {
         _results = new Db2eResultSet(_db, this, _statement, _maxFieldSize);
         _meta = (Db2eResultSet.Db2eRsmd)_results.getMetaData();
      }
      else
         _results.reset(resetColumns);
      _count = -1;
      return _meta.getColumnCount() > 0;
   }

   /** This method adds a SQL statement to a SQL batch.  A driver is not
     * required to implement this method.
     *
     * @param sql The sql statement to add to the batch.
     * @exception SQLException If an error occurs.
     *
     */
   public void addBatch(String sql) throws SQLException
   {
      notSupported();
   }

   /** This method clears out any SQL statements that have been populated in
     * the current batch.  A driver is not required to implement this method.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void clearBatch() throws SQLException
   {
      notSupported();
   }

   /** This method clears any SQL warnings that have been attached to this
     * statement.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void clearWarnings() throws SQLException
   {
      _warnings = null;
   }

   /** This method closes the statement and frees any associated resources.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void close() throws SQLException
   {
      try
      {
         _db.freeHandle(Db2e.SQL_HANDLE_STMT, _statement);
         _statement = 0;
         _results = null;
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }
   protected boolean executeSet(String sql) throws DB2Exception
   {
      String upper = sql.toUpperCase();

      // Is this a 'SET attr=val' command?
      if(upper.charAt(0) == 'S' && upper.charAt(1) == 'E' && upper.charAt(3) == ' ')
      {

         // Get attr name.
         int length = upper.length();
         int i;

         // Skip whitespace.
         for(i = 4;i < length && upper.charAt(i) == ' ';i++)
            ;

         // Go until whitespace or '='
         int begin = i;
         char ch;
         for(i++;i < length && (ch = upper.charAt(i)) != ' ' && ch != '=';i++)
            ;

         // Grab attribute name.
         String name = upper.substring(begin, i);

         // Go until value begins
         for(i++;i < length && ((ch = upper.charAt(i)) == ' ' || ch == '=');i++)
            ;

         // Grab value.
         begin = i;
         for(i++;i < length && (ch = upper.charAt(i)) != ' ' && ch != '=';i++)
            ;
         String value = upper.substring(begin, i);
         if(name.equals("DIRTY_BIT_MODE"))
         {
            if(value.equals("SET_BY_SYSTEM"))

               //          Vm.debug("Setting DIRTY_BIT_MODE to SET_BY_SYSTEM.");
               _db.setDirtybitMode(_statement, Db2e.SQL_DIRTYBIT_SET_BY_SYSTEM);
            else
               if(value.equals("SET_BY_APPLICATION"))

                  //          Vm.debug("Setting DIRTY_BIT_MODE to SET_BY_APPLICATION.");
                  _db.setDirtybitMode(_statement, Db2e.SQL_DIRTYBIT_SET_BY_APPLICATION);
         }
         else
            if(name.equals("DELETE_MODE"))
            {
               if(value.equals("MARK_ONLY"))

                  //          Vm.debug("Setting DELETE_MODE to MARK_ONLY.");
                  _db.setDeleteMode(_statement, Db2e.SQL_DELETE_MARK_ONLY);
               else
                  if(value.equals("PHYSICAL_REMOVE"))

                     //          Vm.debug("Setting DELETE_MODE to PHYSICAL_REMOVE.");
                     _db.setDeleteMode(_statement, Db2e.SQL_DELETE_PHYSICAL_REMOVE);
            }
            else
               if(name.equals("READ_MODE"))
               {
                  if(value.equals("EXCLUDE_MARKED_DELETE"))

                     //          Vm.debug("Setting READ_MODE to EXCLUDE_MARKED_DELETE");
                     _db.setReadMode(_statement, Db2e.SQL_READ_EXCLUDE_MARKED_DELETE);
                  else
                     if(value.equals("INCLUDE_MARKED_DELETE"))

                        //          Vm.debug("Setting READ_MODE to INCLUDE_MARKED_DELETE");
                        _db.setReadMode(_statement, Db2e.SQL_READ_INCLUDE_MARKED_DELETE);
               }
               else
                  if(name.equals("REORG_MODE"))
                  {
                     if(value.equals("ENABLED"))

                        //          Vm.debug("Setting REORG_MODE to ENABLED.");
                        _db.setReorgMode(_statement, Db2e.SQL_REORG_ENABLED);
                     else
                        if(value.equals("DISABLED"))

                           //          Vm.debug("Setting REORG_MODE to DISABLED.");
                           _db.setReorgMode(_statement, Db2e.SQL_REORG_DISABLED);
                  }
         return true;
      }
      return false;
   }

   /** This method executes an arbitrary SQL statement of any time.  The
     * methods <code>getResultSet</code>, <code>getMoreResults</code> and
     * <code>getUpdateCount</code> retrieve the results.
     *
     * @return <code>true</code> if a result set was returned, <code>false</code>
     *         if an update count was returned.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean execute(String sql) throws SQLException
   {
      if(_resultOpen)
         throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {

         // Parse this manually if it is a 'SET attr=val' command.
         if(sql.length() > 4 && waba.sys.Convert.toUpperCase(sql.charAt(2)) == 'T' && executeSet(sql))
            return false;
         _db.execDirect(_statement, sql);
         return prepResultSet(true);
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning.
         return false;
      }
   }

   /** This method executes the SQL batch and returns an array of update
     * counts - one for each SQL statement in the batch - ordered in the same
     * order the statements were added to the batch.  A driver is not required
     * to implement this method.
     *
     * @return An array of update counts for this batch.
     * @exception SQLException If an error occurs.
     *
     */
   public int[] executeBatch() throws SQLException
   {
      notSupported();
      return null;
   }

   /** This method executes the specified SQL SELECT statement and returns a
     * (possibly empty) <code>ResultSet</code> with the results of the query.
     *
     * @param sql The SQL statement to execute.
     * @return The result set of the SQL statement.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet executeQuery(String sql) throws SQLException
   {
      if(_resultOpen)
         throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {
         _db.execDirect(_statement, sql);
         prepResultSet(true);
         return getResultSet();
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning.
         return getResultSet();
      }
   }

   /** This method executes the specified SQL INSERT, UPDATE, or DELETE statement
     * and returns the number of rows affected, which may be 0.
     *
     * @param sql The SQL statement to execute.
     * @return The number of rows affected by the SQL statement.
     * @exception SQLException If an error occurs.
     *
     */
   public int executeUpdate(String sql) throws SQLException
   {

      //    if(_resultOpen)

      //      throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {
         _db.execDirect(_statement, sql);
         return _db.getRowCount(_statement);
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning...
         return 0;
      }
   }

   /** This method returns the <code>Connection</code> instance that was
     * used to create this object.
     *
     * @return The connection used to create this object.
     * @exception SQLException If an error occurs.
     *
     */
   public Connection getConnection() throws SQLException
   {
      return _connection;
   }

   /** This method returns the current direction that the driver thinks the
     * result set will be accessed int.
     *
     * @return The direction the result set will be accessed in (????)
     * @exception SQLException If an error occurs.
     *
     */
   public int getFetchDirection() throws SQLException
   {
      return ResultSet.FETCH_FORWARD;
   }

   /** This method returns the number of rows the driver believes should be
     * fetched from the database at a time.
     *
     * @return The number of rows that will be fetched from the database at a time.
     * @exception SQLException If an error occurs.
     *
     */
   public int getFetchSize() throws SQLException
   {
      return 1;
   }

   /** This method returns the maximum length of any column value in bytes.
     *
     * @return The maximum length of any column value in bytes.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxFieldSize() throws SQLException
   {
      return _maxFieldSize;
   }

   /** This method returns the maximum possible number of rows in a result set.
     *
     * @return The maximum possible number of rows in a result set.
     * @exception SQLException If an error occurs.
     *
     */
   public int getMaxRows() throws SQLException
   {
      return 0;
   }

   /** This method advances the result set pointer to the next result set,
     * which can then be retrieved using <code>getResultSet</code>
     *
     * @return <code>true</code> if there is another result set,
     *         <code>false</code> otherwise (for example, the next result is an
     *         update count).
     * @exception SQLException If an error occurs.
     * @see #execute
     *
     */
   public boolean getMoreResults() throws SQLException
   {
      return false;
   }

   /** The method returns the number of seconds a statement may be in process
     * before timing out.  A value of 0 means there is no timeout.
     *
     * @return The SQL statement timeout in seconds.
     * @exception SQLException If an error occurs.
     *
     */
   public int getQueryTimeout() throws SQLException
   {
      return 0;
   }

   /** This method returns the result set of the SQL statement that was
     * executed.  This should be called only once per result set returned.
     *
     * @return The result set of the query, or <code>null</code> if there was
     *         no result set (for example, if the statement was an UPDATE).
     * @exception SQLException If an error occurs.
     * @see #execute
     *
     */
   public ResultSet getResultSet() throws SQLException
   {
      if(_results == null)
      {
         _results = new Db2eResultSet(_db, this, _statement, _maxFieldSize);
         _meta = (Db2eResultSet.Db2eRsmd)_results.getMetaData();
      }
      return _results;
   }

   /** This method returns the concurrency type of the result set for this
     * statement. This will be one of the concurrency types defined in
     * <code>ResultSet</code>.
     *
     * @return The concurrency type of the result set for this statement.
     * @exception SQLException If an error occurs.
     * @see ResultSet
     *
     */
   public int getResultSetConcurrency() throws SQLException
   {
      return ResultSet.CONCUR_READ_ONLY;
   }

   /** This method returns the result set type for this statement.  This will
     * be one of the result set types defined in <code>ResultSet</code>.
     *
     * @return The result set type for this statement.
     * @exception SQLException If an error occurs.
     * @see ResultSet
     *
     */
   public int getResultSetType() throws SQLException
   {
      return _resultSetType;
   }

   /** This method returns the update count of the SQL statement that was
     * executed.  This should be called only once per executed SQL statement.
     *
     * @return The update count of the query, or -1 if there was no update
     *         count (for example, if the statement was a SELECT).
     * @exception SQLException If an error occurs.
     * @see #execute
     *
     */
   public int getUpdateCount() throws SQLException
   {
      if(_count == -1)
      {
         try
         {
            _count = _db.getRowCount(_statement);
         }
         catch(DB2Exception error)
         {
            handleError(error);

            // If we get here, then it was a warning.
            return 0;
         }
      }
      return _count;
   }

   /** This method returns the first SQL warning attached to this statement.
     * Subsequent warnings will be chained to this one.
     *
     * @return The first SQL warning for this statement.
     * @exception SQLException If an error occurs.
     *
     */
   public SQLWarning getWarnings() throws SQLException
   {
      return _warnings;
   }

   /** This method sets the cursor name that will be used by the result set.
     *
     * @param name The cursor name to use for this statement.
     * @exception SQLException If an error occurs.
     *
     */
   public void setCursorName(String name) throws SQLException
   {
      notSupported();
   }

   /** This method sets the local escape processing mode on or off.  The
     * default value is on.
     *
     * @param escape <code>true</code> to enable local escape processing,
     *        <code>false</code> to disable it.
     * @exception SQLException If an error occurs.
     *
     */
   public void setEscapeProcessing(boolean enable) throws SQLException
   {
      notSupported();
   }

   /** This method informs the driver which direction the result set will
     * be accessed in.
     *
     * @param direction The direction the result set will be accessed in (?????)
     * @exception SQLException If an error occurs.
     *
     */
   public void setFetchDirection(int direction) throws SQLException
   {
      if(direction != ResultSet.FETCH_FORWARD)
         throw new SQLException("Unsupported fetch direction.");
   }

   /** This method informs the driver how many rows it should fetch from the
     * database at a time.
     *
     * @param numrows The number of rows the driver should fetch at a time
     *        to populate the result set.
     * @exception SQLException If an error occurs.
     *
     */
   public void setFetchSize(int rows) throws SQLException
   {
      if(rows != 1)
         throw new SQLException("Unsupported fetch size.");
   }

   /** This method sets the limit for the maximum length of any column in bytes.
     *
     * @param maxsize The new maximum length of any column in bytes.
     * @exception SQLException If an error occurs.
     *
     */
   public void setMaxFieldSize(int max) throws SQLException
   {
      _maxFieldSize = max;
      if(_results != null)
         _results.setMaxFieldSize(max);
   }

   /** This method sets the maximum number of rows that can be present in a
     * result set.
     *
     * @param maxrows The maximum possible number of rows in a result set.
     * @exception SQLException If an error occurs.
     *
     */
   public void setMaxRows(int max) throws SQLException
   {
      notSupported();
   }

   /** This method sets the number of seconds a statement may be in process
     * before timing out.  A value of 0 means there is no timeout.
     *
     * @param timeout The new SQL statement timeout value.
     * @exception SQLException If an error occurs.
     *
     */
   public void setQueryTimeout(int seconds) throws SQLException
   {

   }
   void closeResultSet(Db2eResultSet rs)
   {
      _resultOpen = false;
   }

   // Methods for PreparedStatement below.

   /** This method adds a set of parameters to the batch for JDBC 2.0.
     * @exception SQLException If an error occurs.
     *
     */
   public void addBatch() throws SQLException
   {
      notSupported();
   }

   /** This method clears all of the input parameter that have been
     * set on this statement.
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void clearParameters() throws SQLException
   {
      try
      {
         _db.resetStatementParams(_statement);
         _params = null;
         _paramBound = null;
         _paramType = null;
      }
      catch(DB2Exception error)
      {
         handleError(error);
      }
   }

   /** This method executes a prepared SQL query.
     * Some prepared statements return multiple results; the execute method
     * handles these complex statements as well as the simpler form of
     * statements handled by executeQuery and executeUpdate.
     *
     * @return The result of the SQL statement.
     * @exception SQLException If an error occurs.
     *
     */
   public boolean execute() throws SQLException
   {
      if(_resultOpen)
         throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {
         _db.execute(_statement);
         return prepResultSet(false);
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning.
         return false;
      }
   }

   /** This method executes a prepared SQL query and returns its ResultSet.
     *
     * @return The ResultSet of the SQL statement.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSet executeQuery() throws SQLException
   {
      if(_resultOpen)
         throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {
         _db.execute(_statement);
         prepResultSet(false);
         return getResultSet();
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning.
         return getResultSet();
      }
   }

   /** This method executes an SQL INSERT, UPDATE or DELETE statement.  SQL
     * statements that return nothing such as SQL DDL statements can be executed.
     *
     * @return The result is either the row count for INSERT, UPDATE or DELETE
     *         statements; or 0 for SQL statements that return nothing.
     * @exception SQLException If an error occurs.
     *
     */
   public int executeUpdate() throws SQLException
   {

      //    if(_resultOpen)

      //      throw new SQLException("There is a ResultSet currently open against this Statement.");
      try
      {
         _db.execute(_statement);
         return (_count = _db.getRowCount(_statement));
      }
      catch(DB2Exception error)
      {
         handleError(error);

         // If we get here, then it was a warning...
         return 0;
      }
   }

   /** This method returns meta data for the result set from this statement.
     *
     * @return Meta data for the result set from this statement.
     * @exception SQLException If an error occurs.
     *
     */
   public ResultSetMetaData getMetaData() throws SQLException
   {
      if(_meta == null)
         getResultSet();
      return _meta;
   }

   /** @since 1.4
     *
     */
   public ParameterMetaData getParameterMetaData() throws SQLException
   {
      notSupported();
      return null;
   }
   protected void ensureParamSize(int parameterIndex)
   {

      //    Vm.debug("ensureParamSize(" + parameterIndex + ")");

      // We grow by 10 at a time.
      if(_params == null)
      {

         //      Vm.debug("_params == null");

         //      Vm.debug("_params = new Object[parameterIndex + 10];");
         _params = new Object[parameterIndex + 10];

         //      Vm.debug("_paramBound = new boolean[parameterIndex + 10];");
         _paramBound = new boolean[parameterIndex + 10];

         //      Vm.debug("_paramType = new int[parameterIndex + 10];");
         _paramType = new int[parameterIndex + 10];
      }
      else
      {

         //      Vm.debug("_params != null");
         Object[] newParams = new Object[parameterIndex + 10];

         //      Vm.debug("Vm.copyArray 1");
         Vm.copyArray(_params, 0, newParams, 0, _params.length);
         _params = newParams;
         boolean[] newBound = new boolean[parameterIndex + 10];

         //      Vm.debug("Vm.copyArray 2");
         Vm.copyArray(_paramBound, 0, newBound, 0, _paramBound.length);
         _paramBound = newBound;
         int[] newType = new int[parameterIndex + 10];

         //      Vm.debug("Vm.copyArray 3");
         Vm.copyArray(_paramType, 0, newType, 0, _paramType.length);
         _paramType = newType;
      }
   }

   /**
     * In DB2e, parameters must be bound in order.  In other words, you cannot
     * bind parameter #3 until you have first bound parameter #1 and then
     * bound parameter #2.  WDBC, however, does not have this restriction, and
     * so we have to make sure that we bind parameters carefully.
     * bindParameters() works by ensuring parameters are bound "in order".  The
     * algorithm assumes that all parameters will be set before execute() is
     * called (which is a requirement anyway).  Whenever called, bindParameters()
     * will bind any parameters that have not yet been bound, in order, until an
     * empty parameter value is encountered, at which point bindParameters() will
     * return and leave any parameters that have been set after the empty
     * parameter unbound.  This works, because as long as every parameter is
     * eventually set before a call to execute(), the algorithm guarantees that
     * all parameters will be bound, and in order.
     * This ends up being effecient for a prepared statement that is executed
     * many times, because once the parameter has been set, this method will
     * no longer be called when the parameter is set again.
     */

   /*  private void bindParameters() throws SQLException
   {
   //    Vm.debug("bindParameters()");
   Object paramObj;
   Object[] params = _params;
   boolean[] paramBound = _paramBound;
   int[] paramType = _paramType;
   int i, length;

   try
   {
   // Loop until an empty parameter is encounted.
   for(i = 1, length = params.length;i < length && (paramObj = params[i]) != null;i++)
   {
   //        Vm.debug("i = " + i + ", paramObj = " + paramObj);
   // Bind this parameter if it has not yet been bound.
   if(!paramBound[i])
   {
   //          Vm.debug("param has not been bound.");
   // Bind according to parameter type.
   switch(paramType[i])
   {
   case Db2e.SQL_INTEGER:
   //              Vm.debug("Binding as SQL_INTEGER.");
   _db.bindIntegerParam(_statement, i, (BoundInt)paramObj);
   break;

   case Db2e.SQL_VARCHAR:
   //              Vm.debug("Binding as SQL_VARCHAR.");
   _db.bindVarcharParam(_statement, i, (BoundString)paramObj);
   break;

   case Db2e.SQL_CHAR:
   //              Vm.debug("Binding as SQL_CHAR.");
   _db.bindCharParam(_statement, i, (BoundString)paramObj);
   break;

   case Db2e.SQL_SMALLINT:
   //              Vm.debug("Binding as SQL_SMALLINT.");
   _db.bindSmallIntParam(_statement, i, (BoundInt)paramObj);
   break;

   case Db2e.SQL_TYPE_DATE:
   //              Vm.debug("Binding as SQL_TYPE_DATE.");
   _db.bindDateParam(_statement, i, (BoundDate)paramObj);
   break;

   case Db2e.SQL_TYPE_TIME:
   //              Vm.debug("Binding as SQL_TYPE_TIME.");
   _db.bindTimeParam(_statement, i, (BoundTime)paramObj);
   break;

   case Db2e.SQL_TYPE_TIMESTAMP:
   //              Vm.debug("Binding as SQL_TYPE_TIMESTAMP.");
   _db.bindTimestampParam(_statement, i, (BoundTimestamp)paramObj);
   break;

   case Db2e.SQL_BLOB:
   //              Vm.debug("Binding as SQL_BLOB.");
   _db.bindBlobParam(_statement, i, (BoundBlob)paramObj);
   break;

   default:
   throw new SQLException("Unsupported type in bindParameters().");
   }

   //          Vm.debug("paramBound[" + i + "] = true");
   paramBound[i] = true;
   }
   }
   }
   catch(DB2Exception db2e)
   {
   handleError(db2e);
   }

   //    Vm.debug("Leaving bindParameters()");
   } */

   /*  protected void ensureIntParamSize(int parameterIndex)
   {
   Vm.debug("ensureIntParamSize(" + parameterIndex + ")");
   if(_intParams == null)
   {
   _intParams = new BoundInt[parameterIndex + 10];
   }
   else
   {
   BoundInt[] newParams = new BoundInt[parameterIndex + 10];
   Vm.copyArray(_intParams, 0, newParams, 0, _intParams.length);
   _intParams = newParams;
   }
   }

   protected void ensureStringParamSize(int parameterIndex)
   {
   Vm.debug("ensureStringParamSize(" + parameterIndex + ")");
   if(_strParams == null)
   {
   _strParams = new BoundString[parameterIndex + 10];
   }
   else
   {
   BoundString[] newParams = new BoundString[parameterIndex + 10];
   Vm.copyArray(_strParams, 0, newParams, 0, _strParams.length);
   _strParams = newParams;
   }
   } */

   /** This method sets the specified parameter from the given Java
     * <code>Blob</code> value.  The default object type to SQL type mapping
     * will be used.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setBlob(int parameterIndex, Blob x) throws SQLException
   {
      Db2eBoundBlob bb;
      int ilen = (int)x.length(); // Do this here so it throws a real NullPointerException.
      try
      {
         bb = (Db2eBoundBlob)_params[parameterIndex];
         try
         {

            // Is the bound buffer already big enough?
            if(bb.blob.length >= ilen)
            {

               // This is the absolute most effecient code path.
               Vm.copyArray(x.getByteBuffer(), 0, bb.blob, 0, ilen);
               bb.length = ilen;
               return ;
            }
            else
            {

               // Make a bigger buffer.
               bb.blob = new byte[ilen];

               // Rebind.
               try
               {
                  _db.bindBlobParam(_statement, parameterIndex, bb);
               }
               catch(DB2Exception error)
               {
                  handleError(error);
               }

               // Copy data.
               Vm.copyArray(x.getByteBuffer(), 0, bb.blob, 0, ilen);
               bb.length = ilen;
               return ;
            }
         }
         catch(NullPointerException npe)
         {


         // If bb (_params[parameterIndex]) is null.

         // Fall through to shared code.
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }

      // Create a new BoundBlob.
      bb = new Db2eBoundBlob();

      // Make it big enough to hold the Blob.
      bb.blob = new byte[ilen];

      // Cache it in _params.
      _params[parameterIndex] = bb;
      _paramType[parameterIndex] = Db2e.SQL_BLOB;

      // Bind
      try
      {
         _db.bindBlobParam(_statement, parameterIndex, bb);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

      //    bindParameters();

      // Copy data.
      Vm.copyArray(x.getByteBuffer(), 0, bb.blob, 0, ilen);
      bb.length = ilen;
   }

   /** This method sets the specified parameter from the given Java
     * <code>boolean</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setBoolean(int parameterIndex, boolean x) throws SQLException
   {
      setShort(parameterIndex, x ? (short)1 : (short)0);
   }

   /** This method sets the specified parameter from the given Java
     * <code>byte</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setByte(int parameterIndex, byte x) throws SQLException
   {
      setShort(parameterIndex, (short)x);
   }

   /** This method sets the specified parameter from the given Java
     * <code>byte</code> array value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setBytes(int parameterIndex, byte[] x) throws SQLException
   {
      Db2eBoundBlob bb;
      Db2eBoundString bs;
      int ilen = x.length; // Do this here so it throws a real NullPointerException.
      try
      {
         switch(_paramType[parameterIndex])
         {
            case Db2e.SQL_BLOB:

               {
                  bb = (Db2eBoundBlob)_params[parameterIndex];
                  try
                  {

                     // If the bound buffer already big enough?
                     if(bb.blob.length >= ilen)
                     {

                        // This is the absolute most effecient code path.
                        Vm.copyArray(x, 0, bb.blob, 0, ilen);
                        bb.length = ilen;
                        return ;
                     }
                     else
                     {

                        // Make a bigger buffer.
                        bb.blob = new byte[ilen];

                        // Rebind.
                        try
                        {
                           _db.bindBlobParam(_statement, parameterIndex, bb);
                        }
                        catch(DB2Exception error)
                        {
                           handleError(error);
                        }

                        // Copy data.
                        Vm.copyArray(x, 0, bb.blob, 0, ilen);
                        bb.length = ilen;
                        return ;
                     }
                  }
                  catch(NullPointerException npe)
                  {


                  // If bb (_params[parameterIndex]) is null.

                  // Fall through to shared code.
                  }
                  break;
               }

            case Db2e.SQL_VARCHAR:
            case Db2e.SQL_CHAR:

               {
                  bs = (Db2eBoundString)_params[parameterIndex];
                  try
                  {

                     // Is the bound buffer big enough?
                     if(!bs.setValueCheck(x))
                        return ;

                     // Rebind.
                     try
                     {
                        _db.bindVarcharParam(_statement, parameterIndex, bs);
                     }
                     catch(DB2Exception error)
                     {
                        handleError(error);
                     }
                     return ;
                  }
                  catch(NullPointerException npe)
                  {


                  // if bs (_params[parameterIndex]) is null.
                  }
                  break;
               }

            case 0:
               break;

            default:
               throw new SQLException("Unsupported type for setBytes()");
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }

      // Well, now, things get tricky here, don't they?  We really don't know

      // what type the param is at this point, so I'm going to just make something

      // up.  We will assume that "strings" are more common than blobs, so we

      // will make the parameter a varchar type.
      bs = new Db2eBoundString(x.length);

      // Cache it in _params.
      _params[parameterIndex] = bs;
      _paramType[parameterIndex] = Db2e.SQL_VARCHAR;

      // Bind
      try
      {
         _db.bindVarcharParam(_statement, parameterIndex, bs);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

      //    bindParameters();

      // Copy data.
      bs.setValue(x);
   }

   /** This method sets the specified parameter from the given Java
     * <code>byte</code> array value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setBytes(int parameterIndex, byte[] x, int start, int length) throws SQLException
   {
      Db2eBoundBlob bb;
      Db2eBoundString bs;
      try
      {
         switch(_paramType[parameterIndex])
         {
            case Db2e.SQL_BLOB:

               {
                  bb = (Db2eBoundBlob)_params[parameterIndex];
                  try
                  {

                     // If the bound buffer already big enough?
                     if(bb.blob.length >= length)
                     {

                        // This is the absolute most effecient code path.
                        Vm.copyArray(x, start, bb.blob, 0, length);
                        bb.length = length;
                        return ;
                     }
                     else
                     {

                        // Make a bigger buffer.
                        bb.blob = new byte[length];

                        // Rebind.
                        try
                        {
                           _db.bindBlobParam(_statement, parameterIndex, bb);
                        }
                        catch(DB2Exception error)
                        {
                           handleError(error);
                        }

                        // Copy data.
                        Vm.copyArray(x, start, bb.blob, 0, length);
                        bb.length = length;
                        return ;
                     }
                  }
                  catch(NullPointerException npe)
                  {


                  // If bb (_params[parameterIndex]) is null.

                  // Fall through to shared code.
                  }
                  break;
               }

            case Db2e.SQL_VARCHAR:
            case Db2e.SQL_CHAR:

               {
                  bs = (Db2eBoundString)_params[parameterIndex];
                  try
                  {

                     // Is the bound buffer big enough?
                     if(!bs.setValueCheck(x, start, length))
                        return ;

                     // Rebind.
                     try
                     {
                        _db.bindVarcharParam(_statement, parameterIndex, bs);
                     }
                     catch(DB2Exception error)
                     {
                        handleError(error);
                     }
                     return ;
                  }
                  catch(NullPointerException npe)
                  {


                  // if bs (_params[parameterIndex]) is null.
                  }
                  break;
               }

            case 0:
               break;

            default:
               throw new SQLException("Unsupported type for setBytes()");
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }

      // Well, now, things get tricky here, don't they?  We really don't know

      // what type the param is at this point, so I'm going to just make something

      // up.  We will assume that "strings" are more common than blobs, so we

      // will make the parameter a varchar type.
      bs = new Db2eBoundString(length);

      // Cache it in _params.
      _params[parameterIndex] = bs;
      _paramType[parameterIndex] = Db2e.SQL_VARCHAR;

      // Bind
      try
      {
         _db.bindVarcharParam(_statement, parameterIndex, bs);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

      //    bindParameters();

      // Copy data.
      bs.setValue(x, start, length);
   }

   /** This method sets the specified parameter from the given Java
     * <code>java.sql.Date</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setDate(int parameterIndex, Date x) throws SQLException
   {
      Db2eBoundDate bd;
      try
      {
         bd = (Db2eBoundDate)_params[parameterIndex];
         try
         {
            bd.year = x.getYear();
            bd.month = x.getMonth();
            bd.day = x.getDate();
            bd.nullIndicator = 0;
            bd.translateToDb2e();
            return ;
         }
         catch(NullPointerException npe)
         {


         // If bts is null.

         // Drop through to shared code.
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params isn't big enough.
         ensureParamSize(parameterIndex);
      }
      bd = new Db2eBoundDate(x.getYear(), x.getMonth(), x.getDate());
      _params[parameterIndex] = bd;
      _paramType[parameterIndex] = Db2e.SQL_TYPE_DATE;
      try
      {
         _db.bindDateParam(_statement, parameterIndex, bd);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    bindParameters();
   }

   /** This method sets the specified parameter from the given Java
     * <code>double</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setDouble(int parameterIndex, double x) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>float</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setFloat(int parameterIndex, float x) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>int</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setInt(int parameterIndex, int x) throws SQLException
   {
      Db2eBoundInt bi;
      try
      {
         bi = (Db2eBoundInt)_params[parameterIndex];
         try
         {
            bi.value = x;
            bi.nullIndicator = 0;
            return ;
         }
         catch(NullPointerException npe)
         {


         //        Vm.debug("setInt - NPE2.");

         // If _params[parameterIndex] == null.

         // Fall through to common code.
         }
      }
      catch(NullPointerException npe)
      {

         //        Vm.debug("setInt - NPE1.");

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         //      Vm.debug("setInt - AIOOBE.");

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }

      //    Vm.debug("setInt - entering common.");
      bi = new Db2eBoundInt();
      bi.value = x;
      _params[parameterIndex] = bi;
      _paramType[parameterIndex] = Db2e.SQL_INTEGER;
      try
      {
         _db.bindIntegerParam(_statement, parameterIndex, bi);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    bindParameters();
   }

   /** This method sets the specified parameter from the given Java
     * <code>long</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setLong(int parameterIndex, long x) throws SQLException
   {
      setInt(parameterIndex, (int)(x & 0xFFFFFFFF));
   }

   /** This method populates the specified parameter with a SQL NULL value
     * for the specified type.
     *
     * @param index The index of the parameter to set.
     * @param type The SQL type identifier of the parameter from <code>Types</code>
     *
     * @exception SQLException If an error occurs.
     *
     */
   public void setNull(int parameterIndex, int sqlType) throws SQLException
   {
      try
      {

         // This will throw NullPointerException if _params == null and

         // will throw ArrayIndexOutOfBoundsException if _params is not big enough.
         Object o = _params[parameterIndex];
         switch(sqlType)
         {
            case Types.BLOB:

               {
                  try
                  {
                     Db2eBoundBlob bb = (Db2eBoundBlob)o;
                     bb.length = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundBlob bb = new Db2eBoundBlob();

                     // Just some arbitrary size to give it something to bind.
                     bb.blob = new byte[16];
                     bb.length = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bb;
                     _paramType[parameterIndex] = Db2e.SQL_BLOB;
                     _db.bindBlobParam(_statement, parameterIndex, bb);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.INTEGER:

               {
                  try
                  {
                     Db2eBoundInt bi = (Db2eBoundInt)o;
                     bi.nullIndicator = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundInt bi = new Db2eBoundInt();
                     bi.nullIndicator = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bi;
                     _paramType[parameterIndex] = Db2e.SQL_INTEGER;
                     _db.bindIntegerParam(_statement, parameterIndex, bi);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.BOOLEAN:
            case Types.SMALLINT:
            case Types.TINYINT:

               {
                  try
                  {
                     Db2eBoundInt bi = (Db2eBoundInt)o;
                     bi.nullIndicator = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundInt bi = new Db2eBoundInt();
                     bi.nullIndicator = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bi;
                     _paramType[parameterIndex] = Db2e.SQL_SMALLINT;
                     _db.bindSmallIntParam(_statement, parameterIndex, bi);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.DATE:

               {
                  try
                  {
                     Db2eBoundDate bd = (Db2eBoundDate)o;
                     bd.nullIndicator = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundDate bd = new Db2eBoundDate();
                     bd.nullIndicator = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bd;
                     _paramType[parameterIndex] = Db2e.SQL_TYPE_DATE;
                     _db.bindDateParam(_statement, parameterIndex, bd);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.TIME:

               {
                  try
                  {
                     Db2eBoundTime bt = (Db2eBoundTime)o;
                     bt.nullIndicator = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundTime bt = new Db2eBoundTime();
                     bt.nullIndicator = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bt;
                     _paramType[parameterIndex] = Db2e.SQL_TYPE_TIME;
                     _db.bindTimeParam(_statement, parameterIndex, bt);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.TIMESTAMP:

               {
                  try
                  {
                     Db2eBoundTimestamp bts = (Db2eBoundTimestamp)o;
                     bts.nullIndicator = Db2e.SQL_NULL_DATA;
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundTimestamp bts = new Db2eBoundTimestamp();
                     bts.nullIndicator = Db2e.SQL_NULL_DATA;
                     _params[parameterIndex] = bts;
                     _paramType[parameterIndex] = Db2e.SQL_TYPE_TIMESTAMP;
                     _db.bindTimestampParam(_statement, parameterIndex, bts);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.CHAR:

               {
                  try
                  {
                     Db2eBoundString bs = (Db2eBoundString)o;
                     bs.setLength(Db2e.SQL_NULL_DATA);
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundString bs = new Db2eBoundString(16);
                     bs.setLength(Db2e.SQL_NULL_DATA);
                     _params[parameterIndex] = bs;
                     _paramType[parameterIndex] = Db2e.SQL_CHAR;
                     _db.bindCharParam(_statement, parameterIndex, bs);

                  //            bindParameters();
                  }
                  return ;
               }

            case Types.VARCHAR:

               {
                  try
                  {
                     Db2eBoundString bs = (Db2eBoundString)o;
                     bs.setLength(Db2e.SQL_NULL_DATA);
                  }
                  catch(NullPointerException npe)
                  {
                     Db2eBoundString bs = new Db2eBoundString(16);
                     bs.setLength(Db2e.SQL_NULL_DATA);
                     _params[parameterIndex] = bs;
                     _paramType[parameterIndex] = Db2e.SQL_VARCHAR;
                     _db.bindVarcharParam(_statement, parameterIndex, bs);

                  //            bindParameters();
                  }
                  return ;
               }

            default:
               throw new SQLException("Unsupported type.");
         }
      }
      catch(NullPointerException npe)
      {
         ensureParamSize(parameterIndex);
         setNull(parameterIndex, sqlType);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {
         ensureParamSize(parameterIndex);
         setNull(parameterIndex, sqlType);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }
   }

   /** This method populates the specified parameter with a SQL NULL value
     * for the specified type.
     *
     * @param index The index of the parameter to set.
     * @param type The SQL type identifier of the parameter from <code>Types</code>
     * @param name The name of the data type, for user defined types.
     * @exception SQLException If an error occurs.
     *
     */
   public void setNull(int paramIndex, int sqlType, String typeName) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>Object</code> value.  The default object type to SQL type mapping
     * will be used.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setObject(int parameterIndex, Object x) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>Object</code> value.  The specified SQL object type will be used.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @param type The SQL type to use for the parameter, from <code>Types</code>
     * @exception SQLException If an error occurs.
     * @see Types
     *
     */
   public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>Object</code> value.  The specified SQL object type will be used.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @param type The SQL type to use for the parameter, from <code>Types</code>
     * @param scale The scale of the value, for numeric values only.
     * @exception SQLException If an error occurs.
     * @see Types
     *
     */
   public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>Ref</code> value.  The default object type to SQL type mapping
     * will be used.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setRef(int i, Ref x) throws SQLException
   {
      notSupported();
   }

   /** This method sets the specified parameter from the given Java
     * <code>short</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setShort(int parameterIndex, short x) throws SQLException
   {
      Db2eBoundInt bi;
      try
      {
         bi = (Db2eBoundInt)_params[parameterIndex];
         try
         {
            bi.value = x;
            bi.nullIndicator = 0;
            return ;
         }
         catch(NullPointerException npe)
         {


         // If _params[parameterIndex] == null.

         // Fall through to common code.
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }
      bi = new Db2eBoundInt();
      bi.value = x;
      _params[parameterIndex] = bi;
      _paramType[parameterIndex] = Db2e.SQL_SMALLINT;
      try
      {
         _db.bindSmallIntParam(_statement, parameterIndex, bi);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    bindParameters();
   }

   /** This method sets the specified parameter from the given Java
     * <code>String</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setString(int parameterIndex, String x) throws SQLException
   {
      Db2eBoundString bs;
      try
      {

         //      Vm.debug("bs = (Db2eBoundString)_params[parameterIndex];");
         bs = (Db2eBoundString)_params[parameterIndex];
         try
         {

            //        Vm.debug("bs.setNewBuf(x)");
            bs.setNewBuf(x);
            try
            {

               //          Vm.debug("_db.bindVarcharParam(_statement, parameterIndex, bs)");
               _db.bindVarcharParam(_statement, parameterIndex, bs);
            }
            catch(DB2Exception error)
            {
               handleError(error);
            }

            //        Vm.debug("Returning from main path.");
            return ;
         }
         catch(NullPointerException npe)
         {


         //        Vm.debug("setString - NPE 2");

         // If _params[parameterIndex] == null.

         // Fall through to common code.
         }
      }
      catch(NullPointerException npe)
      {

         //      Vm.debug("setString - NPE 1");

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         //      Vm.debug("setString - AIOOBE");

         // If _params is not big enough.
         ensureParamSize(parameterIndex);
      }

      //    Vm.debug("setString - entering common.");
      bs = new Db2eBoundString(x);
      _params[parameterIndex] = bs;
      _paramType[parameterIndex] = Db2e.SQL_VARCHAR;
      try
      {
         _db.bindVarcharParam(_statement, parameterIndex, bs);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    Vm.debug("Calling bindParameters()");

   //    bindParameters();
   }

   /** This method sets the specified parameter from the given Java
     * <code>java.sql.Time</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setTime(int parameterIndex, Time x) throws SQLException
   {
      Db2eBoundTime bt;
      try
      {
         bt = (Db2eBoundTime)_params[parameterIndex];
         try
         {
            bt.hour = x.getHour();
            bt.minute = x.getMinute();
            bt.second = x.getSecond();
            bt.translateToDb2e();
            return ;
         }
         catch(NullPointerException npe)
         {


         // If bts is null.

         // Drop through to shared code.
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params isn't big enough.
         ensureParamSize(parameterIndex);
      }
      bt = new Db2eBoundTime(x.getHour(), x.getMinute(), x.getSecond());
      _params[parameterIndex] = bt;
      _paramType[parameterIndex] = Db2e.SQL_TYPE_TIME;
      try
      {
         _db.bindTimeParam(_statement, parameterIndex, bt);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    bindParameters();
   }

   /** This method sets the specified parameter from the given Java
     * <code>java.sql.Timestamp</code> value.
     *
     * @param index The index of the parameter value to set.
     * @param value The value of the parameter.
     * @exception SQLException If an error occurs.
     *
     */
   public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException
   {
      Db2eBoundTimestamp bts;
      try
      {
         bts = (Db2eBoundTimestamp)_params[parameterIndex];
         try
         {
            bts.year = x.getYear();
            bts.month = x.getMonth();
            bts.day = x.getDate();
            bts.hour = x.getHour();
            bts.minute = x.getMinute();
            bts.second = x.getSecond();
            bts.translateToDb2e();
            return ;
         }
         catch(NullPointerException npe)
         {


         // If bts is null.

         // Drop through to shared code.
         }
      }
      catch(NullPointerException npe)
      {

         // If _params is null.
         ensureParamSize(parameterIndex);
      }
      catch(ArrayIndexOutOfBoundsException ai)
      {

         // If _params isn't big enough.
         ensureParamSize(parameterIndex);
      }
      bts = new Db2eBoundTimestamp(x.getYear(), x.getMonth(), x.getDate(), x.getHour(), x.getMinute(), x.getSecond(), 0);
      _params[parameterIndex] = bts;
      _paramType[parameterIndex] = Db2e.SQL_TYPE_TIMESTAMP;
      try
      {
         _db.bindTimestampParam(_statement, parameterIndex, bts);
      }
      catch(DB2Exception db2e)
      {
         handleError(db2e);
      }

   //    bindParameters();
   }
}
