
/*********************************************************************************
  *  SuperWaba Virtual Machine, version 4                                         *
  *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
  *                 and Andy DePue <sir_anodos@yahoo.com>                         *
  *  All Rights Reserved                                                          *
  *                                                                               *
  *  This library and virtual machine is free software; you can redistribute      *
  *  it and/or modify it under the terms of the Amended GNU Lesser General        *
  *  Public License distributed with this software.                               *
  *                                                                               *
  *  This library and virtual machine is distributed in the hope that it will     *
  *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
  *                                                                               *
  *  All software that links or interacts with this library and virtual machine,  *
  *  or any derivative works, require the software to prominently display the     *
  *  following notice:                                                            *
  *                                                                               *
  *                   Created with SuperWaba                                      *
  *                  http://www.superwaba.org                                     *
  *                                                                               *
  *  Please see the software license located at SuperWabaSDK/license.txt          *
  *  for more details.                                                            *
  *                                                                               *
  *  You should have received a copy of the License along with this software;     *
  *  if not, write to                                                             *
  *                                                                               *
  *     Guilherme Campos Hazan                                                    *
  *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
  *     Rio de Janeiro / RJ - Brazil                                              *
  *     Cep: 22050-000                                                            *
  *     E-mail: support@superwaba.com.br                                          *
  *                                                                               *
  *********************************************************************************/
package superwaba.ext.xplat.sql.db2e.db2ex;
import waba.sys.*;

public class Db2eBoundString
{

   // WARNING!  DO NOT CHANGE THE ORDER THAT THESE FIELDS APPEAR IN THIS

   // CLASS, AS THE NATIVE CODE REFERENCES THE FIELDS BY #.

   // IF YOU NEED TO ADD FIELDS, ADD THEM AFTER THE LAST FIELD DEFINED HERE.

   // IF YOU NEED TO REMOVE FIELDS, BE SURE TO MODIFY THE NATIVE CODE.
   private byte[] _buf;
   private int _curLength;

   // BoundString doesn't actually use the field below, but it is there to give

   // people a convenient place to store a string related to this.
   public String str;
   public Db2eBoundString(int maxStringSize)
   {
      _buf = new byte[maxStringSize];
      _curLength = 0;
   }
   public Db2eBoundString(String str)
   {
      _buf = str.getBytes();
      _curLength = _buf.length;
   }

   // The DB2e CLI (ODBC) interface keeps a pointer to the buffer once the

   // the buffer has been bound.  Because of this, we cannot allocate a new

   // buffer and must try to reuse the one that is already bound to the

   // parameter.  If the buffer isn't big enough, the user of this code must

   // allocate a new buffer and rebind the parameter.
   public void setValueTruncate(String str)
   {
      byte[] converted = str.getBytes();
      int count = (converted.length >= _buf.length) ? _buf.length : converted.length;
      Vm.copyArray(converted, 0, _buf, 0, count);
      _curLength = count;
   }

   /**
     * Returns false if the string is too large to fit in current buffer.
     */
   public boolean setValue(String str)
   {
      if(str.length() > _buf.length)
         return false;
      byte[] converted = str.getBytes();
      if(converted.length > _buf.length)
         return false;
      Vm.copyArray(converted, 0, _buf, 0, converted.length);
      _curLength = converted.length;
      return true;
   }
   public boolean setValueCheck(String str)
   {
      byte[] converted = str.getBytes();
      int len;
      if((len = converted.length) > _buf.length)
      {
         _buf = converted;
         _curLength = converted.length;
         return true;
      }
      Vm.copyArray(converted, 0, _buf, 0, len);
      _curLength = len;
      return false;
   }
   public boolean setValue(byte[] bstr)
   {
      if(bstr.length > _buf.length)
         return false;
      Vm.copyArray(bstr, 0, _buf, 0, bstr.length);
      _curLength = bstr.length;
      return true;
   }
   public boolean setValueCheck(byte[] bstr)
   {
      int len = bstr.length;
      if(len > _buf.length)
      {
         byte[] newBuf = new byte[len];
         Vm.copyArray(bstr, 0, newBuf, 0, len);
         _buf = newBuf;
         _curLength = len;
         return true;
      }
      Vm.copyArray(bstr, 0, _buf, 0, len);
      _curLength = len;
      return false;
   }
   public boolean setValue(byte[] bstr, int begin, int length)
   {
      if(length > _buf.length)
         return false;
      Vm.copyArray(bstr, begin, _buf, 0, length);
      _curLength = length;
      return true;
   }
   public boolean setValueCheck(byte[] bstr, int begin, int length)
   {
      if(length > _buf.length)
      {
         byte[] newBuf = new byte[length];
         Vm.copyArray(bstr, begin, newBuf, 0, length);
         _buf = newBuf;
         _curLength = length;
         return true;
      }
      Vm.copyArray(bstr, begin, _buf, 0, length);
      _curLength = length;
      return false;
   }

   /**
     * Return true if the buffer should be rebound.
     */
   public void setNewBuf(String str)
   {
      _buf = str.getBytes();
      _curLength = _buf.length;
   }
   public byte[] getBuf()
   {
      return _buf;
   }
   public int getMax()
   {
      return _buf.length;
   }
   public int getLength()
   {
      return _curLength;
   }
   public void setLength(int newLength)
   {
      _curLength = newLength;
   }
   public String toString()
   {

      //    if(_curLength == Db2e.SQL_NULL_DATA)

      //      return null;

      //    else if(_curLength < 1)

      //      return "";
      return new String(_buf, 0, _curLength);
   }
}
