package superwaba.ext.xplat.ui;

import waba.fx.*;
import waba.sys.*;
import waba.ui.*;

/** A popup window that displays a ComboBox with choices, to let the
  * user choose a value. The ComboBox is given as parameter to the constructor.
  */

public class ChoicesDialog extends Window
{
   private Label msg;
   private PushButtonGroup btns;
   private ComboBox cb;
   private int selected = -1;

   /** The text will be used in a Labe, must be one line only! */
   public ChoicesDialog(String title, String text, ComboBox cb)
   {
      this(title, text, cb, new String[]{"Ok","Cancel"});
   }

   /** Tip: the combobox must have at least one item, with spaces that will
     * determine the preferred size */
   public ChoicesDialog(String title, String text, ComboBox cb, String[] buttonCaptions)
   {
      super(title,ROUND_BORDER);
      highResPrepared = true;
      setDoubleBuffer(true);
      btns = new PushButtonGroup(buttonCaptions,false,-1,4,6,1,false,PushButtonGroup.BUTTON);
      msg = new Label(text,Label.CENTER);
      this.cb = cb;

      int wb = btns.getPreferredWidth();
      int hb = btns.getPreferredHeight();
      int wm = Math.min(msg.getPreferredWidth()+1,Settings.screenWidth-6);
      int hm = msg.getPreferredHeight();
      int we = cb.getPreferredWidth();
      int he = cb.getPreferredHeight();
      FontMetrics fm2 = getFontMetrics(titleFont); // guich@220_28
      int captionH = fm2.getHeight()+10;

      int h = captionH + hb + hm + he;
      int w = Math.max(Math.max(Math.max(wb,wm),we),fm2.getTextWidth(title))+6; // guich@200b4_29
      w = Math.min(w,Settings.screenWidth); // guich@200b4_28: dont let the window be greater than the screen size
      setRect(CENTER,CENTER,w,h);
      add(msg);
      add(btns);
      add(cb);
      msg.setRect(4,TOP,wm,hm);
      cb.setRect(CENTER,AFTER+2,we,he);
      btns.setRect(CENTER,AFTER+2,wb,hb);
   }

   protected void onColorsChanged(boolean colorsChanged) // guich@220_47
   {
      if (colorsChanged)
      {
         msg.setForeColor(foreColor);
         msg.setBackColor(backColor);
      }
   }

   /** handle scroll buttons and normal buttons */
   public void onEvent(Event e)
   {
      switch (e.type)
      {
         case ControlEvent.PRESSED:
            if (e.target == btns && (selected=btns.getSelected()) != -1)
            {
               btns.setSelected(-1);
               unpop();
            }
            break;
      }
   }

   /** returns the pressed button index, starting from 0 */
   public int getPressedButtonIndex()
   {
      return selected;
   }

   /** returns the ComboBox associated */
   public ComboBox getComboBox()
   {
      return cb;
   }
}