package superwaba.ext.xplat.ui;

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;

/** Implements a ListBox where colors can be choosed from.
  * The only functional methods are getSelectedItem and getSelectedIndex.
  * This class works well in 2 and 4 bpp (gray scale).
  * Next an example of how to use this class as a combobox color chooser:
  * <pre>
  * add(foreCombo = new ComboBox(new ColorList()), CENTER, BOTTOM);
  * </pre>
  */

public class ColorList extends ListBox
{
   int w;
   private static Color []colors = Palette.SystemPalette.getColors();

   public ColorList()
   {
      super(colors);
      w = fm.getTextWidth("CCCCCC")+8;
   }

   protected void drawItem(Graphics g, int index, int dx, int dy)
   {
      if (0 <= index && index <= colors.length)
      {
         g.setBackColor(colors[index]);
         g.fillRect(dx-1,dy,btnX,fmH);
         g.setForeColor(colors[index].getAlpha() > 128?Color.BLACK:Color.WHITE);
         g.drawText(colors[index].toString(), dx+4, dy);
      }
   }
   protected void drawCursor(Graphics g, int sel, boolean on)
   {
      if (offset <= sel && sel < visibleItems+offset && sel < itemCount)
      {
         int dx = 3;
         int dy = 4;
         if (Settings.uiStyle == Settings.PalmOS) dy--;
         if (simpleBorder) {dx--; dy--;}
         dy += (sel-offset) * fmH;
         g.setClip(dx-1,dy-2,btnX-dx+1,Math.min(fmH * visibleItems, this.height-dy)); // guich@200b4_83: fixed selection overflowing paint area
         g.drawCursorOutline(dx-1,dy-2,btnX-1,fmH+fm.getDescent()); // only select the Object - guich@200b4_130
      }
   }
   public int getPreferredWidth()
   {
      return w + (simpleBorder?4:6) + sbar.getPreferredWidth();
   }
   protected int getItemWidth(int index)
   {
      return w;
   }
   /** Will return a String with the rrggbb in hex */
   public Object getSelectedItem()
   {
      return selectedIndex!=-1 ? colors[selectedIndex] : Color.BLACK;
   }
}