/* Created by Jean Rissoto. With help of Guilherme Campos Hazan.
 * Origine=Waba Edit Class
 * First modification: create a multiEdit Class to edit on several lines
 * Second mofification: create a scrollBarMultiEditClass to edit on several lines and move screen with a scrollbar
 * Third modificatin: create a menuMultiEditClass to have a popupWindow to copy cut paste save restore text
 * Fourth modification: merge multiEdit Class and scrollBarMultiEditCalss in one class MultiEdit with auto-appearance of the new scrollBar enhanced by Guilherme Campos Hazan
 * Fifth modification: merge Waba Edit Class and new Edit in one class=newEdit
 * Sixth modification: change algorithm: manage only word length and blanks position instead of charWidth.
 * Seventh modification: because Guilherme decision newEdit don't substitute to Edit. Thus newEdit is separte in two versions:
 *                         newEdit both several lines editing and one line editing
 *                          multiEdit several lines editing.
 *
 * change 8: guich had optimized/fixed code here for SuperWaba 3.2
 */

/*
 * Todo: separate Initiate wordsWidth, endsWordPlace first changed First from adding new words
 */

package superwaba.ext.xplat.ui;

import waba.ui.*;
import waba.io.*;
import waba.fx.*;
import waba.sys.*;
import waba.util.*;

/**
 * MultiEdit is an Edit with support for multiple lines. A static scrollbar is added, but disabled/enabled as needed.
 * <p>
 * Here is an example showing an Edit control being used:
 *
 * <pre>
 * import superwaba.ext.xplat.ui.MultiEdit;
 *
 * public class MyProgram extends MainWindow
 * {
 *    MultiEdit mEdit;
 *
 *    public void onStart()
 *    {
 *       // the constructor method is called with the mask, the number of lines
 *       // and the vertical interval in pixel between two lines
 *       mEdit = new MultiEdit("",3,1);
 *       add(mEdit,LEFT,TOP);
 *       // add/setRect must precede setText
 *       mEdit.setText("What text you want"); //eventually
 *    }
 * }
 * </pre>
 */


public class MultiEdit extends Container
{
   public static final char ENTER = (char)13;
   private static char SPACE = ' ';
   private Graphics drawg; // only valid while the edit has focus --   original
   private Timer blinkTimer; // only valid while the edit has focus --   original
   private boolean hasFocus = false;
   private boolean cursorShowing = false;
   private boolean editable = true;
   /** Set to false if you don't want the cursor to blink when the edit is not editable */
   public boolean hasCursorWhenNotEditable = true; // guich@340_23
   // begin JR adds
   private IntVector first = new IntVector(5); //JR@0.4.  indices of first character of each line. the value of last is len(text)+1
   private boolean forceDrawAll;
   private int firstToDraw=0; //JR@0.4
   private int numberTextLines=0; // JR@0.4
   private int hLine; // JR@0.4. height of a line
   // end JR@0.4 adds
   /** Area excluding borders and scrollbar */
   private Rect boardRect;
   /** Area boardRect - gaps */
   private Rect textRect;
   private final int charsStep=8; // JR@0.5. Step to inscrease size of the two arrays chars and charWidths
   // end JR@0.5 adds
   private boolean isCommand;
   private int len = 0;
   private char chars[] = new char[charsStep];
   private int insertPos;
   private int startSelectPos;
   private int newInsertPos;
   private int pushedInsertPos; //protected in Edit
   private int pushedStartSelectPos; //protected in Edit
   private int pushedxOffset; //used in Edit is it really useful?
   private ScrollBar sb;
   private int spaceBetweenLines;//
   private int rowCount; // guich@320_28
   private boolean drawDots = true;
   private int tempGap, tempRowCount; // used on popup keyboard

   private int maxLength; // guich@200b4
   private IntVector wordsWidth = new IntVector(20); //JR@0.8
   private IntVector wordsEndPos=new IntVector(20); //JR@0.8
   private Color fColor,back0,back1; //JR@0.8
   private Color fourColors[] = new Color[4]; //JR@0.8
   private byte kbdType=Edit.KBD_KEYBOARD;
   protected String mask;

   public static MultiEditMenu menu;
   public int gap=1; // guich@320_28: made public to remove 2 almost unused methods

   private String validChars;
   private byte capitalise;

   /** Constructor for a text Edit with a vertical scroll Bar, gap is 1
    by default and control's bounds must be given with a setRect. Space between lines may be 0. */
   public MultiEdit(int rowCount, int spaceBetweenLines)  // vertical space between 2 lines in pixel
   {
      this.rowCount = rowCount;
      this.hLine = fmH + spaceBetweenLines;
      this.spaceBetweenLines = spaceBetweenLines;
      this.clearPosState();
      add(this.sb = new ScrollBar(ScrollBar.VERTICAL));
      sb.setLiveScrolling(true);
      // don't let the scrollbar steal focus from us
      sb.setFocusLess(true);
   }

   /** Constructor for a text Edit with a vertical scroll Bar, gap is 1
     * by default and control's bounds must be given with a setRect. Space between lines may be 0.
     * The mask is used to compute the PREFERRED width of the control. Note that the mask does not <i>masks</i>
     * the input. If mask is "", the FILL width is choosen.
     */
   public MultiEdit(String mask, int rowCount, int spaceBetweenLines)
   {
      this(rowCount, spaceBetweenLines);
      this.mask = mask;
   }

   public int getPreferredHeight()
   {
      return hLine*rowCount+ (Settings.uiStyle==Settings.PalmOS?2:4) + 2*gap;//2*(fmH+2)+(Settings.uiStyle==Settings.PalmOS?0:2); //+2= minimal space between 2 lines
   }

   public int getPreferredWidth()
   {
      return mask==null?(waba.sys.Settings.screenWidth>>2):(mask.length()==0)?FILL:(fm.getTextWidth(mask) + ((Settings.uiStyle == Settings.PalmOS)?4:10)); // guich@200b4_202: from 2 -> 4 is PalmOS style - guic@300_52: empty mask means FILL
   }

   /** Used to change the default keyboard to be used with this Edit control.
     * Use the constants Edit.KBD_NONE and Edit.KBD_KEYBOARD.
     */
   public void setKeyboard(byte kbd)
   {
      this.kbdType = kbd == Edit.KBD_NONE ? Edit.KBD_NONE : Edit.KBD_KEYBOARD;
   }

   /** If true, a dotted line appears under each row of text (on by default) */
   public void setDrawDots(boolean on)
   {
      this.drawDots = on;
   }

   /** Used to set the number of rows of this MultiEdit. You must call setRect
     * after this to resize the control in height
     */
   public void setNumberOfRows(int rowCount)
   {
      this.rowCount = rowCount;
   }

   /** Sets the capitalise settings for this MultiEdit. Use Edit.ALL_NORMAL,
     * Edit.ALL_UPPER and Edit.ALL_LOWER. Text entered will made as is, uppercase or
     * lowercase
     */
   public void setCapitaliseMode(byte cap)
   {
      this.capitalise = cap;
   }

   /** sets the valid chars that can be entered in this edit. if null is passed, any char can be entered. (case insensitive). added by guich */
   public void setValidChars(String validCharsString)
   {
      if (validCharsString != null)
         validChars = Convert.toUpperCase(validCharsString);
      else
         validChars = null;
   }

   /** Sets the desired maximum length for text entered in the Edit. @since SuperWaba2.0beta4 */
   public void setMaxLength(int length)
   {
      this.maxLength = length;
   }

   /**
    * Returns the text displayed in the edit control.
    */
   public String getText()
   {
      return new String(chars, 0, len);
   }

   /**
    * Sets the text displayed in the edit control.
    */
   public void setText(String s)
   {
      chars = s.toCharArray();
      len = chars.length;
      newInsertPos = numberTextLines = 0;
      calculateWords();
      if (textRect!=null)
         calculateFirst();
      forceDrawAll=true;
      clearPosState();
   }

   /** Saves the contents of this TextArea to the file MEme.PROV.DATA. Used with command+S. */
   public void saveText()
   {
      Catalog cat = new Catalog("MEme.PROV.DATA",Catalog.READ_WRITE);
      if (cat.isOpen())
         cat.delete();
      cat = new Catalog("MEme.PROV.DATA",Catalog.CREATE);
      ResizeStream rs = new ResizeStream(cat,512);
      DataStream ds = new DataStream(rs);
      if (rs.startRecord())
      {
         ds.writeString(getText());
         rs.endRecord();
      }
      cat.close();
   }

   /** Restores the contents of this TextArea from the file MEme.PROV.DATA. Used with command+R. */
   public void restoreText()
   {
      Catalog cat = new Catalog("MEme.PROV.DATA",Catalog.READ_ONLY);
      if (cat.isOpen())
      {
         ResizeStream rs = new ResizeStream(cat,512);
         DataStream ds = new DataStream(rs);
         if (cat.setRecordPos(0))
            setText(ds.readString());
         cat.close();
      }
      //why not delete catalog then : if user want to restore again.
   }

   /** Sets if the control accepts input from the user */
   public void setEditable(boolean on)
   {
      editable = on;
   }

   /** Gets if the control accepts input from the user */
   public boolean isEditable()
   {
      return editable;
   }

   /** Gets total number of lines in the text */
   public int getNumberOfTextLines()
   {
      return numberTextLines;
   }

   /** user method to popup the keyboard/calendar/calculator for this edit. */
   public void popupKCC()
   {
      if (kbdType == Edit.KBD_NONE) return;
      if (Settings.virtualKeyboard)
         onEvent(new Event(ControlEvent.FOCUS_IN,this,0)); // simulate a focus in event. Sorry, i know this is ugly
      else
      {
         Sound.beep(); // guich@200b4_119: add a click
         requestFocus(); // guich@200b4: bring focus back
         if (Edit.keyboard == null) Edit.keyboard = new Keyboard();
         pushPosState();
         if (removeTimer(blinkTimer)) // guich@200b4_167
               blinkTimer = null;
         // guich@320: modify and restore later our state, bc the Keyboard needs a shrinked control
         tempGap = gap;
         tempRowCount = rowCount;
         gap = 0;
         rowCount = 2;
         Window.getTopMost().popupModal(Edit.keyboard);
         popPosState();
         requestFocus();
      }
   }

   // popup the Menu for this edit. //JR @0.6
   public void popupMenu()
   {
       if (menu == null) menu = new MultiEditMenu();
       pushPosState();
       if (removeTimer(blinkTimer)) // guich@200b4_167
          blinkTimer = null;
       Window.getTopMost().popupModal(menu);
   }

   private int zToCharPos(Coord z)
   {
      int lines,i;
      if (z.x < 0) z.x=0; //see later here is a solution to scroll up when extending)
      if (z.y < 0) z.y=0;
      if (z.x > this.width ) z.x = this.width;
      if (z.y > this.height) z.y = this.height;//see later here is solution to scrolldown when extending)
      lines = firstToDraw + (z.y - textRect.y) / hLine; //first what's the line
      int cx = boardRect.x;
      if (lines+1 >= first.items.length) // guich@341_10: fixed problem when clicking on a row greater than the current number of lines.
         return len;
      int n = first.items[lines+1];
      for (i = first.items[lines]; i < n; i++)
         if (i < len)
         {
            int cw = fm.getCharWidth(chars[i]);
            if (z.x <= cx + (cw >> 1)) //cw >>1 equivalent cw/2
               return i;
            cx += cw;
         }
      i = first.items[lines+1]; // guich@341_10
      return i==0?len:i;
   }

   private Coord charPosToZ(int n)
   {
      Coord z = new Coord(textRect.x,textRect.y);
      // no string or pos 0?
      if (len == 0 || n == 0)
         return z;

      if (n > len)
         n = len;

      int i = 0;
      while (first.items[i] < n)
         i++;

      if (i > 0)
         z.y = --i;
      else
         z.y = 0;

      z.x = textRect.x + fm.getTextWidth(chars,first.items[i],n-first.items[i]);
      z.y -= firstToDraw;
      z.y *= hLine;
      z.y += textRect.y;
      return z;
   }

   protected void onBoundsChanged()
   {
      drawg = null;
      int zOffset = Settings.uiStyle==Settings.PalmOS?0:2; // size of borders
      boardRect = new Rect(zOffset,zOffset,this.width-2*zOffset-sb.getPreferredWidth(),this.height-2*zOffset);    //JR @0.5
      textRect = boardRect.modifiedBy(gap,gap,-2*gap,-2*gap);
      rowCount = textRect.height / this.hLine; // kambiz@350_5: update rowCount according to the new size of the text area
      sb.setRect(RIGHT,TOP,PREFERRED,FILL);
      sb.setValues(0, rowCount, 0, rowCount);
      sb.setEnabled(false);
      numberTextLines = 0;
      firstToDraw = 0;
      forceDrawAll = true;
      calculateWords();
      if (len > 0)
         calculateFirst();
   }

   /*
    * Calculate wordsWidth and wordsEndPos arrays.
    * Else we recalculate all and then  wordsWidth is useless
    * Else this should be enhanced to don't calculate all words, but only
    * from the word wich precede the one where we are writing
    * to enhance: distinghish initiate Ends and Words from actualise
    * Ends and Words
    */
   private void calculateWords() // guich@320_28: completely redesigned
   {
      wordsWidth.clear();
      wordsEndPos.clear();
      wordsEndPos.add(0);
      int wordWidth=0;
      boolean lastWasSpace=false;
      if (len > 0)
         for (int i=0; ; i++) // guich@320_28: avoid duplicating code
         {
            char c = (i == len) ? 0 : chars[i];
            if (c == 0 || (!lastWasSpace && (c == SPACE || c == ENTER || c == 10))) // guich@350_6: added char(10) test
            {
               wordsWidth.add(wordWidth);
               wordsEndPos.add(c==SPACE ? (i+1) : i); // the space is included in the preceding word; else is included in the next word
               if (c == 0)
                  break;
               wordWidth = 0;
            }
            lastWasSpace = c == SPACE;
            wordWidth += fm.getCharWidth(c);
         }
      // debug: for (int i =0; i < wordsEndPos.size(); i++) Vm.debug("wordsEndPos["+i+"]: "+wordsEndPos.items[i]);
   }

   /** Compute the index of the first character of each line */
   private void calculateFirst() // guich@320_28: completely redesigned
   {
      int originalLineCount = numberTextLines;
      first.clear();
      first.add(0); // in line 0, the first char is always 0

      int words = wordsWidth.size();
      int currentLineWidth = 0;
      int firstWordOfThisLine = 0;
      int wordStart=0, wordEnd = 0, wordWidth, i;

      for (i =0; i < words; i++)
      {
         wordStart = wordsEndPos.items[i];
         wordEnd   = wordsEndPos.items[i+1];
         wordWidth = wordsWidth.items[i];
         currentLineWidth += wordWidth;
         char ch = (wordStart < chars.length ? chars[wordStart] : ' ');
         boolean isEnter = ch == ENTER || ch == 10; // not sure why sometimes wordStart not < len

         // if appending a new word to the current line, does it oversizes the textRect.width?
         if (currentLineWidth > textRect.width || isEnter)
         {
            // now we need to wrap the line.
            // case 1: this is the first word of the line and its too big
            if (firstWordOfThisLine == i)
            {
               int cutPos = wordStart, letterWidth;
               while (true)
               {
                  currentLineWidth = 0;
                  // break this word while it is greater than the width
                  while (cutPos < wordEnd && currentLineWidth + (letterWidth=fm.getCharWidth(chars[cutPos])) <= textRect.width)
                  {
                     currentLineWidth += letterWidth;
                     cutPos++;
                  }
                  // cut the word
                  if (cutPos < wordEnd)
                     first.add(cutPos);
                  else
                     break; // don't add last line
               }
            }
            else
            // case 2: already exists other words, so we break now and
            // make the current word the first one of the next line
            {
               first.add(wordStart);
               firstWordOfThisLine = i;
               currentLineWidth = 0;
               i--; // force recompute this word (maybe its too big to fit)
            }
         }
      }
      // add the rest of the last line
      first.add(wordEnd);

      numberTextLines = first.size()-1;

      // debug: for (i =0; i <= numberTextLines; i++) Vm.debug("first["+i+"]: "+first.items[i]);

      // has the number of lines changed? enable/disable scrollbar
      if (numberTextLines != originalLineCount)
      {
         forceDrawAll = true;
         boolean needScroll = numberTextLines > rowCount;
         sb.setEnabled(needScroll);
         sb.setMaximum(needScroll ? numberTextLines : 0);
      }

      // compute the new line of the cursor
      for (i=0; newInsertPos > first.items[i]; i++) {}
      firstToDraw = Math.max(0, i - rowCount);

      // need to change scrollbar position?
      if (sb.getValue() != firstToDraw)
      {
         forceDrawAll=true;
         sb.setValue(firstToDraw);
      }
   }

   /** Called by the system to pass events to the edit control. */
   public void onEvent(Event event)
   {
       if (event.target == this)
       {
           if (wordsWidth == null) return; // widths have not been initialized - not added to hierarchy
           boolean redraw = false;
           boolean extendSelect = false;
           boolean clearSelect = false;
           newInsertPos = insertPos;
           switch (event.type)
           {
               case ControlEvent.TIMER:
                   if (parent != null)
                      draw(drawg, true);
                   event.consumed=true;     //astein@230_5: prevent blinking cursor event from propagating
                   return;
               case ControlEvent.FOCUS_IN:
                   // guich@300_43: this is needed bc when popupKCC is called, the focus comes back to here; also, when the popped up window is closed, the focus comes back again, so we could enter in an infinite loop
                   if (kbdType != Edit.KBD_NONE && Settings.virtualKeyboard) // if running on a PocketPC device, set the bounds of Sip in a way to not cover the edit
                      getParentWindow().setStatePosition(0, getAbsoluteRect().y < (Settings.screenHeight>>1) ? Window.VK_BOTTOM : Window.VK_TOP);
                   if (drawg==null)
                       drawg = createGraphics();
                   hasFocus = true;
                   if (blinkTimer == null)
                       blinkTimer = addTimer(350);
                   break;
               case ControlEvent.FOCUS_OUT:
                   if (Settings.virtualKeyboard) // if running on a PocketPC device, set the bounds of Sip in a way to not cover the edit
                      getParentWindow().setStatePosition(Window.VK_HIDE,Window.VK_HIDE);
                   hasFocus = false;
                   //                  see what to do when popup
                   //                  clearPosState(); should be there before going on popUp
                   //                  newInsertPos = 0;
                   //                  redraw = true;
                   if (removeTimer(blinkTimer))
                      blinkTimer = null;
                   // kambisDarabi@310_7 : remove the cursor, if it is currently shown
                   if (cursorShowing)
                      draw(drawg, true);
                   hasFocus = false;
                   break;
               case KeyEvent.KEY_PRESS:
                   if (editable)
                   {
                       forceDrawAll=false;
                       KeyEvent ke = (KeyEvent)event;
                       boolean isControl = (ke.modifiers & IKeys.CONTROL) != 0; // guich@320_46
                       boolean isPrintable;
                       if (ke.key < 65536 && (ke.modifiers & IKeys.ALT) == 0 && (ke.modifiers & IKeys.CONTROL) == 0)
                           isPrintable = true;
                       else
                           isPrintable = false;
                       // print state -- original
                       if (ke.key == IKeys.COMMAND)
                       {
                           popupMenu(); // guich@320_28: now the menu is popped up with the command.
                           isCommand = !isCommand;
                           break; // command char? break --original
                       }
                       if ((ke.key == IKeys.KEYBOARD_ABC || ke.key == IKeys.KEYBOARD_123) && (Edit.keyboard == null || !Edit.keyboard.isVisible()))
                       {
                           popupKCC();
                           return;
                       }
                       boolean isDelete = (ke.key == IKeys.DELETE);
                       boolean isBackspace = (ke.key == IKeys.BACKSPACE);
                       boolean isEnter = (ke.key == IKeys.ENTER);
                       int del1 = -1;
                       int del2 = -1;
                       int sel1 = startSelectPos;
                       int sel2 = insertPos;
                       if (sel1 > sel2)
                       {
                           int temp = sel1;
                           sel1 = sel2;
                           sel2 = temp;
                       }
                       // clipboard -- original
                       if (isCommand || isControl)
                       {
                           isCommand = false;
                           if (isControl)
                           {
                              ke.key += 64;
                              ke.modifiers &= ~IKeys.CONTROL; // remove control
                           }
                           char key=Convert.toUpperCase((char)ke.key);
                           switch (key)
                           {
                              case 'S':
                                 saveText(); // Save
                                 Sound.beep();
                                 return;
                              case 'R':
                                 restoreText(); //Restore
                                 Sound.beep();
                                 return;
                              case 'X':
                              case 'C':
                                 if (sel1 != -1)
                                 { // cut/copy --original
                                    Vm.clipboardCopy(new String(chars).substring(sel1,sel2));
                                    if (key == 'X')
                                    { // cut -- original
                                       ke.key = IKeys.BACKSPACE;
                                       onEvent(ke);
                                    }
                                    else Sound.beep();
                                 }
                                 break;
                              case ' ':
                                 setText("");
                                 return;
                              case 'P':
                              case 'V':
                                 String pasted = Vm.clipboardPaste();
                                 if (pasted == null || pasted.length() == 0)
                                    Sound.beep();
                                 else
                                 {
                                    char newschars[];
                                    char ch[] = pasted.toCharArray();
                                    if (chars.length<len+ch.length)
                                          newschars= new char[len+ch.length];
                                    else
                                          newschars=new char[chars.length];
                                    Vm.copyArray(chars,0,newschars,0,newInsertPos);
                                    //actually it is newInertPos-1 but newInsertPos is well modified by next line
                                    Vm.copyArray(ch,0,newschars,newInsertPos,ch.length);
                                    Vm.copyArray(chars,newInsertPos,newschars,newInsertPos+ch.length,len-newInsertPos);
                                    chars=newschars;
                                    newInsertPos += ch.length;
                                    len+=ch.length;
                                    calculateWords();
                                    calculateFirst();
                                    forceDrawAll=true;
                                 }
                                 break;
                              default: Sound.beep();
                           }
                           clearSelect=true;
                           break;
                       }
                       if (isPrintable)
                       {
                           if (capitalise == Edit.ALL_NORMAL)
                              ;
                           else
                           if (capitalise == Edit.ALL_UPPER)
                              ke.key = Convert.toUpperCase((char)ke.key);
                           else
                           if (capitalise == Edit.ALL_LOWER)
                              ke.key = Convert.toLowerCase((char)ke.key);

                           if (!isCharValid((char)ke.key)) // guich@101: tests if the key is in the valid char set - moved to here because an valid clipboard char can be an invalid edit char
                           {
                              Sound.beep();
                              break;
                           }
                       }
                       if (sel1 != -1 && (isPrintable || isDelete || isBackspace))
                       {
                           del1 = sel1;
                           del2 = sel2 - 1;
                       }
                       else if (isDelete)
                       {
                           del1 = insertPos;
                           del2 = insertPos;
                       }
                       else if (isBackspace)
                       {
                           del1 = insertPos - 1;
                           del2 = insertPos - 1;
                       }
                       if (isEnter)
                       {
                          ke.key=ENTER;
                          isPrintable=true;
                       }
                       if (del1 >= 0 && del2 < len)
                       {
                           int deleteCount = del2 - del1 + 1;
                           int numOnRight = len - del2 - 1;
                           if (numOnRight > 0)
                              Vm.copyArray(chars, del2 + 1, chars, del1, numOnRight);
                           len -= deleteCount;
                           newInsertPos = del1;
                           forceDrawAll = true;
                           clearSelect = true;
                       }
                       if (isPrintable && (maxLength == 0 || len < maxLength))
                       {
                           // grow the array if required (grows by charsStep) --   original
                           if (len == chars.length)
                           {
                              char newChars[] = new char[len + charsStep];
                              Vm.copyArray(chars, 0, newChars, 0, len);
                              chars = newChars;
                           }
                           char c = (char)ke.key;
                           if (newInsertPos != len)
                           {
                              int i = newInsertPos;
                              int lines = len - newInsertPos;
                              Vm.copyArray(chars, i, chars, i + 1, lines);
                           }
                           else
                           if (wordsWidth.size() > 0)
                           {
                              int words = wordsWidth.size();
                              wordsEndPos.items[words-1]=newInsertPos;
                              wordsWidth.items[words-1]+=fm.getCharWidth(c);
                           }
                           chars[newInsertPos] = c;
                           len++;
                           newInsertPos++;
                           redraw = true;
                           clearSelect = true;
                       }
                       boolean isMove = true;
                       switch (ke.key)
                       {
                           case IKeys.HOME  :
                               newInsertPos = 0;
                               if (firstToDraw!=0)
                               {
                                   firstToDraw=0;
                                   forceDrawAll=true;
                                   sb.setValue(0);
                               }
                               break;
                           case IKeys.END   :
                               newInsertPos = len;
                               if (numberTextLines>rowCount)
                               {
                                   firstToDraw=numberTextLines-rowCount;
                                   forceDrawAll=true;
                                   sb.setValue(firstToDraw);
                               }
                               break;
                           case IKeys.UP    :
                               if (newInsertPos>=first.items[1])
                               {
                                   Coord nzPos=charPosToZ(newInsertPos);
                                   if (nzPos.y<=textRect.y)
                                   {
                                       int ii =sb.getValue();
                                       if (ii>sb.getMinimum()) firstToDraw--;
                                       sb.setValue(--ii);
                                   }
                                   else
                                       nzPos.y-=hLine;
                                   newInsertPos=zToCharPos(nzPos);
                                   forceDrawAll=true;
                               }
                               break;
                           case IKeys.LEFT  :
                               newInsertPos--;
                               if (newInsertPos<0)
                                   newInsertPos=0;
                               while (newInsertPos<first.items[firstToDraw])
                               {
                                   firstToDraw--;
                                   forceDrawAll=true;
                                   sb.setValue(sb.getValue()-1);
                               }
                               break;
                           case IKeys.RIGHT :
                               newInsertPos++;
                               if (newInsertPos>len)
                                   newInsertPos=len;
                               if (numberTextLines>rowCount)
                               {
                                   while (newInsertPos>=first.items[firstToDraw+rowCount])
                                   {
                                       firstToDraw++;
                                       forceDrawAll=true;
                                       sb.setValue(sb.getValue()+1);
                                   }
                               }
                               break;
                           case IKeys.DOWN  :
                               if (newInsertPos<=first.items[numberTextLines-1]) //-1
                               {
                                   Coord nzPos=charPosToZ(newInsertPos);
                                   if (nzPos.y>=textRect.height-hLine)
                                   {
                                       int ii =sb.getValue();
                                       if (ii<sb.getMaximum())
                                           firstToDraw++;
                                       sb.setValue(++ii);
                                   }
                                   else
                                       nzPos.y+=hLine;
                                   forceDrawAll=true;
                                   newInsertPos=zToCharPos(nzPos);
                               }
                               break;
                           default: isMove = false;
                       }
                       if (isMove && newInsertPos != insertPos)
                       {
                           if ((ke.modifiers & IKeys.SHIFT) > 0)
                               extendSelect = true;
                           else
                               clearSelect = true;
                       }
                       if (!isMove)
                       {
                          calculateWords();
                          calculateFirst();
                       }
                   }
                   break;
               case PenEvent.PEN_DOWN:
               {
                   PenEvent pe = (PenEvent)event;
                   newInsertPos=zToCharPos(new Coord(pe.x,pe.y));
                   if ((pe.modifiers & IKeys.SHIFT) > 0) extendSelect = true; // shift
                   else  clearSelect = true;
                   break;
               }
               case PenEvent.PEN_DRAG:
               {
                   PenEvent pe = (PenEvent)event;
                   newInsertPos = zToCharPos(new Coord(pe.x,pe.y));
                   if (newInsertPos != insertPos)
                       extendSelect = true;
                   else
                       return; // guich@320_28: avoid unnecessary repaints
                   break;
               }
               default:
                  if (event.type == Keyboard.KEYBOARD_ON_UNPOP)
                  {
                     gap = tempGap;
                     rowCount = tempRowCount;
                  }
                  else
                  if (event.type == Keyboard.KEYBOARD_POST_UNPOP)
                     requestFocus();
                  return;
           }
           if (extendSelect)
           {
               if (startSelectPos == -1)
                   startSelectPos = insertPos;
               else if (newInsertPos == startSelectPos)
                   startSelectPos = -1;
               redraw = true;
           }
           if (clearSelect && (startSelectPos != -1))
           {
               startSelectPos = -1;
               redraw = true;
           }
           if (newInsertPos > len)  newInsertPos = len;
           if (newInsertPos < 0) newInsertPos = 0;
           boolean insertChanged = (newInsertPos != startSelectPos );
           if (insertChanged && cursorShowing)
               draw(drawg, true); // erase cursor at old insert position
           insertPos = newInsertPos;
           if (redraw || /*forceDrawAll || */insertChanged)
              draw(drawg, !insertChanged);
           if (event.type == ControlEvent.FOCUS_OUT)
              drawg = null;
       }
       else
       if (event.target==sb && event.type == ControlEvent.PRESSED)
       {
          firstToDraw = sb.getValue();
          repaint(); // alexgross@340_17
       }
   }
   protected void draw(Graphics g, boolean cursorOnly)
   {
       if (g == null) return;
       if (forceDrawAll)
       {
          g.setBackColor(back0);
          g.clearClip();
          int x2 = this.width-sb.getPreferredWidth();
          g.fillRect(0,0,x2,this.height);
          if (Settings.uiStyle != Settings.PalmOS)
              g.draw3dRect(0,0,x2, this.height, Graphics.R3D_CHECK, false,false, fourColors);
       }
       g.setClip(boardRect);
       // draw the text and/or the selection --original
       if (!cursorOnly || forceDrawAll)
       {
           int i;
           int h  = textRect.y;
           int dh = textRect.y + fm.getAscent();
           g.setForeColor(fColor);
           g.setBackColor(back0);
           for (i = firstToDraw; i < numberTextLines; i++, h += hLine, dh += hLine)
           {
               if (!forceDrawAll) g.fillRect(boardRect.x, h, boardRect.width-1, hLine); // erase drawing area
               int ww = first.items[i+1]-first.items[i];
               g.drawText(chars, first.items[i], ww < len ? ww : len, textRect.x, h);
               if (drawDots)
               {
                  g.drawDots(textRect.x, dh, textRect.x2(), dh); // guich@320_28: draw the dotted line
                  g.setBackColor(back0);
               }
               // debug: Vm.debug("drawing "+new String(chars, first.items[i], ww < len ? ww : len)+" - "+(ww < len ? ww : len));
           }

           // guich@320_28: draw the dotted lines
           if (!forceDrawAll || drawDots)
               for (; i < firstToDraw+rowCount; i++, h += hLine, dh += hLine)
               {
                  if (!forceDrawAll) g.fillRect(boardRect.x, h, boardRect.width-1, hLine); // erase drawing area
                  if (drawDots) g.drawDots(textRect.x, dh, textRect.x2(), dh);
                  g.setBackColor(back0);
               }

           if (startSelectPos != -1)
           {
               // character regions are: --   original
               // 0 to (sel1-1) .. sel1 to (sel2-1) .. sel2 to last_char --  original
               int sel1 = Math.min(startSelectPos,insertPos);
               int sel2 = Math.max(startSelectPos,insertPos);
               g.setForeColor(back0);   // inverse color writing
               g.setBackColor(back1); // color of "selected block"-- JR@0.8
               Coord zSel1 = charPosToZ(sel1);
               Coord zSel2 = charPosToZ(sel2);
               if (zSel1.y==zSel2.y)
                  g.eraseRect(zSel1.x,zSel1.y,zSel2.x-zSel1.x,fmH);
               else
               {
                  g.eraseRect(zSel1.x,zSel1.y,textRect.x2()-zSel1.x+1,hLine);
                  if (zSel2.y > zSel1.y)
                     g.eraseRect(textRect.x,zSel1.y+hLine,textRect.width,zSel2.y-zSel1.y- hLine);
                  g.eraseRect(textRect.x,zSel2.y,zSel2.x-textRect.x,fmH);
               }
           }
       }
       forceDrawAll=false;
       if (hasFocus && (editable || hasCursorWhenNotEditable))
       {
           // draw cursor -- original
           Coord z=charPosToZ(insertPos);
           int zx;
           if (z.x-1>boardRect.x)
               zx = z.x - 1;
           else
               zx = boardRect.x;
           g.drawCursor(zx, z.y - (spaceBetweenLines>>1), 1, hLine);
           if (cursorOnly)
               cursorShowing = !cursorShowing;
           else
               cursorShowing = true;
       }
       else
           cursorShowing = false;
   }

   protected void onWindowPaintFinished()
   {
       if (editable && !hasFocus) onEvent(new Event(ControlEvent.FOCUS_IN,this,0)); // this event is called on the focused control of the parent window. so, if we are not in FOCUS state, set it now. --original - guich@350_7: added the editable check
   }

   public void onPaint(Graphics g)
   {
      forceDrawAll=true;
      draw(g, false);
   }

   private void clearPosState()
   {
      insertPos = 0;
      startSelectPos = -1;
   }

   protected void pushPosState()
   {
      pushedInsertPos = insertPos;
      pushedStartSelectPos = startSelectPos;
   }

   protected void popPosState()
   {
      if (cursorShowing)
         draw(drawg,true);
      insertPos = pushedInsertPos;
      startSelectPos = pushedStartSelectPos;
   }

   /** Return true if the given char exists in the set of valid characters for this Edit */
   private boolean isCharValid(char c)
   {
      if (validChars == null) return true;
      c = Convert.toUpperCase(c);
      return validChars.indexOf(c) != -1;
   }

   protected void onColorsChanged(boolean colorsChanged)
   {
      fColor = getForeColor();
      back0  = getBackColor().brighter();
      back1  = back0.equ != Color.WHITE.equ?backColor:back0.getCursorColor();//guich@300_20: use backColor instead of: back0.getCursorColor();
      Graphics.compute3dColors(enabled,backColor,foreColor,fourColors);
      sb.setBackForeColors(backColor, foreColor);
   }

   /** Sets the rect for this MultiEdit. Note that height is recomputed based
     * in the value for rowCount given in the constructor if the height is PREFERRED
     */
   public void setRect(int x, int y, int width, int height)
   {
      if (height == PREFERRED) // kambiz@330_24: use preferred height only if user wants
         height = getPreferredHeight();
      super.setRect(x,y,width,height);
   }
   protected void onFontChanged() // guich@320_28
   {
      hLine = fmH + spaceBetweenLines;
   }
}
