package superwaba.ext.xplat.ui;

import waba.ui.*;
import waba.fx.*;

/** A basic progress bar, with the bar and a text */

public class ProgressBar extends Control
{
	/**
	 * A optional String that can be displayed on the progress bar.
	 * The default is null. Setting this to a non-null value does not
	 * imply that the String will be displayed.
	 */
	private String progressString;

	/**
	 * The default minimum for a progress bar is 0.
	 */
	static final private int defaultMinimum = 0;
	/**
	 * The default maximum for a progress bar is 100.
	 */
	static final private int defaultMaximum = 100;

	/**
	 * The minimum value of a progress bar.
	 */
	private int min;

	/**
	 * The maximum value of a progress bar.
	 */
	private int max;

	/**
	 * The current value of a progress bar.
	 */
	private int value;

	/**
	 * The text color of the text of a progress bar.
	 */
	public Color textColor = waba.sys.Settings.isColor?new Color(255, 0, 0):Color.WHITE; // guich@340_5

	/**
	 * The color used to fill progress bar.
	 */
	public Color fillColor = waba.sys.Settings.isColor?new Color(0, 0, 255):Color.BLACK; // rogerio@341_13

	/**
	 * Creates a progress bar, String is set to <code>null</code>, which is the default.
	 * Uses the default minimum and maximum.
	 * Uses the default minimum for the initial value of the progress bar.
	 */
	public ProgressBar()
	{
		this(defaultMinimum, defaultMaximum);
	}

	/**
	 * Creates a progress bar, String is set to <code>null</code>, which is the default.
	 * Uses the specified minimum and maximum.
	 * Uses the specified minimum for the initial value of the progress bar.
	 */
	public ProgressBar(int mn, int mx)
	{
		min = value = mn;
		max = mx;
		if (!waba.sys.Settings.isColor)
		{
	      foreColor = Color.BLACK; // rogerio@341_13
         backColor = Color.DARK;  // rogerio@341_13
      }
	}

	 /**
	 * Returns the current value of the Progress String.
	 * If you are providing a custom Progress String via this method,
	 * you will want to ensure that you call setString() before
	 * you call getString();
	 *
	 * @return the value of the percent string
	 * @see    #setString
	 */
	public String getString()
	{
		if (progressString != null)
			return progressString;
		else
		{
			int pc = (int)Math.round(100 * getPercentComplete());
			return pc + "%";
		}
	}

	 /**
	 * Sets the value of the Progress String. By default,
	 * this String is set to <code>null</code>.
	 * If you are providing a custom Progress String via this method,
	 * you will want to ensure that you call setString() before
	 * you call getString().
	 * If you have provided a custom String and want to revert to
	 * the built-in behavior, set the String back to <code>null</code>.
	 *
	 * @param  s       the value of the percent string
	 * @see    #getString
	 */
	public void setString(String s)
	{
		String oldValue = progressString;
		if (s != null)
		{
		   while (fm.getTextWidth(s) > width-4)
		   {
				s = s.substring(0, s.length()-1);
			}
		}
		progressString = s;
		if (progressString == null || oldValue == null || !progressString.equals(oldValue))
			repaintNow();
	}

	/**
	 * Returns the percentage/percent complete for the progress bar.
	 * Note that, as a double, this number is between 0.00 and 1.00.
	 *
	 * @return the percent complete for this progress bar.
	*/
	public double getPercentComplete()
	{
	   double dif = max-min; // guich@400_60
		return dif == 0 ? 100 : (double)(value - min) / (double)(max - min);
	}

   public int getPreferredWidth()
   {
      return parent==null ? waba.sys.Settings.screenWidth : (parent.getSize().x-6);
   }

   public int getPreferredHeight()
   {
      return fmH+4;
   }

	/**
	 * Returns the model's current value. The value is always between the
	 * model's minimum and maximum values, inclusive. By default, the value
	 * equals the minimum.
	 *
	 * @return  the value
	 * @see     #setValue
	 */
	public int getValue() { return value; }

	 /**
	 * Returns the model's minimum value.
	 * By default, this is <code>0</code>.
	 *
	 * @return  an int -- the model's minimum
	 * @see     #setMinimum
	 */
	public int getMinimum() { return min; }

	/**
	 * Returns the model's maximum value.
	 * By default, this is <code>100</code>.
	 *
	 * @return  an int -- the model's maximum
	 * @see     #setMaximum
	 */
	public int getMaximum() { return max; }

	/**
	 * Sets the model's current value to <I>x</I>.
	 * The underlying BoundedRangeModel will handle any mathematical
	 * issues arrising from assigning faulty values.
	 *
	 * @param   n       the new value
	 * @see     #getValue
	 */
	public void setValue(int n)
	{
		if (n<min) n = min;
		else
		if (n>max) n = max;
		value = n;
		repaintNow();
	}

	/**
	 * Sets the model's minimum to <I>n</I>.
	 *
	 * @param  n       the new minimum
	 * @see    #getMinimum
	 */
	public void setMinimum(int n) { min = n; }

	/**
	 * Sets the model's maximum to <I>n</I>.
	 * @param  x       the new maximum
	 * @see    #getMaximum
	 */
	public void setMaximum(int n) { max = n; }

	public void onPaint(Graphics g)
	{
		g.setForeColor(foreColor); // rogerio@341_13
		g.setBackColor(backColor); // rogerio@341_13
		g.fillRect(0, 0, width, height);
		g.drawRect(0, 0, width, height);

		int pc = (int)Math.round(width * getPercentComplete()); // danjones@340_26: changed 100 by width
		if (value > min)
		{
         g.setBackColor(fillColor); // rogerio@341_13
			g.fillRect(1, 1, pc-1, height-1);
		}

		g.setForeColor(textColor);
		String st = getString();
      g.drawText(st, (width-fm.getTextWidth(st))/2, (height-fmH)/2); // rogerio@340_27
	}
}
