// Copyright Guilherme Campos Hazan (c) 2000-2003

package superwaba.ext.xplat.ui;

import waba.fx.*;
import waba.util.*;
import waba.ui.*;
import waba.sys.*;

/** Creates a control with two arrows, so you can scroll values and show
  * the current one. */

public class SpinList extends Control
{
   String []choices;
   int selected=0;
   Timer timer;
   public int timerInterval=300;
   boolean goingUp;

   public SpinList(String[] choices, int selected)
   {
      int ini;
      if (choices == null) choices = new String[]{""};
      else
      {
         Vector v = new Vector(choices.length+10);
         for (int i =0; i < choices.length; i++)
            if ((ini=choices[i].indexOf('[')) != -1)
            {
               int fim = choices[i].indexOf(']');
               String prefix = choices[i].substring(0,ini);
               String sufix  = choices[i].substring(fim+1);
               int j;
               int start = Convert.toInt(choices[i].substring(ini+1,j=choices[i].indexOf(',',ini+1)));
               int end   = Convert.toInt(choices[i].substring(j+1,fim));
               for (int k =start; k <= end; k++)
                  v.add(prefix+k+sufix);
            }
            else v.add(choices[i]);
         if (choices.length != v.getCount())
         {
            // choices = (String[])v.toObjectArray();
            // was getting a ClassCastException
            choices = new String[v.getCount()];
            for (int i =0; i < choices.length; i++)
               choices[i] = (String)v.get(i);
         }
      }
      this.choices = choices;
      this.selected = selected;
   }
   public int getPreferredWidth()
   {
      int w=0;
      for (int i =0; i < choices.length; i++)
         w = Math.max(w,fm.getTextWidth(choices[i]));
      if (w == 0) w = 30;
      return w+10;
   }

   public void setChoices(String []choices)
   {
   	this.choices = choices;
      selected = 0;
      repaint();
   }
   public String[] getChoices()
   {
      String []ch = new String[choices.length];
      Vm.copyArray(choices,0,ch,0,choices.length);
   	return ch;
   }

   public String getSelected()
   {
      return choices[selected];
   }

   public int getSelectedIndex()
   {
      return selected;
   }

	public void setSelectedIndex(int i)
	{
  	   if (0 <= i && i < choices.length)
  	   {
		   selected = i;
		   repaint();
		}
	}
	public String removeAt(int index)
	{
   	String ret = choices[index];
   	int last = choices.length-1;
   	String []ch = new String[last];
		Vm.copyArray(choices,0,ch,0,index);
      if (index < last)
      	Vm.copyArray(choices,index+1,ch,index,last-index);
      else
         selected--;
      this.choices = ch;
      repaint();
      return ret;
	}
   public String removeCurrent()
   {
   	return removeAt(selected);
   }
   public int compareTo(String s1, String s2)
   {
   	int len1 = s1.length();
   	int len2 = s2.length();
   	int n = (len1<len2)?len1:len2;
   	int i = 0;

   	while (n-- != 0)
   	{
   	   char c1 = s1.charAt(i);
   	   char c2 = s2.charAt(i++);
   	   if (c1 != c2)
   		   return c1 - c2;
   	}
	   return len1 - len2;
   }
   public int indexOf(String elem)
   {
      for (int i = 0; i < choices.length; i++)
         if (choices[i].equals(elem))
            return i;
      return -1;
   }
	public void insertInOrder(String elem)
	{
   	// obtm a posicao a inserir
      int index = 0;
      while (index < choices.length && compareTo(elem,choices[index]) > 0)
         index++;
      if (index == choices.length || !elem.equals(choices[index]))
      {
	      String []ch = new String[choices.length+1];
	      Vm.copyArray(choices,0,ch,0,index);
	      ch[index] = elem;
	      if (index < choices.length)
	      	Vm.copyArray(choices,index,ch,index+1,choices.length-index);
	      choices = ch;
	      selected = index;
	      repaint();
      }
	}

   public void onPaint(Graphics g)
   {
      g.setBackColor(getBackColor()); // guich@341_3
      g.fillRect(0,0,width,height);
      g.setForeColor(getForeColor());
      int yoff = (height - fm.getHeight()) / 2 + 1;
     	if (choices.length > 0) g.drawText(choices[selected],10,yoff-1);
      for (int i =0; i <= 3; i++)
      {
      	g.drawLine(3-i,yoff+i,4+i,yoff+i);
         g.drawLine(0+i,yoff+5+i,7-i,yoff+5+i);
      }
   }
   private void scroll(boolean up)
   {
      int max = choices.length-1;
  	   if (up)
      {
   	   selected--;
	      if (selected < 0) selected = max;
      }
      else
      {
         selected++;
	      if (selected > max) selected = 0;
      }
      repaint();
      postEvent(new ControlEvent(ControlEvent.PRESSED,this));
   }
   int tick;
   public void onEvent(Event event)
   {
      switch (event.type)
      {
         case KeyEvent.KEY_PRESS:
            int key = ((KeyEvent)event).key;
            if (key == ' ') selected = 0; else
            if (key == IKeys.PAGE_UP)
               scroll(true); else
            if (key == IKeys.PAGE_DOWN)
               scroll(false);
            else
            {
               if ('a' <= key && key <= 'z') // converte pra maiuscula
                  key -= 32;
               for (int i =0; i < choices.length; i++)
                  if (choices[i].charAt(0) == (char)key)
                  {
                     selected = i;
                     repaint();
                     break;
                  }
            }
            break;
	      case PenEvent.PEN_DOWN:
   	      if (((PenEvent)event).x < 8)
      	   {
        	 	   goingUp = (((PenEvent)event).y) > height /2;
            	scroll(goingUp);
            }
				if (timer == null)
            {
	            tick = 0;
					timer = addTimer(timerInterval);
            }
            break;
	      case PenEvent.PEN_UP:
       		if (timer != null)
       	 	{
					removeTimer(timer);
					timer = null;
				}
	 	   	break;
 		   case ControlEvent.TIMER:
	      	if (tick++ > 3) scroll(goingUp);
   	}
   }
}