package superwaba.ext.xplat.ui;

import waba.fx.*;
import waba.ui.*;
import waba.sys.Settings;
import waba.sys.Convert;

/**
* <PRE>
* ToolTip : Allows to display a tooltip when user select control. The
control must have
* been added to the container, otherwise you must call <code>added</code>
method so
* the ToolTip can initialize itself.
* ----------------------------------------------
* Exemple :<bR>
* Button b = new Button("a button");<br>
* add(b);<br>
* b.setRect(LEFT, TOP, PREFERRED, PREFFERED);<br>
* ToolTip toolTip = new ToolTip(b, "Hi, it's a button");<br>
* <bR>
* You can write a multi lines tooltip using the char '|' in your message<br>
* Exemple : ToolTip tip = new ToolTip(control, "Hi! | It's Me");
* ----------------------------------------------
* </PRE>
* @version : 1.0 - 20/06/2003
* @author : Arnaud FARINE - http://www.superwaba-france.com
**/
public class ToolTip extends Control
{
   private final int SHOWING = 0;
   private final int HIDDEN = -1;
   private Control control; // The control which supports the tip
   private Container container;
   private  String text; // The message to display

   private int posX; // Position x
   private int posY; // position y
   private int w; // width
   private int h; // height

   private int stage = HIDDEN; // The stage to paint or not the tip

   private Color borderColor; // the color around the rect

   int tX, tY;

   Timer delay;
   int millisDelay = 1000;
   Timer display;
   int millisDisplay = 2000;
   int nbLine = 0; // Text is larger than screen width
   Event eventSave;
   String[] textToWrite;
   /**
   * Constructor
   * @param Control : the control which supports the tip
   * @param message : the message which will be written in the tip. You can
   * made multiLine, using | character like a Label control.
   **/
   public ToolTip(Control control, String message)
   {
      this.control = control;
      if (message!=null && message.length()>0)
      {
         textToWrite = Convert.tokenizeString(message,'|');
         if (textToWrite.length==0)
            textToWrite = new String[]{message};
      }
      else textToWrite = new String[]{"ToolTip"};

      nbLine = textToWrite.length;
      added();
   }

   public void added()
   {
      container = control.getParent();
      if (container == null)
         return;

      container.add(this);
      Rect r = control.getRect();
      setRect(r);

      tX = Settings.screenWidth/160;
      tY = Settings.screenHeight/160;

      posX = r.x;
      posY = r.y + (r.height+2*tY);

      w = getMaxWidth() + 4 * tX;
      h = fmH * (nbLine+1*tY); //@afe 19062003 - To be more compatible for all resolution

      Rect rp = container.getRect();
      if (posX+w+rp.x > Settings.screenWidth) // @afe 17062003 - If position + width>screen size.
         posX = Settings.screenWidth - w;

     if (posY+h+rp.y  > Settings.screenHeight)// @afe 19062003 - If position + height>screen size.
         posY = Settings.screenHeight - h;

      delay = new Timer();
      display = new Timer();
   }

   private int getMaxWidth()
   {
      int tmpMax = -1;
      for (int i = nbLine; --i >= 0;)
      {
         int lengthCurrent = fm.getTextWidth(textToWrite[i]);
         if (lengthCurrent > tmpMax)
            tmpMax = lengthCurrent;
      }
      return tmpMax;
   }

   /**
   * Delay before to display the tooltip - Default 1000
   * @param int : millisDelay - Time in milliseconds
   **/
   public void setDelay(int millisDelay)
   {
      this.millisDelay = millisDelay;
   }

   /**
   * Time during which the message is diplayed - Default 2000
   * @param int : millisDisplay - Time in milliseconds
   **/
   public void setDisplay(int millisDisplay)
   {
      this.millisDisplay = millisDisplay;
   }

   /**
   * By the system to produce the draw the tip
   * @param Event : e
   **/
   public void onEvent(Event e)
   {
      control.onEvent(e);

      if (e.type==PenEvent.PEN_DOWN && stage == HIDDEN)
      {
         delay = addTimer(millisDelay);
         stage = SHOWING;
      } else
      if (delay.triggered && stage == SHOWING)
      {
         drawMessage();
         removeTimer(delay);
         display = addTimer(millisDisplay);
      } else
      if (e.type == PenEvent.PEN_UP)
      {
         removeTimer(display);
         removeTimer(delay);
         stage=HIDDEN;
          getParentWindow().repaint(); // to force delete the message of the screen (if message is bigger than the Container.
      } else
      if (display.triggered)
      {
         removeTimer(display);
         stage = HIDDEN; // guich: changed from 1 to -1
         getParentWindow().repaint();//@afe : 19062003 to force delete the message of the screen (if message is bigger than the Container.
      }
   }

   /**
   * Set the border color for the tip. By default there's no border color.
   **/
   public void setBorderColor(Color borderColor)
   {
      this.borderColor = borderColor;
   }

   /**
   * Draw the tooltip
   **/
   private void drawMessage()
   {
      Graphics g = container.createGraphics();

      g.setBackColor(backColor);
      g.fillRoundRect(posX, posY, w, h, 4*tX);

      //@afe18062003 : invert order drawRoundRect with fillRoundRect to draw and see the border
      if (borderColor != null)
      {
         g.setForeColor(borderColor);
         g.drawRoundRect(posX, posY, w, h, 4*tX);
      }

      g.setForeColor(foreColor);
      int yy = posY + tY;
      for (int i=0; i < nbLine; i++, yy += fmH+tY)
         g.drawText(textToWrite[i],posX+1*tX, yy);
   }
}
         