package superwaba.ext.xplat.util.datergf;

// import Date.Date;

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;


/**
 The Calendar class is a mimic of the palms Calendar.  It looks and functions the basically the same.
 It uses the Date class for all operations.<br>
 Guich made some modifications to it act like an Window. Instead of create a new instance (which consumes memory), you may
 use the Edit's static field <i>calendar</i>.

 <b>Added by Allan C. Solomon</b>
 @version 1.0 16 Aug 2000
 @author Allan C. Solomon

 <p>[rgf, hint: to cancel (close without selection) the popup calendar press the soft-keys 'abc' or '123' in the
    Graffiti area.

    <br>Intercept the ControlEvent.WINDOW_CLOSED targeted at the appropriate
        CalendarRGF control and retrieve the selected Date with
        {@link #getSelectedDate()}; if the calendar popup was cancelled, then
        this method returns <code>null</code>.
    ]

 <p>Additions/changes from Rony (2001-04-03...2001-04-05):
 <ul>
 <li>working with Date.Date instead of waba.util.Date,

 <li>allows for localizing the chars for the names of the abbreviated weekdays
     using Date's present monthnames,

 <li>allows for localizing the beginning of the week using Date's present
     setting,

 <li>if in/decreasing months and the day was on the last day of month and the
     new month has less days, then the last day of the new month is set.
 </ul>


 <p><em>Please note:</em> if displaying a month in which the switch to the
    Gregorian calendar occurred, then <em>all</em> days are shown. The reason:
    it is unlikely that anybody might really need this particular feature. And
    if so, then just change the code here-in to reflect this.
    (Or you can convince me that adding this feature is really very important
    and not just bloating the code).
    <br>
    Well, today, I convinced myself to support this special case...
    <br>;==)
    <br>cf. <em>TestCalRGF_w.java</em>

 <p>author of these changes: Rony G. Flatscher, 2001-04-07; adapted to Date, allow for internationalization
*/

public class Calendar extends Window
{

//  static boolean onJava= (waba.sys.Vm.getPlatform().equals("Java"));

 /**
  * Field to contain the selected {@link superwaba.ext.xplat.util.datergf.Date} date or <code>null</code>.
  */
 private Date dSel = null;
 private Date dTmp = null;   // contains the Date of the first day of that month

 /** Holds the ordinal number of the first day of the month, which depends
   * on the day which begins a week (e.g. Sunday in the U.S., but Monday in Austria/Europe).
   */
 private int fdOrd   = 0;       // DOW_ORDINAL of first day of month

 /** Holds the maximum number of days for that month. */
 private int maxDays = 0;       // maximum days in month

 /** Indicates if date is in a month in which the change to the Gregorian calendar occurred. */
 private boolean isGCD=false;
 private int lJCD=0, fGCD=0, diff=0;

 /** First day of Gregorian calendar usage. */
 static private Date dGCD=Date.getGregorianChange();

 /** Last day of Julian calendar, if in month in which the change to the Gregorian calendar occurred. */


 private final int CELLW = 16;
 private final int CELLH = 16;
 private final int CELLX0 = 6;
 private final int CELLY0 = 28;


/**
 Constructs Calendar set to the current day.
*/
 public Calendar()
 {
   super("",RECT_BORDER); // no title yet

   setDoubleBuffer(true);
   setRect(CENTER,CENTER,CELLX0*2+CELLW*7,CELLY0+CELLX0+CELLH*6); // same gap in all corners


//   if (!onJava) waba.sys.Vm.setTrace(false, 100, 20, 8);
 }


 private void drawArrow(Graphics g, int x, int y, boolean leftArrow, boolean enabled)
 {
        if (enabled)
           g.setForeColor(Color.WHITE);
        else
           g.setForeColor(Color.DARK);
        y += 5;
        if (!leftArrow) x += 5;
        for (int i = 1; i <= 6; i++, x += (leftArrow?1:-1))
        {
           int r = i*2-1;
           if (enabled)
                  g.drawLine(x,y-r/2,x,y+r/2);
           else
                  g.drawDots(x,y-r/2,x,y+r/2);
        }
 }


 public void onEvent(Event event)
 {
   int rightArrowX = width-8;
   int leftArrowX  = width-8-8-4;

   // closes this window without selecting any day
   if (   event.type == KeyEvent.KEY_PRESS          &&
       (((KeyEvent)event).key == IKeys.KEYBOARD_ABC ||
        ((KeyEvent)event).key == IKeys.KEYBOARD_123)
      )
   {
          dSel=null;    // indicate no valid selection
          unpop();
   }
   else

  if (event.type == PenEvent.PEN_DOWN)
   {
        PenEvent penEvent = (PenEvent)event;
        if ((penEvent.y >=CELLY0) && ((penEvent.x >=CELLX0)&&(penEvent.x <= (width-CELLX0*2)))) // in dates?
        {
           int col = ((penEvent.x -CELLX0) / CELLW);
           int row = ((penEvent.y-CELLY0) / CELLH);
           int d = (row * 7) - fdOrd + (col+1) + 1;

           if (d >0 && d <= maxDays)      // on a valid day? Hilight!
           {
               Graphics g = createGraphics();
               g.fillRect(CELLX0 +(col * CELLW),CELLY0+(row * CELLH),CELLW,CELLH);
               g.setForeColor(Color.WHITE);
                        // take care of Gregorian calendar change month
               g.drawText(Convert.toString( ((isGCD&&d>lJCD) ? d+diff : d ) ),CELLX0+2 +(col * CELLW),CELLY0+(row * CELLH));
           }
        }
        else if  (penEvent.y <=13) // in title?
        {
           Graphics g = createGraphics();
           if ((penEvent.x >= leftArrowX) && (penEvent.x <= leftArrowX+6))
           {
               drawArrow(g,leftArrowX,1,true,false); // g.drawImage(gleftArrow,140,1);
           }
           if ((penEvent.x >=rightArrowX) && (penEvent.x <= rightArrowX+6))
           {
               drawArrow(g,rightArrowX,1,false,false); // g.drawImage(grightArrow,153,1);
           }
        }
   }

  else if (event.type == PenEvent.PEN_UP)
   {
        PenEvent penEvent = (PenEvent)event;
        if (penEvent.y <=13)            // month-arrow was pressed
         {
             if ((penEvent.x >=leftArrowX) && (penEvent.x <=leftArrowX+6))
             {

                dTmp.set(DTC.MONTH_FIELD,(dTmp.get(DTC.MONTH_FIELD) - 1));
             }
             else if ((penEvent.x >=rightArrowX) && (penEvent.x <=rightArrowX+6))
             {
                dTmp.set(DTC.MONTH_FIELD,
                         (dTmp.get(DTC.MONTH_FIELD) + 1));

             }
                // month changed, get ordinal day of week of first day and total days of month
             fdOrd  =dTmp.get(DTC.DOW_ORDINAL);       // get ordinal day of week of first day of month
             maxDays=dTmp.get(DTC.DAYS_IN_MONTH);     // get maximum days in month

             int d=dSel.get(DTC.DAY_FIELD); // get old day

             dSel.assign(dTmp);
             if (d != 1)        // has day to be changed?
             {
                // if new month has less days then the previous, set last day of new month
                dSel.set(DTC.DAY_FIELD, ( d > maxDays ? maxDays : d) );
             }

                 // month in which change to Gregorian calendar occurred?
             isGCD = (
                      (dGCD.get(DTC.YEAR_FIELD ) == dSel.get(DTC.YEAR_FIELD)  ) &&
                      (dGCD.get(DTC.MONTH_FIELD) == dSel.get(DTC.MONTH_FIELD) )
                    );

             repaint();

         }
        else if ((penEvent.y >=CELLY0) && ((penEvent.x >=CELLX0)&&(penEvent.x <= (width-CELLX0*2))))
        {
                 int row = ((penEvent.y-CELLY0) / CELLH);
                 int col = ((penEvent.x -CELLX0) / CELLW)+1;


                 int d = (row * 7) - fdOrd + (col+1);


                 if (d <1 || d > maxDays)      // on a valid day? Hilight!
                 {
                    repaint();
                    return;
                 }
                        // take care of Gregorian calendar change month
                 dSel.set(DTC.DAY_FIELD, ((isGCD && d>lJCD) ? d+diff : d)) ;    // set to chosen day

                 unpop(); // closes this window
        }
   }
 }


 public void onPaint(Graphics g)
 {
    //Draw title with appropriote month and year
    paintTitle(Date.getString(DTC.MN, dTmp.get(DTC.MONTH_FIELD))
               + " "
               + dTmp.get(DTC.YEAR_FIELD),
               g
              );

    //Draw arrows
    int rightArrowX = width-8;
    int leftArrowX = width-8-8-4;
    drawArrow(g,leftArrowX,1,true,true); // g.drawImage(leftArrow,140,1);
    drawArrow(g,rightArrowX,1,false,true); // g.drawImage(rightArrow,153,1);


    //Draw day of week labels
    int weekStart=Date.getStatic(DTC.WEEK_START);

    String[] s = new String [8];
    {   // create heading; this uses Date daynames, which could be localized
        int i=weekStart; // ISO day number
        int j=0;

        for ( ; j<7; i++, j++)
        {
           if (i>7) i=1;
           s[j]=Date.getString(DTC.DN, i).substring(0,1);
        }
    }

    int xPos = CELLX0;
    int yPos = CELLY0-1;

    FontMetrics fm = getFontMetrics(new Font("Helvetica", Font.BOLD, 14));
    g.setForeColor(Color.BLACK);

    for(int x = 0;x<s.length;x++)
    {
          g.drawText(s[x],xPos+((CELLW-fm.getTextWidth(s[x]))/2),CELLY0/2);
          xPos+=CELLW;
    }
    g.setFont(new Font("Helvetica", Font.PLAIN, 12));



    //Draw horz. lines
    for(int i=0;i<7;i++)
    {
          g.drawLine(CELLX0,yPos,CELLX0+7*CELLW,yPos);
          yPos+=CELLH;
    }

    //Draw Vert. lines
    xPos = CELLX0;
    for(int i=0;i<8;i++)
    {
          g.drawLine(xPos,CELLY0-1,xPos,6*CELLH+CELLY0-1);
          xPos+=CELLW;
    }

    //Draw dates out
    int cPos = fdOrd;

    g.setColors(Color.BLACK);
    yPos = CELLY0;
    xPos = CELLX0+2 + (CELLW*(cPos-1));

    {   // supporting the month in which the change to the Gregorian calendar occurred
       lJCD=fGCD=diff=0;

       if (isGCD)               // o.k. in this month, draw the numbers correctly!
       {
          fGCD=dGCD.get(DTC.DAY_FIELD); // get first day of Gregorian calendar

          lJCD=(                // get last day of Julian calendar
                 ( (Date)dGCD.copy() ).add(-1).get(DTC.DAY_FIELD)
               );
          diff=fGCD-lJCD-1;     // get difference in days
       }

       for(int i=1;i<=maxDays;i++)      // paint days
       {
          if (isGCD)            // month of change to Gregorian calendar?
          {
             g.drawText(Convert.toString( (i>lJCD ? i+diff : i) ), xPos, yPos);
          }
          else
             g.drawText(Convert.toString(i), xPos, yPos);

          if (cPos%7==0){yPos+=CELLH;xPos=CELLX0+2;cPos=1;}
          else {xPos+=CELLW;cPos++;}
       }

             // highlight day in calendar
       {
          int   d = dSel.get(DTC.DAY_FIELD);
          int col = dSel.get(DTC.DOW_ORDINAL) - 1;

          int d_pos = ( // account for missing days in Gregorian change month
                        (isGCD && d>lJCD) ? (d-diff) : d
                      );
          int row = (d_pos + fdOrd - col - 1) / 7;

                // with of black filled rectangle depends on number of digits to display
          if (d >9) {g.fillRect(CELLX0 +(col * CELLW),CELLY0+(row * CELLH),12,11);}
          else      {g.fillRect(CELLX0 +(col * CELLW),CELLY0+(row * CELLH),10,11);}

          g.setForeColor(Color.WHITE);
          g.drawText(Convert.toString(d),CELLX0+2+(col*CELLW),CELLY0+(row*CELLH));
       }
    }

    g.setFont(new Font("Helvetica", Font.PLAIN, 12));
 }



 protected void onPopup()
 {
    if (dSel==null)          // no selected Date so far?
       setSelectedDate(null);

    setTitle(Date.getString(DTC.MN, dSel.get(DTC.MONTH_FIELD))
             + " " + dSel.get(DTC.YEAR_FIELD) );
 }



/**
 Returns selected Date.

 @return Date object set to the selected day. <code>null</code>,
          if calendar popup was cancelled.
*/
 public Date getSelectedDate()
 {
     return dSel;
 }


 /** Sets the selection Date to the Date specified.
     If it is <code>null</code>, sets the date to today.

     @param  d a Date object or <code>null</code>.
     @return the Date object this CalendarRGF was set to.
*/
 public Date setSelectedDate(Date d)
 {
   if (d==null)         // no object received, use present local day
   {
      dSel = new Date().update();  // create a new Date, set to the local date of the machine
   }
   else
   {
      if (dSel==null) dSel=new Date();       // no object as of yet, create one
      dSel.assign(d);
   }

   dTmp=((Date)dSel.copy()).set(DTC.DAY_FIELD,1); // set clone to first day of month

   fdOrd  =dTmp.get(DTC.DOW_ORDINAL);       // get ordinal day of week of first day of month
   maxDays=dTmp.get(DTC.DAYS_IN_MONTH);     // get maximum days in month

       // month in which change to Gregorian calendar occurred?
   isGCD= (
            (dGCD.get(DTC.YEAR_FIELD ) == dSel.get(DTC.YEAR_FIELD) ) &&
            (dGCD.get(DTC.MONTH_FIELD) == dSel.get(DTC.MONTH_FIELD))
          );

   return dSel;
 }


/* --->
  static void println(String aha)
  {
     if (onJava) System.err.println(aha);
     else        waba.sys.Vm.debug(aha, 100);
  }
<--- */

}
