package superwaba.ext.xplat.util.props;

import waba.io.*;
import waba.util.*;

/** Class used to store properties pairs (key,value). A hashtable is used
 * to store them. Currently, the key must be a String and the value must be
 * a Value (Properties.Str, Properties.Int, Properties.Double,
 * Properties.Long, Properties.Boolean.
 * The properties can be saved and loaded to/from a DataStream.
 */
public class Properties
{
  private Hashtable props;

  public Properties()
  {
    props = new Hashtable(20);
  }

  /** Stores the given keys/values pairs in a new Properties. */
  public Properties(String []keys, Value []values)
  {
    int n = keys.length;
    props = new Hashtable(n<<1);
    for (int i = 0; i < n; i++)
      props.put(keys[i], values[i]);
  }

  /** Represents a generic value that can be stored here. */
  public static class Value
  {
     /** read-only property */
     public char type;
  }
  /** Implements a value of type String */
  public static class Str extends Value
  {
    public final static char TYPE='S';
    public String value;
    public Str(String value) {this.value = value; type = TYPE;}
  }
  /** Implements a value of type int */
  public static class Int extends Value
  {
    public final static char TYPE='I';
    public int value;
    public Int(int value)    {this.value = value; type = TYPE;}
  }
  /** Implements a value of type double */
  public static class Double extends Value
  {
    public final static char TYPE='D';
    public double value;
    public Double(double value) {this.value = value; type = TYPE;}
  }
  /** Implements a value of type boolean */
  public static class Boolean extends Value
  {
    public final static char TYPE='B';
    public boolean value;
    public Boolean(boolean value) {this.value = value; type = TYPE;}
  }
  /** Implements a value of type long */
  public static class Long extends Value
  {
    public final static char TYPE='L';
    public long value;
    public Long(long value)  {this.value = value; type = TYPE;}
  }

  /** Put the given key/value pair in the hashtable that stores the properties. */
  public void put(String key, Value v)
  {
    props.put(key,v);
  }
  /** Get the value given the key from the hashtable that stores the properties. */
  public Value get(String key)
  {
    return (Value)props.get(key);
  }
  /** Returns the number of properties */
  public int size()
  {
    return props.size();
  }
  /** Returns a Vector with the current keys */
  public Vector getKeys()
  {
    return props.getKeys();
  }
  /** Clears this property */
  public void clear()
  {
    props.clear();
  }
  /** Remove a value from the property */
  public void remove(String key)
  {
    props.remove(key);
  }
  /** Save all properties in the given DataStream */
  public void save(DataStream ds)
  {
    // get all keys, store everything in database
    Vector vec = props.getKeys();
    int n = vec.size();
    for (int i =0; i < n; i++)
    {
      String key = (String)vec.items[i];
      Value v = (Value)props.get(key);
      byte type = (byte)v.type;
      ds.writeByte(type);
      ds.writeString(key);
      switch (type)
      {
        case Str.TYPE:     ds.writeString(((Str)v).value); break;
        case Int.TYPE:     ds.writeInt(((Int)v).value); break;
        case Double.TYPE:  ds.writeDouble(((Double)v).value); break;
        case Boolean.TYPE: ds.writeBoolean(((Boolean)v).value); break;
        case Long.TYPE:    ds.writeLong(((Long)v).value); break;
      }
    }
    ds.writeByte(255);
  }

  /** Load all properties from the given DataStream. */
  public void load(DataStream ds)
  {
    props.clear();
    // read and populate the options hashtable from pdb
    while (true)
    {
      byte type = ds.readByte();
      if (type == -1) break;
      String key = ds.readString();
      Value v=null;
      switch (type)
      {
        case Str.TYPE:     v = new Str(ds.readString()); break;
        case Int.TYPE:     v = new Int(ds.readInt()); break;
        case Double.TYPE:  v = new Double(ds.readDouble()); break;
        case Boolean.TYPE: v = new Boolean(ds.readBoolean()); break;
        case Long.TYPE:    v = new Long(ds.readLong()); break;
      }
      if (v != null)
         props.put(key, v);
    }
  }
}