/*
 * The contents of this file are subject to the "Lesser GNU Public License"
 * (LGPL); you may not use this file except in compliance with the License.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 *
 */

package superwaba.ext.xplat.util.xml;

/** Listener that must be extended to process the events thrown by HttpLiteParser.
  */
public abstract class HttpListener
{
   /** Buffer that stores the contents for the current tag/attribute.
     * Every time a method is called, the contents of this buffer from
     * 0 to the given len stores the current attribute/tag.
     */
   public byte []buffer;
   /** Current length of the string */
   public int len;
   /** Starting position of the current string in the buffer */
   public int start;

    /**
     * This is the method called by parser when the HTTP version is found
     */
    public abstract void foundVersion();
    
    /**
     * This is the method called by parser when HTTP response code is found
     */
    public abstract void foundCode();
    
    /**
     * This is the method called by parser when HTTP response reason is found
     */
    public abstract void foundReason();
    
    /**
     * This is the method called by parser when A HTTP Header name is found
     */
    public abstract void foundHttpHeader();
    
    /**
     * This is the method called by parser when a HTTP Header value is found
     */
    public abstract void foundHttpValue();
}
