/*
 * The contents of this file are subject to the "Lesser GNU Public License"
 * (LGPL); you may not use this file except in compliance with the License.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 *
 */

package superwaba.ext.xplat.util.xml;

import waba.io.Stream;

/**
 * <p>This is a fast and simple parser for a lite version of XML within a
 * HTTP packet.
 * You can use this parser with SuperWaba VM to parse data from an open
 * Stream from  HTTP server.</p>
 *
 * @deprecated  It is better to user HttpLiteParser and XmlListParser separately.
 */
public class HttpXmlLiteParser
{
    protected HttpXmlListener httpXmlListener;
    protected byte eol=0;


    /** Constructor.
     * @param listener
     */
    public HttpXmlLiteParser(HttpXmlListener listener)
    {
        httpXmlListener=listener;
    }

    /**
     * This method parses the bytes from the passed stream into HTTP and XML callbacks.
     * The algorithm is based on DFA (Deterministic Finite Automata) theory and
     * Regular Expression theory.
     * @param input Input stream.
     * @param bufferSize The bufer size to use. No tag or content can be
     * larger than the buffer.
     * @exception ParserException
     */
    public void parse(Stream input,int bufferSize) throws ParserException
    {
       // Create buffer
       StreamBuffer sbuf = new StreamBuffer(input,bufferSize);
       parse(sbuf);
    }

    /**
     * This method parses the bytes from the passed stream buffer into HTTP and XML callbacks.
     * The algorithm is based on DFA (Deterministic Finite Automata) theory and
     * Regular Expression theory.
     * @param sbuff A buffer that can be refilled from a stream
     * @exception ParserException
     */
    public void parse(StreamBuffer sbuf) throws ParserException
    {
        HttpLiteParser httpParser = new HttpLiteParser(new Http2HttpXml());
        httpParser.parse(sbuf);
        if (httpParser.state==0)
        {
            XmlLiteParser xmlParser = new XmlLiteParser(httpXmlListener);
            xmlParser.parse(sbuf);
        }
    }

    /**
     * This method parses the bytes from the passed byte array into HTTP and XML callbacks.
     * The algorithm is based on DFA (Deterministic Finite Automata) theory and
     * Regular Expression theory.
     * @param size
     * @exception ParserException
     */
    public void parse(byte []input,  int size) throws ParserException
    {
        HttpLiteParser httpParser = new HttpLiteParser(new Http2HttpXml());
        httpParser.parse(input,size);
        if (httpParser.state==0)
        {
            XmlLiteParser xmlParser = new XmlLiteParser(httpXmlListener);
            xmlParser.pos=httpParser.pos;
            xmlParser.state=0;
            xmlParser.parseBuffer(input,size);
        }
    }


    /**
     * This HttpListener converts HttpListener callbacks to HttpXmlCallbacks.
     */
    private class Http2HttpXml extends HttpListener
    {
        /**
         * This is the method called by parser when the HTTP version is found.
         * Delegates from HttpListener to HttpXmlListener.
         */
        public void foundVersion()
        {
            httpXmlListener.buffer=buffer;
            httpXmlListener.start=start;
            httpXmlListener.len=len;
            httpXmlListener.foundVersion();
        }

        /**
         * This is the method called by parser when HTTP response code is found.
         * Delegates from HttpListener to HttpXmlListener.
         */
        public void foundCode()
        {
            httpXmlListener.buffer=buffer;
            httpXmlListener.start=start;
            httpXmlListener.len=len;
            httpXmlListener.foundCode();
        }

        /**
         * This is the method called by parser when HTTP response reason is found.
         * Delegates from HttpListener to HttpXmlListener.
         */
        public void foundReason()
        {
            httpXmlListener.buffer=buffer;
            httpXmlListener.start=start;
            httpXmlListener.len=len;
            httpXmlListener.foundReason();
        }

        /**
         * This is the method called by parser when A HTTP Header name is found.
         * Delegates from HttpListener to HttpXmlListener.
         */
        public void foundHttpHeader()
        {
            httpXmlListener.buffer=buffer;
            httpXmlListener.start=start;
            httpXmlListener.len=len;
            httpXmlListener.foundHttpHeader();
        }

        /**
         * This is the method called by parser when a HTTP Header value is found.
         * Delegates from HttpListener to HttpXmlListener.
         */
        public void foundHttpValue()
        {
            httpXmlListener.buffer=buffer;
            httpXmlListener.start=start;
            httpXmlListener.len=len;
            httpXmlListener.foundHttpValue();
        }
    }
}
