/*
 * XmlLiteParser version 1.0
 * Created on September 26, 2002
 *
 * Copyright (C) 2002
 * Giovanni Remigi
 * via Procche n.28 36030 Zugliano (Vicenza),
 * Italy. All Rights Reserved.
 *
 * The contents of this file are subject to the "Lesser GNU Public License"
 * (LGPL); you may not use this file except in compliance with the License.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 *
 * Thanks to Fabiano Dal Castello for providing various fixes and hints.
 *
 * Reestructured and optimized by Guich for SuperWaba 3.3.
 */

package superwaba.ext.xplat.util.xml;

/**
 * <p>This is a fast and simple parser for a lite version of XML format. You can
 * use this parser with SuperWaba VM to parse data from an open Stream.</p>
 * <p>The parser looks for:</p>
 * <ul>
 * <li>start-tag name,</li>
 * <li>end-tag name,</li>
 * <li>attribute name,</li>
 * <li>attribute value,</li>
 * <li>content between a start-tag and an end-tag.</li>
 * </ul>
 *
 * <p>When the parser finds these elements it calls respectively:</p>
 * <ul>
 * <li><tt>foundStartTagName();</tt></li>
 * <li><tt>foundEndTagName();</tt></li>
 * <li><tt>foundAttributeName();</tt></li>
 * <li><tt>foundAttributeValue();</tt></li>
 * <li><tt>foundContent();</tt></li>
 * </ul>
 *
 * <p>The parsing algorithm is based on DFA (Deterministic Finite Automata)
 * theory and Regular Expression theory. See the following schema: <img src="../../../../../xmldfa.gif">
 * </p>
 *
 * <p>In order to use this parser you need to extend the XmlListener class and
 * override the above methods. To start parsing, call the method
 * parse(byte inputBytes, int length). </p>
 * See also samples/util/xml.
 *
 * @author Giovanni Remigi
 * @email g.remigi@tiscalinet.it
 * @author Greg Wilkins
 * @version 0.01
 */

 // guich@330: incorporated into superwaba.ext and optimized the code.

public class LiteParser
{
   // Terminal symbols.
   static byte COLON = (byte)':';
   static byte UNDERSCORE = (byte)'_';
   static byte HYPHEN = (byte)'-';
   static byte FULL_STOP = (byte)'.';
   static byte QUOTATION_MARK = (byte)'"';
   static byte GREATER_THAN = (byte)'>';
   static byte LESS_THAN = (byte)'<';
   static byte FRACTION_SLASH = (byte)'/';
   static byte EQUAL_SIGN = (byte)'=';
   static byte SPACE = 0x20;
   static byte CARRIAGE_RETURN = 0x0D;
   static byte LINE_FEED = 0x0A;
   static byte TAB = 0x09;

   /** current state */
   protected int state = 0;

   /** pos in buffer */
   protected int pos = 0;
}
