/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package superwaba.ext.xplat.util.xml;

import waba.io.*;
import waba.util.*;
import waba.sys.*;

/** This class adds support for parsing a Xml file splitted into different records of a Catalog.
  * It is useful if your xml is too big to fit in a single record.
  * Important! Each record must be written with DataStream.writeString and it must
  * contain a single String, because the first 2 bytes of each record are skipped
  * (they store the length of the String).
  */

public class RecordsStreamBuffer extends superwaba.ext.xplat.util.xml.StreamBuffer
{
   protected int endRec;
   protected Catalog cat;

   /** The records used in the catalog will be parsed from startRec to endRec */
   public RecordsStreamBuffer(Catalog cat, int size, int startRec, int endRec)
   {
      super(cat,size);
      this.cat = cat;
      this.endRec = endRec;
      // position in first record
      if (!cat.setRecordPos(startRec))
         throw new RuntimeException("Can't setRecordPos("+startRec+")"); // unlike to occur
      else
         cat.skipBytes(2); // skip <length> of string
   }

   /** Called from XmlLiteParser to get more data to parse */
   public int fill()
   {
      int count=0;
      // save data after the mark.
      if (mark>=0 && mark<size)
      {
         Vm.copyArray(buffer,mark,buffer,0,size-mark);
         size -= mark;
         pos  -= mark;
         mark=0;
      }
      // save data after the pos
      else
      if (size>0 && pos<size)
      {
          Vm.copyArray(buffer,pos,buffer,0,size-pos);
          size -= pos;
          pos=0;
          mark=-1;
      }
      else
      {
          size=0;
          pos=0;
          mark=-1;
      }

      // read some more data
      int recsize = cat.getRecordSize();
      int recoff = cat.getRecordOffset();
      int maxRead = Math.min(buffer.length-size,recsize-recoff);
      count=cat.readBytes(buffer,size,maxRead);
      if (count>0)
          size += count;
      else // no more bytes in current record? move to next record
      {
         int p = cat.getRecordPos();
         if (p < endRec) // still has records?
         {
            if (cat.setRecordPos(p+1))
            {
               cat.skipBytes(2); // skip <length> of string
               count=cat.readBytes(buffer,size,buffer.length-size);
               if (count>0)
                  size += count;
            }
         } else return -1;
      }
      return count;
   }
}