/*
 * XmlLiteParser version 1.0
 * Created on September 26, 2002
 *
 * Copyright (C) 2002
 * Giovanni Remigi
 * via Procche n.28 36030 Zugliano (Vicenza),
 * Italy. All Rights Reserved.
 *
 * The contents of this file are subject to the "Lesser GNU Public License"
 * (LGPL); you may not use this file except in compliance with the License.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific terms governing rights and limitations under the License.
 *
 * Thanks to Fabiano Dal Castello for providing various fixes and hints.
 *
 * Reestructured and optimized by Guich for SuperWaba 3.3.
 */

package superwaba.ext.xplat.util.xml;

/** Listener that must be extended to process the events thrown by XmlLiteParser.
  * See also samples/util/xml for an example.
  */

public abstract class XmlListener
{
   /** Buffer that stores the contents for the current tag/attribute.
     * Every time a method is called, the contents of this buffer from
     * 0 to the given len stores the current attribute/tag.
     */
   public byte []buffer;
   /** Current length of the string */
   public int len;
   /** Starting position of the current string in the buffer */
   public int start;

   /**
    * This is the default method called by parser when a start tag name is
    * found.
    */
   public abstract void foundStartTagName();

   /**
    * This is the default method called by parser when an attribute name is
    * found.
    */
   public abstract void foundAttributeName();

   /**
    * This is the default method called by parser when an end tag name is
    * found.
    */
   public abstract void foundEndTagName();

   /**
    * This is the default method called by parser when an attribute value is
    * found.
    */
   public abstract void foundAttributeValue();

   /**
    * This is the default method called by parser when a content text is
    * found.
    */
   public abstract void foundContent();

   /** Returns the current string being represented in this moment.
     * If you want just to check if the current string is equal to a desired one,
     * please use the <code>hashCode</code> method instead.
     */
   public String getString()
   {
      if (start == -1 || len == 0) return "";
      return new String(buffer, start, len);
   }

   /** Returns the hashcode for the string in the current tag.
     * Efficient to check if this is the tag you want without
     * allocating a String and doing an <code>equals</code>.
     * Note that jdk 1.1.x has a different method for computing
     * the hashcode, comparing with the other jdks (and i can't
     * guarantee they won't change it in the future!).
     * This method uses the same one of SuperWaba VM: jdk 1.2.2.
     */
   public int hashCode()
   {
      if (start == -1 || len == 0) return 0;
      int i = 0;
      int j = start;
      byte ac[] = buffer;
      int k = len;
      while (k-- > 0)
         i = (i << 5) - i + ac[j++]; // i*31 = i*32-i
      return i;
   }
}