/**
 * Modified for SuperWaba November 2002 by Alexander Gro
 */

/* -*-mode:java; c-basic-offset:2; -*- */
/* JZlib -- zlib in pure Java
 *
 * Copyright (C) 2000 ymnk, JCraft, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 * This program is based on zlib-1.1.3, so all credit should go authors
 * Jean-loup Gailly(jloup@gzip.org) and Mark Adler(madler@alumni.caltech.edu)
 * and contributors of zlib.
 */

package superwaba.ext.xplat.util.zip;
import waba.sys.*;

final public class JZlib
{
  // compression levels
  static final public int Z_NO_COMPRESSION=0;
  static final public int Z_BEST_SPEED=1;
  static final public int Z_BEST_COMPRESSION=9;
  static final public int Z_DEFAULT_COMPRESSION=(-1);

  // compression strategy
  static final public int Z_FILTERED=1;
  static final public int Z_HUFFMAN_ONLY=2;
  static final public int Z_DEFAULT_STRATEGY=0;

  static final public int Z_NO_FLUSH=0;
  static final public int Z_PARTIAL_FLUSH=1;
  static final public int Z_SYNC_FLUSH=2;
  static final public int Z_FULL_FLUSH=3;
  static final public int Z_FINISH=4;

  static final public int Z_OK=0;
  static final public int Z_STREAM_END=1;
  static final public int Z_NEED_DICT=2;
  static final public int Z_ERRNO=-1;
  static final public int Z_STREAM_ERROR=-2;
  static final public int Z_DATA_ERROR=-3;
  static final public int Z_MEM_ERROR=-4;
  static final public int Z_BUF_ERROR=-5;
  static final public int Z_VERSION_ERROR=-6;

  /** String that contains the inflate/deflate status, already separated using |. */
  public static String status;

    /** Inflates a compressed buffer */
    public static byte[] inflate(byte[] txtbyte) throws Exception
    {
       // error variable
       int err;
       // lenght of compressed data
       int comprLen   = txtbyte.length;
       // lenght of uncompressed data
       int uncomprLen   = comprLen*3;
       // init buffer for uncompressed data
       byte[] uncompr   = new byte[uncomprLen];
       // length of compressed data
       int len       = txtbyte.length;

      // init the zstream object
      ZStream d_stream=new ZStream();

      // set next_in length and data
      d_stream.next_in=txtbyte;
      d_stream.next_in_index=0;
      // set next_out length and data
      d_stream.next_out=uncompr;
      d_stream.next_out_index=0;

      // start inflating with first chunk of data
      err=d_stream.inflateInit(13);
      CHECK_ERR(d_stream, err, "inflateInit");

      int str = Vm.getTimeStamp();

      // inflate the rest of the data
      while(d_stream.total_out < uncomprLen && d_stream.total_in < comprLen)
      {
         d_stream.avail_in = d_stream.avail_out = 1;
         err = d_stream.inflate(Z_NO_FLUSH);
         if(err==Z_STREAM_END) break;
         if (err != Z_OK)
            throw new Exception((d_stream.msg!=null?d_stream.msg:"")+" inflate error: "+err);
      }

      // end inflating
      err = d_stream.inflateEnd();
      CHECK_ERR(d_stream, err, "inflateEnd");

      // init uncompressed data array
      byte[] fin = new byte[(int)d_stream.total_out];
      Vm.copyArray(uncompr, 0, fin, 0, (int)d_stream.total_out);

      int end = Vm.getTimeStamp();

      status = "inflated "+d_stream.total_in+"|bytes to "+d_stream.total_out+" bytes|in "+Vm.getTimeElapsed(end,str)+" millis";

      // free the zstream object
      d_stream.free();

      return fin;
   }

   /** Deflate an uncompressed buffer. The parameter compressionLevel must be 0 (no compression), 1 (min compression) to 9 (max compression) */
   public static byte[] deflate(byte[] txtbyte, int compressionLevel) throws Exception
   {
      int err;
      int uncomprLen = txtbyte.length;
      int comprLen   = uncomprLen;
      byte[] compr = new byte[comprLen];

      int str = Vm.getTimeStamp();

      ZStream c_stream = new ZStream();

      c_stream.next_in=txtbyte;
      c_stream.next_in_index=0;

      c_stream.next_out=compr;
      c_stream.next_out_index=0;

      err=c_stream.deflateInit(compressionLevel, 13);
      CHECK_ERR(c_stream, err, "deflate");

      while(c_stream.total_in != txtbyte.length && c_stream.total_out < comprLen)
      {
         c_stream.avail_in = c_stream.avail_out = txtbyte.length;
         err = c_stream.deflate(Z_NO_FLUSH);
         if (err != Z_OK)
            throw new Exception((c_stream.msg!=null?c_stream.msg:"")+" deflate error: "+err);
      }

      while(true)
      {
         c_stream.avail_out=1;
         err=c_stream.deflate(Z_FINISH);
         if (err==Z_STREAM_END) break;
         if (err != Z_OK)
            throw new Exception((c_stream.msg!=null?c_stream.msg:"")+" deflate error: "+err);
      }

      err=c_stream.deflateEnd();
      CHECK_ERR(c_stream, err, "deflateEnd");

      byte[] fin = new byte[(int)c_stream.total_out];
      Vm.copyArray(compr, 0, fin, 0, (int)c_stream.total_out);

      int end = Vm.getTimeStamp();

      status = "deflated "+c_stream.total_in+"|bytes to "+c_stream.total_out+" bytes|in "+Vm.getTimeElapsed(end, str)+" millis";

      c_stream.free();

      return fin;
    }

   // error checking method
   private static void CHECK_ERR(ZStream z, int err, String msg) throws Exception
   {
      if (err != Z_OK)
      {
         if (z.msg!=null) msg = z.msg+". "+msg;
         throw new Exception(msg+" error: "+err);
      }
   }
}
