/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2001-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 2000 Dave Slaughter                                            *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

/*
 * Note: Everything that calls SuperWaba code in these classes must be
 * synchronized with respect to the Applet uiLock object to allow SuperWaba
 * programs to be single threaded. This is because of the multi-threaded
 * nature of Java and because timers use multiple threads.
 *
 * Because all calls into SuperWaba are synchronized and users can't call this code,
 * they can't deadlock the program in any way. If we moved the synchronization
 * into SuperWaba code, we would have the possibility of deadlock.
 */

import waba.ui.*;
import waba.sys.*;
import waba.fx.Graphics;

public class Applet extends java.applet.Applet implements waba.fx.ISurface
{
   static class Frame extends java.awt.Frame
   {
      public boolean handleEvent(java.awt.Event event)
      {
         if (event.id == java.awt.Event.WINDOW_DESTROY)
         {
            Applet.currentApplet.destroy();
            System.exit(0);
         }
         else
         if (Applet._winCanvas != null && event.id != java.awt.Event.LOST_FOCUS && getFocusOwner() != Applet._winCanvas && !Applet.currentApplet.destroyed) // guich@320_39
            Applet._winCanvas.requestFocus();  // guich@200b4: correct a bug that sometimes key events was not being sent anymore to the canvas.
         return super.handleEvent(event);
      }
   }

   public static Applet currentApplet;
   String className;
   Frame frame = null;
   public MainWindow mainWindow = null;
   public boolean isApplication = false;
   public boolean isColor = false;
   private static final int defaultBPP = 4;
   public static int bpp = defaultBPP; // guich@200b4_58: this must be set before the Applet construction
   public int width;
   public int height;
   public static int appX; //dtauschke
   public static int appY; //dtauschke
   public String dataPath = null; // jeanrissoto@200b4_75: fix the app path problem (changed "" to null)
   public WinTimer _winTimer;
   public String commandLine = "";
   public static String appTitle;

   public static WinCanvas _winCanvas;
   boolean started; // guich@120
   public static boolean toPalm;
   public static Graphics mainG; // guich@200b2: this is the "official" screen

   public boolean destroyed; // guich@230_24
   public void destroy()
   {
      if (mainWindow == null || destroyed)
         return;
      _winTimer.stopGracefully();
      WinCanvas.invokeInEventThread(true, new Runnable()
      {
        public void run()
        {
          mainWindow.appEnding();
          JavaBridge.storeSettings();
        }
      });
      destroyed = true;
   }
   public Applet()
   {
      currentApplet  = this;
  	   try {System.runFinalizersOnExit(true);} catch (Throwable t) {} // guich@300_31
      // guich@200b4: instantiate the classes that will handle the native methods
      //  new waba.sys.NativeMethods4JDK(); // must be initialized first due to Vm. debug
   }
   public void init()
   {
      try
      {
         // print instructions
         System.out.println("Key emulations:");
         System.out.println("F1-F4 : HARD1 to HARD4");
		   System.out.println("F5 : COMMAND");
		   System.out.println("F6 : MENU");
		   System.out.println("F7 : CALC");
		   System.out.println("F8 : FIND");
		   System.out.println("F9 : LAUNCH (HOME)");
		   System.out.println("F10: ACTION (Center button press)"); // guich@400
		   System.out.println("F11: KEYBOARD_ABC");
		   System.out.println("F12: KEYBOARD_123");

         setLayout(new java.awt.BorderLayout());
         // NOTE: getParameter() and size() don't work in a
         // java applet constructor, so we need to call them here
         if (!isApplication)
         {
            className = getParameter("appClass");
            if (className == null) throw new Exception("please use a parameter \"appClass\" specifying the class name"); // guich@120
            dataPath = getParameter("appPath"); // guich@120
            if (dataPath == null) dataPath = getParameter("dataPath");
            if (getParameter("useSonyFonts") != null) // guich@220_30
               JavaBridge.useSonyFonts = true;
            // get the bpp parameter
            String s = getParameter("bpp");
            int nbpp;
            if (s != null)
               try
               {
                  nbpp = Integer.parseInt(s);
                  if (nbpp != 2 && nbpp != 4 && nbpp != 8)
                     System.out.println("ERROR: bpp must be 2,4 or 8");
                  else
                  {
                     bpp = nbpp;
                     isColor = nbpp > 4; // guich@200b3: now the isColor depends on the nbpp.
                  }
               }
               catch (Exception e) {System.out.println("ERROR: bad bpp on parameter 'bpp'");}
            // get the scale parameter
            if ((s=getParameter("scale")) != null)
               try
               {
                  int scale = Integer.parseInt(s);
                  if (1 <= scale && scale <= 4)
                     JavaBridge.scale = scale;
               }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad scale");
               }

            width = size().width / JavaBridge.scale;
            height = size().height / JavaBridge.scale;

            s = getParameter("uiStyle");
            if (s != null && s.equalsIgnoreCase("PalmOS"))
               toPalm = true;
         }

         if (dataPath != null && !dataPath.endsWith("/")) dataPath += "/"; // guich@120
         if (dataPath == null)
            dataPath = "./"; // guich@200b4_4

         // guich@200b4: instantiate the classes that will handle the native methods
         //new waba.fx.NativeMethods4JDK();
//         new waba.io.NativeMethods4JDK();
//         new waba.ui.NativeMethods4JDK();
//         new java.lang.NativeMethods4JDK();

         JavaBridge.fillSettings();
         try
         {
            Class c = Class.forName(className); // guich@200b2: applets dont let you specify the path. it must be set in the codebase param
            JavaBridge._class = c;
            mainWindow = (MainWindow)c.newInstance();
            if (toPalm)
               Settings.setPalmOSStyle(true); // now is safe
         }
         catch (LinkageError le) {System.out.println("Fatal Error when running applet: there is an error in the constructor of the class "+className+" and it could not be instantiated. Stack trace: "); le.printStackTrace(); if (isApplication) System.exit(0);}
         catch (Exception e) { e.printStackTrace(); }
         // NOTE: java will call a partially constructed object
         // if show() is called before all the objects are constructed
         if (frame != null)
         {
            frame.show();
            WinCanvas.currentCanvas.requestFocus();
         }
         //repaint(); guich@200b4_59: avoid flickering when start.
      } catch (Exception ee) {ee.printStackTrace();} // guich@120
   }

   void startApp()
   {
      if (!started) // guich@120 - make sure that the canvas is available for drawing when starting the application. called by WinCanvas.paint.
      {
         try
         {
           WinCanvas.invokeInEventThread(true, new Runnable()
           {
             public void run() { mainWindow.appStarting(); } // guich@200b4_107
           });
         } catch (Throwable e) {e.printStackTrace();}
         started = true;
      }
   }
   public static void main(String args[])
   {
      int width = 160;
      int height = 160;
      int count = args.length;
      String newDataPath = null;
      int nbpp = -1;
      boolean toPalm = false;
      String cmdLine="";
      System.out.println("Current path : "+System.getProperty("user.dir"));
      if (count == 0)
      {
         System.out.println("ERROR: you must supply a class name as the last argument");
         System.out.println("Possible Arguments (in any order and case insensitive): /x <x> /y <y> /w <width> /h <height> /uiStyle <WinCE_or_PalmOS> /bpp <bitsPerPixel_2_4_or_8> /scale <scale_1_to_4> /t <title> /dataPath <path> /useSonyFonts /cmdLine <the rest of arguments are passed to the app as the commandline>");
         System.out.println("The class name must always be the last argument");
         System.out.println("The dataPath is used to set where the catalog and images are stored (note: appPath was changed to dataPath to make more sense, but appPath is still used for backwards compatibility)");
         return;
      }
      for (int i = 0; i < count - 1; i++)
      {
         if (args[i].equalsIgnoreCase("/useSonyFonts")) // guich@220_30
            JavaBridge.useSonyFonts = true;
         else
         if (args[i].equalsIgnoreCase("/appPath") || args[i].equalsIgnoreCase("/dataPath"))
         {
            if (++i < count - 1)
               try
               {
                  newDataPath = args[i].replace('\\','/');
                  java.io.File f = new java.io.File(newDataPath);
                  if (!f.isDirectory())
                     System.out.println("ERROR: dataPath specified is not a directory or does not exist! "+newDataPath);
               }
               catch (SecurityException e) {/* should not occur */}
         }
         else if (args[i].equalsIgnoreCase("/t"))   // OliE@330_20
         {
            if (++i < count - 1)
               appTitle = args[i];
         }
         else
         if (args[i].equalsIgnoreCase("/w"))
         {
            if (++i < count - 1)
               try { width = Integer.parseInt(args[i]); }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad width");
               }
         }
         else if (args[i].equalsIgnoreCase("/h"))
         {
            if (++i < count - 1)
               try { height = Integer.parseInt(args[i]); }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad height");
               }
         }
         else if (args[i].equalsIgnoreCase("/cmdline"))
         {
            cmdLine = "";
            while (++i < count - 1)
               cmdLine += args[i] + ((i < count-2)?" ":"");
            System.out.println("Command line : '"+cmdLine+"'");
         }
         // Added by Daniel Tauschke 07.11.01 to set x and y coordinates
         // Very usefull if you use a dual screen solution!
         else if (args[i].equalsIgnoreCase("/x"))
         {
            if (++i < count - 1)
               try { appX = Integer.parseInt(args[i]); }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad x coordinate");
               }
         }
         else if (args[i].equalsIgnoreCase("/y"))
         {
            if (++i < count - 1)
               try { appY = Integer.parseInt(args[i]); }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad y coordinate");
               }
         }
         else if (args[i].equalsIgnoreCase("/uiStyle"))
         {
            if (++i < count - 1 && args[i].equalsIgnoreCase("PalmOS"))
               toPalm = true;
         }
         else if (args[i].equalsIgnoreCase("/color") && nbpp == -1)
         {
            nbpp = 8;
         }
         else if (args[i].equalsIgnoreCase("/bpp"))
         {
            if (++i < count - 1)
               try
               {
                  nbpp = Integer.parseInt(args[i]);
                  if (nbpp != 2 && nbpp != 4 && nbpp != 8)
                     System.out.println("ERROR: bpp must be 2,4 or 8");
               }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad bpp");
               }
         }
         else if (args[i].equalsIgnoreCase("/scale"))
         {
            if (++i < count - 1)
               try
               {
                  int scale = Integer.parseInt(args[i]);
                  if (1 <= scale && scale <= 4)
                     JavaBridge.scale = scale;
               }
               catch (Exception e)
               {
                  System.out.println("ERROR: bad scale");
               }
         }
      }
      if (nbpp == -1) nbpp = defaultBPP;
      Applet.bpp = nbpp; // guich@200b4_58: this must be set before the applet construction
      Applet applet = new Applet();
      applet.className = args[count - 1];
      applet.isApplication = true;
      applet.isColor = nbpp > 4; // guich@200b3: now the isColor depends on the nbpp.
      applet.width = width;
      applet.height = height;
      applet.toPalm = toPalm;
      applet.commandLine = cmdLine;
      if (newDataPath != null) applet.dataPath = newDataPath;
      Frame frame = new Frame();
      if (appTitle != null) frame.setTitle(appTitle); // oliE@330_20
      frame.setLayout(new java.awt.BorderLayout());
      frame.add("Center", applet);
      frame.resize(width*JavaBridge.scale, height*JavaBridge.scale);
      frame.setLocation(appX,appY); // added by Daniel Tauschke
      frame.show();
      applet.frame = frame;
      // NOTE: java requires us to do this to make sure things paint
      frame.hide();
      java.awt.Insets insets;
      try
      {
         insets = frame.getInsets();
      }
      catch (NoSuchMethodError e)
      {
         insets = frame.insets(); // this is the JDK 1.02 call to get insets
      }
      if (insets == null)
         insets = new java.awt.Insets(0, 0, 0, 0);
      frame.resize(width*JavaBridge.scale + insets.left + insets.right, height*JavaBridge.scale + insets.top + insets.bottom);
      applet.init();
   }
   public void start()
   {
      mainWindow = MainWindow.getMainWindow();
   }

   ///////// guich@200b2: to make the vm easier to port, i removed all methods from the SuperWaba classes that uses the jdk classes /////////
   public void eraseScreen()
   {
      java.awt.Graphics g = WinCanvas.currentCanvas.getGraphics();
      g.setColor(java.awt.Color.white);
      g.fillRect(0,0,Applet.currentApplet.getBounds().width,Applet.currentApplet.getBounds().height); // erase the contents of the window
   }
   public void registerMainWindow(waba.ui.MainWindow main)
   {
	   _winTimer = new WinTimer(main);
	   _winCanvas = new WinCanvas(main); //ds @110
	   add(_winCanvas); // ds@110
	   _winCanvas.setBounds(0,0,width*JavaBridge.scale,height*JavaBridge.scale); // guich@120
	   _winCanvas.requestFocus(); // ds@110
	   mainG = null;
      mainG = new Graphics(main);
  }
   // guich@120: called by the exec to destroy the current mainWindow so other can take place
   public void destroyCurrentMainWindow()
   {
      // erase the screen prior to destroying the object. maybe a call to onExit?
      eraseScreen();
	   _winTimer.stopGracefully(); // guich@120
	   remove(_winCanvas); // guich@120 - Vm.exec may replace the current MainWindow
	   MainWindow.destroyZStack();
   }
   public void _setTimerInterval(int milliseconds)
   {
	   _winTimer.setInterval(milliseconds);
   }
   public void exit(int exitCode)
   {
      destroy(); // guich@230_24
      System.exit(exitCode);
   }

   public void paint(Graphics g)
   {
   }
   public void update(Graphics g)
   {
      paint(g);
   }
   public void print(Graphics g)
   {
   }

}