/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;

import waba.fx.Font;

/**
  * This class was created to implement all methods of Sun's JDK and
  * thus making easier to port this package to other java platforms
  * <p>
  * This methods are for internal use only. <b>Never</b> call them directly!
  */

  // Created by guich@200b2

public final class JavaBridge
{
   private static boolean nonGUIApp = false;
   /** IMPORTANT!!! To create an app just to read or write a pdb file, without a Graphical
    * User Interface, you must call this method.
    */
   public static void setNonGUIApp()
   {
      nonGUIApp = true;
      //new java.lang.NativeMethods(); guich@241_16
      new waba.sys.NativeMethods4JDK();
      new waba.io.NativeMethods4JDK();
   }
   public static void resetNonGUIApp()
   {
      nonGUIApp = false;
   }
   /** show 1:1, doubled, 3x or 4x */
   public static int scale = 2;

   static boolean useSonyFonts; // guich@220_30
   static Class _class; // used by the openInputStream method.
   /////////////////////////////////////////////////////////////////////////////////////////
   /** set to false if youre deploying your JDK application, to hide the
     * warning messages. USE THIS WITH CAUTION, because any errors that the
     * VM would warn, you will miss, and this may lead to real crashes in the
     * device.
     */
   public static boolean showMsgs = true;
   /** Calls System.out.println. Superwaba system debugging uses this
      method.  See also debug(String s). */
   public static void print(String s)
   {
      if (showMsgs) System.err.println(s);
   }
   ////  font and font metrics //////////////////////////////////////////////////////
   static String getFontPrefix()
   {
      return (waba.sys.Settings.screenWidth==320)?(useSonyFonts?"HSW":"5SW"):(waba.sys.Settings.screenWidth==160)?"LSW":"MSW";
   }
   static Hashtable htLoadedFonts = new Hashtable(10);
   public static waba.applet.UserFont getFont(waba.fx.Font f)
   {
      UserFont uf=null;
      // verify if its in the cache.
      String sf = waba.sys.Convert.toLowerCase(f.getName()+","+f.getSize()+","+f.getStyle());
      if ((uf=(waba.applet.UserFont)htLoadedFonts.get(sf)) != null)
         return uf;
      boolean bold = f.getStyle() == Font.BOLD;
      boolean large = f.getSize() >= 14;
      String fontName = f.getName();
      if (fontName.equalsIgnoreCase("SW")) // guich@220_30
         fontName = getFontPrefix();
      String sufix = ""+(large?"Large":"Small")+(bold?"Bold":"");
      // first, try to load the font itself using the sw font pattern
      uf = new UserFont(fontName,sufix);
      if (!uf.loaded)
      {
         // ok, the pattern didnt work, use the font name given
         uf = new UserFont(fontName,"");
         if (!uf.loaded)
         {
            // humm, the font is not found, use the internal ones
            fontName = getFontPrefix(); // guich@220_30
            uf = new UserFont(fontName,sufix);
         }
      }
      if (!uf.loaded)
      {
         print("No fonts found! be sure to place the files ?SW.pdb in the same directory from where youre running your application or in your applet's codebase or in a jar file!\n(? must be L,M,H,5)\nVM CRASHED!");
         System.exit(2);
         uf = null;
      }
      else
         htLoadedFonts.put(sf,uf);
	   return uf;
   }

   /** called by FontMetrics */
   public static UserFont getFontMetrics(waba.fx.Font f)
   {
      return getFont(f); // the UserFont has all methods that are needed
   }

   //// Graphics ////////////////////////////////////////////////////////////////////
   // Bresenham algorithm to draw lines
   static void drawQuadLine(Graphics g, int Ax, int Ay, int Bx, int By)
   {
      // INITIALIZE THE COMPONENTS OF THE ALGORITHM THAT ARE NOT AFFECTED BY THE SLOPE OR DIRECTION OF THE LINE
      int dX = Math.abs(Bx-Ax);    // store the change in X and Y of the line endpoints
      int dY = Math.abs(By-Ay);

      int CurrentX = Ax;      // store the starting point (just point A)
      int CurrentY = Ay;

      // DETERMINE "DIRECTIONS" TO INCREMENT X AND Y (REGARDLESS OF DECISION)
      int Xincr, Yincr;

      // guich: if its a pixel, draw only the pixel
      if (dX == 0 && dY == 0)
         g.fillRect(Ax*scale,Ay*scale,scale,scale); else
      // guich: if they are vertical or horizontal lines, use specific methods
      if (dY == 0) // horizontal line?
         g.fillRect(Math.min(Ax,Bx)*scale,Math.min(Ay,By)*scale,(dX+1)*scale,scale); else
      if (dX == 0) // vertical line?
         g.fillRect(Math.min(Ax,Bx)*scale,Math.min(Ay,By)*scale,scale,(dY+1)*scale);
      else // diagonal
      {
         if (Ax > Bx) { Xincr=-1; } else { Xincr=1; }    // which direction in X?
         if (Ay > By) { Yincr=-1; } else { Yincr=1; }    // which direction in Y?

         // DETERMINE INDEPENDENT VARIABLE (ONE THAT ALWAYS INCREMENTS BY 1 (OR -1) )
         // AND INITIATE APPROPRIATE LINE DRAWING ROUTINE (BASED ON FIRST OCTANT
         // ALWAYS). THE X AND Y'S MAY BE FLIPPED IF Y IS THE INDEPENDENT VARIABLE.
         if (dX >= dY)   // if X is the independent variable
         {
               int dPr     = dY<<1;                        // amount to increment decision if right is chosen (always)
               int dPru    = dPr - (dX<<1);                // amount to increment decision if up is chosen
               int P       = dPr - dX;                     // decision variable start value

               for (; dX>=0; dX--)                         // process each point in the line one at a time (just use dX)
               {
                  g.fillRect(CurrentX*scale,CurrentY*scale,scale,scale); // plot the pixel
                  CurrentX+=Xincr;                        // increment independent variable
                  if (P > 0)                              // is the pixel going right AND up?
                  {
                     CurrentY+=Yincr;                    // increment dependent variable
                     P+=dPru;                            // increment decision (for up)
                  }
                  else                                    // is the pixel just going right?
                     P+=dPr;                             // increment decision (for right)
               }
         }
         else            // if Y is the independent variable
         {
               int dPr     = dX<<1;                        // amount to increment decision if right is chosen (always)
               int dPru    = dPr - (dY<<1);                    // amount to increment decision if up is chosen
               int P       = dPr - dY;                     // decision variable start value

               for (; dY>=0; dY--)                         // process each point in the line one at a time (just use dY)
               {
                  g.fillRect(CurrentX*scale,CurrentY*scale,scale,scale);   // plot the pixel
                  CurrentY+=Yincr;                        // increment independent variable
                  if (P > 0)                              // is the pixel going up AND right?
                  {
                     CurrentX+=Xincr;                    // increment dependent variable
                     P+=dPru;                            // increment decision (for up)
                  }
                  else                                    // is the pixel just going up?
                     P+=dPr;                             // increment decision (for right)
               }
         }
      }
   }

   public static int lastColor=-1; // guich@321_10
   public static waba.fx.ISurface lastSurface; // guich@321_10

   public static void GraphicsLine(waba.fx.ISurface surface, int x1, int y1, int x2, int y2, int color)
   {
      if (!(surface instanceof waba.fx.Image)) // Images don't draw on screen, only in memory
      {
         if (color != lastColor || lastSurface != surface)
         {
            WinCanvas._g.setColor(getColor(color));
            lastColor = color;
            lastSurface = surface;
         }
         if (scale == 1) // guich@200b4_56: double or triple if necessary
            WinCanvas._g.drawLine(x1,y1,x2,y2);
         else
            drawQuadLine(WinCanvas._g,x1,y1,x2,y2);
      }
   }
   /** gets the awt color from the SuperWaba color */
   public static Color getColor(int color)
   {
      return new Color(color);
   }
   ///////////////////////        I/O        /////////////////////////////////////
   public static Hashtable htOpenedAt = new Hashtable(10); // guich@200b4_82
   private static File[] getClassPathDirectories() throws Exception
   {
      char dirSeparator = File.pathSeparatorChar;
      File[] classPath;
      String pathstr = System.getProperty("java.class.path");
      // Count the number of path separators
      int i=0;
      int n=0;
      int j=0;
      while ((i = pathstr.indexOf(dirSeparator, i)) != -1)
      {
         n++;
         i++;
      }
      // Build the class path
      File[] path = new File[n+1];
      int len = pathstr.length();
      for (i = n = 0; i < len; i = j + 1)
      {
         if ((j = pathstr.indexOf(dirSeparator, i)) == -1)
            j = len;
         if (i != j)
         {
            File file = new File(pathstr.substring(i, j));
            if (file.isDirectory())
               path[n++]=file;
         }
      }
      // Trim class path to exact size
      classPath = new File[n];
      System.arraycopy(path, 0, classPath, 0, n);
      return classPath;
   }

   private static InputStream readJavaInputStream(java.io.InputStream is)
   {
      if (is == null) return null;
      ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
      byte []buf = new byte[128];
      int len;
      while (true)
      {
         try
         {
            len = is.read(buf);
         }
         catch (IOException e) {break;}
         if (len > 0)
            baos.write(buf,0,len);
         else
            break;
      }
      return new ByteArrayInputStream(baos.toByteArray());
   }

   private static String getPathOf(String pathAndFileName)
   {
      char []chars = pathAndFileName.toCharArray();
      for (int i = chars.length-1; i >=0; i--)
         if (chars[i] == '\\' || chars[i] == '/')
            return new String(chars,0,i);
      return ""; // no path
   }

   /** used in some classes so they can correctly open files. now can open jar files. */
   // added by guich@120
   // enhanced by Scott Cytacki@20010222
   // corrected again by guich.
   public static InputStream openInputStream(String path)
   {
      print("\nopening for read "+path);
      if (Applet.currentApplet == null && !nonGUIApp) // guich@200b4: let non gui app work too
      {
         print("Applet.currentApplet is null. This can be caused by trying to run waba.applet.Applet with files both in directory/classpath and inside a jar");
         return null;
      }
      InputStream stream = null;
      try
      {
         if (nonGUIApp) // guich@200b4: let non gui app work too
            stream = new FileInputStream(path);
         else
         if (Applet.currentApplet.isApplication)
         {
            // search in the dataPath
            try
            {
               String p = isOk(Applet.currentApplet.dataPath)?(Applet.currentApplet.dataPath+path):path;
               print("#1 - dataPath");
               stream = new FileInputStream(p);
               htOpenedAt.put(path, getPathOf(p)); // guich@200b4_82 - jr: i changed getPathOf(path) to getPathOf(p)
               print("found in "+p);
            } catch (Exception e) {stream = null;}
            // search in the classpath
            if (stream == null)
            {
               print("#2 - classpath");
               File []dirs = getClassPathDirectories();
               File f = null;
               for (int i = 0; i < dirs.length; i++)
                  try
                  {
                     f = new File(dirs[i],path);
                     if (f.isFile())
                     {
                        htOpenedAt.put(path,dirs[i].getAbsolutePath()); // guich@200b4_82 - jr: changed dirs[i].getAbsolutePath
                        print("found in "+dirs[i].getAbsolutePath());
                        break;
                     }
                     else f = null; // guich@400_8: fixed problem when file was not found so the #3 can be tried below
                  } catch (Exception e) {f = null;}
               if (f != null)
                  stream = new FileInputStream(f);
            }
            if (stream == null) // guich@400_6: now the fonts can be read from the jar file
            {
               print("#3 - jar file");
               try
               {
                  InputStream is = (InputStream)_class.getResourceAsStream("/"+path);
                  if (is != null)
                  {
                     stream = readJavaInputStream(is);
                     print("found in "+is);
                  }
               } catch (Throwable tt) {System.out.println(tt.getMessage());}
            }
         }
         else
         {
            URL url;
            // zero in the jar file (normal way)
            InputStream is=null;
            try
            {
               is = (InputStream)_class.getResourceAsStream("/"+path);
            } catch (Throwable tt) {System.out.println(tt.getMessage());}
            print("#0 - resource: "+is); // guich@200b4_59
            if (is != null)
               stream = readJavaInputStream(is);
            // first in the jar file
            // guich@200b4: using this in Internet makes the archive be fetched from the server at each call of this function.
            if (stream == null)
            {
               String archive = Applet.currentApplet.getParameter("archive");
               print("#1 - archive: "+archive);
               if (isOk(archive) && !archive.equals("null"))
               {
                  if(archive.startsWith("null"))
                     archive = archive.substring(4, archive.length());
                  URL codeBase = Applet.currentApplet.getCodeBase();
                  url = new URL(codeBase+"/"+archive);
                  try
                  {
                     ZipInputStream zIn = new ZipInputStream(url.openStream());
                     ZipEntry zEntry = zIn.getNextEntry();
                     while(!zEntry.getName().equals(path))
                     {
                        zEntry = zIn.getNextEntry();
                        if (zEntry == null)
                           throw new Exception("doh");
                     }
                     print("found under zip entry.");
                     // guich@200b2: ok. the zIn.available() returns 1 and not the real size of the zip entry. so, here we read all into a byte stream
                     stream = readJavaInputStream(zIn);
                  } catch (Exception e){if (!e.getMessage().equals("doh")) e.printStackTrace();/* doh didn't find it in the jar thing */}
               }
            }
            // second under the codebase
            if (stream == null)
            try
            {
               URL codeBase = Applet.currentApplet.getCodeBase();
               print("#2 - codeBase: "+codeBase);
               String cb = codeBase.toString();
               char lastc = cb.charAt(cb.length() - 1);
               char firstc = path.charAt(0);
               if (lastc != '/' && firstc != '/')
                  cb += "/";
               url = new URL(cb + path);
               stream = url.openStream();
               print("found under codebase: "+url+". bytes: "+stream.available());
            }
            catch (FileNotFoundException ee) {}
            catch (Exception e) {e.printStackTrace();/* neither in the codebase */}
            // third in the localhost
            if (stream == null)
            try
            {
               print("#3- url: file://localhost/"+Applet.currentApplet.dataPath + path);
               url = new URL("file://localhost/"+Applet.currentApplet.dataPath + path); // guich@120
               stream = url.openStream();
               print("found under localhost: "+url);
            } catch (Exception e) {};
         }
         if (stream == null) print("file not found");
      }
      catch (FileNotFoundException ee) {print("file not found");}
      catch (Exception e) {/*if (!msgShowed) */print("error in JavaBridge.openInputStream: "+e.getMessage()); return null;} // guich@120
      return stream;
   }
   private static OutputStream openOutputUrl(URL url)
   {
	   try
	   {
         URLConnection con = url.openConnection();
         con.setUseCaches(false);
         con.setDoOutput(true);
         con.setDoInput(false);
         return con.getOutputStream();
      }
      catch (Exception u) // try another way
      {
         try
         {
            String path = url+"";
            return new FileOutputStream(isOk(Applet.currentApplet.dataPath)?(Applet.currentApplet.dataPath+path):path);
         } catch (Exception ee) {return null;}
      }
   }
   /** used in some classes so they can correctly open files.
     * used internally by readBytes.
     */
   public static OutputStream openOutputStream(String path)
   {
      print("\nopening for write "+path);
      if (Applet.currentApplet == null && !nonGUIApp) // guich@200b4: let non gui app work too
      {
         print("Applet.currentApplet is null. This can be caused by trying to run waba.applet.Applet with files both in directory/classpath and inside a jar");
         return null;
      }
      OutputStream stream = null;
      try
      {
         if (nonGUIApp) // guich@200b4: let non gui app work too
            stream = new FileOutputStream(path);
         else
         if (Applet.currentApplet.isApplication)
         {
            // search in the place where it was readed - guich@200b4_82
            String readed = (String)htOpenedAt.get(path);
            if (readed == null) // guich@400_59
            try
            {
               print("#0 - readed path");
               stream = new FileOutputStream(new File(readed,path));
               print("found in "+readed);
            }
            catch (Exception e) {stream = null;}
            if (stream != null)
               return stream;
            // search in the dataPath
            try
            {
               String p = isOk(Applet.currentApplet.dataPath)?(Applet.currentApplet.dataPath+path):path;
               print("#1 - dataPath");
               stream = new FileOutputStream(p);
               print("found in "+p);
            }
            catch (Exception e) {stream = null;}
            // search in the classpath
            if (stream == null)
            {
               print("#2 - classpath");
               File []dirs = getClassPathDirectories();
               File f = null;
               for (int i = 0; i < dirs.length; i++)
                  try
                  {
                     f = new File(dirs[i],path);
                     if (f.isFile())
                     {
                        print("found in "+dirs[i]);
                        break;
                     }
                  } catch (Exception e) {f = null;}
               if (f == null)
                  print("could not find file in the classpath");
               else
                  stream = new FileOutputStream(f);
            }
         }
         else
         {
            URL url;
            // first under the codebase
            if (stream == null)
            try
            {
               URL codeBase = Applet.currentApplet.getCodeBase();
               print("#1- codeBase: "+codeBase);
               String cb = codeBase.toString();
               char lastc = cb.charAt(cb.length() - 1);
               char firstc = path.charAt(0);
               if (lastc != '/' && firstc != '/')
                  cb += "/";
               url = new URL(cb + path);
               stream = openOutputUrl(url);
               print("found under codebase: "+url);
            }
            catch (Exception e) {e.printStackTrace();/* neither in the codebase */}
            // third in the localhost
            if (stream == null)
            try
            {
               print("#2- url: file://localhost/"+Applet.currentApplet.dataPath + path);
               url = new URL("file://localhost/"+Applet.currentApplet.dataPath + path); // guich@120
               stream = openOutputUrl(url);
               print("found under localhost: "+url);
            } catch (Exception e) {};
         }
         if (stream == null) print("file not found");
      }
      catch (FileNotFoundException ee) {print("file not found");}
      catch (Exception e) {/*if (!msgShowed) */print("error in Vm.openOutputStream: "+e.getMessage()); return null;} // guich@200
      return stream;
   }
   /** read the available bytes from the stream getted with openInputStream.
     * called by waba.fx.Image and waba.io.Catalog
     */
   public static byte [] readBytes(String path)
   {
      byte [] bytes = null;
      try
      {
	      InputStream is = openInputStream(path);
	      if (is != null)
	      {
		      int n = is.available();
		      bytes = new byte[n];
		      is.read(bytes);
		      is.close();
		   }
		} catch (Exception e) {e.printStackTrace();}
		return bytes;
   }
   /** write the available bytes to the stream getted with openOutputStream.
     * called by waba.io.Catalog
     */
   public static boolean writeBytes(String path, byte []buf, int len)
   {
      boolean ret = true;
      try
      {
	      OutputStream os = openOutputStream(path);
	      if (os != null)
	      {
	         if (buf != null)
	         {
   	         os.write(buf,0,len);
   	         os.close(); // pietj@330_1
   	      }
   	      else
   	         print("ATT: you sent to stream.writeBytes a null buffer!");
		   }
		} catch (Exception e) {e.printStackTrace(); ret = false;}
		return ret;
   }

   /** called by waba.fx.Sound.beep and Sound.tone */
   public static void beep()
   {
      Toolkit.getDefaultToolkit().beep();
   }
   /** return true is the string is valid. called by openInputStream and openOutputStream in this class. */
   public static boolean isOk(String s)
   {
      return s != null && s.length() > 0;
   }
   /** called by methods in the waba.io.Catalog class */
   public static long getTime()
   {
      return new Date().getTime();
   }
   /** called by waba.sys.Time constructor */
   public static void fillTime(waba.sys.Time t)
   {
	   java.util.Date d = new java.util.Date();
	   t.year = d.getYear() + 1900;
	   t.month = d.getMonth() + 1;
	   t.day = d.getDate();
	   t.hour = d.getHours();
	   t.minute = d.getMinutes();
	   t.second = d.getSeconds();
	   t.millis = (int)(d.getTime() % 1000L);
   }
   public static String getDefaultCrid(String name)
   {
      if (name.equalsIgnoreCase("waba.ui.Welcome"))
         return "SWAB";

      if (name.indexOf('.') != -1)
         name = name.substring(name.lastIndexOf('.')+1);
      int i;
      int n = name.length();
      int hash = 0;
      byte[] creat=new byte[4];
      for (i = 0; i < n; i++)
         hash += (byte)name.charAt(i);
      for (i = 0; i < 4; i++)
      {
         creat[i] = (byte)((hash % 26) + 'a');
         if ((hash & 64)>0)
            creat[i] += ('A'-'a');
         hash = hash / 2;
      }
      return new String(creat);
   }

   public static void storeSettings()
   {
      try
      {
         String crid = waba.sys.Settings.appCreatorId;
         // first verify if the catalog is created but the String is null
         showMsgs = false; // guich@340_49
         waba.io.Catalog cat = new waba.io.Catalog("Settings4"+crid+".SWVM."+crid,waba.io.Catalog.READ_WRITE); // guich@241_17: changed READ_ONLY to READ_WRITE to fix "operation invalid" error
         if (cat.isOpen() && waba.sys.Settings.appSettings == null)
            cat.delete(); // no more strings stored
         else
         if (waba.sys.Settings.appSettings != null || waba.sys.Settings.appSecretKey != null) // has settings, store them
         {
            if (!cat.isOpen())
               cat = new waba.io.Catalog("Settings4"+crid+".SWVM."+crid,waba.io.Catalog.CREATE);
            if (cat.isOpen())
            {
               waba.io.ResizeStream rs = new waba.io.ResizeStream(cat,256);
               waba.io.DataStream ds = new waba.io.DataStream(rs);

               if (cat.setRecordPos(0))
                  cat.deleteRecord();
               rs.startRecord();
               // store the appSettings record
               ds.writeString(waba.sys.Settings.appSettings);
               ds.writeString(waba.sys.Settings.appSecretKey);
               rs.endRecord();
               cat.close();
            }
         }
         showMsgs = true;
      }
      catch (Throwable t) {System.out.println("Settings can't be stored!");}
   }

   private static String[] getAppSettings(String crid)
   {
      String []sets = null;
      try
      {
         showMsgs = false; // guich@340_49
         waba.io.Catalog cat = new waba.io.Catalog("Settings4"+crid+".SWVM."+crid,waba.io.Catalog.READ_ONLY);
         waba.io.DataStream ds = new waba.io.DataStream(cat);
         if (cat.isOpen() && cat.setRecordPos(0))
         {
            sets = new String[2];
            sets[0] = ds.readString();
            try
            {
               sets[1] = ds.readString();
            } catch (Throwable t) {System.out.println("Reading an old settings file; no appSecretKey available.");}
            cat.close();
         }
         showMsgs = true; // guich@340_49
      }
      catch (Throwable t) {sets = null;}
      return sets;
   }
   private static char getFirstSymbol(String s)
   {
      char []c = s.toCharArray();
      for (int i =0; i < c.length; i++)
         if (c[i] != ' ' && !('0' <= c[i] && c[i] <= '9'))
            return c[i];
      return ' ';
   }
   /** called by waba.applet.Applet.init() */
   public static void fillSettings()
   {
      java.util.Calendar cal = java.util.Calendar.getInstance();
      // guich@340_34: since java can't provide us good methods to return these values, we use parse the return of some formatting methods
      cal.set(2002,11,25,20,0,0); // guich@401_32
      java.text.DateFormat df = java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT); // guich@401_32: fixed wrong results in some systems
      String d = df.format(cal.getTime());
      waba.sys.Settings.dateFormat = d.startsWith("25") ? waba.sys.Settings.DATE_DMY
                                   : d.startsWith("12") ? waba.sys.Settings.DATE_MDY
                                   : waba.sys.Settings.DATE_YMD;
      waba.sys.Settings.dateSeparator = getFirstSymbol(d);
      df = java.text.DateFormat.getTimeInstance(java.text.DateFormat.SHORT); // guich@401_32
      d = df.format(cal.getTime());

      waba.sys.Settings.is24Hour = d.toLowerCase().indexOf("am") == -1 && d.toLowerCase().indexOf("pm") == -1;
      waba.sys.Settings.timeSeparator = getFirstSymbol(d);
      //

      java.util.TimeZone tz = java.util.TimeZone.getDefault(); // guich@340_33
      waba.sys.Settings.daylightSavings = tz.useDaylightTime();
      waba.sys.Settings.timeZone = tz.getRawOffset() / (60*60*1000);
      waba.sys.Settings.weekStart = (byte) (cal.getFirstDayOfWeek() - 1);

      java.text.DecimalFormatSymbols dfs = new java.text.DecimalFormatSymbols();
      waba.sys.Settings.thousandsSeparator = dfs.getGroupingSeparator();
      waba.sys.Settings.decimalSeparator = dfs.getDecimalSeparator();
      waba.sys.Settings.isColor = Applet.currentApplet.isColor;
      waba.sys.Settings.maxColors = 1 << Applet.currentApplet.bpp;
      waba.sys.Settings.romVersion = 0x02000000;
      waba.sys.Settings.uiStyle = waba.sys.Settings.WinCE;
      waba.sys.Settings.screenWidth = Applet.currentApplet.width;
      waba.sys.Settings.screenHeight = Applet.currentApplet.height;
      waba.sys.Settings.onDevice = false;
      waba.sys.Settings.platform = "Java";
      waba.sys.Settings.appCreatorId = getDefaultCrid(Applet.currentApplet.className);
      waba.sys.Settings.virtualKeyboard = false;
      String []sets = getAppSettings(waba.sys.Settings.appCreatorId); // guich@330_47
      if (sets != null)
      {
         waba.sys.Settings.appSettings = sets[0];
         waba.sys.Settings.appSecretKey = sets[1];
      }
      try
      {
    	   waba.sys.Settings.userName = !Applet.currentApplet.isApplication?null:java.lang.System.getProperty("user.name");
    	} catch (SecurityException se) {waba.sys.Settings.userName = null;}
   }
   //////////////////////////////////////////////////////////////////////////////////////////
}