/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
 *                 and Andy DePue                                                *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

import waba.ui.*;

class SWEventThread implements Runnable
{
   protected MainWindow _win;
   protected static Queue  _eventQueue;
   protected int _timerTickEvent;
   protected static int _invokeInEventThread = Event.getNextAvailableEventId();
   protected java.lang.Thread _thread;  // 9/16/02 - Andy - Needs to know its thread to fix deadlock when exit() called from event handler.

   public SWEventThread(MainWindow win)
   {
      _win = win;
      _eventQueue = new Queue();
   }

   public void run()
   {
      for(;;)
      {
        try
        {
          handleOneEvent();
        }
        catch(java.lang.Throwable t)
        {
          System.out.println("---------------------------");
          System.out.println(">>>>>>> CAUGHT UNHANDLED EXCEPTION IN SuperWaba EVENT THREAD:");
          t.printStackTrace();
        }
      }
   }

   public boolean eventAvailable()
   {
      return _eventQueue.getSize() > 0;
   }

   public void handleOneEvent()
   {
      // This gives the system CPU some breathing room when in a tight event
      // loop and no events are posted.
      SWEvent event = (SWEvent)_eventQueue.popWait(100);
      if (event != null)
      {
         if (event.type == _invokeInEventThread)
         {
            event.r.run();
            // If they are waiting for this, then notify.
            if (event.synch != null)
            {
               synchronized(event.synch)
               {
                  event.synch.notify();
               }
            }
         }
         else
            _win._postEvent(event.type, event.key, event.x, event.y, event.modifiers, event.timestamp);
      }
   }

   public void pushEvent(int type, int key, int x, int y, int modifiers, int timestamp)
   {
      _eventQueue.push(new SWEvent(type, key, x, y, modifiers, timestamp));
   }

   public void invokeInEventThread(boolean wait, Runnable r)
   {
      if (!wait)
         _eventQueue.push(new SWEvent(_invokeInEventThread, r));
      else
      {
         // 9/16/02 - Andy - I have modified the code below so that it now checks
         // to see if we are currently in the event thread.  If we are in the
         // event thread, then we will simply call r.run() directly, thus
         // avoiding a deadlock if invokeInEventThread() was called from an event
         // handler.

         // Are we currently in the event thread?
         java.lang.Thread current = java.lang.Thread.currentThread();
         if (current.equals(_thread))
         {
            // Execute directly.
            r.run();
         }
         else
         {
            // We are not in the event thread, so push to event thread.
            // We have to create a new Object, in case there is more than one
            // thread waiting on an invoke.
            Object synch = new Object();
            synchronized(synch)
            {
               _eventQueue.push(new SWEvent(_invokeInEventThread, r, synch));
               try { synch.wait(); } catch(Exception ie) { }
            }
         }
      }
   }

   // 9/16/02 - Andy - These two methods added for bug fix (deadlock on exit() when called from event handler)
   public void setThread(java.lang.Thread thread)
   {
      _thread = thread;
   }
   public java.lang.Thread getThread()
   {
      return _thread;
   }
}
