/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

import waba.io.*;

/** Represents the internal font structure, readed from a pdb file. used internally. */
// created by guich@200b2

public class UserFont
{
   public short fontType;         // font type
   public short firstChar;        // ASCII code of first character
   public short lastChar;         // ASCII code of last character
   public short maxWidth;         // maximum character width
   public short kernMax;          // negative of maximum character kern
   public short nDescent;         // negative of descent
   public short fRectWidth;       // width of font rectangle
   public short fRectHeight;      // height of font rectangle
   public short owTLoc;           // offset to offset/width table
   public short ascent;           // ascent
   public short descent;          // descent
   public short leading;          // leading
   public short rowWords;         // row width of bit image / 2

   public int   rowWidthInBytes;
   public int   numberOfChars;
   public int   bitmapTableSize;
   public byte  []bitmapTable;
   public short []bitIndexTable;

   public boolean loaded;

   public UserFont(String fontName, String sufix)
   {
      Catalog c;
      // guich@340_47: added support for joined font files.
      // first check if the font is inside a packaged file.
      c = new Catalog(fontName+".Font.Font",Catalog.READ_ONLY);
      if (!c.isOpen())
         c = new Catalog(fontName+sufix+".Font.Font",Catalog.READ_ONLY);
      if (c.isOpen())
      {
         fontName += sufix;
         DataStream ds = new DataStream(c);
         int n = c.getRecordCount();
         if (n > 1) // multifont file?
         {
            boolean found = false;
            String name;
            for (int i =0; i < n; i++)
               if (c.setRecordPos(i) && (name=ds.readCString()).equals(fontName)) // we found the desired font in this record
               {
                  int l = name.length()+1;
                  ds.skip((int)((l+3) / 4)*4 - l); // skip the pad bytes

                  found = true;
                  break;
               }
            if (!found)
            {
               c.close();
               return; // loaded = false
            }
         }
         else c.setRecordPos(0);
         fontType    = ds.readShort();
         firstChar   = ds.readShort();
         lastChar    = ds.readShort();
         maxWidth    = ds.readShort();
         kernMax     = ds.readShort();
         nDescent    = ds.readShort();
         fRectWidth  = ds.readShort();
         fRectHeight = ds.readShort();
         owTLoc      = ds.readShort();
         ascent      = ds.readShort();
         descent     = ds.readShort();
         leading     = ds.readShort();
         rowWords    = ds.readShort();

         rowWidthInBytes = rowWords << 1;
         numberOfChars   = lastChar - firstChar+1;
         bitmapTableSize = (int)rowWidthInBytes * (int)fRectHeight;

         bitmapTable     = new byte[bitmapTableSize];
         ds.readBytes(bitmapTable);
         bitIndexTable   = new short[numberOfChars+2]; // guich@200b4: read 2 more indexes
         for (int i=0; i < bitIndexTable.length; i++)
            bitIndexTable[i] = ds.readShort();

         loaded = true;
         c.close();
      }
   }

   public int charWidth(char ch)
   {
      int index = (short)ch - firstChar;
      if (index < 0 || ch > lastChar) index = numberOfChars; // MCS index
      // Get the source x coordinate and width of the character
      int offset = bitIndexTable[index];
      int width = (byte)(bitIndexTable[index+1] - offset);
      if (width == 0 && ch != 13) // guich@340_46 - guich@341_9: fixed problem with <enter>
         width = (byte)(bitIndexTable[numberOfChars+1] - bitIndexTable[numberOfChars]); // width of MCB
      return width;
   }

   public int stringWidth(String s)
   {
      char []ac = s.toCharArray();
      int sum = 0;
      for (int i =ac.length-1; i >= 0; i--)
         sum += charWidth(ac[i]);
      return sum;
   }

   public int getAscent()
   {
      return ascent;
   }
   public int getDescent()
   {
      return descent;
   }
   public int getLeading()
   {
      return leading;
   }
}
