/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2001-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 2000 Dave Slaughter                                            *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

import waba.ui.*;
import waba.sys.*;

public class WinCanvas extends java.awt.Canvas
{
   MainWindow win;
   public static WinCanvas currentCanvas;
   protected static java.awt.Graphics _g;
   protected static SWEventThread _eventThread;

   public WinCanvas(MainWindow win)
   {
      currentCanvas = this; // only 1 canvas per applet
      this.win = win;
      createEventThread(win);
   }
   public void addNotify()
   {
      super.addNotify();
      _g = getGraphics(); // guich@300_50: now only one global Graphics object
   }

   private int actionKeyValue(int action)
   {
      int key = 0;
      switch (action)
      {
         case java.awt.Event.PGUP:       key = IKeys.PAGE_UP; break;
         case java.awt.Event.PGDN:       key = IKeys.PAGE_DOWN; break;
         case java.awt.Event.HOME:       key = IKeys.HOME; break;
         case java.awt.Event.END:        key = IKeys.END; break;
         case java.awt.Event.UP:         key = IKeys.UP; break;
         case java.awt.Event.DOWN:       key = IKeys.DOWN; break;
         case java.awt.Event.LEFT:       key = IKeys.LEFT; break;
         case java.awt.Event.RIGHT:      key = IKeys.RIGHT; break;
         case java.awt.Event.INSERT:     key = IKeys.INSERT; break;
         case java.awt.Event.ENTER:      key = IKeys.ENTER; break;
         case java.awt.Event.TAB:        key = IKeys.TAB; break;
         case java.awt.Event.BACK_SPACE: key = IKeys.BACKSPACE; break;
         case java.awt.Event.ESCAPE:     key = IKeys.ESCAPE; break;
         case java.awt.Event.DELETE:     key = IKeys.DELETE; break;
         // guich@120 - emulate more keys
         case java.awt.Event.F1:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_HARD1) != 0) key = IKeys.HARD1; break;
         case java.awt.Event.F2:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_HARD2) != 0) key = IKeys.HARD2; break;
         case java.awt.Event.F3:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_HARD3) != 0) key = IKeys.HARD3; break;
         case java.awt.Event.F4:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_HARD4) != 0) key = IKeys.HARD4; break;
         case java.awt.Event.F5:         key = IKeys.COMMAND; break;
         case java.awt.Event.F6:         key = IKeys.MENU; break;
         case java.awt.Event.F7:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_CALC) != 0) key = IKeys.CALC; break;
         case java.awt.Event.F8:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_FIND) != 0) key = IKeys.FIND; break;
         case java.awt.Event.F9:         if ((Vm.interceptSystemKey & waba.sys.Vm.SK_LAUNCH) != 0) key = IKeys.LAUNCH; break;
         case java.awt.Event.F10:        key = IKeys.ACTION; break; // guich@400_64
         case java.awt.Event.F11:        key = IKeys.KEYBOARD_ABC; break;
         case java.awt.Event.F12:        key = IKeys.KEYBOARD_123; break;
      }
      return key;
   }
   public int currentKeysState = 0;
   private void setKeysState(int javaType, int javaKey) // guich@200b4_7
   {
      if (javaType == java.awt.Event.KEY_ACTION)
         switch (javaKey)
         {
            case java.awt.Event.F1:   currentKeysState |= waba.sys.Vm.SK_HARD1; break;
            case java.awt.Event.F2:   currentKeysState |= waba.sys.Vm.SK_HARD2; break;
            case java.awt.Event.F3:   currentKeysState |= waba.sys.Vm.SK_HARD3; break;
            case java.awt.Event.F4:   currentKeysState |= waba.sys.Vm.SK_HARD4; break;
            case java.awt.Event.PGUP: if ((currentKeysState & waba.sys.Vm.SK_PAGE_DOWN) == 0) // dont let up/down be pressed simultaneously
                                         currentKeysState |= waba.sys.Vm.SK_PAGE_UP; break;
            case java.awt.Event.PGDN: if ((currentKeysState & waba.sys.Vm.SK_PAGE_UP) == 0)   // dont let up/down be pressed simultaneously
                                         currentKeysState |= waba.sys.Vm.SK_PAGE_DOWN; break;
         }
      else // KEY_RELEASE
         switch (javaKey)
         {
            case java.awt.Event.F1:   currentKeysState &= ~waba.sys.Vm.SK_HARD1; break;
            case java.awt.Event.F2:   currentKeysState &= ~waba.sys.Vm.SK_HARD2; break;
            case java.awt.Event.F3:   currentKeysState &= ~waba.sys.Vm.SK_HARD3; break;
            case java.awt.Event.F4:   currentKeysState &= ~waba.sys.Vm.SK_HARD4; break;
            case java.awt.Event.PGUP: currentKeysState &= ~waba.sys.Vm.SK_PAGE_UP; break;
            case java.awt.Event.PGDN: currentKeysState &= ~waba.sys.Vm.SK_PAGE_DOWN; break;
         }
      //System.out.println("currentKeysState: "+currentKeysState); want to test it?
   }

   public boolean handleEvent(java.awt.Event event)
   {
      int type = 0;
      int key = 0;
      int x = 0;
      int y = 0;
      int modifiers = 0;
      if ((event.modifiers & java.awt.Event.SHIFT_MASK) > 0)
         modifiers |= IKeys.SHIFT;
      if ((event.modifiers & java.awt.Event.CTRL_MASK) > 0)
         modifiers |= IKeys.CONTROL;
      boolean doPostEvent = false;
      switch (event.id)
      {
         //case java.awt.Event.MOUSE_MOVE: - guich@120: no more mousemove event
         case java.awt.Event.MOUSE_DRAG:
            type = PenEvent.PEN_MOVE;
            x = event.x;
            y = event.y;
            doPostEvent = true;
            break;
         case java.awt.Event.MOUSE_DOWN:
            type = PenEvent.PEN_DOWN;
            x = event.x;
            y = event.y;
            doPostEvent = true;
            break;
         case java.awt.Event.MOUSE_UP:
            type = PenEvent.PEN_UP;
            x = event.x;
            y = event.y;
            doPostEvent = true;
            break;
         case java.awt.Event.KEY_PRESS:
            type = KeyEvent.KEY_PRESS;
            key = keyValue(event.key, modifiers);
            doPostEvent = true;
            break;
         case java.awt.Event.KEY_ACTION:
            key = actionKeyValue(event.key);
            if (key != 0)
            {
               type = KeyEvent.KEY_PRESS;
               doPostEvent = true;
            }
            setKeysState(event.id,event.key);
            break;
         case java.awt.Event.KEY_ACTION_RELEASE:
            setKeysState(event.id,event.key);
            break;
      }
      if (doPostEvent)
      {
         int timestamp = (int)event.when;
         x /= JavaBridge.scale;
         y /= JavaBridge.scale;
         _eventThread.pushEvent(type, key, x, y, modifiers, timestamp);
      }
      return super.handleEvent(event);
      }
   public static int keyValue(int key, int mod)
   {
      switch (key)
      {
         case 8:
            key = IKeys.BACKSPACE;
            break;
         case 10:
            key = IKeys.ENTER;
            break;
         case 127:
            key = IKeys.DELETE;
            break;
      }
      return key;
   }
   public void paint(java.awt.Graphics g)
   {
      java.awt.Rectangle r = null;
      // getClipRect() is missing in the Kaffe distribution for Linux
      try { r = g.getClipBounds(); } catch (NoSuchMethodError e) { r = g.getClipRect(); }

      if (!Applet.currentApplet.started) // guich@120 - only call onStart after the canvas is valid
         Applet.currentApplet.startApp();

      // anodos@321_9: Do a dummy call to invokeInEventThread to see if it messes up painting, even
      // though painting will still happen in this method and not in event thread.
      invokeInEventThread(true, new Runnable() { public void run() { } });
      // repaint all parents of this window
      // guich@120
      try
      {
         MainWindow.getMainWindow()._doPaint(-5000,0,0,0);
      }
      catch (Exception e) {System.out.println("Exception in WinCanvas.paint"); e.printStackTrace();}
   }

   public void update(java.awt.Graphics g)
   {
      paint(g);
   }

   public static void createEventThread(MainWindow win)
   {
      if(_eventThread == null)
      {
         _eventThread = new SWEventThread(win);
         java.lang.Thread thread = new java.lang.Thread(_eventThread, "SW Event Thread");
         thread.setDaemon(true);
         _eventThread.setThread(thread); // 9/16/02 - Andy - So the event thread will know exactly which thread it is.
                                         // This is to fix the issue with deadlock when exit() called from an event handler.
         thread.start();
      }
   }

   public static boolean eventIsAvailable()
   {
      return _eventThread.eventAvailable();
   }

   public static void pumpEvents()
   {
      _eventThread.handleOneEvent();
   }

   public static void invokeInEventThread(boolean wait, Runnable runnable)
   {
      _eventThread.invokeInEventThread(wait, runnable);
   }
}