/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2001-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 2000 Dave Slaughter                                            *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.applet;

import waba.ui.*;

public class WinTimer extends Thread
{
   MainWindow win;
   private int interval = 0;
   private boolean shouldStop = false;
   public static int threadCount;
   private Queue threadSynch = new Queue();

   public WinTimer(MainWindow win)
   {
      this.win = win;
      start();
   }

   public void run()
   {
     // NOTE: because we have created an official event queue/thread, which now
     // resembles the device event queue much more closely, we must be
     // sure that all timers and SW threads are run in that event thread.  This
     // will ensure that such things as blinking cursors will continue to work
     // if there is a blocking modal dialog open.  This also means that SW JDK
     // threads will act much more like the device threads... in that, threads
     // will not run unless a message pump is running.
      while (!shouldStop)
      {
         boolean doTick = true;
         int millis = interval;
         if (millis <= 0)
         {
            // NOTE: Netscape navigator doesn't support interrupt()
            // so we sleep here less than we would normally need to
            // (1 second) if we're not doing anything to check if
            // the timer should start in case interrupt didn't work
            millis = 1 * 1000;
            doTick = false;
         }
         // guich@200b4_84: implement the simple thread
         long first = System.currentTimeMillis();
         while ((System.currentTimeMillis()-first) < millis)
         {
            if (threadCount > 0)
            {
              long t = System.currentTimeMillis();
              // Since I am passing 'true' to this method, then it will not
              // return until our 'run' method below has finished in the event
              // thread.
              WinCanvas.invokeInEventThread(true, new Runnable() {
                public void run() { win._runThreads(); }
              });
              try { Thread.sleep(20); } catch(InterruptedException ie) {}
              millis -= System.currentTimeMillis() - t;
            }
            else
               try
               {
                  sleep(millis);
                  doTick = true; // guich@230_3
                  break; // guich@230_3
               }
               catch (InterruptedException e)
               {
                  doTick = false;
                  break; // guich@230_4
               }
         }
         if (doTick)
         {
           WinCanvas.invokeInEventThread(false, new Runnable() {
             public void run() { win._onTimerTick(); }
           });
         }
      }
   }

   public void setInterval(int millis)
   {
      //System.out.println("setInterval "+millis);
      interval = millis<55 ? 55 : millis; // guich@230_3
      interrupt();
   }

   public void stopGracefully()
   {
      // NOTE: It's not a good idea to call stop() on threads since
      // it can cause the JVM to crash.
      shouldStop = true;
      interrupt();
   }
}