/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.fx;

import waba.sys.*;
import waba.util.*;
/**
 * Palette represents a ordered group of colors.
 * Palette only supports up to 256 colors, since memory is critical, supporting 16k would fill all SuperWaba memory heap.
 * To make this Palette active, use Graphics.applyPalette.
 * In PalmOS, the System palette is as follows:
 <p>
 * In Color devices:
 <p>
<table border="0" cellpadding="3">
  <tr>
    <td bgcolor="FFFFFF">00 - FFFFFF</td>
    <td bgcolor="FFCCFF">01 - FFCCFF</td>
    <td bgcolor="FF99FF">02 - FF99FF</td>
    <td bgcolor="FF66FF">03 - FF66FF</td>
    <td bgcolor="FF33FF">04 - FF33FF</td>
    <td bgcolor="FF00FF">05 - FF00FF</td>
  </tr>
  <tr>
    <td bgcolor="FFFFCC">06 - FFFFCC</td>
    <td bgcolor="FFCCCC">07 - FFCCCC</td>
    <td bgcolor="FF99CC">08 - FF99CC</td>
    <td bgcolor="FF66CC">09 - FF66CC</td>
    <td bgcolor="FF33CC">0A - FF33CC</td>
    <td bgcolor="FF00CC">0B - FF00CC</td>
  </tr>
  <tr>
    <td bgcolor="FFFF99">0C - FFFF99</td>
    <td bgcolor="FFCC99">0D - FFCC99</td>
    <td bgcolor="FF9999">0E - FF9999</td>
    <td bgcolor="FF6699">0F - FF6699</td>
    <td bgcolor="FF3399">10 - FF3399</td>
    <td bgcolor="FF0099">11 - FF0099</td>
  </tr>
  <tr>
    <td bgcolor="CCFFFF">12 - CCFFFF</td>
    <td bgcolor="CCCCFF">13 - CCCCFF</td>
    <td bgcolor="CC99FF">14 - CC99FF</td>
    <td bgcolor="CC66FF">15 - CC66FF</td>
    <td bgcolor="CC33FF">16 - CC33FF</td>
    <td bgcolor="CC00FF">17 - CC00FF</td>
  </tr>
  <tr>
    <td bgcolor="CCFFCC">18 - CCFFCC</td>
    <td bgcolor="CCCCCC">19 - CCCCCC</td>
    <td bgcolor="CC99CC">1A - CC99CC</td>
    <td bgcolor="CC66CC">1B - CC66CC</td>
    <td bgcolor="CC33CC">1C - CC33CC</td>
    <td bgcolor="CC00CC">1D - CC00CC</td>
  </tr>
  <tr>
    <td bgcolor="CCFF99">1E - CCFF99</td>
    <td bgcolor="CCCC99">1F - CCCC99</td>
    <td bgcolor="CC9999">20 - CC9999</td>
    <td bgcolor="CC6699">21 - CC6699</td>
    <td bgcolor="CC3399">22 - CC3399</td>
    <td bgcolor="CC0099">23 - CC0099</td>
  </tr>
  <tr>
    <td bgcolor="99FFFF">24 - 99FFFF</td>
    <td bgcolor="99CCFF">25 - 99CCFF</td>
    <td bgcolor="9999FF">26 - 9999FF</td>
    <td bgcolor="9966FF">27 - 9966FF</td>
    <td bgcolor="9933FF">28 - 9933FF</td>
    <td bgcolor="9900FF">29 - 9900FF</td>
  </tr>
  <tr>
    <td bgcolor="99FFCC">2A - 99FFCC</td>
    <td bgcolor="99CCCC">2B - 99CCCC</td>
    <td bgcolor="9999CC">2C - 9999CC</td>
    <td bgcolor="9966CC">2D - 9966CC</td>
    <td bgcolor="9933CC">2E - 9933CC</td>
    <td bgcolor="9900CC">2F - 9900CC</td>
  </tr>
  <tr>
    <td bgcolor="99FF99">30 - 99FF99</td>
    <td bgcolor="99CC99">31 - 99CC99</td>
    <td bgcolor="999999">32 - 999999</td>
    <td bgcolor="996699">33 - 996699</td>
    <td bgcolor="993399"><font color="#FFFFFF">34 - 993399</td>
    <td bgcolor="990099"><font color="#FFFFFF">35 - 990099</td>
  </tr>
  <tr>
    <td bgcolor="66FFFF">36 - 66FFFF</td>
    <td bgcolor="66CCFF">37 - 66CCFF</td>
    <td bgcolor="6699FF">38 - 6699FF</td>
    <td bgcolor="6666FF">39 - 6666FF</td>
    <td bgcolor="6633FF">3A - 6633FF</td>
    <td bgcolor="6600FF">3B - 6600FF</td>
  </tr>
  <tr>
    <td bgcolor="66FFCC">3C - 66FFCC</td>
    <td bgcolor="66CCCC">3D - 66CCCC</td>
    <td bgcolor="6699CC">3E - 6699CC</td>
    <td bgcolor="6666CC">3F - 6666CC</td>
    <td bgcolor="6633CC"><font color="#FFFFFF">40 - 6633CC</font></td>
    <td bgcolor="6600CC"><font color="#FFFFFF">41 - 6600CC</font></td>
  </tr>
  <tr>
    <td bgcolor="66FF99">42 - 66FF99</td>
    <td bgcolor="66CC99">43 - 66CC99</td>
    <td bgcolor="669999">44 - 669999</td>
    <td bgcolor="666699">45 - 666699</td>
    <td bgcolor="663399"><font color="#FFFFFF">46 - 663399</font></td>
    <td bgcolor="660099"><font color="#FFFFFF">47 - 660099</font></td>
  </tr>
  <tr>
    <td bgcolor="33FFFF">48 - 33FFFF</td>
    <td bgcolor="33CCFF">49 - 33CCFF</td>
    <td bgcolor="3399FF">4A - 3399FF</td>
    <td bgcolor="3366FF">4B - 3366FF</td>
    <td bgcolor="3333FF">4C - 3333FF</td>
    <td bgcolor="3300FF">4D - 3300FF</td>
  </tr>
  <tr>
    <td bgcolor="33FFCC">4E - 33FFCC</td>
    <td bgcolor="33CCCC">4F - 33CCCC</td>
    <td bgcolor="3399CC">50 - 3399CC</td>
    <td bgcolor="3366CC">51 - 3366CC</td>
    <td bgcolor="3333CC"><font color="#FFFFFF">52 - 3333CC</font></td>
    <td bgcolor="3300CC"><font color="#FFFFFF">53 - 3300CC</font></td>
  </tr>
  <tr>
    <td bgcolor="33FF99">54 - 33FF99</td>
    <td bgcolor="33CC99">55 - 33CC99</td>
    <td bgcolor="339999">56 - 339999</td>
    <td bgcolor="336699">57 - 336699</td>
    <td bgcolor="333399"><font color="#FFFFFF">58 - 333399</font></td>
    <td bgcolor="330099"><font color="#FFFFFF">59 - 330099</font></td>
  </tr>
  <tr>
    <td bgcolor="00FFFF">5A - 00FFFF</td>
    <td bgcolor="00CCFF">5B - 00CCFF</td>
    <td bgcolor="0099FF">5C - 0099FF</td>
    <td bgcolor="0066FF">5D - 0066FF</td>
    <td bgcolor="0033FF">5E - 0033FF</td>
    <td bgcolor="0000FF">5F - 0000FF</td>
  </tr>
  <tr>
    <td bgcolor="00FFCC">60 - 00FFCC</td>
    <td bgcolor="00CCCC">61 - 00CCCC</td>
    <td bgcolor="0099CC">62 - 0099CC</td>
    <td bgcolor="0066CC">63 - 0066CC</td>
    <td bgcolor="0033CC"><font color="#FFFFFF">64 - 0033CC</font></td>
    <td bgcolor="0000CC"><font color="#FFFFFF">65 - 0000CC</font></td>
  </tr>
  <tr>
    <td bgcolor="00FF99">66 - 00FF99</td>
    <td bgcolor="00CC99">67 - 00CC99</td>
    <td bgcolor="009999">68 - 009999</td>
    <td bgcolor="006699">69 - 006699</td>
    <td bgcolor="003399"><font color="#FFFFFF">6A - 003399</font></td>
    <td bgcolor="000099"><font color="#FFFFFF">6B - 000099</font></td>
  </tr>
  <tr>
    <td bgcolor="FFFF66">6C - FFFF66</td>
    <td bgcolor="FFCC66">6D - FFCC66</td>
    <td bgcolor="FF9966">6E - FF9966</td>
    <td bgcolor="FF6666">6F - FF6666</td>
    <td bgcolor="FF3366">70 - FF3366</td>
    <td bgcolor="FF0066">71 - FF0066</td>
  </tr>
  <tr>
    <td bgcolor="FFFF33">72 - FFFF33</td>
    <td bgcolor="FFCC33">73 - FFCC33</td>
    <td bgcolor="FF9933">74 - FF9933</td>
    <td bgcolor="FF6633">75 - FF6633</td>
    <td bgcolor="FF3333">76 - FF3333</td>
    <td bgcolor="FF0033">77 - FF0033</td>
  </tr>
  <tr>
    <td bgcolor="FFFF00">78 - FFFF00</td>
    <td bgcolor="FFCC00">79 - FFCC00</td>
    <td bgcolor="FF9900">7A - FF9900</td>
    <td bgcolor="FF6600">7B - FF6600</td>
    <td bgcolor="FF3300">7C - FF3300</td>
    <td bgcolor="FF0000">7D - FF0000</td>
  </tr>
  <tr>
    <td bgcolor="CCFF66">7E - CCFF66</td>
    <td bgcolor="CCCC66">7F - CCCC66</td>
    <td bgcolor="CC9966">80 - CC9966</td>
    <td bgcolor="CC6666">81 - CC6666</td>
    <td bgcolor="CC3366">82 - CC3366</td>
    <td bgcolor="CC0066">83 - CC0066</td>
  </tr>
  <tr>
    <td bgcolor="CCFF33">84 - CCFF33</td>
    <td bgcolor="CCCC33">85 - CCCC33</td>
    <td bgcolor="CC9933">86 - CC9933</td>
    <td bgcolor="CC6633">87 - CC6633</td>
    <td bgcolor="CC3333">88 - CC3333</td>
    <td bgcolor="CC0033">89 - CC0033</td>
  </tr>
  <tr>
    <td bgcolor="CCFF00">8A - CCFF00</td>
    <td bgcolor="CCCC00">8B - CCCC00</td>
    <td bgcolor="CC9900">8C - CC9900</td>
    <td bgcolor="CC6600">8D - CC6600</td>
    <td bgcolor="CC3300">8E - CC3300</td>
    <td bgcolor="CC0000">8F - CC0000</td>
  </tr>
  <tr>
    <td bgcolor="99FF66">90 - 99FF66</td>
    <td bgcolor="99CC66">91 - 99CC66</td>
    <td bgcolor="999966">92 - 999966</td>
    <td bgcolor="996666">93 - 996666</td>
    <td bgcolor="993366">94 - 993366</td>
    <td bgcolor="990066">95 - 990066</td>
  </tr>
  <tr>
    <td bgcolor="99FF33">96 - 99FF33</td>
    <td bgcolor="99CC33">97 - 99CC33</td>
    <td bgcolor="999933">98 - 999933</td>
    <td bgcolor="996633">99 - 996633</td>
    <td bgcolor="993333">9A - 993333</td>
    <td bgcolor="990033">9B - 990033</td>
  </tr>
  <tr>
    <td bgcolor="99FF00">9C - 99FF00</td>
    <td bgcolor="99CC00">9D - 99CC00</td>
    <td bgcolor="999900">9E - 999900</td>
    <td bgcolor="996600">9F - 996600</td>
    <td bgcolor="993300">A0 - 993300</td>
    <td bgcolor="990000">A1 - 990000</td>
  </tr>
  <tr>
    <td bgcolor="66FF66">A2 - 66FF66</td>
    <td bgcolor="66CC66">A3 - 66CC66</td>
    <td bgcolor="669966">A4 - 669966</td>
    <td bgcolor="666666">A5 - 666666</td>
    <td bgcolor="663366"><font color="#FFFFFF">A6 - 663366</font></td>
    <td bgcolor="660066"><font color="#FFFFFF">A7 - 660066</font></td>
  </tr>
  <tr>
    <td bgcolor="66FF33">A8 - 66FF33</td>
    <td bgcolor="66CC33">A9 - 66CC33</td>
    <td bgcolor="669933">AA - 669933</td>
    <td bgcolor="666633">AB - 666633</td>
    <td bgcolor="663333"><font color="#FFFFFF">AC - 663333</font></td>
    <td bgcolor="660033"><font color="#FFFFFF">AD - 660033</font></td>
  </tr>
  <tr>
    <td bgcolor="66FF00">AE - 66FF00</td>
    <td bgcolor="66CC00">AF - 66CC00</td>
    <td bgcolor="669900">B0 - 669900</td>
    <td bgcolor="666600">B1 - 666600</td>
    <td bgcolor="663300"><font color="#FFFFFF">B2 - 663300</font></td>
    <td bgcolor="660000"><font color="#FFFFFF">B3 - 660000</font></td>
  </tr>
  <tr>
    <td bgcolor="33FF66">B4 - 33FF66</td>
    <td bgcolor="33CC66">B5 - 33CC66</td>
    <td bgcolor="339966">B6 - 339966</td>
    <td bgcolor="336666">B7 - 336666</td>
    <td bgcolor="333366"><font color="#FFFFFF">B8 - 333366</font></td>
    <td bgcolor="330066"><font color="#FFFFFF">B9 - 330066</font></td>
  </tr>
  <tr>
    <td bgcolor="33FF33">BA - 33FF33</td>
    <td bgcolor="33CC33">BB - 33CC33</td>
    <td bgcolor="339933">BC - 339933</td>
    <td bgcolor="336633">BD - 336633</td>
    <td bgcolor="333333"><font color="#FFFFFF">BE - 333333</font></td>
    <td bgcolor="330033"><font color="#FFFFFF">BF - 330033</font></td>
  </tr>
  <tr>
    <td bgcolor="33FF00">C0 - 33FF00</td>
    <td bgcolor="33CC00">C1 - 33CC00</td>
    <td bgcolor="339900">C2 - 339900</td>
    <td bgcolor="336600">C3 - 336600</td>
    <td bgcolor="333300"><font color="#FFFFFF">C4 - 333300</font></td>
    <td bgcolor="330000"><font color="#FFFFFF">C5 - 330000</font></td>
  </tr>
  <tr>
    <td bgcolor="00FF66">C6 - 00FF66</td>
    <td bgcolor="00CC66">C7 - 00CC66</td>
    <td bgcolor="009966">C8 - 009966</td>
    <td bgcolor="006666">C9 - 006666</td>
    <td bgcolor="003366"><font color="#FFFFFF">CA - 003366</font></td>
    <td bgcolor="000066"><font color="#FFFFFF">CB - 000066</font></td>
  </tr>
  <tr>
    <td bgcolor="00FF33">CC - 00FF33</td>
    <td bgcolor="00CC33">CD - 00CC33</td>
    <td bgcolor="009933">CE - 009933</td>
    <td bgcolor="006633">CF - 006633</td>
    <td bgcolor="003333"><font color="#FFFFFF">D0 - 003333</font></td>
    <td bgcolor="000033"><font color="#FFFFFF">D1 - 000033</font></td>
  </tr>
  <tr>
    <td bgcolor="00FF00">D2 - 00FF00</td>
    <td bgcolor="00CC00">D3 - 00CC00</td>
    <td bgcolor="009900">D4 - 009900</td>
    <td bgcolor="006600">D5 - 006600</td>
    <td bgcolor="003300"><font color="#FFFFFF">D6 - 003300</font></td>
    <td bgcolor="111111"><font color="#FFFFFF">D7 - 111111</font></td>
  </tr>
  <tr>
    <td bgcolor="222222"><font color="#FFFFFF">D8 - 222222</font></td>
    <td bgcolor="444444"><font color="#FFFFFF">D9 - 444444</font></td>
    <td bgcolor="555555"><font color="#FFFFFF">DA - 555555</font></td>
    <td bgcolor="777777">DB - 777777</td>
    <td bgcolor="888888">DC - 888888</td>
    <td bgcolor="AAAAAA">DD - AAAAAA</td>
  </tr>
  <tr>
    <td bgcolor="BBBBBB">DE - BBBBBB</td>
    <td bgcolor="DDDDDD">DF - DDDDDD</td>
    <td bgcolor="EEEEEE">E0 - EEEEEE</td>
    <td bgcolor="C0C0C0">E1 - C0C0C0</td>
    <td bgcolor="800000">E2 - 800000</td>
    <td bgcolor="800080">E3 - 800080</td>
  </tr>
  <tr>
    <td bgcolor="008000">E4 - 008000</td>
    <td bgcolor="008080">E5 - 008080</td>
    <td bgcolor="000000"><font color="#FFFFFF">E6 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">E7 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">E8 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">E9 - 000000</font></td>
  </tr>
  <tr>
    <td bgcolor="000000"><font color="#FFFFFF">EA - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">EB - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">EC - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">ED - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">EE - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">EF - 000000</font></td>
  </tr>
  <tr>
    <td bgcolor="000000"><font color="#FFFFFF">F0 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F1 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F2 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F3 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F4 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F5 - 000000</font></td>
  </tr>
  <tr>
    <td bgcolor="000000"><font color="#FFFFFF">F6 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F7 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F8 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">F9 - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">FA - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">FB - 000000</font></td>
  </tr>
  <tr>
    <td bgcolor="000000"><font color="#FFFFFF">FC - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">FD - 000000</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">FE - 000000</font></td>
    <td bgcolor="060003"><font color="#FFFFFF">FF - 060003</font></td>
  </tr>
</table>
<p>
 * In GrayScale devices with PalmOS version greater or equal to 3.5 that supports 16 colors:
<p>
<table border="0" cellpadding="5">
  <tr>
    <td bgcolor="FFFFFF">00 - FFFFFF</td>
    <td bgcolor="EEEEEE">01 - EEEEEE</td>
    <td bgcolor="DDDDDD">02 - DDDDDD</td>
    <td bgcolor="CCCCCC">03 - CCCCCC</td>
    <td bgcolor="BBBBBB">04 - BBBBBB</td>
    <td bgcolor="AAAAAA">05 - AAAAAA</td>
  </tr>
  <tr>
    <td bgcolor="999999">06 - 999999</td>
    <td bgcolor="888888">07 - 888888</td>
    <td bgcolor="777777">08 - 777777</td>
    <td bgcolor="666666">09 - 666666</td>
    <td bgcolor="555555">0A - 555555</td>
    <td bgcolor="444444">0B - 444444</td>
  </tr>
  <tr>
    <td bgcolor="333333"><font color="#FFFFFF">0C - 333333</font></td>
    <td bgcolor="222222"><font color="#FFFFFF">0D - 222222</font></td>
    <td bgcolor="111111"><font color="#FFFFFF">0E - 111111</font></td>
    <td bgcolor="000000"><font color="#FFFFFF">0F - 000000</font></td>
  </tr>
</table>
<p>
 * In GrayScale devices with PalmOS version prior to 3.5 that supports 4 colors:
<p>
<table border="0" cellpadding="5">
  <tr>
    <td bgcolor="FFFFFF">00 - FFFFFF</td>
    <td bgcolor="BFBFBF">01 - BFBFBF</td>
    <td bgcolor="7F7F7F">02 - 7F7F7F</td>
    <td bgcolor="000000"><font color="#FFFFFF">03 - 000000</font></td>
  </tr>
</table>
<p>
 * The following code generate a new palette with the 216 web safe colors:
 * <pre>
 * int[] samples = {0x00, 0x33, 0x66, 0x99, 0xcc, 0xff};
 * int index = 0;
 * for (int r = 0; r < 6; r++)
 *    for (int g = 0; g < 6; g++)
 *       for (int b = 0; b < 6; b++)
 *       {
 *          Color c = new Color(samples[r], samples[g], samples[b]);
 *       }
 * </pre>
 */

public final class Palette
{
   // In the device, white (FFFFFF) is the absense of color (0) and
   // black (000000) is the pixel drawed at maximum intensity (3 or 15).
   // Thus, they have inverted palettes
   public static final int default16PaletteScr2Dev[] =
   {
      0xFFFFFF,0xEEEEEE,0xDDDDDD,0xCCCCCC,0xBBBBBB,0xAAAAAA,0x999999,0x888888,
      0x777777,0x666666,0x555555,0x444444,0x333333,0x222222,0x111111,0x000000
   };
   public static final int default16PaletteDev2Scr[] =
   {
      0x000000,0x111111,0x222222,0x333333,0x444444,0x555555,0x666666,0x777777,
      0x888888,0x999999,0xAAAAAA,0xBBBBBB,0xCCCCCC,0xDDDDDD,0xEEEEEE,0xFFFFFF
   };
   public static final int default4PaletteScr2Dev[]  =
   {
      0xFFFFFF,0xBFBFBF,0x7F7F7F,0x000000
   };
   public static final int default4PaletteDev2Scr[]  =
   {
      0x000000,0x7F7F7F,0xBFBFBF,0xFFFFFF
   };

   private Color []colors;
   private IntHashtable ihMatchedColors;
   private int maxIndex;
   /** System palette. In Palm OS, if color, this is the WebSafe palette; if gray scale, this is a gray palette with 4 or 16 colors */
   public static final Palette SystemPalette = new Palette();

   /** Create a palette containing the current applied palette. In PalmOS it is the WebSafe palette if in color or a gray palette if not color. You can also use the SystemPalette to get the default system palette. */
   public Palette()
   {
      setColors(Settings.maxColors ==  16 ? default16PaletteDev2Scr :
                Settings.maxColors ==   4 ? default4PaletteDev2Scr
                                          : NativeMethods4JDK.paletteGetCurrentPalette(this));
      maxIndex = this.colors.length-1;
   }
   /** Create a Palette with the given colors array. if the array length is > 256 or the array is null, the system palette is assigned. */
   public Palette(Color []colors)
   {
      if (colors == null || colors.length > 256)
         setColors(NativeMethods4JDK.paletteGetCurrentPalette(this));
      else
         this.colors = colors;
      maxIndex = this.colors.length-1;
   }
   /** construct the palette getting the rgb values from the int array */
   public Palette(int []colorRGBs)
   {
      setColors(colorRGBs);
      maxIndex = this.colors.length-1;
   }
   private void setColors(int []colorRGBs)
   {
      colors = new Color[colorRGBs.length];
      int n = colors.length;
      for (int i =0; i < n; i++)
         colors[i] = new Color(colorRGBs[i]);
   }
   /** return an int array containing the rgb values of this palette, from
    * startIndex to endIndex (including it). You can pass -1,-1 to include
    * all palette; or 100,-1 to get from 100 up to the last element. This method
    * is useful to save this palette into a stream.
    * @return a sub-array containing only the colors whoose indexes you asked for.
    */
   public int []toRGBArray(int startIndex, int endIndex)
   {
      if (startIndex < 0) startIndex = 0;
      if (endIndex < 0) endIndex = maxIndex;
      if (endIndex < startIndex) return null;
      int []ai = new int[endIndex-startIndex+1];
      for (int i =startIndex,j=0; i <= endIndex; i++,j++)
         ai[j] = colors[i].rgb;
      return ai;
   }

   /** returns a copy of the color array assigned with this Palette */
   public Color []getColors()
   {
      Color []cs = new Color[colors.length];
      Vm.copyArray(colors,0,cs,0,colors.length);
      return cs;
   }
   /** sets the color in the given index */
   public void setColor(int index, Color color)
   {
      if (index <= maxIndex)
         colors[index] = color;
   }
   /** sets a range of colors in this palette, replacing all colors from toIndex to toIndex+aColor.length-1. */
   public void setColors(Color []aColor, int toIndex)
   {
      Vm.copyArray(aColor,0,colors,toIndex,aColor.length);
   }
   /** gets the color from the given index. it puts index in the range, if necessary. */
   public Color getColor(int index)
   {
      return colors[index<0?0 : index>maxIndex?maxIndex : index];
   }
   /** returns the number of colors in this palette */
   public int size()
   {
      return colors.length;
   }
   /** does a linear search for the given color in this palette and returns its index. if not found, return -1. */
   public int find(Color color)
   {
      int rgb = color.rgb;
      int n = colors.length;
      for (int i =0; i < n; i++)
         if (colors[i].rgb == rgb)
            return i;
      return -1;
   }
   /** search for the nearest color of the given color in this palette and return its index. */
   public int findNearest(Color color)
   {
      int nc=0;
      int sdist = 255 * 255 * 255 + 1;
      int r = color.red;
      int g = color.green;
      int b = color.blue;
      int n = colors.length;
      for (int i =0; i < n; i++)
      {
         Color c = colors[i];
         int dist = (r-c.red)*(r-c.red) + (g-c.green)*(g-c.green) + (b-c.blue)*(b-c.blue);
         if (dist < sdist)
         {
            nc = i;
            sdist = dist;
         }
      }
      return nc;
   }

   /** fade the colors from startIndex to endIndex out (to black) or in (to white) and return true if any color had changed
   public boolean fade(boolean out, int startIndex, int endIndex)
   {
      boolean b = false;
      if (startIndex < 0) startIndex = 0;
      if (endIndex < 0) endIndex = maxIndex;
      int step = out?-1:1;
      int limit = out?0:255;
      for (int i = startIndex; i <= endIndex; i++)
      {
         Color c = colors[i];
         if (c.red != limit) {c.red += step; b = true;}
         if (c.green != limit) {c.green += step; b = true;}
         if (c.blue != limit) {c.blue += step; b = true;}
         c.rgb = (c.red << 16) | (c.green << 8) | c.blue;
      }
      return b;
   } guich@320: removed this method. */

   /** Gets the color index of the given color in this Palette. If the
     * color isnt in this palette, returns the nearest color index. The
     * searched colors are stored in a hash table to improve performance
     */
   public int getColorIndex(Color color)
   {
      int b;
      if (ihMatchedColors == null) ihMatchedColors = new IntHashtable(colors.length*3/2);
      // convert from rgb to the active's palette index
      int key = color.equ; // or rgb?
      if ((b = ihMatchedColors.get(key)) == IntHashtable.INVALID) // already found?
      {
         if (Settings.isColor)
         {
            if ((b = find(color)) == -1) // no, find the exact match
               b = findNearest(color); // color not found, find the nearest match
         }
         else b = Settings.maxColors-1-color.equ;
         ihMatchedColors.put(key,b);
      }
//      Vm.debug("match "+Convert.toString(color.rgb,16)+" = "+b+" ("+Convert.toString(colors[b].rgb,16)+')');
      return b;
   }
   public static int[] getDeviceRGBs()
   {
      return Settings.maxColors>=256?SystemPalette.toRGBArray(-1,-1):Settings.maxColors==16?default16PaletteScr2Dev:default4PaletteScr2Dev;
   }
}

