/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.fx;


/**
 * Rect is a rectangle.
 */

public class Rect
{
   /** x position */
   public int x;
   /** y position */
   public int y;
   /** rectangle width */
   public int width;
   /** rectangle height */
   public int height;

   public static final int SAME = 0; // 0 makes more sense

   /** Constructs a rectangle with all args = 0. */
   public Rect()
   {
      // they are already 0 as default.
   }

   /** Constructs a rectangle with the given x, y, width and height. */
   public Rect(int x, int y, int width, int height)
	{
	   this.x = x;
	   this.y = y;
	   this.width = width;
	   this.height = height;
	}

   /** Constructs a rectangle with the given coords */
   public Rect(Coord topleft, Coord bottomright)
   {
      set(topleft.x,topleft.y,bottomright.x-topleft.x+1,bottomright.y-topleft.y+1);
   }

   /** sets the coords of this rect. */
   public void set(int x, int y, int width, int height)
   {
	   this.x = x;
	   this.y = y;
	   this.width = width;
	   this.height = height;
	}

   /** sets the coords of this rect. */
   public void set(Rect r) // guich@210_4
   {
	   this.x = r.x;
	   this.y = r.y;
	   this.width = r.width;
	   this.height = r.height;
	}

   /** returns true if the point xx,yy is inside this rect. added by guich */
   public boolean contains(int xx, int yy)
   {
      return x <= xx && xx < x+width && y <= yy && yy < y+height;
   }
   public String toString()
   {
      return ""+x+','+y+','+width+','+height;
   }
   /** translates this rect. the new positions will be this.x+xx,this.y+yy. */
   public void translate(int deltaX, int deltaY)
   {
      this.x += deltaX;
      this.y += deltaY;
   }
   /** returns a new rect modified in the specified parameters. you can use the constant SAME so the value is not modified (or use 0). */
   public Rect modifiedBy(int deltaX, int deltaY, int deltaW, int deltaH)
   {
      return new Rect(deltaX==SAME?x:(x+deltaX), deltaY==SAME?y:(y+deltaY), deltaW==SAME?width:(width+deltaW), deltaH==SAME?height:(height+deltaH));
   }
   /** modifies this rect with the specified parameters. you can use the constant SAME so the value is not modified (or use 0). */
   public void modify(int deltaX, int deltaY, int deltaW, int deltaH)
   {
      x += deltaX;
      y += deltaY;
      width += deltaW;
      height += deltaH;
   }
   /** returns x+width-1 */
   public int x2()
   {
      return x+width-1;
   }
   /** returns y+height-1 */
   public int y2()
   {
      return y+height-1;
   }
   /** returns true if this rectangle intersects with the given one */
   public boolean intersects(Rect r)
   {
	   return !((r.x + r.width <= x) || (r.y + r.height <= y) || (r.x >= x + width) || (r.y >= y + height));
   }
   /** Modify this Rect by doing the intersection with the given rect. Returns <code>this</code>. */
   public Rect intersectWith(Rect r)
   {
	   int x1 = Math.max(x, r.x);
	   int x2 = Math.min(x + width, r.x + r.width);
	   int y1 = Math.max(y, r.y);
	   int y2 = Math.min(y + height, r.y + r.height);
	   this.x = x1;
	   this.y = y1;
	   this.width  = x2 - x1;
	   this.height = y2 - y1;
	   return this;
   }

   /** Modify this Rect by doing an union with the given rect. Returns <code>this</code> */
   public Rect unionWith(Rect r)
   {
	   int x1 = Math.min(x, r.x);
	   int x2 = Math.max(x + width, r.x + r.width);
	   int y1 = Math.min(y, r.y);
	   int y2 = Math.max(y + height, r.y + r.height);
	   this.x = x1;
	   this.y = y1;
	   this.width  = x2 - x1;
	   this.height = y2 - y1;
	   return this;
   }

   /** Returns true if the bounds of this Rect and the given one are the same */
   public boolean equals(Object other) // guich@240_2
   {
      if (other instanceof Rect)
      {
         Rect r = (Rect)other;
         return r.x == this.x && r.y == this.y && r.width == this.width && r.height == this.height;
      }
      return false;
   }
}