/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.fx;


/**
 * SoundClip is a sound clip.
 * <p>
 * Support for sound clips varies between platforms. Under Java, sound clips are
 * only supported by Java applets - not applications. This is primarily because
 * AudioClips in Java weren't supported in Java applications until the JDK 1.2
 * without using undocumented method calls. Even when using applets, some Java
 * virtual machines support .wav and .au sound files and some versions don't
 * seem to support either format.
 * <p>
 * Using a SuperWaba Virtual Machine, .wav format sound clips are supported under
 * Win32, WinCE and Palm OS 5. The SuperWaba VM under PalmOS smaller than 5 has no support for sound clips.
 * Under Win32 and WinCE, the .wav files for sound clips must exist in a file
 * outside of the programs warp (resource) file. In Palm OS 5, the wav must be added to the pdb
 * (Warp does this automagically. Just reference a .wav and it will be added)
 * <p>
 * If you're playing a sound clip under a Windows CE device and you don't hear
 * anything, make sure that the device is set to allow programs to play sounds.
 * To check the setting, look at:
 * <p>
 * Start->Settings->Volume & Sounds
 * <p>
 * for the check box:
 * <p>
 * Enable sounds for: Programs
 * <p>
 * If it is not checked on, sound clips won't play.
 * <p>
 * Here is an example that plays a sound:
 *
 * <pre>
 * SoundClip s = new SoundClip("sound.wav");
 * s.play();
 * </pre>
 * Under Palm OS 5, the currently supported formats are: uncompressed (PCM) or IMA
 * 4-bit adaptive differential (IMA ADPCM). The ADPCM type is also known as DVI
 * ADPCM; in a WAVE file, it's known as format 0x11. One or two-channels; All
 * normal sampling rates (8k, 11k, 22.05k, 44.1k, 48, 96k). Also, since it is going
 * to be stored in a pdb, the wav's size is limited to 64kb.
 */
public final class SoundClip
{
   protected String path;
   boolean loaded;

   /**
   * Loads and constructs a sound clip from a file. In Windows, you must specify the full path. In Palm OS 5
   */
   public SoundClip(String path)
   {
	   this.path = path;
   }

   /**
   * Plays the sound clip. Returns true if the sound starts playing and false otherwise.
   */
   public boolean play()
   {
      return NativeMethods4JDK.soundClipPlay(this);
   }

}