/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.io;

import waba.sys.*;

/** creates a byte array stream. it can be useful with the method
  * Catalog.inspectRecord, since that method returns a byte array.
  * When writting, the buffer expands itself if needed.
  * This class cannot be used for output AND input, but only for output OR input. */

public final class ByteArrayStream extends Stream
{
   private int pos;
   private byte []buffer;

   /** sets the internal buffer to be the specified buffer param. */
   public ByteArrayStream(byte []buffer)
   {
      this.buffer = buffer;
      pos = 0;
   }
   /** creates a new buffer with the specific size. CANNOT BE 0! */
   public ByteArrayStream(int size)
   {
      this(new byte[size]);
   }
   /** does nothing.  */
   public boolean close()
   {
      // dont set buffer to null here or the Catalog class will stop working!
      return true;
   }
   /** gets the buffer used. the size of the buffer may be different of the array length. get its size in count(). */
   public byte []getBuffer()
   {
     return buffer;
   }
   /** returns the current position in the buffer. */
   public int count()
   {
      return pos;
   }
   /** transfers count bytes from class buffer to buf. <b>Does not perform range checking!</b>
   @return the number of bytes read. */
   public int readBytes(byte buf[], int start, int count)
   {
      Vm.copyArray(buffer,pos,buf,start,count);
      pos += count;
      return count;
   }
   /** resets the position to 0 so the buffer can be reused. */
   public void reset()
   {
      pos = 0;
   }
   
   /** Skips the number of bytes from the current position. It may be < than 0. 
     * @returns The number of bytes skipped, or -1 if error.
     * @since SuperWaba 4.01
     */
   public int skipBytes(int count) // guich@401_33
   {
      int off = pos + count;
      if (off < 0 || off > buffer.length) // guich@210_11: added support to go backwards
         return -1;
      pos += count;
      return count;
   }
   
   /** Reuses the already read part of the buffer. This method shifts the buffer
     * from the current position to 0, so you can reuse the buffer, avoiding
     * exceed the memory limits.
     * @returns The number of bytes shifted
     * @since SuperWaba 4.01
     */
   public int reuse() // guich@401_34
   {
      int shifted = pos;
      if (pos > 0)
      {
         Vm.copyArray(buffer, pos, buffer, 0, buffer.length-pos);
         pos = 0;
      }
      return shifted;
   }
   
   /** this writes to the byte array, expanding it if necessary. returns the number of bytes written.
   @since SuperWaba2_0beta2 */
   public int writeBytes(byte buf[], int start, int count)
   {
      if (buffer.length < (count+pos)) // grow the buffer
      {
         byte []newBuffer = new byte[(count+pos)*12/10]; // grows 20% above the new needed capacity
         Vm.copyArray(buffer,0,newBuffer,0,pos);
         buffer = newBuffer;
      }
      Vm.copyArray(buf,start,buffer,pos,count);
      pos += count;
      return count;
   }

   public boolean isOpen() // guich@330_40
   {
      return true;
   }
}