/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.io;

import waba.util.Hashtable;
import waba.util.Vector;
import waba.sys.Vm;
import waba.sys.Convert;
import waba.applet.JavaBridge;
import waba.io.ByteArrayStream;
import waba.io.DataStream;
import waba.io.Stream;
import waba.io.File;
import java.lang.reflect.*;

/** This class is for internal use only. There are no public methods. */

final public class NativeMethods4JDK
{
   ////////////////////////////////////////////////////////////////////////////////////
   //                                waba.io.SerialPort                              //
   ////////////////////////////////////////////////////////////////////////////////////
   // guich@250: added support for Serial at desktop. Thanks to Michal Hobot
   // hobot@320_22
   static void serialPortCreate(SerialPort obj, int number, int baudRate, int bits, boolean parity, int stopBits)
   {
      try
      {
         // guich@330_38: use introspection to be able to compile everything without the comm.jar
         // javax.comm.CommPortIdentifier portId = javax.comm.CommPortIdentifier.getPortIdentifier(getPortName(number));
         Class cpi = Class.forName("javax.comm.CommPortIdentifier"); // get the class

         String portName = getPortName(number);
         System.out.println("Port found: "+portName);

         Method mGetPortIdentifier = cpi.getMethod("getPortIdentifier", new Class[]{String.class}); // get the method getPortIdentifier(String)
         Object portId = mGetPortIdentifier.invoke(null,new Object[]{portName}); // invoke the getPortIdentifier(getPortName(number)) method

         // javax.comm.SerialPort port = (javax.comm.SerialPort)portId.open("SuperWaba", 1000);
         Method mOpen = portId.getClass().getMethod("open", new Class[]{String.class,int.class}); // get the method getPortIdentifier(String)
         Object port = mOpen.invoke(portId,new Object[]{"SuperWaba",new Integer(1000)}); // invoke the portId.open method

         // port.setSerialPortParams(baudRate, bits, stopBits, parity?javax.comm.SerialPort.PARITY_EVEN:javax.comm.SerialPort.PARITY_NONE);
         Class cSerialPort = port.getClass().getSuperclass();
         Method mSetSerialPortParams = cSerialPort.getMethod("setSerialPortParams", new Class[]{int.class,int.class,int.class,int.class}); // get the method setSerialPortParams
         Field parityField = cSerialPort.getField(parity ? "PARITY_EVEN" : "PARITY_NONE");
         mSetSerialPortParams.invoke(port,new Object[]{new Integer(baudRate), new Integer(bits), new Integer(stopBits), new Integer(parityField.getInt(port))});

         // port.enableReceiveTimeout(100);
         Method mEnableReceiveTimeout = cSerialPort.getMethod("enableReceiveTimeout", new Class[]{int.class});
         mEnableReceiveTimeout.invoke(port,new Object[]{new Integer(100)});

         // obj.thisInputStream = port.getInputStream();
         Method mGetInputStream = cSerialPort.getMethod("getInputStream", null);
         obj.thisInputStream = mGetInputStream.invoke(port,null);

         // obj.thisOutputStream = port.getOutputStream();
         Method mGetOutputStream = cSerialPort.getMethod("getOutputStream", null);
         obj.thisOutputStream = mGetOutputStream.invoke(port,null);

         obj.thisSerialPort = port;
      }
      catch (java.lang.reflect.InvocationTargetException ule)
      {
         ule.printStackTrace();
         System.out.println("Exception thrown when creating SerialPort: "+ule.getMessage());
         System.out.println("The probable cause of this message is that the port is in use (You must kill HotSync)\nor that the win32com.dll is not accessible in your path. Please add 'SuperwabaSDK/utils/xplat/commapi' to your path or add '-Djava.library.path=SuperwabaSDK/utils/xplat/commapi' to your java.exe command");
         obj.thisSerialPort = null;
         obj.lastError = 65525; // NO_SERIAL_LIBRARY_AVAILABLE
      }
      catch (java.lang.NoClassDefFoundError e)
      {
         System.out.println("Cannot find javax.comm.CommPortIdentifier. You must add the SuperwabaSDK/utils/xplat/commapi/comm.jar file to your classpath. If you're not using Windows, then you must download the respective Comm api at http://java.sun.com/products/javacomm/");
         obj.thisSerialPort = null;
         obj.lastError = 65525; // NO_SERIAL_LIBRARY_AVAILABLE
      }
      catch (Throwable t)
      {
         System.out.println("Exception occured when creating serial port: "+t.toString());
         t.printStackTrace();
         obj.thisSerialPort = null;
         obj.lastError = 771; // NoMem (?)
      }
      if (obj.thisSerialPort == null)
         System.exit(0);
   }

   private static String portDEFAULT, portIRCOMM, portSIR, portUSB;

   private static void initSWserial()
   {
      java.util.ResourceBundle swserial;
      try
      {
         swserial = java.util.ResourceBundle.getBundle("swserial");
         portDEFAULT = swserial.getString("DEFAULT");
         portIRCOMM = swserial.getString("IRCOMM");
         portSIR = swserial.getString("SIR");
         portUSB = swserial.getString("USB");
      }
      catch (java.util.MissingResourceException e)
      {
         System.out.println("Error: missing resource bundle: "+e.getClassName());
         System.out.println("Using defaults..."); // guich@400_20
         portDEFAULT="COM1";
         portIRCOMM="COM2";
         portSIR="COM3";
         portUSB="COM4";
      }
   }

   /**
    * @return When number == Logical port number defined in waba.io.SerialPort -
    * name of port assigned in swserial.properties file.
    * For other values - returns first port name with given number inside of it.
    */
   static private String getPortName(int number)
   {
      if (portDEFAULT == null)
         initSWserial();

      switch(number)
      {
         case SerialPort.DEFAULT: return portDEFAULT;
         case SerialPort.IRCOMM:  return portIRCOMM;
         case SerialPort.SIR:     return portSIR;
         case SerialPort.USB:     return portUSB;
      }

      try
      {
         //java.util.Enumeration portIds = javax.comm.CommPortIdentifier.getPortIdentifiers();
         Class cpi = Class.forName("javax.comm.CommPortIdentifier"); // get the class
         Method mGetPortIdentifiers = cpi.getMethod("getPortIdentifiers", null);
         java.util.Enumeration portIds = (java.util.Enumeration)mGetPortIdentifiers.invoke(null,null);

         if (number == 0)
         {
            if (portIds.hasMoreElements())
            {
               // return ((javax.comm.CommPortIdentifier)portIds.nextElement()).getName();
               Object oCommPortIdentifier = portIds.nextElement();
               Method mGetName = oCommPortIdentifier.getClass().getMethod("getName", null);
               String portName = (String)mGetName.invoke(oCommPortIdentifier,null);
               return portName;
            }
         }
         else
         {
            while (portIds.hasMoreElements())
            {
               // portName = ((javax.comm.CommPortIdentifier)portIds.nextElement()).getName();
               Object oCommPortIdentifier = portIds.nextElement();
               Method mGetName = oCommPortIdentifier.getClass().getMethod("getName", null);
               String portName = (String)mGetName.invoke(oCommPortIdentifier,null);
               if (portName.indexOf(number+'0') != -1)
                  return portName;
            }
         }
      } catch (Exception ee) {ee.printStackTrace();}
      return null;
   }
   static boolean serialPortClose(SerialPort obj)
   {
      try
      {
         // ((javax.comm.SerialPort)obj.thisSerialPort).close();
         Method m = obj.thisSerialPort.getClass().getSuperclass().getMethod("close", null);
         m.invoke(obj.thisSerialPort,null);
         return true;
      }
      catch (Throwable t)
      {
         t.printStackTrace();
         obj.lastError = 65525; // NO_SERIAL_LIBRARY_AVAILABLE
         return false;
      }
   }
   static boolean serialPortIsOpen(SerialPort obj)
   {
      return obj.thisSerialPort != null;
   }
   static boolean serialPortSetFlowControl(SerialPort obj, boolean on)
   {
      try
      {
         //((javax.comm.SerialPort)obj.thisSerialPort).setFlowControlMode(on?javax.comm.SerialPort.FLOWCONTROL_RTSCTS_IN|javax.comm.SerialPort.FLOWCONTROL_RTSCTS_OUT:javax.comm.SerialPort.FLOWCONTROL_NONE);
         Field fRTSCTS_IN = obj.thisSerialPort.getClass().getSuperclass().getField("FLOWCONTROL_RTSCTS_IN");
         Field fRTSCTS_OUT = obj.thisSerialPort.getClass().getSuperclass().getField("FLOWCONTROL_RTSCTS_OUT");
         Field fNONE = obj.thisSerialPort.getClass().getSuperclass().getField("FLOWCONTROL_NONE");
         Method m = obj.thisSerialPort.getClass().getSuperclass().getMethod("setFlowControlMode", new Class[]{int.class});
         m.invoke(obj.thisSerialPort,new Object[]{on ? new Integer(fRTSCTS_IN.getInt(obj.thisSerialPort)|fRTSCTS_OUT.getInt(obj.thisSerialPort)) : new Integer(fNONE.getInt(obj.thisSerialPort))});
         return true;
      }
      catch (Throwable t)
      {
         t.printStackTrace();
         obj.lastError = 778; // NotSupported
         return false;
      }
   }
   static boolean serialPortSetReadTimeout(SerialPort obj, int millis)
   {
      try
      {
         // ((javax.comm.SerialPort)obj.thisSerialPort).enableReceiveTimeout(millis);
         Method mEnableReceiveTimeout = obj.thisSerialPort.getClass().getSuperclass().getMethod("enableReceiveTimeout", new Class[]{int.class});
         mEnableReceiveTimeout.invoke(obj.thisSerialPort,new Object[]{new Integer(millis)});
         return true;
      }
      catch (Throwable t)
      {
         t.printStackTrace();
         obj.lastError = 778; // NotSupported
         return false;
      }
   }
   static int serialPortReadBytes(SerialPort obj, byte buf[], int start, int count)
   {
      try
      {
         Method m = obj.thisInputStream.getClass().getSuperclass().getMethod("read", new Class[]{byte[].class,int.class,int.class});
         // guich@330_42: make sure we have read everything
         int total = count;
         do
         {
            //return ((java.io.InputStream)obj.thisInputStream).read(buf, start, count);
            Integer i = (Integer)m.invoke(obj.thisInputStream,new Object[]{buf,new Integer(start),new Integer(count)});
            int read = i.intValue();
            start += read;
            count -= read;
         }
         while (count > 0);
         return total;
      }
      catch (Throwable t)
      {
         t.printStackTrace();
         obj.lastError = 778; // NotSupported
      }
      return -1;
   }
   static int serialPortReadCheck(SerialPort obj)
   {
      try
      {
         //return ((java.io.InputStream)obj.thisInputStream).available();
         Method m = obj.thisInputStream.getClass().getSuperclass().getMethod("available", null);
         Integer i = (Integer)m.invoke(obj.thisInputStream,null);
         return i.intValue();
      }
      catch (Throwable t)
      {
         t.printStackTrace();
      }
      return -1;
   }

   static int serialPortWriteBytes(SerialPort obj, byte buf[], int start, int count)
   {
      try
      {
         // guich@330_42: make sure we have wrote everything
         //((java.io.OutputStream)obj.thisOutputStream).write(buf, start, count);
         // note: write returns void
         Method m = obj.thisOutputStream.getClass().getSuperclass().getMethod("write", new Class[]{byte[].class,int.class,int.class});
         m.invoke(obj.thisOutputStream,new Object[]{buf,new Integer(start),new Integer(count)});
         return count;
      }
      catch (Throwable t)
      {
         t.printStackTrace();
      }
      return -1;
   }
   ////////////////////////////////////////////////////////////////////////////////////
   //                                 waba.io.Socket                                 //
   ////////////////////////////////////////////////////////////////////////////////////
   static void socketCreate(Socket obj, String host, int port, int timeout, boolean noLinger)
   {
      obj._isOpen = false;
      try
      {
         obj._socket = new java.net.Socket(host, port);
         ((java.net.Socket)obj._socket).setSoTimeout(timeout);
         obj._in = ((java.net.Socket)obj._socket).getInputStream();
         obj._out = ((java.net.Socket)obj._socket).getOutputStream();
         obj._isOpen = true;
      }
      catch (Exception e) { e.printStackTrace(); }

      if (!obj._isOpen && obj._socket != null)
      {
         try { ((java.net.Socket)obj._socket).close(); }
         catch (Exception e) {}
      }
   }
   static boolean socketClose(Socket obj)
   {
      if (!obj._isOpen)
         return false;
      obj._isOpen = false;
      boolean pass = true;
      try
      {
         ((java.net.Socket)obj._socket).close();
      }
      catch (Exception e) { pass = false; }
      return pass;
   }
   static boolean socketIsOpen(Socket obj)
   {
      return obj._isOpen;
   }
   static boolean socketSetReadTimeout(Socket obj, int millis)
   {
      if (millis < 0)
         return false;
      if (millis == 0)
         millis = 1; // 0 means infinity in setSoTimeout()
      try
      {
         ((java.net.Socket)obj._socket).setSoTimeout(millis);
         return true;
      }
      catch (Exception e) { e.printStackTrace(); }
      return false;
   }
   static int socketReadBytes(Socket obj, byte buf[], int start, int count)
   {
      if (!obj._isOpen)
         return -1;
      int nread = -1;
      try
      {
         nread = ((java.io.InputStream)obj._in).read(buf, start, count);
      }
      catch (java.io.InterruptedIOException e) { return 0; }
      catch (Exception e) { e.printStackTrace(); }
      return nread;
   }
   static int socketWriteBytes(Socket obj, byte buf[], int start, int count)
   {
      if (!obj._isOpen)
         return -1;
      int nwrite = -1;
      try
      {
         ((java.io.OutputStream)obj._out).write(buf, start, count);
         nwrite = count;
      }
      catch (Exception e) { e.printStackTrace(); }
      return nwrite;
   }
   static boolean socketDisconnect()
   {
      System.out.println("Not implemented in this platform");
      return true;
   }
   ////////////////////////////////////////////////////////////////////////////////////
   //                                 waba.io.Catalog                                //
   ////////////////////////////////////////////////////////////////////////////////////
   static Hashtable _dbHash = new Hashtable(10);
   //static Hashtable htBags = new Hashtable(10); // guich@400_92
   static class CatalogBag
   {
      boolean _isOpen;
      String _name;
      int _mode;
      Vector _records = new Vector();
      Vector _attrs = new Vector(); // guich@200
      int _recordPos;
      String _creator; // guich@120
      String _type; // guich@120
      int originalModificationNumber; // guich@120
      boolean deleted; // guich@400_3

      public CatalogBag()
      {
         //htBags.put(this,this);
      }
      /** The following structure declaration represents an application
      * information block for an application that uses standard Palm OS
      * category information:
      * <pre>
      * typedef struct
      * {
      *    UInt16 renamedCategories;
      *    Char categoryLabels[16][16];
      *    UInt8 categoryUniqIDs[16];
      *    UInt8 lastUniqID;
      *    UInt8 padding;
      * } AppInfoType;.PDB and PRC Database Formats
      * </pre>
      * The Sort Information Block<br>
      * Field Descriptions<br>
      * . renamedCategories Specifies which categories have been renamed.<br>
      * . categoryLabel An array of 16 null-terminated category labels, each of which is 16 bytes long.<br>
      * . categoryID An array of 16 category ID values, each of which is one byte long.<br>
      * . lastUniqID The last unique category ID assigned.<br>
      * . padding Unused.<br>
      *
      */
      byte []appInfoBlock; // guich@200b4: store here the appinfoblock (with the categories)
      byte []sortInfoBlock; // guich@200b4

      /** The creation date of the database, specified as the number of seconds since
      * 12:00 A.M. on January 1, 1904.
      */
      int creationDate;
      /** The date of the most recent modification of the database, specified as the
      * number of seconds since 12:00 A.M. on January 1, 1904.
      */
      int modificationDate;
      /** The date of the most recent backup of the database, specified as the number
      * of seconds since 12:00 A.M. on January 1, 1904.
      */
      int lastBackupDate;
      /** The modification number of the database. */
      int modificationNumber;
      /** The attribute flags for the database. */
      short attributes = Catalog.DB_ATTR_BACKUP; // guich@330_44: don't force the Attrbackup bc the user may want to clear it
      /** The application-specific version of the database layout. */
      short version = 1;
      /** The local offset from the beginning of the database header data to the start
      * of the optional, application-specific appInfo block. This value is set to NULL
      * for databases that do not include an appInfo block.
      */
      int appInfoOffset;
      /** The local offset from the beginning of the PDB header data to the start of the
      * optional, application-specific sortInfo block. This value is set to NULL for
      * databases that do not include an sortInfo block.
      */
      int sortInfoOffset;
      /** Used internally by the Palm OS to generate unique identifiers for records on the
      * Palm device when the database is loaded into the device. For PRC databases, this
      * value is normally not used and is set to 0. For PQA databases, this value is not
      * used, and is set to 0.
      */
      int uniqueIDSeed;
      /** The local chunk ID of the next record list in this database. This is 0 if there
      * is no next record list, which is almost always the case. <b>Important!</b> In
      * SuperWaba, this type of database is not supported!
      */
      int nextRecordListID;

      // created by Mr. Tines to correct the date error
      private long getNow()
      {
         long timeX=new java.util.Date().getTime()/1000; // seconds since 1970
         // add of 66 years - Mac date is based in # of seconds since 1904
         // Don't forget the leap days!
         timeX += ((66 * 365) + 17) * 24 * 60 * 60;
         return timeX;
      }

      // created by Mr. Tines to correct the date error
      private void doSetDate()
      {
         long timeX=getNow();
         creationDate = (int)timeX;
         modificationDate = creationDate;
         timeX -= 365 * 24 * 60 * 60; // sub 1 year
         lastBackupDate = (int)timeX; // make sure we will be hotsync'ed!
      }
      /** converts the records to a pdb file. @param records a vector of array of bytes */
      // added by guich@120
      public void toPDB(Stream outStream, Vector records, String creatorId, String dbName, String typeId, Vector _attrs) throws Exception
      {
         byte name[] = new byte[32];
         byte type[]            = typeId.getBytes();
         byte creator[]         = creatorId.getBytes();

         // guich@200b4: verifies the db name
         int slash = -1;
         if ((slash=dbName.lastIndexOf('\\')) != -1 || (slash=dbName.lastIndexOf('/')) != -1)
            dbName = dbName.substring(slash+1);
         if (dbName.length() > 31)
            dbName = dbName.substring(0,31);
         // copies the db name to inside the array <name>
         byte []bn = dbName.getBytes();
         for (int i =0; i < name.length; i++)
            name[i] = (i < bn.length)?bn[i]:0;

         int numRecords = records.size(); // guich@340_48
         DataStream os = new DataStream(outStream);

         int offset = 80+numRecords*8;
         if (appInfoBlock != null)
         {
            appInfoOffset = offset;
            offset += appInfoBlock.length;
         }
         if (sortInfoBlock != null)
         {
            sortInfoOffset = offset;
            offset += sortInfoBlock.length;
         }

         //guich@330_44: commented out -> attributes |= Catalog.DB_ATTR_BACKUP;
         modificationDate = (int)getNow();

         // DatabaseHdrType
         os.writeBytes(name);
         os.writeShort(attributes);
         os.writeShort(version);
         os.writeInt(creationDate);
         os.writeInt(modificationDate);
         os.writeInt(lastBackupDate);
         os.writeInt(modificationNumber);
         os.writeInt(appInfoOffset);
         os.writeInt(sortInfoOffset);
         os.writeBytes(type);
         os.writeBytes(creator);
         os.writeInt(uniqueIDSeed);

         // RecordListType
         int nextRecordListID = 0;
         os.writeInt(nextRecordListID);
         os.writeShort(numRecords);
         byte []recUniqueID = {0,0,0}; // let the system create them again
         for (int i = 0; i < numRecords; i++)
         {
            os.writeInt(offset); // LocalChunkID
            if (_attrs == null || i >= _attrs.size())
               os.writeByte(0); // attributes
            else
               os.writeByte(((Byte)_attrs.elementAt(i)).byteValue() & ~0x20); // guich@300_64: disable all busy attributes
            // get back the uniqueID if already stored
            os.writeBytes(recUniqueID);
            int recSize = ((byte [])records.elementAt(i)).length;
            if (recSize > 65520)
               waba.applet.JavaBridge.print("CAUTION! RECORD "+i+" HAS A SIZE GREATER THAN 65520 AND WILL BE REJECTED BY PALM OS!"); // guich@230_12
            offset += recSize;
         }
         os.writeShort(0); // pad

         // AppInfoBlock (guich@200b4)
         if (appInfoBlock != null)
            os.writeBytes(appInfoBlock);

         for (int i=0; i < numRecords; i++)
            os.writeBytes((byte [])records.elementAt(i));
         os.close();
      }

      /** reads a pdb file and returns a vector of records. added by guich@120 */
      public Vector fromPDB(byte []all, String creatorId, String typeId, Vector _attrs) throws Exception
      {
         // ps: i had to read everything to an array because reading one record at a time was reading trash.
         DataStream is = new DataStream(new ByteArrayStream(all));

         if (all.length == 0) throw new Exception("Invalid File with 0 length!");
         // DatabaseHdrType
         byte name[] = new byte[32]; // ps: the writted string is in c++ format, so this routine doesnt loads the name correctly (comes trash with it)
         byte type[] = new byte[4];
         byte creator[] = new byte[4];

         is.readBytes(name);
         attributes = is.readShort();
         version = is.readShort();
         creationDate = is.readInt();
         modificationDate = is.readInt();
         lastBackupDate = is.readInt();
         modificationNumber = is.readInt();
         appInfoOffset = is.readInt();
         sortInfoOffset = is.readInt();
         is.readBytes(type);
         is.readBytes(creator);
         uniqueIDSeed = is.readInt();
         originalModificationNumber = modificationNumber;

         // verify if the creatorId is valid
         if (!creatorId.equals(new String(creator)) || !typeId.equals(new String(type)))
         {
            if (all.length >= 8 && new String(all,0,8).trim().equalsIgnoreCase("<HTML>")) // for some reason we got a webpage?
               return new Vector(); // return a empty db
            throw new Exception("Error: invalid pdb file! creatorId '"+creatorId+"' != '"+new String(creator)+"' and/or typeId '"+typeId+"' != '"+new String(type)+"' \n"+new String(all));
         }

         // RecordListType
         nextRecordListID = is.readInt();
         if (nextRecordListID != 0)
            throw new Exception("Invalid database file! SuperWaba does not support multiple record or resource lists in a Database!!!");
         int numRecords = is.readUnsignedShort(); // guich@340_48
         // reads the header (meaningless)
         int recOffsets[] = new int[numRecords+1];
         byte recAttributes;
         byte recUniqueID[] = new byte[3];
         if (_attrs != null) _attrs.clear();
         for (int i=0; i < numRecords; i++)
         {
            recOffsets[i] = is.readInt();  // offset
            recAttributes = is.readByte();
            is.readBytes(recUniqueID);
            if (_attrs != null) _attrs.addElement(new Byte(recAttributes));
         }
         recOffsets[numRecords] = all.length; // add the total size so we can compute the size of each record

         is.readShort(); // pad
         int offset = 80+numRecords*8;

         // guich@200b4: read the appInfoBlock and sortInfoBlock and store it in a safe place
         if (appInfoOffset > 0)
         {
            int len = sortInfoOffset > 0 ? (sortInfoOffset-offset) : (recOffsets[0]-offset);
            appInfoBlock = new byte[len];
            is.readBytes(appInfoBlock);
            offset += len;
         }
         if (sortInfoOffset > 0)
         {
            int len = recOffsets[0]-offset;
            sortInfoBlock = new byte[len];
            is.readBytes(sortInfoBlock);
            offset += len;
         }

         // the records were writted in sequence from here
         Vector v = new Vector(numRecords);
         int size=0;
         for (int i=0;  i < numRecords; i++)
         {
            size = recOffsets[i+1] - recOffsets[i];
            if (size <= 0)
               throw new Exception("Invalid record "+i+"! Size <= 0 ("+size+")");
            byte []bytes = new byte[size];
            is.readBytes(bytes);
            v.addElement(bytes);
         }
         is.close();

         return v;
      }
      protected void finalize() throws Throwable // guich@300_31
      {
         try
         {
            if (_isOpen && !deleted)
            {
               JavaBridge.print("Catalog "+_name+" being closed by GC...");
               NativeMethods4JDK.catalogClose(this);
            }
         } catch (Exception e) {}
      }
   }
   static final CatalogBag getCB(Catalog obj)
   {
      return (CatalogBag)obj.bag;
   }
   static CatalogBag createCB() // the creation of a inner class can't be in a static method
   {
      return new CatalogBag();
   }
   /** returns the file name of this catalog */
   static String getFileName(CatalogBag cb)
   {
      return cb._name+".pdb";//+_creator+"."+_type;
   }
   static void catalogCreate(Catalog obj)
   {
      obj.bag = /*((NativeMethods4JDK)nm).*/createCB();
      getCB(obj).doSetDate();
   }
   static void catalogCreate(Catalog obj, String name, int mode)
   {
      obj.bag = /*((NativeMethods4JDK)nm).*/createCB();
      CatalogBag cb = (CatalogBag)obj.bag;
      // parse name
      String []st = Convert.tokenizeString(name,'.');
      if (st == null || st.length < 3) // guich@340_30: changed != to <
      {
         waba.applet.JavaBridge.print("Invalid catalog name: "+name+". It must be in the form MyCatalog.CRTR.TYPE (see documentation)");
         return;
      }
      // guich@340_30: fixes problem when the catalog path has a . inside
      if (st.length == 3)
         cb._name = st[0];
      else
         cb._name = name.substring(0,name.length()-10);
      cb._creator = st[st.length-2];
      cb._type = st[st.length-1];

      cb._mode = mode;

      // read the catalog from disk
      byte []bytes = JavaBridge.readBytes(getFileName(cb));
      // guich@220_36: match the behaviour of Device: if catalog found and mode = create, continue.
      if (bytes == null) // catalog not found?
      {
         if (mode != Catalog.CREATE) // if not creating, exit
            return;
         cb.doSetDate();
      }
      else
         try
         {
            cb._records = cb.fromPDB(bytes,cb._creator,cb._type, cb._attrs);
         } catch (Exception e) {waba.applet.JavaBridge.print("EXCEPTION OCCURED WITH '"+name+"' DATABASE"); e.printStackTrace(); return;}
      _dbHash.put(name, obj);
      cb._isOpen = true;
   }
   static boolean catalogRename(Catalog obj, String newName)
   {
      CatalogBag cb = getCB(obj);
      if (!cb._isOpen)
         return false;
      String []st = Convert.tokenizeString(newName,'.');
      if (st == null || st.length != 3)
      {
         waba.applet.JavaBridge.print("Invalid catalog name: "+newName+". It must be in the form MyCatalog.CRTR.TYPE (see documentation)");
         return false;
      }
      // update the htOpenedAt with the new name, otherwise the file will be saved in a different location
      String path = removeCatalogFromDisk(cb); // guich@400_2: putted the delete routine in a method.
      cb._name = st[0];
      cb._creator = st[1];
      cb._type = st[2];
      cb.deleted = false; // guich@400_58: was set to true in removeCatalogFromDisk
      cb.modificationNumber++; // guich@400_58: needed for Catalog.close.
      if (path != null)
         JavaBridge.htOpenedAt.put(getFileName(cb),path);
      return true;
   }
   static int catalogAddRecord(Catalog obj, int size)
   {
      CatalogBag cb = getCB(obj);
      if (!cb._isOpen)
         return -1;
      cb._recordPos = cb._records.size();
      cb._records.addElement(new byte[size]);
      cb._attrs.addElement(new Byte(Catalog.REC_ATTR_DIRTY));
      obj.hvRecordOffset = 0;
      cb.modificationNumber++;
      return cb._recordPos;
   }
   static int catalogAddRecord(Catalog obj, int size, int pos)
   {
      CatalogBag cb = getCB(obj);
      if (!cb._isOpen || size < 0)
         return -1;
      if (pos > cb._records.size()) // guich@320_29
      {
         waba.applet.JavaBridge.print("Trying to add a record at an invalid position ("+pos+" > db size + 1).");
         pos = cb._records.size();
      }

      try
      {
         cb._records.insertElementAt(new byte[size], pos);
         cb._attrs.insertElementAt(new Byte(Catalog.REC_ATTR_DIRTY), pos);
         cb.modificationNumber++;
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         return -1;
      }
      cb._recordPos = pos;
      obj.hvRecordOffset = 0;
      return cb._recordPos;
   }
   static boolean catalogResizeRecord(Catalog obj, int size)
   {
      CatalogBag cb = getCB(obj);
      if (cb._recordPos == -1)
         return false;
      byte oldRec[] = (byte[])cb._records.elementAt(cb._recordPos);
      byte newRec[] = new byte[size];
      int copyLen;
      if (oldRec.length < newRec.length)
         copyLen = oldRec.length;
      else
         copyLen = newRec.length;
      System.arraycopy(oldRec, 0, newRec, 0, copyLen);
      cb._records.setElementAt(newRec, cb._recordPos);
      cb.modificationNumber++;
      // _cursor = 0; guich@120
      return true;
   }
   static boolean catalogClose(CatalogBag cb) // guich@300_31
   {
      boolean ret = true;
      if (!cb._isOpen)
         return false;

      if (!cb.deleted && cb.modificationNumber != cb.originalModificationNumber) // guich@120 - write only if modified - guich@200b4_95: use now the modificationNumber to see if we had changed - guich@400_3: now checks if the file was deleted before writting
      {
         try
         {
            ByteArrayStream bas = new ByteArrayStream(2048);
            cb.toPDB(bas,cb._records,cb._creator,cb._name,cb._type,cb._attrs);
            JavaBridge.writeBytes(getFileName(cb),bas.getBuffer(),bas.count());
         } catch (Exception e) {e.printStackTrace(); ret = false;}
      }
      cb._isOpen = false;
      cb._recordPos = -1;
      return ret;
   }
   static boolean catalogClose(Catalog obj)
   {
      return catalogClose(getCB(obj));
   }
   static String removeCatalogFromDisk(CatalogBag cb) // guich@400_2
   {
      //catalogClose(cb);
      String name = getFileName(cb);
      String path = (String)JavaBridge.htOpenedAt.get(name);
      try
      {
         if (path == null)
            path = System.getProperty("user.dir");
         if (path == null)
            path = ".";
         java.io.File f = new java.io.File(path,name);
         JavaBridge.print(f+" - exists?"+f.exists());
         if (!f.exists())  // guich@400_58
         {
            path = "";
            f = new java.io.File(path,name);
            JavaBridge.print(f+" - exists?"+f.exists());
         }
         cb.deleted = true; // guich@400_3
         if (f.delete() || !f.exists()) // guich@400_3: if the file was created in the same session it is being deleted, it may not have been written to disk, so we can simply return as if it was created.
            return path;
         JavaBridge.print("Cannot delete file phisically. Removing logically...");
      } catch (Exception e) {e.printStackTrace();}
      return null;
   }
   static boolean catalogDelete(Catalog obj)
   {
      CatalogBag cb = getCB(obj);
      if (!cb._isOpen)
         return false;
      _dbHash.remove(cb._name);

      cb._records.clear(); // erases the vector
      cb._attrs.clear();
      cb._isOpen = false;
      cb._recordPos = -1;
      try
      {
         // guich@300_19: try to delete the file from disk first
         String path;
         if ((path=removeCatalogFromDisk(cb)) != null) // guich@400_2: putted the delete routine in a method.
            return true;
         // could not phisicaly delete the file?
         ByteArrayStream bas = new ByteArrayStream(1024);
         cb.toPDB(bas,cb._records,cb._creator,cb._name,cb._type,cb._attrs);
         JavaBridge.writeBytes(getFileName(cb),bas.getBuffer(),bas.count());
      } catch (Exception e) {e.printStackTrace(); return false;}
      return true;
   }
   static private int makeInt(byte b1, byte b2, byte b3, byte b4)
   {
      return (b1 << 24) | (b2 << 16) | (b3 << 8) | b4;
   }
   static String []catalogListCatalogs(int creatorIdWild, int typeWild)
   {
      // guich@220_8: now returns also the prc/pdb files of the current dir
      // gets the already open files
      Vector v = new Vector(_dbHash.getKeys().toObjectArray());
      // guich@330_17: check the creators of the current items
      if (creatorIdWild != 0 || typeWild != 0)
      {
         for (int i =v.size()-1; i >=0; i--)
         {
            byte[]b = v.items[i].toString().getBytes();
            int l = b.length;
            int crtrI = makeInt(b[l-9],b[l-8],b[l-7],b[l-6]);
            int typeI = makeInt(b[l-4],b[l-3],b[l-2],b[l-1]);
            if ((creatorIdWild != 0 && creatorIdWild != crtrI) ||
                (typeWild != 0 && typeWild != typeI))
               v.removeElementAt(i);
         }
      }

      // gets from the current dir
      java.io.File f = new java.io.File(".");
      String []s = f.list();
      byte []buf = new byte[68];
      for (int i =0; i < s.length; i++)
         if (v.find(s[i]) == -1 && (s[i].toLowerCase().endsWith("prc") || s[i].toLowerCase().endsWith("pdb")))
         try
         {
            // get the type and creator for this file
            java.io.FileInputStream fis = new java.io.FileInputStream(new java.io.File(".",s[i]));
            fis.read(buf);
            int typeI = makeInt(buf[60],buf[61],buf[62],buf[63]); // guich@330_17
            int crtrI = makeInt(buf[64],buf[65],buf[66],buf[67]);
            if (creatorIdWild != 0 && creatorIdWild != crtrI) continue;
            if (typeWild != 0 && typeWild != typeI) continue;
            String type = ""+(char)buf[60]+(char)buf[61]+(char)buf[62]+(char)buf[63];
            String crtr = ""+(char)buf[64]+(char)buf[65]+(char)buf[66]+(char)buf[67];
            String fullName = s[i].substring(0,s[i].indexOf('.'))+'.'+crtr+'.'+type;
            if (v.find(fullName) == -1) // guich@300_18
               v.add(fullName);
         } catch (java.io.IOException e) {System.out.println("Error in Catalog.listCatalogs while reading "+s[i]); e.printStackTrace();}
      return (String[])v.toObjectArray(); // guich@200b4_27: fix array cast exception - guich@220_32: now we can use the cast.
   }
   static boolean catalogDeleteRecord(Catalog obj)
   {
      CatalogBag cb = getCB(obj);
      if (cb._recordPos == -1)
         return false;
      cb._records.removeElementAt(cb._recordPos);
      cb._attrs.removeElementAt(cb._recordPos);
      cb._recordPos = -1;
      cb.modificationNumber++;
      return true;
   }
   static int catalogGetRecordCount(Catalog obj)
   {
      CatalogBag cb = getCB(obj);
      if (!cb._isOpen)
         return -1;

      return cb._records.size();
   }
   static int catalogGetRecordSize(Catalog obj)
   {
      CatalogBag cb = getCB(obj);
      if (cb._recordPos == -1)
         return -1;
      byte rec[] = (byte[])cb._records.elementAt(cb._recordPos);
      return rec.length;
   }
   static boolean catalogIsOpen(Catalog obj)
   {
      return getCB(obj)._isOpen;
   }
   static boolean catalogSetRecordPos(Catalog obj, int pos)
   {
      CatalogBag cb = getCB(obj);
      if (pos < 0 || pos >= cb._records.size())
      {
         cb._recordPos = -1;
         return false;
      }
      if (cb._attrs != null && (( (Byte)cb._attrs.elementAt(pos) ).byteValue() & Catalog.REC_ATTR_DELETE) != 0)
      {
         waba.applet.JavaBridge.print("You cannot setRecordPos in a deleted record");
         return false;
      }
      cb._recordPos = pos;
      obj.hvRecordOffset = 0;
      return true;
   }
   static int _readWriteBytes(Catalog obj, byte buf[], int start, int count, boolean isRead)
   {
      byte rec[] = null;
      try
      {
         CatalogBag cb = getCB(obj);
         if (cb._recordPos == -1 || start < 0 || count < 0 || count > buf.length) // guich@401_29: changed the last condition from count+offset to count only.
            return -1;
         if ((cb._mode == Catalog.READ_ONLY && !isRead) || (cb._mode == Catalog.WRITE_ONLY && isRead))
         {
            waba.applet.JavaBridge.print("Invalid operation at catalog for this mode");
            return -1;
         }
         if (cb._recordPos >= cb._records.size())
         {
            waba.applet.JavaBridge.print("Record not correctly added. RecordPos: "+cb._recordPos+" but RecordCount: "+cb._records.size());
            return -1;
         }

         rec = (byte[])cb._records.elementAt(cb._recordPos);
         if (obj.hvRecordOffset + count > rec.length)
            return -1;
         if (isRead) // guich@120: System.arraycopy is faster than a for loop.
            System.arraycopy(rec,obj.hvRecordOffset,buf,start,count);
         else
         {
            System.arraycopy(buf,start,rec,obj.hvRecordOffset,count);
            if (cb._attrs != null) catalogSetRecordAttributes(obj, cb._recordPos, (byte)(catalogGetRecordAttributes(obj, cb._recordPos) | Catalog.REC_ATTR_DIRTY)); // set dirty
            cb.modificationNumber++;
         }
         obj.hvRecordOffset += count;
      } catch (Exception ee) {System.out.println("Exception "+ee+" in _readWriteBytes: rec.length: "+(rec==null?"null":""+rec.length)+", buf.length: "+buf.length+", start: "+start+", count: "+count+", isRead? "+isRead); count = -1;}
      return count;
   }
   static int catalogReadBytes(Catalog obj, byte buf[], int start, int count)
   {
      return _readWriteBytes(obj, buf, start, count, true);
   }
   static int catalogSkipBytes(Catalog obj, int count)
   {
      CatalogBag cb = getCB(obj);
      if (cb._recordPos == -1)
         return -1;
      byte rec[] = (byte[])cb._records.elementAt(cb._recordPos);
      int off = obj.hvRecordOffset + count;
      if (off < 0 || off > rec.length) // guich@210_11: added support to go backwards
         return -1;
      obj.hvRecordOffset += count;
      return count;
   }
   static int catalogWriteBytes(Catalog obj, byte buf[], int start, int count)
   {
      return _readWriteBytes(obj, buf, start, count, false);
   }
   static int catalogInspectRecord(Catalog obj, byte buf[], int recPosition)
   {
      CatalogBag cb = getCB(obj);
      byte rec[] = (byte[]) cb._records.elementAt(recPosition);
      int count = Math.min(buf.length, rec.length);
      System.arraycopy(rec, 0, buf, 0, count);
      return count;
   }
   static byte catalogGetRecordAttributes(Catalog obj, int recordPos)
   {
      CatalogBag cb = getCB(obj);
      if (recordPos < 0 || cb._attrs == null || recordPos >= cb._attrs.size()) // guich@400_42: use the given recordPos instead of the current one
         return 0;
      Byte b = (Byte)cb._attrs.elementAt(recordPos);
      return b.byteValue();
   }
   static void catalogSetRecordAttributes(Catalog obj, int recordPos, byte attr)
   {
      CatalogBag cb = getCB(obj);
      if (recordPos < 0 || cb._attrs == null || recordPos >= cb._attrs.size()) // guich@400_42
      {
         waba.applet.JavaBridge.print("could not change record attributes");
         return;
      }
      cb._attrs.setElementAt(new Byte(attr), recordPos);
      cb.modificationNumber++;
   }
   static int catalogGetAttributes(Catalog obj)
   {
      return getCB(obj).attributes;
   }
   static void catalogSetAttributes(Catalog obj, int attributes)
   {
      CatalogBag cb = getCB(obj);
      cb.attributes = (short)attributes;
      cb.modificationNumber++;
   }
   static int catalogGetRecordPos(Catalog obj)
   {
      return getCB(obj)._recordPos;
   }
   ////////////////////////////////////////////////////////////////////////////////////
   //                                   waba.io.File                                 //
   ////////////////////////////////////////////////////////////////////////////////////
   static private java.io.RandomAccessFile getRaf(waba.io.File obj)
   {
      return (java.io.RandomAccessFile)obj.jraf;
   }
   static private java.io.File getFile(File obj)
   {
      return (java.io.File)obj.jfile;
   }

   static boolean fileIsAvailable() // always available under JDK
   {
      return true;
   }
   static void fileCreate(File obj, String path, int mode)
   {
      obj.jfile = new java.io.File(path);
      if (mode != File.DONT_OPEN)
      {
         String rafMode;
         if (mode == File.READ_ONLY)
            rafMode = "r";
         else
         if (mode == File.WRITE_ONLY)
            rafMode = "w";
         else
         if (mode == File.READ_WRITE)
            rafMode = getFile(obj).exists() ? "rw" : "r";
         else // CREATE
            rafMode = "rw";
         try
         {
            obj.jraf = new java.io.RandomAccessFile(path, rafMode);
         }
         catch (Exception e) {obj.jraf = null;}
      }
   }

   static boolean fileClose(File obj)
   {
      try {getRaf(obj).close();} catch(Exception e) {}
      obj.jraf = null;
      return true;
   }

   static boolean fileCreateDir(File obj)
   {
      return getFile(obj).mkdir();
   }

   static boolean fileDelete(File obj)
   {
      fileClose(obj);
      return getFile(obj).delete();
   }

   static boolean fileExists(File obj)
   {
      return getFile(obj).exists();
   }

   static int fileGetSize(File obj)
   {
      return (int)getFile(obj).length();
   }

   static boolean fileIsDir(File obj)
   {
      return getFile(obj).isDirectory();
   }

   static boolean fileIsOpen(File obj)
   {
      return obj.jraf != null;
   }

   static String []fileListFiles(File obj)
   {
      String path = obj.path;
      String []files = getFile(obj).list();
      for (int i =0; i < files.length; i++)
         if (new java.io.File(path+files[i]).isDirectory())
            files[i] += '/'; // add a / to a directory end
      return files;
   }

   static int fileReadBytes(File obj, byte b[], int off, int len)
   {
      try
      {
         return getRaf(obj).read(b, off, len);
      }
      catch (Exception e) {}
      return -1;
   }

   static boolean fileRename(File obj, String path)
   {
      fileClose(obj);
      try
      {
         return getFile(obj).renameTo(new java.io.File(path));
      }
      catch (Exception e) {}
      return false;
   }

   static boolean fileSetPos(File obj, int pos)
   {
      try
      {
         getRaf(obj).seek(pos);
         return true;
      }
      catch (Exception e) { }
      return false;
   }

   static int fileWriteBytes(File obj, byte b[], int off, int len)
   {
      try
      {
         getRaf(obj).write(b, off, len);
         return len;
      }
      catch (Exception e) {}
      return -1;
   }

   static void fileSetAttributes(File obj, int attr)
   {
   }

   static int fileGetAttributes(File obj)
   {
      return 0;
   }

   static void fileSetTime(File obj, byte whichTime, waba.sys.Time time)
   {
   }

   static waba.sys.Time fileGetTime(File obj, byte whichTime)
   {
      return null;
   }
}