/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.sys;

/** This class is used to correctly handle international character convertions.
  * The default character scheme converter is the 8859-1. If you want to use a different one,
  * you must extend this class, implementing the bytes2chars and chars2bytes methods, and then
  * assign the public member of java.lang.String charConverter to use your class instead of this default one.
  * Note that the java.lang.String mentioned above is located at SuperwabaSDK/src/java.lang.String.
  * To find out which <code>sun.io.CharacterEncoder</code> you're using, do:
  * <p><code>System.out.println(""+sun.io.ByteToCharConverter.getDefault());</code>
  */

public class CharacterConverter
{
   public static char[] bytes2chars(byte bytes[], int offset, int length)
   {
      char []value = new char[length];

      int i = 0;
      while (length-- > 0)
      {
         byte b = bytes[offset++];
         value[i++] = (char)(b >= 0 ? b : (256 + b));
      }
      return value;
   }
   public static byte[] chars2bytes(char chars[], int offset, int length)
   {
      byte []bytes = new byte[length];
      int end = offset+length-1;
      int i = 0;
      byte b;
      for (; offset <= end; offset++)
      {
         char c = chars[offset];
         if (c <= '\377')
            b = (byte)c;
         else
         {
            if ('\uD800' <= c && c <= '\uDBFF')
            {
               if (offset == end) // we must skip one byte, but no more bytes avail?
                  break;
               offset++;
            }
            b = (byte)'?';
         }
         bytes[i++] = b;
      }
      if (i != length)
      {
         byte []temp = new byte[i];
         Vm.copyArray(bytes,0,temp,0,i);
         bytes = temp;
      }
      return bytes;
   }
}