/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.sys;

/**
 * Convert is used to convert between objects and basic types.
 */

public final class Convert
{
   /** The bytes are converted to char and vice-versa using the CharacterConverter associated in this charConverter member. */
   public static waba.sys.CharacterConverter charConverter = new waba.sys.CharacterConverter();

   public static final long MIN_LONG_VALUE = 0x8000000000000000L;
   public static final long MAX_LONG_VALUE = -0x7fffffffffffffffL;
   private Convert()
   {
   }

   /** Creates a copy of the given array */
   public static String [] cloneStringArray(String []strs) // guich@220_2
   {
      String []ret = null;
      if (strs != null)
      {
         int n = strs.length;
         ret = new String[n];
         Vm.copyArray(strs,0,ret,0,n);
      }
      return ret;
   }

   /** Converts the given object array to a String array by calling toString in each object */
   public static String [] toStringArray(Object []objs) // guich@200b4_26
   {
      String as[] = null;
      if (objs != null)
      {
         int n = objs.length;
         as = new String[n];
         for (int i =0; i < n; i++)
            if (objs[i] != null)
               as[i] = objs[i].toString();
            else
               as[i] = null;
      }
      return as;
   }

   /**
    * Converts the given String to an int. If the string passed is not a valid
    * integer, 0 is returned.
    */
   public static int toInt(String s)
   {
      return NativeMethods4JDK.convertToInt(s);
   }

   /** Converts the given boolean to a String. */
   public static String toString(boolean b)
   {
      return b?"true":"false";
   }

   /** Converts the given char to a String. */
   public static String toString(char c)
   {
      return NativeMethods4JDK.convertToString(c);
   }

   /** Converts the given float to its bit representation in IEEE 754 format. */
   public static int toIntBitwise(float f)
   {
      return NativeMethods4JDK.convertToIntBitwise(f);
   }

   /** Converts the given IEEE 754 bit representation of a float to a float. */
   public static float toFloatBitwise(int i)
   {
      return NativeMethods4JDK.convertToFloatBitwise(i);
   }

   /** Converts the given float to a String. */
   public static String toString(float f)
   {
      return NativeMethods4JDK.convertToString(f);
   }

   /** Converts the given int to a String. */
   public static String toString(int i)
   {
      return NativeMethods4JDK.convertToString(i);
   }

   /** Converts the given double to a String, formatting it with <precision> decimal places.
   @since 2.0 */
   public static String toString(double d, int precision)
   {
      return NativeMethods4JDK.convertToString(d,precision);
   }

   /** Converts the String to a double.
   @since 2.0 */
   public static double toDouble(String s)
   {
      return NativeMethods4JDK.convertToDouble(s);
   }

   /** Converts the given float to a String, formatting it with d decimal places. added by guich */
   public static String toString(float f, int d)
   {
      return NativeMethods4JDK.convertToString(f,d);
   }

   /** Converts the given String to a float. If the string passed is not a valid float, 0 is returned. */
   public static float toFloat(String s)
   {
      return NativeMethods4JDK.convertToFloat(s);
   }

   /** formats a String as a double, <b>rounding</b> with n decimal places. if the number is invalid, "0" is returned. */
   public static String toString(String doubleValue, int n)
   {
      return NativeMethods4JDK.convertToString(doubleValue,n);
   }

    /**
     * Returns a representation of the specified floating-point value
     * according to the IEEE 754 floating-point "double
     * format" bit layout.
     * <p>
     * Bit 63 represents the sign of the floating-point number. Bits
     * 62-52 represent the exponent. Bits 51-0 represent
     * the significand (sometimes called the mantissa) of the
     * floating-point number.
     * <p>
     * If the argument is positive infinity, the result is
     * <code>0x7ff0000000000000L</code>.
     * <p>
     * If the argument is negative infinity, the result is
     * <code>0xfff0000000000000L</code>.
     * <p>
     * If the argument is NaN, the result is
     * <code>0x7ff8000000000000L</code>.
     *
     * @param   value   a double precision floating-point number.
     * @return  the bits that represent the floating-point number.
     * @since   SuperWaba2.0
     */
   public static long doubleToLongBits(double value)
   {
      return NativeMethods4JDK.convertDoubleToLongBits(value);
   }

   /**
     * Returns the double-float corresponding to a given bit represention.
     * The argument is considered to be a representation of a
     * floating-point value according to the IEEE 754 floating-point
     * "double precision" bit layout. That floating-point
     * value is returned as the result.
     * <p>
     * If the argument is <code>0x7f80000000000000L</code>, the result
     * is positive infinity.
     * <p>
     * If the argument is <code>0xff80000000000000L</code>, the result
     * is negative infinity.
     * <p>
     * If the argument is any value in the range
     * <code>0x7ff0000000000001L</code> through
     * <code>0x7fffffffffffffffL</code> or in the range
     * <code>0xfff0000000000001L</code> through
     * <code>0xffffffffffffffffL</code>, the result is NaN. All IEEE 754
     * NaN values are, in effect, lumped together by the Java language
     * into a single value.
     *
     * @param   bits   any <code>long</code> integer.
     * @return  the <code>double</code> floating-point value with the same
     *          bit pattern.
     * @since   SuperWaba2.0
     */
   public static double longBitsToDouble(long bits)
   {
      return NativeMethods4JDK.convertDoubleFromLongBits(bits); // must change name because it conflicts doubleToLongBits (same hashcode)
   }

   /** returns the value of the digit in the specified radix. this method is simplified to only handle the standard ascii table.
   @since 2.0 */
   public static int digitOf(char ch, int radix)
   {
      int value = -1;
      if (2 <= radix && radix <= 16)
      {
         if ('0' <= ch && ch <= '9')
            value = ch - '0';
         else
         if ('A' <= ch && ch <= 'F')
            value = ch - 'A' + 10;
         else
            value = ch - 'a' + 10;
      }
      return (0 <= value && value < radix) ? value : -1;
   }

   /** returns the digit in the corresponding radix.
   @since 2.0 */
   public static char forDigit(int digit, int radix)
   {
      if (digit >= radix || digit < 0 || radix < 2 || radix > 16)
         return '?';
      if (digit < 10)
         return (char)('0' + digit);
      return (char)('a' - 10 + digit);
   }

   /** Converts the long to a String at base 10.
   @since 2.0 */
   public static String toString(long l)
   {
      return toString(l,10);
   }

   /** Converts the long to a String in the given radix. Radix can be from 2 to 16.
   @since 2.0 */

   public static String toString(long i, int radix)
   {
      if (i == 0) return "0";
      if (radix < 2 || radix > 16)
         radix = 10;
      StringBuffer buf = new StringBuffer(radix >= 8 ? 23 : 65);
      boolean negative = (i < 0);
      if (!negative)
         i = -i;
      while (i <= -radix)
      {
         char c = forDigit((int)(-(i % radix)), radix);
         buf.append(c);
         i = i / radix;
      }
      buf.append(forDigit((int)(-i), radix));
      if (negative)
         buf.append('-');
      // reverse the string
      char []chars = buf.toString().toCharArray();
      char []ac = new char[chars.length];
      int j = chars.length-1;
      for (int k =0; k < ac.length; k++,j--)
         ac[k] = chars[j];
      String ss = new String(ac);
      return ss;
   }

   /** Converts the String to a long.
   @since 2.0 */
   public static long toLong(String s)
   {
      return toLong(s,10);
   }

   /** Converts the String to a long in the given radix. Radix can be from 2 to 16. At any error, 0 is returned. The hex digits are in lowercase.
   @since 2.0 */
   public static long toLong(String s, int radix)
   {
      if (s == null || radix < 2 || radix > 16) return 0;

      long result = 0;
      boolean negative = false;
      int i = 0, max = s.length();
      long limit;
      long multmin;
      int digit;

      if (s.charAt(0) == '-')
      {
         negative = true;
         limit = MIN_LONG_VALUE; // smallest long value
         i++;
      }
      else limit = MAX_LONG_VALUE; // largest long value
      multmin = limit / radix;
      if (i < max)
      {
         digit = digitOf(s.charAt(i++),radix);
         if (digit < 0)
            return 0;
         else
            result = -digit;
      }
      while (i < max)
      {
         // Accumulating negatively avoids surprises near MAX_VALUE
         digit = digitOf(s.charAt(i++),radix);
         if (digit < 0)
            return 0;
         if (result < multmin)
            return 0;
         result *= radix;
         if (result < limit + digit)
            return 0;
         result -= digit;
      }
      if (negative)
      {
         if (i > 1)
            return result;
         else // Only got "-"
            return 0;
      }
      else
         return -result;
   }

   /** Converts the given double to a String, formatting it using scientific notation.
   @since 2.0 */
   public static String toString(double d)
   {
      return toString(d,-1); // guich@200b4_201: fixed exp notation
   }

   /** converts the string to upper case letters */
   public static String toUpperCase(String s)
   {
      char[] c = s.toCharArray();
      for (int i=0; i < c.length; i++)
         c[i] = toUpperCase(c[i]);
      return new String(c);
   }

   /** converts the string to lower case letters */
   public static String toLowerCase(String s)
   {
      char[] c = s.toCharArray();
      for (int i=0; i < c.length; i++)
         c[i] = toLowerCase(c[i]);
        return new String(c);
   }

   /** converts the char to lower case letter */
   public static char toLowerCase(char c)
   {
      if (('A' <= c && c <= 'Z') || ('' <= c && c <= ''))
         c += 32;
      return c;
   }

   /** converts the char to upper case letter */
   public static char toUpperCase(char c)
   {
      if (('a' <= c && c <= 'z') || ('' <= c && c <= '' && c != 247))
         c -= 32;
      return c;
   }
   /** pads the string with zeroes at left */
   public static String zeroPad(String s, int size)
   {
      for (int i = size-s.length(); i > 0; i--) // guich@320_4: optimized code
         s = '0'+s;
      return s;
   }

   /** Tokenize the given input string. if there's no delim chars in input, returns the input string */
   public static String[] tokenizeString(String input, char delim)
   {
      int i,n,acl,last;
      String as[] = null;
      char []ac = input.toCharArray();
      n = 1;
      acl = ac.length;
      for (i = acl-1; i >= 0; i--)
         if (ac[i] == delim) n++;
      last = 0;
      as = new String[n];
      n =0;
      for (i =0; i < acl; i++)
         if (ac[i] == delim)
         {
            as[n++] = new String(ac,last,i-last);
            last = i+1;
         }
      as[n++] = new String(ac,last,ac.length-last);
      return as;
   }

   /** This method is useful to insert line breaks into the text used in the MessageBox constructor. It receives
     * a String and returns the parsed String. Here is an example of use:
     * <code>Convert.insertLineBreak(154,'|',getFontMetrics(getFont()),"This is a very long text and i dont want to waste my time parsing it to be fit in the MessageBox!");</code>
     * @param maxWidth the maximum width that is permitted for each line. For a text used in the MessageBox class, use <code>Settings.screenWidth-6</code> or a lower number.
     * @param separator the separator that will divide each line. The MessageBox class uses '|' as the separator
     * @param fm the FontMetrics of the font you will use to display the text
     * @param text the text to be parsed
     * @return the parsed text
     */
   public static String insertLineBreak(int maxWidth, char separator, waba.fx.FontMetrics fm, String text) // guich@200b4_30
   {
      String []words = tokenizeString(text,' ');
      StringBuffer sb = new StringBuffer(text.length()+20);
      String s = "";
      int n = words.length;
      for (int i =0; i < n; i++)
      {
         if (fm.getTextWidth(s+' '+words[i]) > maxWidth)
         {
            if (sb.length() > 0)
               sb.append(separator);
            sb.append(s);
            s = "";
         }
         if (s.length() > 0) s += ' ';
         s += words[i];
      }
      if (s.length() > 0)
      {
         if (sb.length() > 0) // olie@241_8
            sb.append(separator);
         sb.append(s);
      }
      return sb.toString();
   }
   /** Applies the Quick Sort algorithm to the elements of the given array.
     * If they are Strings,
     * the sort will be much faster because a direct cast to String is done;
     * otherwise, if they are not strings, the toString() method is used to return
     * the string that will be used for comparision.
     * You may call this way: Convert.qsort(items,0,items.length-1).
     * You can sort subportions of the array as well.
     */
   public static void qsort(Object []items, int first, int last) // guich@220_34
   {
      int low = first;
      int high = last;
      if (first >= last)
         return;
      if (items[first] instanceof String) // guich@240_9: now the object array can be an array of objects :-)
      {
         String mid = (String)items[(first+last) >> 1];
         do
         {
            while (mid.compareTo((String)items[low])  > 0)
               low++;
            while (mid.compareTo((String)items[high]) < 0)
               high--;
            if (low <= high)
            {
               Object temp = items[low];
               items[low++] = items[high];
               items[high--] = temp;
            }
         } while (low <= high);
      }
      else
      {
         String mid = items[(first+last) >> 1].toString();
         do
         {
            while (mid.compareTo(items[low].toString())  > 0)
               low++;
            while (mid.compareTo(items[high].toString()) < 0)
               high--;
            if (low <= high)
            {
               Object temp = items[low];
               items[low++] = items[high];
               items[high--] = temp;
            }
         } while (low <= high);
      }
      qsort(items,first,high);
      qsort(items,low,last);
   }

   static char []b2h = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
   /** Returns a String with the convertion of the unsigned integer to hexadecimal,
     * using the given number of places (up to 8). This routine is 10 times faster than
     * toString(long, int). The hex digits are in upper case.
     * @since SuperWaba 3.0
     */
   static public String unsigned2hex(int b, int places)   // guich@300_8
   {
      char []c = new char[places];
      for (int i = places-1; i >= 0; i--)
      {
         c[i] = b2h[b & 0xF];
         b >>= 4;
      }
      return new String(c);
   }
  /** Do a rol of n bits in the given i(nt). n must be < <code>bits</code>. This differs from the
    * shift left operator (<<) in that the bits are not lost,
    * they are reinserted in order at the right.
    * Bits may be 4,8,16 or 32.
    */
  public static long rol(long i, int n, int bits) // guich@330_8: moved to here to make Math compatible with JDK versions.
  {
     long nn = bits-1-n;
     long i2 = i >> nn;
     return (i << n) | i2;
  }

  /** Do a ror of n bits in the given i(nt). n must be < <code>bits</code>. This differs from the
    * shift right operator (>>) in that the bits are not lost,
    * they are reinserted in order at the left.
    * Bits may be 4,8,16 or 32.
    */
  public static long ror(long i, int n, int bits)
  {
     long nn = bits-1-n;
     long i2 = (i << nn) & (1L << nn);
     return (i >> n) | i2;
  }

  /** Convert a creator id or a type to an int. */
  public static int chars2int(String fourChars) // guich@330_17
  {
     char []chars = fourChars.toCharArray();
     return (chars[0] << 24) | (chars[1] << 16) | (chars[2] << 8) | chars[3];
  }
}
