/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.sys;

import waba.fx.*;

/** this class provides some preferences from PalmOS config and other Vm settings.
 * All settings are read-only, unless otherwise specified. Changing their values may cause
 * the VM to crash.
 */

public final class Settings
{
   // Set by the VM

   /** Can be one of the following constants: DATE_MDY, DATE_DMY, DATE_YMD; where m = month, d = day and y = year */
   public static byte dateFormat;
   /** The date char separator. */
   public static char dateSeparator;
   /** The week day start. 0 = sunday, 6 = saturday */
   public static byte weekStart;
   /** True if the time format is 24 hour format or if it is the AM/PM format */
   public static boolean is24Hour;
   /** The time char separator */
   public static char timeSeparator;
   /** The thousands separator for numbers */
   public static char thousandsSeparator;
   /** The decimal separator for numbers */
   public static char decimalSeparator;

   /** <b>READ-ONLY</b> variable that represents the device's screen width */
   public static int screenWidth;
   /** <b>READ-ONLY</b> variable that represents the device's screen height */
   public static int screenHeight;
   /** <b>READ-ONLY</b> variable that represents if the device supports color. */
   public static boolean isColor;
   /** <b>READ-ONLY</b> variable that returns the number of colors supported by the device. 0 means 2^32 colors. */
   public static int maxColors;
   /** <b>READ-ONLY</b> variable that defines if were being runned in a handheld device or in a desktop computer. Note that under Win32, onDevice is true. */
   public static boolean onDevice = false;
   /** <b>READ-ONLY</b> variable that returns the ROM version of the device, like 0x02000000 or 0x03010000. In desktop, return the oldest version of the destination platform where SuperWaba can run. */
   public static int romVersion = 0x02000000;
   /** <b>READ-ONLY</b> variable that returns the current platform name. Eg: Java, PalmOS, WindowsCE, PocketPC, Win32, ... */
   public static String platform;
   /**
   * <b>READ-ONLY</b> variable that returns the username of the user running the Virtual Machine. Because of
   * Java's security model, this method will return null when called in a Java
   * applet. This method will also return null under most WinCE devices (that
   * will be fixed in a future release).
   */
   public static String userName;
   /** Application defined setting. If you set the value of this app to something other than null,
       the VM will save it when exiting and load it when restarting. Under PalmOS, the value is stored in the
       unsaved preferrences database, which is not backupped during hot-sync.
       Use this to save small strings, up to 2 or 4 kb maximum.
       At desktop, a file named settings4crtr.pdb stores the appSettings for the current running SW programs.
     */
   public static String appSettings;

   /** Application defined secret key. If you set the value of this app to something other than null,
       the VM will save it when exiting and load it when restarting. Under PalmOS, the value is stored in the
       unsaved preferrences database, which is not backupped during hot-sync.
       Use this to save small strings, up to 2 or 4 kb maximum.
       At desktop, a file named settings4crtr.pdb stores the appSettings for the current running SW programs.
       The String is stored in the saved preferrences with a creator id different of your application's (but
       calculated based in it), so
       it will never be deleted and will be restored even after a hard-reset.
     */
   public static String appSecretKey;

   /** Application creator ID. In desktop, it returns the automatically created Creator ID. In device, it will return the same or the one specified with /c option argument in Warp. */
   public static String appCreatorId;

   // Not set by the VM

   /** The uiStyle used to define the user interface style. It must be set by calling Settings.setPalmOSStyle(boolean whiteBackground). The possible values are WinCE (default) and PalmOS. */
   public static byte uiStyle;
   /**
   * Returns the version of the SuperWaba Virtual Machine. The major version is
   * base 100. For example, version 1.0 has value 100. version 4 has a
   * version value of 400. A beta 0.8 VM will have version 80.
   * ps: Waba 1.0G will return 1.01. SuperWaba = 110 (1.1)
   */
   public static int version = 401;
   /** Returns the version in a string form, like "2.0b4r8" */
   public static String versionStr = "4.01";

   /** Defines a Windows CE user interface style. Used in the uiStyle member. Note for grayscale devices: that WinCE user interfaces looks better with bright (non-white) background */
   public static final byte WinCE = 0;
   /** Defines a PalmOS user interface style. Used in the uiStyle member. Note for grayscale devices: that PalmOS user interfaces looks better with white background */
   public static final byte PalmOS = 1;
   /** Constant used in dateFormat: month day year */
   public static final byte DATE_MDY = 1;
   /** Constant used in dateFormat: day month year */
   public static final byte DATE_DMY = 2;
   /** Constant used in dateFormat: year month day */
   public static final byte DATE_YMD = 3;

   /** Returns true if the device is in daylight savings mode. Note that this is
     * not available in all devices. Only works in desktop, WinCE and Palm OS 4.0 or newer devices;
     * in other devices will always return false.
     * @since SuperWaba 3.4
     */
   public static boolean daylightSavings;

   /** Returns the timezone used for this device. This is the number of hours
     * away from GMT (E.g.: for Brazil it will return -3). Only works in desktop, WinCE and Palm OS 4.0 or newer devices;
     * in other devices will always return 0.
     * @since SuperWaba 3.4
     */
   public static int timeZone;

   /** True if this handheld has a virtual keyboard */
   public static boolean virtualKeyboard;

   /** Changes the user interface style to PalmOS. The default is WinCE. If withWhiteBackground flag is true,
    * the default back color for controls and the MainWindow backgrounds are changed to white. This method must
    * be called in the MainWindow's constructor. E.g.:
    * <pre>
    * public class Foo extends MainWindow
    * {
    *    public Foo()
    *    {
    *       super("Hi bar",TAB_ONLY_BORDER);
    *       waba.sys.Settings.setPalmOSStyle(true);
    * </pre>
    */
	public static void setPalmOSStyle(boolean withWhiteBackground)
	{
	   uiStyle = PalmOS;
	   if (withWhiteBackground)
	   {
         Color.defaultBackColor = Color.WHITE;
         waba.ui.MainWindow.getMainWindow().setBackColor(Color.WHITE);
      }
	}

   protected Settings(byte dateFormat, char dateSeparator, byte weekStart,
                    boolean is24Hour, char timeSeparator,char thousandsSeparator,
                    char decimalSeparator,
                    int screenWidth, int screenHeight, boolean isColor, int maxColors,
                    boolean onDevice, int romVersion, String platform, String userName,
                    String appSettings, String appCreatorId, boolean virtualKeyboard,
                    String appSecretKey, boolean daylightSavings, int timeZone
                    )
   {
      this.dateFormat = dateFormat;
      this.dateSeparator = dateSeparator;
      this.weekStart = weekStart;
      this.is24Hour = is24Hour;
      this.timeSeparator = timeSeparator;
      this.thousandsSeparator = thousandsSeparator;
      this.decimalSeparator = decimalSeparator;
      this.screenWidth = screenWidth;
      this.screenHeight = screenHeight;
      this.isColor = isColor;
      this.maxColors = maxColors;
      this.onDevice = onDevice;
      this.romVersion = romVersion;
      this.platform = platform;
      this.userName = userName;
      this.appSettings = appSettings;
      this.appCreatorId = appCreatorId;
      this.virtualKeyboard = virtualKeyboard;
      this.appSecretKey = appSecretKey;
      this.daylightSavings = daylightSavings;
      this.timeZone = timeZone;
    }
}