/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.sys;

/**
 * Time identifies a date and time.
 * <p>
 * Here is an example of Time being used to display the current date:
 *
 * <pre>
 * Time t = new Time();
 * ...
 * g.drawText("Today is " + t.year + "/" + t.month + "/" + t.day);
 * </pre>
 */

public final class Time
{
   /** The year as its full set of digits (year 2010 is 2010). */
   public int year;

   /** The month in the range of 1 to 12. */
   public int month;

   /** The day in the range of 1 to the last day in the month. */
   public int day;

   /** The hour in the range of 0 to 23. */
   public int hour;

   /** The minute in the range of 0 to 59. */
   public int minute;

   /** The second in the range of 0 to 59. */
   public int second;

   /** Milliseconds in the range of 0 to 999. */
   public int millis;

   /**
   * Constructs a time object set to the current date and time.
   */
   public Time()
   {
	   NativeMethods4JDK.timeCreate(this);
   }

   /** Constructs a time object from the given value. It must have been got with
     * the getTimeLong method.
     * @since SuperWaba 4.0
     */
   public Time(long t)
   {
      second = (int)(t % 100); t /= 100;
      minute = (int)(t % 100); t /= 100;
      hour   = (int)(t % 100); t /= 100;
      day    = (int)(t % 100); t /= 100;
      month  = (int)(t % 100); t /= 100;
      year   = (int)t;
   }

   /** Returns the time in the format YYYYMMDDHHMMSS as a long value. It does
     * not include the millis.
     * @since SuperWaba 4.0
     */
   public long getTimeLong()
   {
      int i = hour * 10000 + minute * 100 + second;
      return year * 10000000000L + month * 100000000L + day * 1000000 + i;
   }

   /** Constructs a new time with the given values. The values are not checked.
     * @since SuperWaba 3.5
     */
   public Time(int year, int month, int day, int hour, int minute, int second, int millis)
   {
      this.year = year;
      this.month = month;
      this.day = day;
      this.hour = hour;
      this.minute = minute;
      this.second = second;
      this.millis = millis;
   }

   /** returns the time in format specified in waba.sys.Settings.
     * To return the date, use class waba.util.Date. */
   public String toString() // guich@300_56
   {
      boolean useAmPm = !Settings.is24Hour;
      int h = (useAmPm && hour > 12) ? (hour-12) : hour;
      StringBuffer sb = new StringBuffer(12);
      if (hour < 10) sb.append('0');
      sb.append(h);
      sb.append(Settings.timeSeparator);
      if (minute < 10) sb.append('0');
      sb.append(minute);
      sb.append(Settings.timeSeparator);
      if (second < 10) sb.append('0');
      sb.append(second);
      if (useAmPm)
         sb.append(hour >= 13 ? " PM":" AM");
      return sb.toString();
   }

   public boolean equals(Object o)
   {
      if (o instanceof Time)
      {
         Time t = (Time)o;
         return year == t.year && month == t.month && day == t.day &&
                hour == t.hour && minute == t.minute && second == t.second && millis == t.millis;
      }
      else return o == this;
   }
}