/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.sys;

/**
 * Vm contains various system level methods.
 * <p>
 * This class contains methods to copy arrays, obtain a timestamp,
 * sleep and get platform and version information.
 */

// NOTE:
// In the future, these methods may include getting unique object id's,
// getting object classes, sleep (for single threaded apps),
// getting amount of memory used/free, etc.
// The reason these methods should appear in this class and not somewhere
// like the Object class is because each method added to the Object class
// adds one more method to every object in the system.

public final class Vm
{
   /** This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_PAGE_UP   = 0x00000002;
   /** This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_PAGE_DOWN = 0x00000004;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_HARD1     = 0x00000008;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_HARD2     = 0x00000010;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_HARD3     = 0x00000020;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. */
   public static final int SK_HARD4     = 0x00000040;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method */
   public static final int SK_LAUNCH    = 0x01000000;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method */
   public static final int SK_FIND      = 0x02000000;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method */
   public static final int SK_CALC      = 0x04000000;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. */
   public static final int SK_ACTION    = 0x08000000;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. */
   public static final int SK_CONTRAST  = 0x10000000; // guich@401_31
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. */
   public static final int SK_CLOCK     = 0x20000000; // guich@401_31
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. Note: SK_ALL does not intercept the SK_LAUNCH, SK_CONTRAST and SK_CLOCK keys (to maintain compatibility with older applications. You must use SK_ALL | SK_LAUNCH (or other excluded SK_) to intercept the home key. */
   public static final int SK_ALL       = 0xFFFFFFFF & ~(SK_LAUNCH|SK_CONTRAST|SK_CLOCK);
   /** used as constant that can be 'or'ed in the interceptSystemKeys method */
   public static final int SK_NONE      = 0;

   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. This key works only in Windows CE and Palm OS 5 or greater.
     * @since SuperWaba 3.0 */
   public static final int SK_PAGE_LEFT = 0x00001000;
   /** used as constant that can be 'or'ed in the interceptSystemKeys method. This one also can be used in the getSystemKeysPressed method. This key works only in Windows CE and Palm OS 5 or greater.
     * @since SuperWaba 3.0 */
   public static final int SK_PAGE_RIGHT = 0x00002000;

   /** Only available at desktop. used internally. Dont set it directly, it will have no effect. */
   public static int interceptSystemKey = SK_ALL;

   /** Internal use only */
   public static boolean actionEqualsMenu = true;

   private Vm()
	{
	}

   /**
   * Copies the elements of one array to another array. This method returns
   * true if the copy is successful. It will return false if the array types
   * are not compatible or if either array is null. If the length parameter
   * would cause the copy to read or write past the end of one of the arrays,
   * an index out of range error will occur. If false is returned then no
   * copying has been performed.
   * @param srcArray the array to copy elements from
   * @param srcStart the starting position in the source array
   * @param dstArray the array to copy elements to
   * @param dstStart the starting position in the destination array
   * @param length the number of elements to copy
   */
   public static boolean copyArray(Object srcArray, int srcStart, Object dstArray, int dstStart, int length)
   {
      return NativeMethods4JDK.vmCopyArray(srcArray,srcStart,dstArray,dstStart,length);
   }

   /**
   * Returns the elapsed time between the given parameters. Due to the 32 bits precision,
   * the getTimeStamp parameter, in the way it is calculated, can wrap around 0. So,
   * use this method to always get the right result.
   * @deprecated The wrapping will only occur after more than 12 days of continuous
   * program run. Since this is something unlike to occur in a PDA, you can safely
   * subtract one timeStamp by the other. This method will be removed in further releases,
   * so if your program can run without exiting for more than 12 days, copy this method
   * to your program and use it.
   * @deprecated I don't know if there exists any program running in a PDA for more than 12 days. So, if you use this method, copy to your code and use it, bc it will be removed in future versions.
   */
   public static int getTimeElapsed(int now, int before) // peterd@200b4_187
   {
      int result = now-before;
      if (result < 0)
         result += 0x40000000; // needed after/every 12.4 days
      return result;
   }

   /**
   * Returns a time stamp in milliseconds. The time stamp is the time
   * in milliseconds since some arbitrary starting time fixed when
   * the VM starts executing. The maximum time stamp value is (1 << 30) and
   * when it is reached, the timer will reset to 0 and will continue counting
   * from there.
   */
   public static int getTimeStamp()
   {
      return NativeMethods4JDK.vmGetTimeStamp();
   }

   /** Sets the system time to the given arguments. Note that in some systems,
     * the millis field is ignored. Some devices require that you add <code>Settings.timeZone</code>.
     * @since SuperWaba 4.01
     */
   public static void setTimeStamp(Time t)
   {
      NativeMethods4JDK.vmSetTimeStamp(t);
   }

   /** Reboots the device. This is the equivalent of a warm boot (or soft reset).
     * Calling this method terminates the program, closes all open files and, finaly,
     * resets the device (MainWindow.onExit is still called). In JDK, this is equivalent
     * to the Vm.exit method (there's no reboot at all).
     * @since SuperWaba 4.01
     */
   public static void exitAndReboot()
   {
      NativeMethods4JDK.vmExitAndReboot();
   }

   /**
   * Executes a command.
   * <p>
   * As an example, the following call could be used to run the command
   * "scandir /p mydir" under Java, Win32 or WinCE:
   * <pre>
   * int result = Vm.exec("scandir", "/p mydir", 0, true);
   * </pre>
   * This example executes the Scribble program under PalmOS:
   * <pre>
   * Vm.exec("Scribble", null, 0, false);
   * </pre>
   * This example executes the web clipper program under PalmOS, telling
   * it to display a web page by using launchCode 54 (CmdGoToURL).
   * <pre>
   * Vm.exec("Clipper", "http://www.yahoo.com", 54, true);
   * </pre>
   * The args parameter passed to this method is the arguments string
   * to pass to the program being executed.
   * <p>
   * The launchCode parameter is only used under PalmOS. Under PalmOS, it is
   * the launch code value to use when the Vm calls SysUIAppSwitch().
   * If 0 is passed, the default launch code (CmdNormalLaunch) is used to
   * execute the program.
   * <p>
   * The wait parameter passed to this method determines whether to execute
   * the command asynchronously. If false, then the method will return without
   * waiting for the command to complete execution. If true, the method will
   * wait for the program to finish executing and the return value of the
   * method will be the value returned from the application under Java, Win32
   * and WinCE.
   * <p>
   * Under PalmOS, the wait parameter is ignored since executing another
   * program terminates the running program.
   * <p><b>Note: this method kills all running threads.</b>
   *
   * @param command the command to execute
   * @param args command arguments
   * @param launchCode launch code for PalmOS applications
   * @param wait whether to wait for the command to complete execution before returning
   */
   public static int exec(String command, String args, int launchCode, boolean wait)
   {
      waba.ui.MainWindow.getMainWindow().killThreads();
      return NativeMethods4JDK.vmExec(command,args,launchCode,wait);
   }


   /**
   * Sets the device's "auto-off" time. This is the time in seconds where, if no
   * user interaction occurs with the device, it turns off. To keep the device always
   * on, pass 0. This method only works under PalmOS. The integer returned is
   * the previous auto-off time in seconds. Remember to restore it when the program exit.
   */
   public static int setDeviceAutoOff(int seconds)
   {
      return NativeMethods4JDK.vmSetDeviceAutoOff(seconds);
   }


   /**
   * Causes the VM to pause execution for the given number of milliseconds.
   * @param millis time to sleep in milliseconds
   */
   public static void sleep(int millis)
   {
      NativeMethods4JDK.vmSleep(millis);
   }

   /** returns the free memory in the device. This memory is where the database info is stored.
   @since SuperWaba2.0beta4 */
   public static int getDeviceFreeMemory()
   {
      return NativeMethods4JDK.vmGetDeviceFreeMemory();
   }

   /** Calls the Garbage Collector. Usualy, this isnt necessary; the gc is called everytime theres no more memory to allocate */
   public static void gc()
   {
      NativeMethods4JDK.vmGC();
   }

   /** Specify which system keys to intercept. use the SK_xxx constants <i>ored</i> together. eg: <code>interceptSystemKeys(HARD1 | HARD2 | FIND)</code>. You can also use SK_ALL or SK_NONE as parameter. Note that the constants differ from the ones in IKeys. in the event handler, you must check for the IKeys values (eg: IKeys.HARD1).
   @since SuperWaba2.0beta3 */
   public static void interceptSystemKeys(int keys)
   {
      interceptSystemKey = keys; // for desktop
      actionEqualsMenu = (keys & SK_ACTION) == 0; // if user wants the action key, disable its use to call the menu
      NativeMethods4JDK.vmInterceptSystemKeys(keys); // for device
   }

   /** sends a text to the "SuperWaba Debug Console" memo, preceeded with the current time stamp and followed by a line feed.
     * Note: if you're debugging a String that contains 0 in it, all chars after the first 0 will be ignored.
     */
   public static void debug(String s)
   {
      NativeMethods4JDK.vmDebug(s);
   }

   /** copies the specific string to the clipboard */
   public static void clipboardCopy(String s)
   {
      NativeMethods4JDK.vmClipboardCopy(s);
   }

   /** gets the last string from the clipboard. if none, returns "". */
   public static String clipboardPaste()
   {
      return NativeMethods4JDK.vmClipboardPaste();
   }

   /** Attaches a library to this application instance. This is useful if you
     * have a local or global library and want to make its classes available
     * to your app. At the initialization, SuperWaba scans all libraries in the
     * device and automatically attach them. But, if the library you're using have a
     * different creator ID or if you saved a bitmap into a file, you may need to
     * attach the file where it is located so you can instantiate its classes or
     * load it using new Image(filename).
     * <p>Note: this method only attaches SuperWaba Local Extension Libraries, ie,
     * files that have the app's creator id and type 'SWAX'. You may also specify
     * another creator id to the name by adding it like "name.crtr". This make
     * possible the use of a local library in another programs (eg: libA used by
     * progB and progC).
     * <b>In the dekstop</b> you must use "name.crtr", otherwise the library will not be loaded.
     * <p>After a library is added, its file cannot be modified.
     * <p>To see what libraries are in the device, you can use Catalog.listCatalogs()
     * and check the files that has type 'SWAX'.
     * <p>At the desktop, you may attach a library to load an image stored in a Catalog.
     * @return true if the library was found and attached, false otherwise.
     */
   public static boolean attachLibrary(String name)
   {
      return NativeMethods4JDK.vmAttachLibrary(name);
   }

   /** Attaches a native library to this application instance (only works at device). You must attach the native
     * library before use any native methods from your library. To learn how to create a
     * native library, you can purchase the tutorial. See at www.superwaba.org.
     */
   public static boolean attachNativeLibrary(String name)
   {
      return NativeMethods4JDK.vmAttachNativeLibrary(name);
   }

   /** Gets the current keys being pressed. You must set which keys you want to receive notification with
     * the interceptSystemKeys method. Note that those keys will not send notifications in the normal way;
     * you must keep pooling with this method to see when a state has changed.
     * @see #SK_PAGE_UP
     * @see #SK_PAGE_DOWN
     * @see #SK_PAGE_LEFT
     * @see #SK_PAGE_RIGHT
     * @see #SK_ACTION
     * @see #SK_HARD1
     * @see #SK_HARD2
     * @see #SK_HARD3
     * @see #SK_HARD4
     * @see #interceptSystemKeys
     */
   public static int getSystemKeysPressed()
   {
      return NativeMethods4JDK.vmGetSystemKeysPressed();
   }
}