/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;

/** This class is used by the Edit class when its type is CURRENCY and displays
  * a calculator with the four basic operations and a numeric pad. The user
  * can paste the value entered or the total value.
  * @author Guilherme Campos Hazan for the VPFinance program and granted for SuperWaba 2.0.
  */

public class Calculator extends Window // guich@200b4: i forgot to set this class public.
{
   private Edit edOper1,edOper2,edOrig;
   private Label lTotal;
   private PushButtonGroup pbgPaste,pbgAction,pbgOper,numericPad;
   private String []numerics = {"1","2","3","4","5","6","7","8","9","0",".","-"};
   private String answer;
   /** Specifies how many decimal places the entered value will be formatted with. 
     * @since SuperWaba 4.01
     */
   public int decimalPlaces=4;

   public Calculator()
   {
      super("Calculator",RECT_BORDER); // with caption and borders
      highResPrepared = true;
      // test if we're running in Sony devices, with big fonts
      float factor = (float)fmH / 11f;
      int ww = factor == 1?110:(int)(factor * 95);// ? 190 : 110;
      int hh = factor == 1?113:(int)(factor * 106);// ? 200 : 113;
      setRect(CENTER,CENTER,ww,hh);
   }

   public void onStart()
   {
      if (Settings.isColor)
         setBackColor(new Color(0,240,160)); // before control definitions!

      String []opers = {" + "," - "," * "," / "," ^ "};
      String []pastes = {"Paste Op 1","Paste Total"};
      String []actions = {"Reuse","Clear","Cancel"}; // guich@320_44: added reuse button
      add(edOper1=new Edit("999999999,99"), LEFT+2,TOP+2);
      edOper1.setMode(Edit.CURRENCY);//ValidChars(Edit.currencyCharsSet); // value - max = 999.999.999,99
      edOper1.setKeyboard(Edit.KBD_NONE);

      add(edOper2=new Edit("999999999,99"), SAME,AFTER+2);
      edOper2.setMode(Edit.CURRENCY);//ValidChars(Edit.currencyCharsSet); // value - max = 999.999.999,99
      edOper2.setKeyboard(Edit.KBD_NONE);

      add(pbgOper=new PushButtonGroup(opers,false,-1,-1,4,0,false,PushButtonGroup.BUTTON),LEFT+((edOper1.width-pbgOper.getPreferredWidth()) >> 1),AFTER+4);

      add(pbgPaste = new PushButtonGroup(pastes,false,-1,2,4,2,true,PushButtonGroup.BUTTON), LEFT+2,BOTTOM-2);
      add(pbgAction= new PushButtonGroup(actions,false,-1,2,4,3,true,PushButtonGroup.BUTTON), RIGHT-2,BOTTOM-2);

      add(lTotal=new Label(""));
      lTotal.setRect(LEFT+2,SAME,FILL-pbgAction.getPreferredWidth()-4,PREFERRED);

      // numeric pad
      add(numericPad=new PushButtonGroup(numerics,false,-1,-1,6,4,true,PushButtonGroup.BUTTON),RIGHT-2,TOP+2);
      numericPad.setFocusLess(true); // guich@320_32

      if (Settings.isColor)
      {
         pbgOper.setBackColor(new Color(200,200,20));
         pbgPaste.setBackColor(new Color(0,160,230));
         pbgAction.setBackColor(new Color(200,0,200));
         numericPad.setBackColor(new Color(0,204,192));
      }

      clear(false);
   }
   private boolean isOk(String s)
   {
      return s != null && s.length() > 0;
   }
   private boolean validate(Edit ed)
   {
      if (this != topMost) return true;
      String s = ed.getText();
      boolean ok = isOk(s);
      String msgErr = null;
      if (ed == edOper1 || ed == edOper2)
      {
         String f="";
         if (ok && ((f=Convert.toString(s,decimalPlaces)).length() <= 0 || f.equals("0")))
            msgErr = "err: "+s;
         else if (isOk(f)) s = f;
      }
      // if error, show it
      if (msgErr != null)
      {
         Sound.beep();
         ed.setText(msgErr);
         Graphics gg = ed.createGraphics();
         ed.onPaint(gg);
         gg.free();
         Vm.sleep(1500);
      }
      if (isOk(s)) ed.setText(s);
      return msgErr == null;
   }
   /** Gets the answer that the user selected to be pasted.
     * It can be the first operator, the total computed or null if canceled
     */
   public String getAnswer() // guich@200b4_193: get the 'pasted' answer
   {
      return answer;
   }
   /** Clears everything in this calculator. If you call it with (true) before the window is popped up, a trash will appear when the window is unpopped. */
   public void clear(boolean requestFocusOnOper1) // guich@320_19: made public
   {
      if (requestFocusOnOper1)
         setFocus(edOper1); // guich@320_19: added this.
      edOper1.setText("");
      edOper2.setText("");
      lTotal.setText("");
      answer = null;
   }
   public void onUnpop()
   {
      setFocus(this);
   }
   public void onPopup()
   {
      clear(false);
      Control c = topMost.getFocus();
      if (c instanceof Edit)
      {
         edOrig = (Edit)c;
         String s = edOrig.getText();
         if (s.length() > 0 && "+-0123456789".indexOf(s.charAt(0)) != -1) // guich@401_16: added support for + and changed the routine
            edOper1.setText(s);
      } else edOrig = null;
   }
   public void postPopup()
   {
      String text1 = edOper1.getText();
      setFocus((text1.length() > 0 && Convert.toDouble(text1) != 0)?edOper2:edOper1); // guich@320_20: check not only if the field isnt empty, but also if its value is valid.
   }
   public void onEvent(Event event)
   {
      switch (event.type)
      {
         case KeyEvent.KEY_PRESS:
            int key = ((KeyEvent)event).key;
            if (key == IKeys.CALC)
            {
               Sound.beep();
               unpop();
            }
            break;
         case ControlEvent.FOCUS_OUT:
            if (event.target instanceof Edit)
               validate((Edit)event.target);
            break;
         case ControlEvent.PRESSED:
            if (event.target == pbgPaste && pbgPaste.getSelected() != -1)
            {
               answer = pbgPaste.getSelected()==0?edOper1.getText():lTotal.getText();
               if (edOrig != null)
                  edOrig.setText(answer);
               unpop();
            }
            else
            if (event.target == pbgAction && pbgAction.getSelected() != -1)
            {
               switch (pbgAction.getSelected())
               {
                  case 0:
                     edOper1.setText(lTotal.getText());
                     edOper2.setText("");
                     lTotal.setText("");
                     break;
                  case 1:
                     clear(true);
                     break;
                  case 2:
                     unpop();
                     break;
               }
            }
            else
            if (event.target == pbgOper && validate(edOper1) && validate(edOper2))
            {
               double oper1 = Convert.toDouble(edOper1.getText());
               double oper2 = Convert.toDouble(edOper2.getText());
               double result = 0;
               switch (pbgOper.getSelected())
               {
                  case 0: result = oper1 + oper2; break;
                  case 1: result = oper1 - oper2; break;
                  case 2: result = oper1 * oper2; break;
                  case 3: result = oper1 / oper2; break;
                  case 4: result = Math.pow(oper1, oper2); break; // guich@320_40
               }
               lTotal.setText(Convert.toString(result,decimalPlaces));
            }
            else
            if (event.target == numericPad && getFocus() instanceof Edit)
            {
               int i = numericPad.getSelected();
               if (i != -1)
               {
                  KeyEvent ke = new KeyEvent();
                  ke.key = numerics[i].charAt(0);
                  ke.target = getFocus();
                  ke.type = KeyEvent.KEY_PRESS;
                  getFocus().onEvent(ke);
               }
            }
            break;
      }
   }
}
