/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2003 Guilherme Campos Hazan <support@superwaba.com.br>         *
 *  Copyright (C) 2000-2001 Allan C. Solomon                                     *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.fx.*;
import waba.sys.*;
import waba.util.*;

/**
 The Calendar class is a mimic of the palms Calendar.  It looks and functions the basically the same.
 It uses the Date class for all operations.<br>
 Guich made some modifications to it act like an Window. Instead of create a new instance (which consumes memory), you may
 use the Edit's static field <i>calendar</i>

 <b>Added by Allan C. Solomon</b>
 @version 1.0 16 Aug 2000
 @author Allan C. Solomon
*/

public class Calendar extends Window
{
   private int day=-1;
   private int month;
   private int year;
   private int sentDay=0;
   private int sentMonth;
   private int sentYear;
   private int lastCol=-1,lastRow=-1;
   private Graphics myg;
   private Font boldFont;
   private FontMetrics fmBold;
   private int cellW,cellH,clientY,arrowW;
   private Color yearColor = Color.BRIGHT;
   private Color monthColor = Color.BRIGHT;
   private Button btnToday, btnClear, btnCancel;
   /** True if the user had canceled without selecting */
   public boolean canceled;

   /**
     * Constructs Calendar set to the current day.
     */
   public Calendar()
   {
      super("",RECT_BORDER); // no title yet
      highResPrepared = true;
      //setDoubleBuffer(true);
      started = true; // avoid calling the onStart method

      // test if we're running in Sony devices, with big fonts
      cellW   = 16;
      cellH   = 16;
      clientY = 28;
      arrowW  = 5;
      if (fmH > 11)
      {
         float factor = (float)fmH / 11f;
         cellW   *= factor;
         cellH   *= factor;
         clientY *= factor;
         arrowW  *= factor;
      }
      if (Settings.isColor)
      {
         yearColor = new Color(0,200,200);
         monthColor = new Color(200,200,0);
         setBackColor(new Color(0,240,160));
      }
      btnToday = new Button("Today");
      btnClear = new Button("Clear");
      btnCancel = new Button("Cancel");
      setRect(CENTER,CENTER,6*2+cellW*7,clientY+6+cellH*6+btnCancel.getPreferredHeight()+6); // same gap in all corners
      onFontChanged();
      Button.commonGap = 2;
      add(btnToday, LEFT+3, BOTTOM-3);
      add(btnClear, CENTER, SAME);
      add(btnCancel, RIGHT-3, SAME);
      Button.commonGap = 0;
      if (Settings.isColor)
      {
         btnToday.setBackColor(new Color(0,200,160));
         btnClear.setBackColor(new Color(0,200,180));
         btnCancel.setBackColor(new Color(200,0,0));
      }
   }

   /**
   Returns selected Date.

   @return Date object set to the selected day.
   */
   public Date getSelectedDate()
   {
      if (day==-1)
         return null;
      Date date = new Date(day,month,year);
      return date;
   }

   /** sets the current day to the Date specified. if its null, sets the date to today. */
   public void setSelectedDate(Date d)
   {
      if (d == null) d = new Date();
      sentMonth = month = d.getMonth();
      sentYear = year = d.getYear();
      sentDay = d.getDay();
   }

   public void onEvent(Event event)
   {
      int rightArrowX = width-arrowW-2;
      int leftArrowX = rightArrowX-arrowW-4;
      int yrLeftArrowX = 2;
      int yrRightArrowX = yrLeftArrowX+4+arrowW;
      PenEvent penEvent;
      switch (event.type)
      {
         case ControlEvent.WINDOW_MOVED:
            if (myg != null) myg.free();
            myg = createGraphics();
            break;
         case KeyEvent.KEY_PRESS:
            if (((KeyEvent)event).key == IKeys.KEYBOARD_ABC || ((KeyEvent)event).key == IKeys.KEYBOARD_123) // closes this window without selecting any day
               unpop();
            break;
         case ControlEvent.PRESSED:
            if (event.target == btnToday)
            {
               setSelectedDate(null);
               day = sentDay; // guich@401_14
               unpop();  // guich@401_14
            } else
            if (event.target == btnClear)
            {
               day = -1;
               unpop();
            } else
            if (event.target == btnCancel)
            {
               canceled = true;
               unpop();
            }
            break;
         case PenEvent.PEN_DOWN:
            if (event.target != this) break; // guich@401_9: make sure is not a button
            penEvent = (PenEvent)event;
            lastCol = lastRow = -1;
            // in dates?
            if (penEvent.y >=clientY && penEvent.x >= 6 && penEvent.x <= width-6*2)
            {
               int col = ((penEvent.x - 6) / cellW);
               int row = ((penEvent.y - clientY) / cellH);
               Date date = new Date(1,month,year);
               int d = ((row * 7) - date.getDayOfWeek()) + (col+1);
               if (1 <= d && d <= date.getDaysInMonth())
               {
                  Color c = getBackColor();
                  myg.setForeColor(c);
                  myg.setBackColor(c.darker());
                  myg.eraseRect(6 +(col * cellW),clientY+(row *cellH),cellW,cellH);
                  lastCol = col;
                  lastRow = row;
               }
            }
            else
            // in title?
            if  (penEvent.y < (clientY>>1))
            {
               if (penEvent.x >= leftArrowX && penEvent.x <= leftArrowX+arrowW)
                  myg.drawArrow(leftArrowX,1,arrowW,Graphics.ARROW_LEFT,false,false,monthColor);

               if (penEvent.x >= rightArrowX && penEvent.x <= rightArrowX+arrowW)
                  myg.drawArrow(rightArrowX,1,arrowW,Graphics.ARROW_RIGHT,false,false,monthColor);

               if (penEvent.x >= yrLeftArrowX && penEvent.x <= yrLeftArrowX+arrowW)
                  myg.drawArrow(yrLeftArrowX,1,arrowW,Graphics.ARROW_LEFT,false,false,yearColor);

               if (penEvent.x >= yrRightArrowX && penEvent.x <= yrRightArrowX+arrowW)
                  myg.drawArrow(yrRightArrowX,1,arrowW,Graphics.ARROW_RIGHT,false,false,yearColor);
            }
            break;
         case PenEvent.PEN_UP:
            if (event.target != this) break; // guich@401_9: make sure is not a button
            penEvent = (PenEvent)event;
            // erase the last selected cell
            if (lastCol != -1 && lastRow != -1)
               onClickedOutside(-1,-1);
            // clicked on arrows?
            if (penEvent.y < (clientY>>1))
            {
               if (penEvent.x >= leftArrowX && penEvent.x <= leftArrowX+arrowW)
               {
                  month--;
                  if (month== 0)
                  {
                     year--;
                     month=12;
                  }
                  repaint();
               }
               else
               if (penEvent.x >= rightArrowX && penEvent.x <= rightArrowX+arrowW)
               {
                  month++;
                  if (month==13)
                  {
                     month=1;
                     year++;
                  }
                  repaint();
               }
               else
               if (penEvent.x >= yrLeftArrowX && penEvent.x <= yrLeftArrowX+arrowW)
               {
                  year--;
                  repaint();
               }
               else
               if (penEvent.x >= yrRightArrowX && penEvent.x <= yrRightArrowX+arrowW)
               {
                  year++;
                  repaint();
               }

            }
            else
            if (penEvent.y >= clientY && penEvent.x >=6 && penEvent.x <= width-6*2)
            {
               int col = ((penEvent.x -6) / cellW)+1;
               int row = ((penEvent.y-clientY) / cellH);
               Date date = new Date(1,month,year);
               int selectedDay = ((row * 7) - date.getDayOfWeek()) + col;
               day = -1;
               if (1 <= selectedDay && selectedDay <= date.getDaysInMonth())
               {
                  day = selectedDay;
                  unpop(); // closes this window
               }
            }
            break;
      }
   }
   protected boolean onClickedOutside(int x, int y)
   {
      int col = lastCol;
      int row = lastRow;
      Color c = getBackColor();
      myg.setForeColor(c.darker());
      myg.setBackColor(c);
      myg.eraseRect(6 +(col * cellW),clientY+(row * cellH),cellW,cellH);
      lastCol = lastRow = -1;
      return true;
   }

   public void onPaint(Graphics g)
   {
      //Draw title with appropriote month and year
      paintTitle(Date.getMonthName(month) + ' ' + year,g);

      g.setBackColor(backColor); // guich@400_40
      //Draw month arrows
      int rightArrowX = width-arrowW-2;
      int leftArrowX = rightArrowX-4-arrowW;
      g.drawArrow( leftArrowX,1,arrowW,Graphics.ARROW_LEFT, false,true,monthColor);
      g.drawArrow(rightArrowX,1,arrowW,Graphics.ARROW_RIGHT,false,true,monthColor);

      //Draw yr arrows
      leftArrowX = 2;
      rightArrowX = leftArrowX+4+arrowW;
      g.drawArrow( leftArrowX,1,arrowW,Graphics.ARROW_LEFT, false,true,yearColor);
      g.drawArrow(rightArrowX,1,arrowW,Graphics.ARROW_RIGHT,false,true,yearColor);

      g.setForeColor(Color.BLACK);
      //Draw day of week labels
      g.setFont(boldFont);
      String[] s = {"S","M","T","W","T","F","S"};
      int xPos = 6;
      int yPos = clientY-1;
      for(int x = 0; x < s.length; x++)
      {
         g.drawText(s[x],xPos+((cellW-fmBold.getTextWidth(s[x])) >> 1),clientY >> 1);
         xPos+=cellW;
      }
      g.setFont(font);

      //Draw horz. lines
      for(int i=0; i < 7; i++,yPos+=cellH)
         g.drawLine(6,yPos,6+7*cellW,yPos);

      //Draw Vert. lines
      xPos = 6;
      for(int i=0; i < 8; i++,xPos+=cellW)
         g.drawLine(xPos,clientY-1,xPos,6*cellH+clientY-1);

      //Draw dates out
      Date date = new Date(1,month,year);
      int cPos = date.getDayOfWeek()+1;

      yPos = clientY;
      xPos = 6+2 + cellW * (cPos-1);

      int n = date.getDaysInMonth();
      for(int i=1; i <= n; i++)
      {
         g.drawText(Convert.toString(i),xPos,yPos);
         if (cPos % 7 == 0)
         {
            yPos+=cellH;
            xPos=6+2;
            cPos=1;
         }
         else
         {
            xPos+=cellW;
            cPos++;
         }
      }

      // check for and select current day

      int d;
      if (sentDay == 0)
         d = date.getDay();
      else
         d = sentDay;
      date = new Date(1,month,year);

      if (month == sentMonth && year == sentYear)
      {
         int dayOfWeek = date.getDayOfWeek();
         int col = ((d + dayOfWeek) % 7)-1;
         int row = (d + dayOfWeek) / 7;
         if (col == -1)
         {
            col = 6;
            row--;
         }
         if (Settings.isColor) g.setBackColor(new Color(0,160,240));
         g.fillRect(7 +(col * cellW),clientY+(row * cellH),fm.getTextWidth(Convert.toString(d))+4,fm.getHeight());
         g.setForeColor(Color.WHITE);
         g.drawText(Convert.toString(d),6+2+(col * cellW),clientY+(row * cellH));
      }
   }

   protected void onPopup()
   {
      canceled = false;
      day = -1;
      setTitle(Date.getMonthName(month) + " " + (new Date().getYear()));
      if (sentDay == 0) setSelectedDate(null); // guich@300_45: makes the sentDate the default
      myg = createGraphics();
   }

   protected void onFontChanged()
   {
      boldFont = font.asBold();
      fmBold = getFontMetrics(boldFont);
   }

   public void onUnpop()
   {
      setFocus(this);
   }
}
