/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.fx.*;
import waba.sys.*;

/**
 * Check is a check control.
 * <p>
 * Here is an example showing a check being used:
 *
 * <pre>
 * public class MyProgram extends MainWindow
 * {
 *    Check check;
 *
 *    public void onStart()
 *    {
 *       add(check = new Check("Check me"), LEFT, AFTER);
 *    }
 *
 *    public void onEvent(Event event)
 *    {
 *       if (event.type == ControlEvent.PRESSED &&
 *       event.target == check)
 *       {
 *          ... handle check being pressed
 * </pre>
 */

public class Check extends Control
{
   private String text;
   //Font font; removed by guich
   private boolean checked;
   private Color cbColor, cfColor;
   private Color fourColors[] = new Color[4];
   private int textW;

   /** Creates a check control displaying the given text. */
   public Check(String text)
   {
      this.text = text;
      checked = false;
      textW = fm.getTextWidth(this.text);
   }

   /** Called by the system to pass events to the check control. */
   public void onEvent(Event event)
   {
      if (event.type == PenEvent.PEN_DOWN)
      {
         checked = !checked;
         repaint();
         PenEvent pe = (PenEvent)event;
         if (0 <= pe.x && pe.x < width && 0 <= pe.y && pe.y < height)
            postEvent(new ControlEvent(ControlEvent.PRESSED, this));
      }
   }

   /** sets the text that is displayed in the check. added by guich */
   public void setText(String text)
   {
      this.text = text;
      onFontChanged();
      repaint();
   }
   /** Gets the text displayed in the check. */
   public String getText()
   {
      return text;
   }

   /** Returns the checked state of the control. */
   public boolean getChecked()
   {
      return checked;
   }

   /** Sets the checked state of the control. */
   public void setChecked(boolean checked)
   {
      if (this.checked == checked)
         return;
      this.checked = checked;
      repaint();
   }

   /** returns the preffered width of this control. */
   public int getPreferredWidth()
   {
      return fm.getTextWidth(text)+getPreferredHeight() + 2;
   }

   /** returns the preffered height of this control. */
   public int getPreferredHeight()
   {
      if (fmH == 11)
         return Settings.uiStyle == Settings.WinCE ? (fmH+2) : fmH;
      return fm.getAscent();
   }

   protected void onColorsChanged(boolean colorsChanged)
   {
      cbColor = getBackColor().brighter();
      cfColor = getForeColor();
      Graphics.compute3dColors(enabled,backColor,foreColor,fourColors);
   }

   protected void onFontChanged()
   {
      textW = fm.getTextWidth(this.text);
   }

   /** Called by the system to draw the check control. */
   public void onPaint(Graphics g)
   {
      int wh = height;
      int xx,yy;

      // guich@200b4_126: repaint the background of the whole control
      g.setBackColor(backColor);
      g.fillRect(0,0,width,height);
      // square paint
      g.setBackColor(cbColor);
      g.fillRect(0,0,wh,wh); // guich@220_28
      g.draw3dRect(0,0,wh,wh,Graphics.R3D_CHECK,false,false,fourColors); // guich@220_28
      g.setForeColor(cfColor);

      if (checked)
      {
         int m = (wh >> 1) - 1;
         if (fmH >= 22) m--;
         yy = m;
         xx = Settings.uiStyle == Settings.PalmOS ? 2 : 3;
         wh -= xx;
         for (int i = xx; i < wh; i++)
         {
            g.drawLine(xx, yy, xx, yy + 2);
            xx++;
            if (i < m)
               yy++;
            else
               yy--;
         }
      }
      // draw label
      yy = (this.height - fmH) >> 1;
      xx = this.width - textW; // guich@300_69
      g.setBackColor(backColor); // guich@400_40: in wince the back color is used as the text background, so we restore it.
      g.drawText(text, xx, yy);
   }
}
