/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.fx.*;
import waba.sys.*;

/**
 * Control is the base class for user-interface objects.
 */

public class Control implements ISurface
{
   /** The control's x location */
   protected int x; // guich@200: VERY IMPORTANT: DONT CHANGE THE LOCATION OF THIS VARIABLE! it must be the # 1
   /** The control's y location */
   protected int y; // guich@200: VERY IMPORTANT: DONT CHANGE THE LOCATION OF THIS VARIABLE! it must be the # 2
   /** The control's width */
   protected int width; // guich@200: VERY IMPORTANT: DONT CHANGE THE LOCATION OF THIS VARIABLE! it must be the # 3
   /** The control's height */
   protected int height; // guich@200: VERY IMPORTANT: DONT CHANGE THE LOCATION OF THIS VARIABLE! it must be the # 4
   /** The parent of the control. */
   protected Container parent;
   /** The control's next sibling. */
   Control next;
   /** The control's previous sibling. */
   Control prev;
   /** True if the control is enabled (accepts events) or false if not */
   protected boolean enabled=true;
   /** The font used by the control. */
   protected Font font;
   /** The fontMetrics corresponding to the controls font. */
   public FontMetrics fm;
   /** True if the control is visible, false otherwise */
   protected boolean visible=true;
   /** Foreground color of this control. When the control is added, its foreground is set to be the same of the parent's. */
   protected Color foreColor;
   /** Background color of this control. When the control is added, its background is set to be the same of the parent's. */
   protected Color backColor;
   /** Application defined constant. This constant is not used; its only a placeholder for the application so it can set to any value it wants */
   public int appId;
   /** Constant used in params width and height in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: PREFERRED+2 or PREFERRED-1. */
   public static final int PREFERRED = 100000;
   /** Constant used in param x in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: LEFT+2 or LEFT-1. */
   public static final int LEFT      = 110000;
   /** Constant used in params x and y in setRect.  You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: CENTER+2 or CENTER-1. */
   public static final int CENTER    = 120000;
   /** Constant used in param x in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: RIGHT+2 or RIGHT-1. */
   public static final int RIGHT     = 130000;
   /** Constant used in param y in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: TOP+2 or TOP-1. */
   public static final int TOP       = 140000;
   /** Constant used in param y in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: BOTTOM+2 or BOTTOM-1. */
   public static final int BOTTOM    = 150000;
   /** Constant used in params width and height in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: FILL+2 or FILL-1. Note that FILL cannot be used with other x/y positioning constants. */
   public static final int FILL      = 160000;
   /** Constant used in param x/y in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: BEFORE+2 or BEFORE-1. */
   public static final int BEFORE    = 170000; // guich@200b4_100
   /** Constant used in params x/y/width/height in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: SAME+2 or SAME-1. */
   public static final int SAME      = 180000; // guich@200b4_100
   /** Constant used in param x/y in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: AFTER+2 or AFTER-1. */
   public static final int AFTER     = 190000; // guich@200b4_100
   /** Constant used in params width and height in setRect. You can use this constant added to a number to specify a increment/decrement to the calculated size. EG: FIT+2 or FIT-1. Note that FIT cannot be used with other x/y positioning constants. FIT will make the control fit between the desired position and the last added control position. */
   public static final int FIT       = 200000; // guich@330_4
   /** Equivalent to fm.getHeight(). Since this is widely used... */
   protected int fmH;

   /** Disabled fore and back colors */
   protected Color foreDis,backDis;

   /** x+width-1 */
   protected int x2;
   /** y+height-1 */
   protected int y2;
   /** If true, this control will receive pen and key events but will never gain focus */
   protected boolean focusLess;

   boolean eventsEnabled = true; // guich@200b4: now we use a different approach to disable events

   public static final int RANGE = 4000;

   // speedup some instance_of containers
   protected Container asContainer;
   protected Window asWindow;

   /** creates the font for this control as the same font of the MainWindow. */
   protected Control()
   {
      font = MainWindow.defaultFont;
      fm = getFontMetrics(font);
      fmH = fm.getHeight();
   }
   /**
    * Adds a timer to a control. Each time the timer ticks, a TIMER
    * event will be posted to the control. The timer does
    * not interrupt the program during its execution at the timer interval,
    * it is scheduled along with application events. The timer object
    * returned from this method can be passed to removeTimer() to
    * remove the timer. Under Windows, the timer has a minimum resolution
    * of 55ms due to the native Windows system clock resolution of 55ms. Under
    * Palm OS, the minimum timer resolution is 10ms.
    *
    * @param millis the timer tick interval in milliseconds
    * @see ControlEvent
    */
   public Timer addTimer(int millis)
   {
      return MainWindow._mainWindow.addTimer(this, millis);
   }

   /**
    * Removes a timer from a control. True is returned if the timer was
    * found and removed and false is returned if the timer could not be
    * found (meaning it was not active).
    */
   public boolean removeTimer(Timer timer)
   {
      return MainWindow._mainWindow.removeTimer(timer);
   }

   /** sets the font of this conrol. */
   public void setFont(Font font)
   {
      this.font = font;
      this.fm = getFontMetrics(font);
      this.fmH = fm.getHeight();
      onFontChanged();
   }

   /** Gets the font of this conrol. */
   public Font getFont()
   {
      return this.font;
   }

   /** Returns the preffered width of this control. */
   public int getPreferredWidth()
   {
      return 30;
   }

   /** Returns the preffered height of this control. */
   public int getPreferredHeight()
   {
      return fmH;
   }

   /** Returns the font metrics for a given font. */
   public FontMetrics getFontMetrics(Font font)
   {
      return MainWindow._mainWindow.getFontMetrics(font);
   }

   /** Sets or changes a control's position and size. */
   public void setRect(Rect r)
   {
      setRect(r.x,r.y,r.width,r.height,null);
   }

   /** Sets or changes a control's position and size.
     * The control used is the last control added.
     * You may use the constants
     * to make it easier. EG: here is an example of relative positioning:
     * <pre>
     * add(new Label("1"),CENTER,CENTER);
     * add(new Label("2"),AFTER,SAME);
     * add(new Label("3"),SAME,AFTER);
     * add(new Label("4"),BEFORE,SAME);
     * add(new Label("5"),BEFORE,BEFORE);
     * </pre>
     * You will see this on screen:
     * <pre>
     * 512
     *  43
     * </pre>
     * Note: add(control, x,y) does: <code>add(control); control.setRect(x,y,PREFERRED,PREFERRED);</code>
     * <p>
     * <b>Important! Always add the control to the container before doing a setRect.</b>
     * <p>The relative positioning does not work well if the control is placed outside screen bounds.
     */
   public void setRect(int x, int y, int width, int height)
   {
      setRect(x,y,width,height,null);
   }

   /** Sets or changes a control's position and size.
     * The relative positioning will be made with the given control (relative).
     * Note that in this case, only the SAME,BEFORE,AFTER are affected by the given control.
     * You may use the constants
     * to make it easier. EG: here is an example of relative positioning:
     * <p>
     * Important note: you can't use FILL/FIT with BEFORE/RIGHT/BOTTOM (for x,y).
     * <pre>
     * add(new Label("1"),CENTER,CENTER);
     * add(new Label("2"),AFTER,SAME);
     * add(new Label("3"),SAME,AFTER);
     * add(new Label("4"),BEFORE,SAME);
     * add(new Label("5"),BEFORE,BEFORE);
     * </pre>
     * You will see this on screen:
     * <pre>
     * 512
     *  43
     * </pre>
     * Note: add(control, x,y) does: <code>add(control); control.setRect(x,y,PREFERRED,PREFERRED);</code>
     * <p>
     * <b>Important! Always add the control to the container before doing a setRect.</b>
     * <p>The relative positioning does not work well if the control is placed outside screen bounds.
     */
   public void setRect(int x, int y, int width, int height, Control relative) // guich@200b4_138
   {
      if (parent != null || asWindow != null)
         repaint();

      if (x+y+width+height >= 50000) // are there any relative coords?
      {
         int px=0,py=0,pw,ph,lpx=0,lpy=0;
         // relative placement
         if (parent == null)
         {
            pw = Settings.screenWidth;
            ph = Settings.screenHeight;
         }
         else
         {
            Rect cli = parent.getClientRect();
            px = cli.x;
            py = cli.y;
            pw = cli.width;
            ph = cli.height;
            lpx = parent.lastX;
            lpy = parent.lastY;
            if (pw == 0 || ph == 0) Vm.debug("Warning! "+parent+" must have its bounds set before calling "+this+".setRect"); // guich@300_28
            if (relative != null)
            {
               parent.lastX = relative.x;
               parent.lastY = relative.y;
               parent.lastW = relative.width;
               parent.lastH = relative.height;
            }
            else
            {
               if (parent.lastX == -1) parent.lastX = px;
               if (parent.lastY == -1) parent.lastY = py;
            }
         }
         boolean prefW = false;
         boolean prefH = false;

         if ((PREFERRED-RANGE) <= width  && width  <= (PREFERRED+RANGE)) {width  = getPreferredWidth() +(width -PREFERRED); prefW = true;}
         if ((PREFERRED-RANGE) <= height && height <= (PREFERRED+RANGE)) {height = getPreferredHeight()+(height-PREFERRED); prefH = true;}
         if ((SAME   -RANGE)   <= width  && width  <= (SAME   +RANGE) && parent != null) width  = parent.lastW + width-SAME;
         if ((SAME   -RANGE)   <= height && height <= (SAME   +RANGE) && parent != null) height = parent.lastH + height-SAME;
         if (x > 50000)
         {
            if ((LEFT      -RANGE) <= x && x <= (LEFT   +RANGE)) x = px + x-LEFT; else
            if ((RIGHT     -RANGE) <= x && x <= (RIGHT  +RANGE)) x = px + pw-width + x-RIGHT; else
            if ((CENTER    -RANGE) <= x && x <= (CENTER +RANGE)) x = px + ((pw-width) >> 1) + x-CENTER;
            if (parent != null)
            {
               if ((AFTER  -RANGE) <= x && x <= (AFTER  +RANGE)) x = parent.lastX + parent.lastW + x-AFTER; else
               if ((BEFORE -RANGE) <= x && x <= (BEFORE +RANGE)) x = parent.lastX - width + x-BEFORE; else
               if ((SAME   -RANGE) <= x && x <= (SAME   +RANGE)) x = parent.lastX + x-SAME;
            }
         }
         if (y > 50000)
         {
            if ((TOP       -RANGE) <= y && y <= (TOP    +RANGE)) y = py + y-TOP; else
            if ((BOTTOM    -RANGE) <= y && y <= (BOTTOM +RANGE)) y = py + ph-height + y-BOTTOM; else
            if ((CENTER    -RANGE) <= y && y <= (CENTER +RANGE)) y = py + ((ph-height) >> 1) + y-CENTER;
            if (parent != null)
            {
               if ((AFTER  -RANGE) <= y && y <= (AFTER  +RANGE)) y = parent.lastY + parent.lastH + y-AFTER; else
               if ((BEFORE -RANGE) <= y && y <= (BEFORE +RANGE)) y = parent.lastY - height + y-BEFORE; else
               if ((SAME   -RANGE) <= y && y <= (SAME   +RANGE)) y = parent.lastY + y-SAME;
            }
         }
         if (width > 50000)
         {
            if ((FILL-RANGE) <= width && width  <= (FILL+RANGE)) width = pw - x + px + width-FILL;
            if (parent != null)
               if ((FIT -RANGE) <= width && width  <= (FIT +RANGE)) width = lpx - x + width-FIT;
         }
         if (height > 50000)
         {
            if ((FILL-RANGE) <= height && height <= (FILL+RANGE)) height = ph - y + py + height-FILL;
            if (parent != null)
               if ((FIT -RANGE) <= height && height <= (FIT +RANGE)) height = lpy - y + height-FIT;
         }

         if (x+y+width+height > 50000)
         {
            x=y=0;
            width=height=10;
            Vm.debug("To use AFTER/BEFORE/SAME you must add first the control "+toString()+" to the parent container.");
         } else
         if ((x+y) < -50000) // guich@300_27
         {
            if (x < -50000)
               Vm.debug("You can't use FILL with BEFORE or RIGHT for control "+toString());
            else
               Vm.debug("You can't use FILL with BEFORE or BOTTOM for control "+toString());
         }
         else // guich@220_50: make sure that the control does not goes beyond the parent's bounds if PREFERRED was choosen
         if (parent != null)
         {
            if (prefW && x+width > px+pw)  {int w = px+pw-x; if (w > 0) width  = w;} // only change if the width is going to be > 0!
            if (prefH && y+height > py+ph) {int h = py+ph-y; if (h > 0) height = h;}
         }
      }
      //
      if (asWindow != null && fmH > 11 && !asWindow.highResPrepared) // guich@240_20
      {
         float factor = (float)fmH / 11f;
         width  *= factor;
         height *= factor;
         x = (Settings.screenWidth-width) >> 1;
         y = (Settings.screenHeight-height) >> 1;
      }
      this.x = x;
      this.y = y;
      this.width = width;
      this.height = height;
      this.x2 = x+width-1;
      this.y2 = y+height-1;
      if (parent != null)
      {
         parent.lastX = x; // guich@200b4_100: save last positions
         parent.lastY = y;
         parent.lastW = width;
         parent.lastH = height;
      }
      if (visible && parent != null || asWindow != null)
         repaint();
      onBoundsChanged();
      if (asContainer != null && !asContainer.started) // guich@340_15;
      {
         asContainer.started = true;
         asContainer.onStart();
      }
   }

   /** Returns the current size (width,height) of this control */
   public Coord getSize()
   {
      return new Coord(width,height);
   }

   /** Returns the current position (x,y) of this control */
   public Coord getPos()
   {
      return new Coord(x,y);
   }

   /** Shows or "hides" this control. The "hide" works setting the control's size to zero. it remains attached to its container. unhidding restores the original size. you can change the controls size when hided if you want. calls repaint. */
   public void setVisible(boolean visible)
   {
      if (visible != this.visible)
      {
         this.visible = visible;
         repaint();
         if (parent != null && parent.parentWindow != null) // guich@200b4_163: make sure, if we hide controls, that the area will be erased and the control will be 'hided'
            parent.parentWindow.eraseBackgroundNow = true;
      }
   }

   /** Returns true if this control is visible, false otherwise */
   public boolean isVisible()
   {
      return visible;
   }

   /**
    * Returns a copy of the control's rectangle. A control's rectangle
    * defines its location and size.
    */
   public Rect getRect()
   {
      return new Rect(this.x, this.y, this.width, this.height);
   }

   /** Returns the absolute coordinates of this control relative to the MainWindow. */
   public Rect getAbsoluteRect() // guich@102: changed name from getRelativeRect to getAbsoluteRect.
   {
      Rect r = getRect();
      Control c = parent;
      while (c != null)
      {
         r.x += c.x;
         r.y += c.y;
         c = c.parent;
      }
      return r;
   }
   /** Returns the control's parent Window or topMost if there's no parent (eg: the control still not added to any container). */
   public Window getParentWindow()
   {
      Container c = parent;
      if (c != null && c.parentWindow != null)
         return c.parentWindow; // cklim@200b4_175

      // guich@200final_17: using again the old algorithm
      while (c != null && c.asWindow == null) // guich@320_2: changed instanceof to .asWindow
         c = c.parent;
      return c!=null ? (Window)c : Window.getTopMost();
   }

   /** Returns the control's parent container. */
   public Container getParent()
   {
      return parent;
   }

   /** Returns the next child in the parent's list of controls. */
   public Control getNext()
   {
      return next;
   }

   /**
    * Returns true if the given x and y coordinate in the parent's
    * coordinate system is contained within this control.
    */
   public boolean contains(int x, int y)
   {
      return this.x <= x && x <= this.x2 && this.y <= y && y <= this.y2;
   }

   /** Marks the control for repaint. Important note: when you call repaint, the absolute
     * area of the control regarding its parent Window is invalidated (marked
     * for repaint); then, the next time a event (a keypress, a timer, a pen event)
     * occurs, the screen is updated. If you call repaint and the control isn't
     * effectively repainted, you have 2 choices: call Window._doPaint(), or,
     * use Control.repaintNow.
     */
   public void repaint() // guich@200: does repaint makes sense for hidden controls?
   {
      int x = 0;
      int y = 0;
      Control c = this;
      while (c.asWindow == null)
      {
         x += c.x;
         y += c.y;
         c = c.parent;
         if (c == null)
            return;
      }
      c.asWindow.damageRect(x, y, this.width, this.height);
   }

   /** Redraws the control immediately. If this control is a Window, the whole window area is
     * marked for repaint (useful if you're removing some controls from a container)
     * @since SuperWaba2.0beta4release3
     */
   public void repaintNow()
   {
      if (asWindow != null) // guich@200b4: if this is a Window, make sure that the parent Window is painted too
      {
         repaint(); // make sure the whole area is marked to be repainted
         asWindow._doPaint(0,0,width,height);
      }
      else
      {
         Graphics g = createGraphics();
         if (g != null)
         {
            onPaint(g);
            if (asContainer != null) // else, if this is a Container, be sure to repaint all its childrens
               asContainer.paintChildren(g, 0,0,width,height);
         }
      }
   }

   /**
     * Creates a Graphics object which can be used to draw in the control.
     * This method finds the surface associated with the control, creates
     * a graphics associated with it and translates the graphics to the
     * origin of the control. It does <b>not</b> set a clipping rectangle on the
     * graphics.
     */
   public Graphics createGraphics()
   {
      int x = 0;
      int y = 0;
      Control c = this;
      while (c.asWindow == null)
      {
         x += c.x;
         y += c.y;
         c = c.parent;
         if (c == null)
            return null;
      }
      Graphics g = new Graphics(this);
      g.translate(x + c.asWindow.x, y + c.asWindow.y); // guich@102: translate into win coords
      g.setFont(font);
      return g;
   }

   /**
     * Posts an event. The event pass will be posted to this control
     * and all the parent controls of this control (all the containers
     * this control is within).
     * @see Event
     */
   public void postEvent(Event event)
   {
      if (!enabled || !eventsEnabled) return;

      Control c,cp;

      c = this;
      while (c != null)
      {
         cp = c.parent;
         c.onEvent(event);
         if (event.consumed) // guich@200b4_132: if the control consumed the event, stop propagation.
         {
            event.consumed = false; // set to false again bc some controls reuse event objects
            break;
         }
         if (cp != c.parent) // guich@200b4: has the parent changed? if yes, dont broadcast the event anymore.
            break;
         c = cp;
      }
   }

   /** Sets if this control can or not accept events.
     * It changes the appearance of many controls to indicate they are disabled. Also,
     * this method must be used after adding the control to its parent container.
     */
   public void setEnabled(boolean enabled)
   {
      if (enabled != this.enabled)
      {
         this.enabled = enabled;
         onColorsChanged(false);
         repaint(); // now the controls have different l&f for disabled states
      }
   }

   /** Returns if this control can or not accept events */
   public boolean isEnabled()
   {
      return this.enabled;
   }

   /**
    * Called to process key, pen, control and other posted events.
    * @param event the event to process
    * @see Event
    * @see KeyEvent
    * @see PenEvent
    */
   public void onEvent(Event event)
   {
   }

   /**
    * Called to draw the control. When this method is called, the graphics
    * object passed has been translated into the coordinate system of the
    * control and the area behind the control has
    * already been painted. The background is painted by the top-level
    * window control.
    * @param g the graphics object for drawing
    * @see Graphics
    */
   public void onPaint(Graphics g)
   {
   }

   /** Called after an setRect. */
   protected void onBoundsChanged()
   {
   }

   /** Called after a setEnabled, setForeColor and setBackColor and when a control has
     * been added to a Container. If colorsChanged
     * is true, it was called from setForeColor/setBackColor/Container.add; otherwise, it was
     * called from setEnabled
     */
   protected void onColorsChanged(boolean colorsChanged) // guich@200b4_152
   {
   }

   /** Called after the window has finished a paint. */
   protected void onWindowPaintFinished()
   {
   }

   /** Called after a setFont */
   protected void onFontChanged() // guich@200b4_153
   {
   }

   /** Set the background and foreground colors at once */
   public void setBackForeColors(Color back, Color fore) // guich@200b4_170
   {
      this.backColor = back;
      this.backDis = backColor.darker();
      this.foreColor = fore;
      this.foreDis = foreColor.brighter();
      onColorsChanged(true);
   }
   /** Set the foreground color of this control.
   @since SuperWaba2.0 */
   public void setForeColor(Color c)
   {
      this.foreColor = c;
      this.foreDis = foreColor.brighter();
      onColorsChanged(true);
   }

   /** Set the background color of this control.
   @since SuperWaba2.0 */
   public void setBackColor(Color c)
   {
      this.backColor = c;
      this.backDis = backColor.darker();
      onColorsChanged(true);
   }

   /** Get the desired foreground color of this control.
   @since SuperWaba2.0 */
   public Color getForeColor()
   {
      return enabled?foreColor:foreDis;
   }

   /** Get the desired background color of this control.
   @since SuperWaba2.0 */
   public Color getBackColor()
   {
      // note: if were in a white back color, return the color without darking
      return (enabled || parent == null)?backColor:(backColor.equ == Color.WHITE.equ && backColor.equ == parent.getBackColor().equ)?Color.WHITE:backDis;
   }

   /** Return true if the parent of this Control is added to somewhere.
    * Some containers, like the TabPanel, has n child containers, but only one is added
    * at a time. With this method, you can discover if your container is the one available.
    */
   public boolean isDisplayed()
   {
      Control c = this;
      while (c.asWindow == null)
      {
         c = c.parent;
         if (c == null)
            return false; // not added yet
      }
      return true;
   }

   /** Sets the focus to this control */
   public void requestFocus()
   {
      getParentWindow().setFocus(this);
   }

   /** Sets this control to be focusless. If this control is a container, sets
     * all its children to be focusless too.
     */
   public void setFocusLess(boolean on)
   {
      this.focusLess = on;
      if (asContainer != null)
      {
         Control c = asContainer.children;
         while (c != null)
         {
            if (c.asContainer != null)
               c.setFocusLess(on);
            else
               c.focusLess = on;
            c = c.next;
         }
      }
   }
}