/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

/**
 * Event is the base class for all events.
 */

public class Event
{
   /** The type of event. */
   public int type;

   /**
   * The target of the event. For user-interface events, this is the
   * control the event is associated with.
   */
   public Object target;

   /**
   * The event's timestamp.
   * @see waba.sys.Vm#getTimeStamp()
   */
   public int timeStamp;

   /**
   * Set to true to break the event propagation up to the parents
   */
   public boolean consumed;

   /** constructs a new event based in the given parameters. */
   public Event(int type, Object target, int timeStamp)
   {
      this.type = type;
      this.target = target;
      this.timeStamp = timeStamp;
   }
   /** Constructs a new event with the current time stamp */
   public Event()
   {
      timeStamp = waba.sys.Vm.getTimeStamp(); // guich@200b4: removed this from the other subclasses and putted here.
   }
   /** Updates the timeStamp parameter with the current time */
   public void touch()
   {
      timeStamp = waba.sys.Vm.getTimeStamp(); // guich@200b4: removed this from the other subclasses and putted here.
   }

   /** The next available event number */
   private static int nextAvailableEventId = 10000;
   /** Register a new event with the system. Use it to avoid
     * conflict of events created by multiple Applications. */
   public static int getNextAvailableEventId()
   {
      return nextAvailableEventId++;
   }

   /** Returns true if there is an event available in the VM event queue. */
   public static boolean isAvailable()
   {
      return NativeMethods4JDK.eventIsAvailable();
   }
}
