/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

/**
 * IKeys is an interface containing values for special keys and modifiers.
 * <p>
 * Below is an example of IKeys being used.
 *
 * <pre>
 * public void onEvent(Event event)
 * {
 *  if (event.type == KeyEvent.KEY_PRESS)
 *  {
 *     KeyEvent ke = (KeyEvent)event;
 *     if ((ke.modifiers & CONTROL) > 0)
 *        ... control key was held down
 *     if (ke.key == IKeys.PAGE_DOWN)
 *        ... page down key pressed
 *     if (ke.key == IKeys.PAGE_UP)
 *        ... page up key pressed
 * </pre>
 */
public interface IKeys
{
   // NOTE: The SuperWaba VM indexes directly to these values

   /** modifier for alt key */
   public static final int ALT       = (1 << 0);
   /** modifier for control key */
   public static final int CONTROL   = (1 << 1);
   /** modifier for shift key */
   public static final int SHIFT     = (1 << 2);

   /** special key */
   public static final int PAGE_UP   = 75000;
   /** special key */
   public static final int PAGE_DOWN = 75001;
   /** special key */
   public static final int HOME      = 75002;
   /** special key */
   public static final int END       = 75003;
   /** special key */
   public static final int UP        = 75004;
   /** special key */
   public static final int DOWN      = 75005;
   /** special key */
   public static final int LEFT      = 75006;
   /** special key */
   public static final int RIGHT     = 75007;
   /** special key */
   public static final int INSERT    = 75008;
   /** special key */
   public static final int ENTER     = 75009;
   /** special key */
   public static final int TAB       = 75010;
   /** special key */
   public static final int BACKSPACE = 75011;
   /** special key */
   public static final int ESCAPE    = 75012;
   /** special key */
   public static final int DELETE    = 75013;
   /** special key */
   public static final int MENU      = 75014;
   /** special key */
   public static final int COMMAND   = 75015;
   // below added by guich
   public static final int KEYBOARD_ABC   = 76000;
   public static final int KEYBOARD_123   = 76001;
   public static final int KEYBOARD       = 76002;
   public static final int HARD1          = 76003;
   public static final int HARD2          = 76004;
   public static final int HARD3          = 76005;
   public static final int HARD4          = 76006;
   /** CALC button under PalmOS, NOTES button under WindowsCE */
   public static final int CALC           = 76007;
   /** Valid only on PalmOS */
   public static final int FIND           = 76008;
   /** Valid only on PalmOS */
   public static final int LAUNCH         = 76009;
   /** Valid only on WindowsCE. If not registered by the user, it is replaced by MENU. */
   public static final int ACTION         = 76010;
   /** Valid only on PalmOS */
   public static final int CONTRAST       = 76011;
   /** Valid only on PalmOS */
   public static final int CLOCK          = 76012;
   /** Supported only in systems that has a Jogdial. Currently, only in Sony. In Handera you must check for PAGE_UP key instead. */
   public static final int JOG_UP         = 76030;
   /** Supported only in systems that has a Jogdial. Currently, only in Sony. In Handera you must check for PAGE_DOWN key instead. */
   public static final int JOG_DOWN       = 76031;
   /** Supported only in systems that has a Jogdial. Currently, only in Sony. In Handera you must check for ENTER key instead. */
   public static final int JOG_PUSH       = 76032;
   /** Supported only in systems that has a Jogdial (and not all systems supports this state). Currently, only in Sony. */
   public static final int JOG_RELEASE    = 76033;
   /** Supported only in systems that has a Jogdial (and not all systems supports this state). Currently, only in Sony. */
   public static final int JOG_PUSHREPEAT = 76034;
   /** Supported only in systems that has a Jogdial (and not all systems supports this state). Currently, only in Sony. */
   public static final int JOG_PUSHUP     = 76035;
   /** Supported only in systems that has a Jogdial (and not all systems supports this state). Currently, only in Sony. */
   public static final int JOG_PUSHDOWN   = 76036;
}