/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;

/** a popup keyboard to be used with the Edit and MultiEdit class.
<p>
Yehuda Miron added Hebrew support to this class. To use it, you need to have hebrew support for your palm in the normal way  (Piloc from palmgear,
or buy it from kalanit distributer).
<p>
Superwaba uses the normal hebrew in the palm, you don't need any special fonts for
using hebrew in the superwaba. if you write an application in superwaba and need the hebrew keyboard there
is a global variable you need to set at the start of your application (once only !)
<p>
<code>Keyboard.setLanguage(Keyboard.LANG_HEBREW);
<p>
and the popup keyboard will start with hebrew and english characters.
the normal ascii codes for hebrew are used in superwaba (not hacking is needed).
<p>
Setting hebrew string in superwaba is done in the normal way:
String s1 = "";
<p>
Note that you can also extend this multi-language feature to other languages. To do
it, just create one more constant for the language and change the setAuxKeys to change
letters you want.
*/

public class Keyboard extends Window // guich@102
{
   private Control destControl;
   private Rect destRect;
   private Container destCont;
   private PushButtonGroup pbs[] = new PushButtonGroup[6];
   private KeyEvent ke = new KeyEvent();
   private int inside=-1;
   private byte []rows = {4,4,4,3,1,1};        // rows in each PushButtonGroup
   public static int []px = {LEFT+2,CENTER,RIGHT-2,CENTER, LEFT+2,   RIGHT-2};      // x position of each PushButtonGroup
   public static int []py = {AFTER+2,   SAME,  SAME,   AFTER+4,BOTTOM-2, BOTTOM-2}; // y position of each PushButtonGroup
   public static byte []ig = {6,2,6,4,4,5};          // internal gap

   /** Event issued by this keybard on the onUnpop method. The edit must call pushPosState. */
   public static final int KEYBOARD_ON_UNPOP = Event.getNextAvailableEventId(); // guich@320_34
   /** Event issued by this keybard on the postUnpop method. The edit must call popPosState. */
   public static final int KEYBOARD_POST_UNPOP = Event.getNextAvailableEventId(); // guich@320_34

   private boolean isCaps, isShift;
   /** Used to access the names member. */
   public final static int NUMERIC_PAD = 0;
   /** Used to access the names member. */
   public final static int SYMBOLS_PAD = 1;
   /** Used to access the names member. */
   public final static int ACCENT_PAD = 2;
   /** Used to access the names member. */
   public final static int TEXT_PAD = 3;
   /** Used to access the names member. */
   public final static int CAPS_PAD = 4;
   /** Used to access the names member. */
   public final static int SPECIAL_PAD = 5;

   // ym@200b4_80: support for other languages
   /** Used in the setLanguage method. */
   public final static int LANG_NORMAL  = 1;
   /** Used in the setLanguage method. */
   public final static int LANG_HEBREW  = 2;
   /** Used in the setLanguage method. */
   public final static int LANG_GERMAN  = 3;
   /** Used in the setLanguage method. */
   public final static int LANG_TURKISH  = 4;
   /** Used in the setLanguage method. */
   public final static int LANG_RUSSIAN_OR_ANOTHER = 5;

   public static String names[][] = // guich@400_4: made static and public
   {
      {"1","2","3","4","5","6","7","8","9",".","0",","},
      {"+","$","&","(",")","-","#","@","[","]","*","%","|","{","}","/","=","\\","<",">"},
      {"","","","","","","","","","","","","","","",""},
      {"q","w","e","r","t","y","u","i","o","p","'","_","a","s","d","f","g","h","j","k","l",":","~","^","z","x","c","v","b","n","m","!","?",";","\"","`"},
      {"Caps","Shift"},
      {"                ","","Done"},
   };
   public Keyboard()
   {
      super("Popup Keyboard",RECT_BORDER);
      highResPrepared = true;

      if (Settings.isColor)
         setBackColor(new Color(0,240,160)); // before control definitions!

      if (Settings.screenWidth == 320)
         names[SPECIAL_PAD][0] = "           "; // guich@400_53
      setDoubleBuffer(true);
      ke.type = KeyEvent.KEY_PRESS;
   }

   /** Sets the current language of the Keyboard. Note that this must be
     * set before any Keyboard creation */
   public static void setLanguage(int lang) // ym@200b4_80
   {
     // ym@200b4_80 and guich@200b4_106
     int i;

     switch (lang)
     {
       case LANG_HEBREW:
          // create the new accentuated keys
          String [] newKeys = new String[28]; // last key intentionally left null
          for (i=0; i < 27; i++)
             newKeys[i] = String.valueOf((char)(224+i));
          names[ACCENT_PAD] = newKeys;
          // set the new positions and gaps of the PushButtonGroup's to make all fit
          px[1] = AFTER+2;
          ig[NUMERIC_PAD] = 4;
          ig[ACCENT_PAD]  = 2;
          break;
       case LANG_GERMAN: // bruckner@300_53
          names[ACCENT_PAD] = new String[]{String.valueOf((char)0xe4),String.valueOf((char)0xc4),String.valueOf((char)0xf6),String.valueOf((char)0xd6),String.valueOf((char)0xfc),String.valueOf((char)0xdc),String.valueOf((char)0xdf),null};
          px[NUMERIC_PAD] = LEFT+8;
          px[ACCENT_PAD] = RIGHT-12;
          names[TEXT_PAD][5] = "z"; // oresch@350_12: swap the z/y keys
          names[TEXT_PAD][24] = "y";
          break;
       case LANG_RUSSIAN_OR_ANOTHER:
          Vm.debug("The Russian language is not implemented yet. Can you provide the new char set?"); // what is the map ?
          break;
       case LANG_TURKISH: // DoganSoykan@340_25
          names[ACCENT_PAD] = new String[]{String.valueOf((char)0xC7), String.valueOf((char)0xD0), String.valueOf((char)0xD6), String.valueOf((char)0xDC), String.valueOf((char)0xDD), String.valueOf((char)0xDE), String.valueOf((char)0xE7), String.valueOf((char)0xF0), String.valueOf((char)0xF6), String.valueOf((char)0xFC), String.valueOf((char)0xFD), String.valueOf((char)0xFE)};
          px[NUMERIC_PAD] = LEFT+2;
          px[ACCENT_PAD] = RIGHT-2;
          break;
       default:
         // normal
     }
   }

   protected void onWindowPaintFinished()
   {
      destControl.requestFocus();
   }


   protected void onPopup()
   {
      isCaps = false;
      isShift = false;

      destControl = Window.topMost.getFocus();
      destRect = destControl.getRect();
      destCont = destControl.getParent();

      if (pbs[0] == null)
      {
         Color colors[] = null;
         if (Settings.isColor)
            colors = new Color[]
            {
               new Color(60,160,200),
               new Color(0,160,230),
               new Color(0,220,180),
               new Color(90,180,220),
               new Color(200,0,200),
               new Color(200,200,0)
            };
         for (int i =0; i < pbs.length; i++)
         {
            add(pbs[i] = new PushButtonGroup(names[i], false, -1, i!=SPECIAL_PAD?-1:2, ig[i], rows[i], i < 4, (i != 4) ? PushButtonGroup.BUTTON : PushButtonGroup.CHECK));
            if (colors != null) pbs[i].setBackColor(colors[i]);
            pbs[i].appId = i;
            pbs[i].setFocusLess(true); // guich@320_32
         }
      }
      add(destControl);
      // repositions the window and the controls based in the control's size. Note that a MultiEdit has a different height of an Edit, so this must be dinamically computed
      int hh = pbs[NUMERIC_PAD].getPreferredHeight() + pbs[TEXT_PAD].getPreferredHeight() + pbs[CAPS_PAD].getPreferredHeight() + destControl.getPreferredHeight() + getPreferredHeight() + 14;
      int ww = pbs[TEXT_PAD].getPreferredWidth()+10;
      setRect(CENTER,CENTER,ww,hh);
      destControl.setRect(CENTER,TOP+2,Math.min(destRect.width,width-10),PREFERRED);
      for (int i =0; i < pbs.length; i++)
         pbs[i].setRect(px[i],py[i], PREFERRED, PREFERRED);

      // since we changed our size, save again the window behind
      if (imgCovered != null) {imgCovered.free(); imgCovered = null;}
      saveBehind();

      ke.target = destControl;
   }


   protected void onUnpop()
   {
      destControl.onEvent(new ControlEvent(KEYBOARD_ON_UNPOP, destControl)); // guich@320_34
      destControl.setRect(destRect.x,destRect.y,destRect.width,destRect.height);
      destCont.add(destControl);
   }

   protected void postUnpop()
   {
      destControl.onEvent(new ControlEvent(KEYBOARD_POST_UNPOP, destControl)); // guich@320_34
   }

   public void onEvent(Event event)
   {
      switch (event.type)
      {
         case KeyEvent.KEY_PRESS:
            switch (((KeyEvent)event).key)
            {
               case IKeys.KEYBOARD_ABC:
               case IKeys.KEYBOARD_123:
                  unpop();
                  break;
            }
            break;
         case ControlEvent.PRESSED:
            if (event.target instanceof PushButtonGroup)
            {
               PushButtonGroup pb = (PushButtonGroup)event.target;
               int sel = pb.getSelected();
               if (sel >= 0) // guich@200b4_105: fix ArrayIndexOutOfBounds exception
               {
                  String st = names[pb.appId][pb.getSelected()];
                  if (pb.appId == SPECIAL_PAD || pb.appId == CAPS_PAD) // special char?
                  {
                     int key = -1;
                     switch (st.charAt(0))
                     {
                        case 'D': pb.setSelected(-1); unpop(); break;
                        case ' ': key = ' '; break;
                        case 'S': isShift = !isShift; isCaps = false; break;
                        case 'C': isCaps =  !isCaps; isShift = false; break;
                        case '': // guich@200b4: this char is not being recognized correctly in the SPT1700.
                        default : key = IKeys.BACKSPACE; break;
                     }
                     if (key != -1) insertKey(key);
                  }
                  else
                  {
                     char c = st.charAt(0);
                     if (isShift || isCaps) c = Convert.toUpperCase(c);
                     insertKey(c);
                     cancelShift();
                  }
               }
            }
            break;
      }
   }
   private void cancelShift()
   {
      if (isShift)
      {
         isShift = false;
         pbs[CAPS_PAD].setSelected(-1);
      }
   }
   private void insertKey(int key)
   {
      ke.key = key;
      destControl.onEvent(ke);
   }
   /** Calls super.loadBehind(true) to make sure the image is released */
   protected void loadBehind(boolean release)
   {
      super.loadBehind(true);
   }
}