/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.fx.*;
import waba.sys.*;

/**
 * Radio is a radio control.
 * <p>
 * Here is an example showing a radio being used:
 *
 * <pre>
 * public class MyProgram extends MainWindow
 * {
 * Radio radioMale, radioFemale;
 *
 * public void onStart()
 * {
 *  RadioGroup rg = new RadioGroup();
 *  add(radioMale = new Radio("Male", rg), LEFT, AFTER);
 *  add(radioFemale = new Radio("Female", rg), AFTER+2, SAME);
 *  rg.clicked(radioMale); // activate the specified one.
 * }
 *
 * public void onEvent(Event event)
 * {
 *  if (event.type == ControlEvent.PRESSED &&
 *      event.target == radioMale)
 *   {
 *   ... handle radio Male being pressed
 * </pre>
 * Radios can be grouped together in a RadioGroup. See the
 * RadioGroup class for more details.
 */

public class Radio extends Control
{
   private String text;
   private boolean checked;
   private RadioGroup radioGroup;
   private Color colors[] = new Color[4];
   private Color cColor,bColor;
   private int textW;

   /** Creates a radio control displaying the given text. */
   public Radio(String text)
   {
      this.text = text;
      checked = false;
      textW = fm.getTextWidth(this.text);
   }

   /** Creates a radio control with the given text attached to the given RadioGroup */
   public Radio(String text, RadioGroup radioGroup)
   {
      this(text);
      this.radioGroup = radioGroup;
      radioGroup.add(this);
   }

   /** Returns the RadioGroup that this radio belongs to, or null
       if none.
     */
   public RadioGroup getRadioGroup()
   {
      return radioGroup;
   }

   /** sets the text. added by guich */
   public void setText(String text)
   {
      this.text = text;
      repaint();
      onFontChanged();
   }
   /** Gets the text displayed in the radio. */
   public String getText()
   {
      return text;
   }

   /** Returns the checked state of the control. */
   public boolean getChecked()
   {
      return checked;
   }

   /** Sets the checked state of the control. */
   public void setChecked(boolean checked)
   {
      if (this.checked == checked)
         return;
      this.checked = checked;
      repaint();
   }

   /** returns the preffered width of this control. added by guich */
   public int getPreferredWidth()
   {
      return fm.getTextWidth(text)+getPreferredHeight() + (fmH>=22 ? 0 : 1);
   }

   /** returns the preffered height of this control. added by guich */
   public int getPreferredHeight()
   {
      if (fmH == 11)
         return Settings.uiStyle == Settings.WinCE ? (fmH+2) : fmH;
      return fm.getAscent();
   }

   /** Called by the system to pass events to the radio control. */
   public void onEvent(Event event)
   {
      if (event.type == PenEvent.PEN_DOWN)
      {
         checked = !checked;
         repaint();
         PenEvent pe = (PenEvent)event;
         if (0 <= pe.x && pe.x < width && 0 <= pe.y && pe.y < height)
         {
            if (radioGroup != null) radioGroup.clicked(this);
            postEvent(new ControlEvent(ControlEvent.PRESSED, this));
         }
      }
   }

   private final static int coords1[] =
   {
	   // dark grey top
	   4, 0, 7, 0,
	   2, 1, 3, 1,
	   8, 1, 9, 1,
	   // dark grey left
	   0, 4, 0, 7,
	   1, 2, 1, 3,
	   1, 8, 1, 9,
	   // black top
	   4, 1, 7, 1,
	   2, 2, 3, 2,
	   8, 2, 9, 2,
	   // black left
	   1, 4, 1, 7,
	   2, 3, 2, 3,
	   2, 8, 2, 8,
	   // light grey bottom
	   2, 9, 3, 9,
	   8, 9, 9, 9,
	   4, 10, 7, 10,
	   // light grey right
	   9, 3, 9, 3,
	   9, 8, 9, 8,
	   10, 4, 10, 7,
	   // bottom white
	   2, 10, 3, 10,
	   8, 10, 9, 10,
	   4, 11, 7, 11,
	   // right white
	   10, 2, 10, 3,
	   10, 8, 10, 9,
	   11, 4, 11, 7
	};
   private final static int coords2[] =
   {
	   // dark grey top
	   5, 0, 9, 0,
	   3, 1, 4, 1,
	   10, 1, 11, 1,
	   12, 2, 12, 2,
	   // dark grey left
	   0, 5, 0, 9,
	   1, 3, 1, 4,
	   1, 10, 1, 11,
	   2, 2, 2, 2,
	   // black top
	   5, 1, 9, 1,
	   3, 2, 4, 2,
	   10, 2, 11, 2,
	   12, 3, 12, 3,
	   // black left
	   1, 5, 1, 9,
	   2, 3, 2, 4,
	   2, 10, 2, 11,
	   2, 10, 2, 11,
	   // light grey bottom
	   3, 12, 4, 12,
	   5, 13, 9, 13,
	   10, 12, 11, 12,
	   10, 12, 11, 12,
	   // light grey right
	   12, 10, 12, 11,
	   13, 5, 13, 9,
	   12, 4, 12, 4,
	   12, 4, 12, 4,
	   // bottom white
	   2, 12, 2, 12,
	   3, 13, 4, 13,
	   5, 14, 9, 14,
	   10, 13, 11, 13,
	   // right white
	   12, 12, 12, 12,
	   13, 10, 13, 11,
	   14, 5, 14, 9,
	   13, 3, 13, 4
	};

   protected void onColorsChanged(boolean colorsChanged)
   {
      cColor = getForeColor();
      bColor = getBackColor().brighter();
      if (Settings.uiStyle == Settings.PalmOS)
         colors[1] = colors[2] = cColor;
      else
      {
         if (Settings.maxColors==4 && !enabled)
            colors[3] = null;
         else
         {
            colors[0] = colors[2] = cColor.brighter();
            colors[3] = bColor;
         }
         colors[1] = cColor;
      }
	}

   protected void onFontChanged()
   {
      textW = fm.getTextWidth(this.text);
   }

   /** Called by the system to draw the radio control. */
   public void onPaint(Graphics g)
   {
      // guich@200b4_126: erase the back always
      g.setBackColor(backColor);
      g.fillRect(0,0,width,height);

      boolean isPalmOS = Settings.uiStyle == Settings.PalmOS;
      boolean big = fmH >= 22;
      int kk = big?8:6; // number of elements per arc
      int xx = isPalmOS?-1:0;
      int yy = (this.height - (big?14:10)) >> 1;
	   g.translate(xx,yy);

      int []coords = big?coords2:coords1;
	   // white center
      g.setBackColor(bColor);
      if (big)
         g.fillCircle(7,7,7);
      else
         g.fillCircle(5,6,4);

      // 3d borders
	   int i = 0;
	   for (int j = 0; j < 4; j++)
	      if (colors[j] != null)
	      {
		      g.setForeColor(colors[j]);
		      for (int k = 0; k < kk; k++)
			      g.drawLine(coords[i++], coords[i++], coords[i++], coords[i++]);
	      }
	      else i += kk << 2;

      // checked
      g.setForeColor(cColor);
      if (checked)
      {
         g.setBackColor(cColor);
         if (big)
            g.fillCircle(7,7,3);
         else
         {
		      g.fillRect(5, 4, 2, 4);
		      g.drawLine(4, 5, 4, 6);
		      g.drawLine(7, 5, 7, 6);
		   }
      }
	   g.translate(-xx,-yy);

      // draw label
      yy = (this.height - fmH) >> 1;
      xx = this.width - textW; // guich@300_69
      g.setBackColor(backColor); // guich@400_40: in wince the back color is used as the text background, so we restore it.
      g.drawText(text, xx, yy);
   }
}
