/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                   Created with SuperWaba                                      *
 *                  http://www.superwaba.org                                     *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *     Guilherme Campos Hazan                                                    *
 *     Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                 *
 *     Rio de Janeiro / RJ - Brazil                                              *
 *     Cep: 22050-000                                                            *
 *     E-mail: support@superwaba.com.br                                          *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.util.*;

/**
 * RadioGroup is a Radio control grouper.
 * It handles the click on a radio, unchecking the last selected one.
 * <p>Use it as:
 * <pre>
 * RadioGroup rg = new RadioGroup();
 * Radio r1 = new Radio("radio 1",rg);
 * Radio r2 = new Radio("radio 2",rg);
 * Radio r3 = new Radio("radio 3",rg);
 * rg.clicked(r1); // activate r1 only
 * </pre>
 * Note: there is no need (and you can't) add a RadioGroup to a container,
 * since a RadioGroup is a class that doesn't extend Control.
 * <p>
 */

public class RadioGroup
{
   Vector members = new Vector();

   public void add(Radio newMember)
   {
      members.add(newMember);
   }

   public void remove(Radio oldMember)
   {
      members.del(oldMember);
   }

   /** Called by the Radio when a click was made */
   public void clicked(Radio who)
   {
      int n = members.size();
      Radio r;
      for (int i = 0; i < n; i++)
         (r=(Radio)members.items[i]).setChecked(r == who); // keep active only who is calling me.
   }
}