/*********************************************************************************
 *  SuperWaba Virtual Machine, version 4                                         *
 *  Copyright (C) 2000-2003 Guilherme Campos Hazan <support@superwaba.com.br>    *
 *  Copyright (C) 1998, 1999 Wabasoft <www.wabasoft.com>                         *
 *  All Rights Reserved                                                          *
 *                                                                               *
 *  This library and virtual machine is free software; you can redistribute      *
 *  it and/or modify it under the terms of the Amended GNU Lesser General        *
 *  Public License distributed with this software.                               *
 *                                                                               *
 *  This library and virtual machine is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         *
 *                                                                               *
 *  For the purposes of the SuperWaba software we request that software using    *
 *  or linking to the SuperWaba virtual machine or its libraries display the     *
 *  following notice:                                                            *
 *                                                                               *
 *                Created with SuperWaba                                         *
 *               http://www.superwaba.org                                        *
 *                                                                               *
 *  Please see the software license located at SuperWabaSDK/license.txt          *
 *  for more details.                                                            *
 *                                                                               *
 *  You should have received a copy of the License along with this software;     *
 *  if not, write to                                                             *
 *                                                                               *
 *  Guilherme Campos Hazan                                                       *
 *  Av. Nossa Senhora de Copacabana 728 apto 605 - Copacabana                    *
 *  Rio de Janeiro / RJ - Brazil                                                 *
 *  Cep: 22050-000                                                               *
 *  E-mail: support@superwaba.com.br                                             *
 *                                                                               *
 *********************************************************************************/

package waba.ui;

import waba.fx.*;
import waba.sys.*;
import waba.util.*;
import waba.io.*;
//import superwaba.ext.xplat.ui.MultiEdit;

/**
 * Welcome is the welcome application.
 * <p>
 * This is the default program run when none is specified or when the VM needs
 * a program to run to show that the VM is functioning on a device.
 */

public class Welcome extends MainWindow
{
   private MenuBar mbar;

   public Welcome()
   {
      super("Welcome",TAB_ONLY_BORDER);
      setDoubleBuffer(true);
      //Settings.setPalmOSStyle(true);
   }

   public void onStart()
   {
      String fileMenu[] =
      {
        "File",
        "*Beam...",
        "-",
        "Exit",
      };
      String optionsMenu[] =
      {
         "VM",
         "?Audible GC",
         "?Dump Memory Stats",
      };
      String helpMenu[] =
      {
        "?",
        "About",
        "-",
        "License",
      };
      setMenuBar(mbar = new MenuBar(new String[][]{fileMenu,optionsMenu,helpMenu}));
      if (Settings.isColor)
      {
         mbar.setBackForeColors(new Color(0,0,255), Color.WHITE);
         mbar.setCursorColor(new Color(100,100,255));
         mbar.setBorderStyle(NO_BORDER);
         mbar.setPopColors(new Color(0,120,255),new Color(0,255,255),null); // use the default cursor color for the popup menu (last null param)
      }

      // now display the UI elements
      Label l = new Label("SuperWaba Virtual Machine");
      l.setInvert(true);
      l.setFont(new Font("SW",Font.BOLD,14));
      add(l,CENTER,TOP+15);

      add(new Label("Version "+Settings.versionStr+" for " + Settings.platform),CENTER,AFTER+3);
      add(new Label("Virtual Machine installed and ready"),CENTER,AFTER+12);

      // gets a list of the catalogs installed
      Vector vProgs = new Vector();
      if (!Settings.onDevice)
         vProgs.add("This program");
      else
      {
         String []catalogs = Catalog.listCatalogs(0,Convert.chars2int("Wrp2"));
         int n = catalogs.length;
         for (int i = 0; i < n; i++)
            if (catalogs[i] != null) // not a system catalog?
            {
               int m = catalogs[i].indexOf('!',0) != -1 ? 1 : 0; // if end with !, remove it
               vProgs.add(catalogs[i].substring(0,catalogs[i].indexOf('.')-m));
            }
         vProgs.del("SuperWaba"); // delete SW, since its not a program, but the VM itself.
         vProgs.qsort();
      }

      if (vProgs.size() == 0)
         add(new Label("No SuperWaba programs installed"),CENTER,CENTER+10);
      else
      {
         add(new Label("List of currently installed programs:"),CENTER,CENTER+10);
         ComboBox cb = new ComboBox(vProgs.toObjectArray());
         add(cb,CENTER,AFTER+5);
         cb.select(0);
         add(new Label("(They can't be launched from here!)"),CENTER,AFTER+2);
      }

      add(new Label("Click title or menu for options"),CENTER,BOTTOM-2);
      //add(new superwaba.ext.xplat.ui.MultiEdit(6,1), LEFT,BOTTOM);

      // recover last appSettings status for the menu items
      if (Settings.appSettings != null && Settings.appSettings.length() == 2)
      {
         boolean audibleGC = Settings.appSettings.charAt(0) == '1';
         boolean dumpMem   = Settings.appSettings.charAt(1) == '1';
         if (audibleGC)
            mbar.setChecked(101,true);
         if (dumpMem)
            mbar.setChecked(102,true);
      }
   }

   public void onExit()
   {
      if (mbar == null) return; // only in debugs
      boolean audibleGC = mbar.isChecked(101);
      boolean dumpMem   = mbar.isChecked(102);
      if (!audibleGC && !dumpMem)
         Settings.appSettings = null; // no options, delete appSettings
      else
         Settings.appSettings = (audibleGC?"1":"0") +
                                (dumpMem  ?"1":"0");
      Vm.debug("onExit");
   }

   public void onEvent(Event e)
   {
      switch (e.type)
      {
         case ControlEvent.WINDOW_CLOSED:
            if (e.target == mbar)
               switch (mbar.getSelectedMenuItem())
               {
                  case -1: break;
                  case 3: // Exit
                     exit(0);
                     break;
                  case 201: // About
                     popupModal(new MessageBox("About","SuperWaba Virtual Machine|www.superwaba.org|Copyright (c) 2000-2003|Guilherme Campos Hazan||SuperWaba is an enhanced version|of the Waba Virtual Machine|Copyright (c) 1998,1999 WabaSoft|www.wabasoft.com"));
                     break;
                  case 203: // License
                     popupModal(new MessageBox("License","Please read it at|www.superwaba.org/license.txt"));
                     break;
               }
            break;
      }
   }
}
